/* 
 * @file    app_comm_cmds.c
 * @author  M91406
 * @date    February 27, 2024
 */

#include <stdint.h> // include standard integer number data types
#include "config/apps.h" // include applicaiton layer header file to get access to other task data
#include "config/hal.h" // include hardware abstraction layer header file 
#include "app_comm_protocol.h" // include proprietary protocol layer header file
#include "svc/svc_comm_engine.h" // include the physical protocol layer driver header file

/* User-defined UART data buffers */

/***********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @struct  PsuStatusData_s
 * @brief   Captures most recent status information of the power supply control task
 * @details
 *  The PSU Status Data structure captures the most recent status information
 *  of the power supply control task. Each value of the data members are 
 *  converted into a generic value format to ease data reception and formatting.
 **********************************************************************************/

struct PsuCommData_s {
    int8_t  Exponent; // 10s exponent of fractional mantissa
    int16_t Mantissa; // Fractional mantissa of the physical quantity
}__attribute__((packed));
typedef struct PsuCommData_s PsuCommData_t;

struct PsuStatusData_s {
    PsuCommData_t VHighVoltage;
    PsuCommData_t VLowVoltage;
    PsuCommData_t ISense;
    PsuCommData_t IAverage;
    PsuCommData_t Temperature;
    PsuCommData_t Status;
    PsuCommData_t OpStateId;
    PsuCommData_t FaultList;
    PsuCommData_t VoltageError;
    PsuCommData_t ReferenceCurrent;
    PsuCommData_t CurrentError;
    PsuCommData_t DutyCycle;
    PsuCommData_t Period;
}__attribute__((aligned));
typedef struct PsuStatusData_s PsuStatusData_t;

PsuStatusData_t statusData = {
    
    .VHighVoltage.Exponent = (int8_t)V48_COMM_DIGITS,
    .VLowVoltage.Exponent = (int8_t)V12_COMM_DIGITS,
    .ISense.Exponent = (int8_t)ISNS_COMM_DIGITS,
    .IAverage.Exponent = (int8_t)ISNS_COMM_DIGITS,
    .Temperature.Exponent = (int8_t)TEMP_COMM_DIGITS,
    .Status.Exponent = (int8_t)0,
    .OpStateId.Exponent = (int8_t)0,
    .FaultList.Exponent = (int8_t)0,
    
    // -- debugging information ----------------
    .VoltageError.Exponent = (int8_t)0,
    .ReferenceCurrent.Exponent = (int8_t)0,
    .CurrentError.Exponent = (int8_t)0,
    .DutyCycle.Exponent = (int8_t)0,
    .Period.Exponent = (int8_t)0
};

int statusData_size = ((sizeof(statusData)/sizeof(statusData.VHighVoltage)) * 3);

/* Private variables and function prototypes */
bool skipStatusFrame = false;
bool sendRawData = false;

static int __attribute__((used)) CommSendResponseFrame8(COMM_FRAME_t* frame, uint8_t data);
static int __attribute__((used)) CommSendResponseFrame16(COMM_FRAME_t* frame, uint16_t data);
static int __attribute__((used)) CommSendResponseFrame32(COMM_FRAME_t* frame, uint32_t data);

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a low-voltage port voltage reference from a host
**********************************************************************************/
int CommSetLvReference(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        uint16_t dataX = (((uint16_t)CmdBuffer[0] << 8) | CmdBuffer[1]);
        dataX = ((dataX > V12_MAX) ? V12_MAX : dataX); // clamp max value
        dataX = ((dataX < V12_MIN) ? V12_MIN : dataX); // clamp min value
        psu.Properties.LVReference = dataX; // set voltage reference

    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent low-voltage port reference voltage to a host 
**********************************************************************************/
int CommGetLvReference(COMM_FRAME_t* frame)
{
    return(CommSendResponseFrame16(frame, (uint16_t)psu.Properties.LVReference));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a high-voltage port voltage reference from a host
**********************************************************************************/
int CommSetHvReference(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        uint16_t dataX = (((uint16_t)CmdBuffer[0] << 8) | CmdBuffer[1]);
        dataX = ((dataX > V48_MAX) ? V48_MAX : dataX); // clamp max value
        dataX = ((dataX < V48_MIN) ? V48_MIN : dataX); // clamp min value
        psu.Properties.HVReference = dataX; // set voltage reference
        
    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent high-voltage port reference voltage to a host 
**********************************************************************************/
int CommGetHvReference(COMM_FRAME_t* frame)
{
    return(CommSendResponseFrame16(frame, (uint16_t)psu.Properties.HVReference));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a new step-down mode current limit from a host
**********************************************************************************/
int CommSetCurrentLimit(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        int16_t dataX = (int16_t)(((uint16_t)CmdBuffer[0] << 8) | (uint16_t)CmdBuffer[1]);
        dataX = ((dataX > ISNS_MAX) ? ISNS_MAX : dataX); // clamp max value
        dataX = ((dataX < ISNS_MIN) ? ISNS_MIN : dataX); // clamp min value
        psu.Control.VLoop.Properties.Maximum = (int16_t)dataX; // set current limit
        
    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent step-down mode current limit to a host
**********************************************************************************/
int CommGetCurrentLimit(COMM_FRAME_t* frame)
{
    return(CommSendResponseFrame16(frame, (uint16_t)psu.Control.VLoop.Properties.Maximum));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a new step-up mode current limit from a host
**********************************************************************************/
int CommSetAltCurrentLimit(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        int16_t dataX = (int16_t)(((uint16_t)CmdBuffer[0] << 8) | (uint16_t)CmdBuffer[1]);
        dataX = ((dataX > ISNS_MAX) ? ISNS_MAX : dataX); // clamp max value
        dataX = ((dataX < ISNS_MIN) ? ISNS_MIN : dataX); // clamp min value
        psu.Control.VLoop.Properties.AltMaximum = dataX; // set current limit
        
    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent step-up mode current limit to a host
**********************************************************************************/
int CommGetAltCurrentLimit(COMM_FRAME_t* frame)
{
    return(CommSendResponseFrame16(frame, (uint16_t)psu.Control.VLoop.Properties.AltMaximum));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets the new ENABLE state of the power supply
**********************************************************************************/
int CommSetPowerSupplyEnable(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (1 == frame->DataLen)
    {
        bool dataX = ((CmdBuffer[0] == 0xFF) ? true : false);
        psu.Properties.Autorun = 0;
        if ((false == dataX) && (false == psu.Status.bits.IsSuspended))
        { appPowerSupply_Suspend(); }
        else if ((true == dataX) && (true == psu.Status.bits.IsSuspended))
        { appPowerSupply_Resume(); psu.Properties.Go = true; }

    } else { frame->DataError++; }

    return(retval);
}
    
/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns ENABLE state of the power supply
**********************************************************************************/
int CommGetPowerSupplyEnable(COMM_FRAME_t* frame)
{
    uint8_t data = (uint8_t)((true == psu.Status.bits.IsSuspended) ? 0x00 : 0xFF);
    return(CommSendResponseFrame8(frame, data));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a new PWM duty cycle from a host
**********************************************************************************/
int CommSetDutyCycle(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        uint16_t dataX = (((uint16_t)CmdBuffer[0] << 8) | CmdBuffer[1]);
        dataX = ((dataX > PWM_DC_MAX) ? PWM_DC_MAX : dataX); // clamp max value
        dataX = ((dataX < PWM_DC_MIN) ? PWM_DC_MIN : dataX); // clamp min value

        // ToDo: Implement intelligent duty cycle adjustment
        //*psu.Control.ILoop[0].Controller->Ports.Target.handle = dataX; // set current limit
        
    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent PWM duty cycle to a host
**********************************************************************************/
int CommGetDutyCycle(COMM_FRAME_t* frame)
{
    uint16_t dataX = (uint16_t)*psu.Control.ILoop[0].Controller->Ports.Target.handle;
    return(CommSendResponseFrame16(frame, dataX));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets the new Open Loop state of the power supply
**********************************************************************************/
int CommSetOpenLoopEnable(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (1 == frame->DataLen)
    {
        bool dataX = ((CmdBuffer[0] == 0xFF) ? true : false);
        psu.Properties.EnforcedOpenLoop = dataX;

        if (psu.StateId.OpStateId == BDBB_OPSTATE_ONLINE)
        {
            if (true == dataX) BdbbLoopControl_Off(&psu);
            else BdbbLoopControl_On(&psu); // ToDo: DANGER !!!
        }
        
    } else { frame->DataError++; }

    return(retval);
}
    
/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the Open Loop state of the power supply
**********************************************************************************/
int CommGetOpenLoopEnable(COMM_FRAME_t* frame)
{
    uint8_t data = (uint8_t)((true == psu.Properties.EnforcedOpenLoop) ? 0xFF : 0x00);
    return(CommSendResponseFrame8(frame, data));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Receives and sets a new conversion mode with swap mode from a host
 * 
 * @code {.c}
 *  Template for Toggling ConversionMode 
 *  psu.Properties.HotSwapEnable = true; 
 *  psu.Properties.ConversionMode ^= 1;  
 *  psu.Properties.Swap = 1;             
 * @endcode
**********************************************************************************/
int CommSetConversionSwap(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    if (2 == frame->DataLen) // validating declared data length
    {
        bool dataX = (bool)((CmdBuffer[0] == 0xFF) ? true : false);
        bool dataY = (bool)((CmdBuffer[1] == 0xFF) ? true : false);

        // Set/clear HotSwap bit
        psu.Properties.HotSwapEnable = dataY;

        // If ConversionMode setting different from recent
        if (psu.Properties.ConversionMode != (bool)dataX)
        { 
            // Initiate swap of ConversionMode
            psu.Properties.ConversionMode = (bool)dataX; 
            psu.Properties.Swap = 1;
        }
        
    } else { frame->DataError++; retval=0; }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns the recent conversion mode with swap mode to a host
**********************************************************************************/
int CommGetConversionSwap(COMM_FRAME_t* frame)
{
    uint8_t dataX = (uint8_t)((true == psu.Properties.HotSwapEnable) ? 0xFF : 0x00);
    uint8_t dataY = (uint8_t)((true == psu.Properties.ConversionMode) ? 0xFF : 0x00);
    uint16_t data = ((((uint16_t)dataY) << 8) | (uint16_t)dataX);
    return(CommSendResponseFrame16(frame, data));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Sends periodic status message transmission Enable state
**********************************************************************************/
int CommSetStatusFrameEnable(COMM_FRAME_t* frame)
{
    int retval = 1;

    if (1 == frame->DataLen)
    {
        skipStatusFrame = ((CmdBuffer[0] == 0xFF) ? true : false);
        
    } else { frame->DataError++; retval=0; }

    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns periodic status message transmission Enable state
**********************************************************************************/
int CommGetStatusFrameEnable(COMM_FRAME_t* frame)
{
    uint8_t data = (uint8_t)((true == skipStatusFrame) ? 0xFF : 0x00);
    return(CommSendResponseFrame8(frame, data));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Switches data format of status messages between raw and normalized format
**********************************************************************************/
int CommSetRawDataFrameEnable(COMM_FRAME_t* frame)
{
    int retval = 1;

    if (1 == frame->DataLen)
    {
        sendRawData = ((CmdBuffer[0] == 0xFF) ? true : false);
        
    } else { frame->DataError++; retval=0; }

    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns if data format of status messages is raw or normalized
**********************************************************************************/
int CommGetRawDataFrameEnable(COMM_FRAME_t* frame)
{
    uint8_t data = (uint8_t)((true == sendRawData) ? 0xFF : 0x00);
    return(CommSendResponseFrame8(frame, data));
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Returns power supply status message (single shot)
**********************************************************************************/
int CommGetStatusFrame(COMM_FRAME_t* frame)
{
    int retval = 1;
    
    // Do not send frame if power supply status frames are sent anyways
    if (false == skipStatusFrame)
    { return(retval); }
    
    // Enforce sending of power supply status frame
    if (0 == frame->DataLen)
    {
        frame->Ready = false;       // Clear READY bit to allow status frame processing
        skipStatusFrame = false;    // Disable skipping of status frame 
        retval &= appComm_SendStatusFrame(); // Send status frame
        skipStatusFrame = true;     // Enable skipping of status frame 
        
    } else { frame->DataError++; retval=0; }

    return(retval);
}

/* ****************************************************************************** */
/* ****************************************************************************** */
/* ****************************************************************************** */

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Builds and sends a 8-bit response data frame
**********************************************************************************/
static int CommSendResponseFrame8(COMM_FRAME_t* frame, uint8_t data)
{
    int retval = 1;
    
    if (0 == frame->DataLen) // validating declared data length
    {
        // Build data frame to be sent
        uint8_t msg[6];
        msg[0] = COMM_SOF;         // add Start Of Frame (SOF)
        msg[1] = frame->Command;   // set command ID
        msg[2] = 0x01;             // set data length
        msg[3] = data;             // place value byte
        msg[4] = COMM_EOM;         // add End Of Message (EOM)
        msg[5] = COMM_EOF;         // add End Of Frame (EOF)

        // Send 8-bit data response
//        retval &= plib33c_Uart_SendData(&Comm, (uint8_t*)&msg[0], 1+5);
//        retval &= plib33c_Uart_SendBufferDma(&Comm, (uint8_t*)&msg, sizeof(msg));
        retval &= CommStageToTransmit((uint8_t*)&msg, sizeof(msg));

    } else { frame->DataError++; retval=0; }

    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Builds and sends a 16-bit response data frame
**********************************************************************************/
static int CommSendResponseFrame16(COMM_FRAME_t* frame, uint16_t data)
{
    int retval = 1;
    
    if (0 == frame->DataLen) // validating declared data length
    {
        // Swap data bytes
        if (true == Comm.Status.ByteSwap)
        { data = SwapWordBytes((uint16_t)data); }
        
        // Build data frame to be sent
        uint8_t msg[7];
        msg[0] = COMM_SOF;         // add Start Of Frame (SOF)
        msg[1] = frame->Command;   // set command ID
        msg[2] = 0x02;             // set data length
        msg[3] = (uint8_t)((data & 0xFF00) >> 8);  // place value byte
        msg[4] = (uint8_t)(data & 0x00FF);         // place value byte
        msg[5] = COMM_EOM;         // add End Of Message (EOM)
        msg[6] = COMM_EOF;         // add End Of Frame (EOF)

        // Send 16-bit data response
//        retval &= plib33c_Uart_SendData(&Comm, (uint8_t*)&msg[0], 2+5);
//        retval &= plib33c_Uart_SendBufferDma(&Comm, (uint8_t*)&msg, sizeof(msg));
        retval &= CommStageToTransmit((uint8_t*)&msg, sizeof(msg));
        
        
    } else { frame->DataError++; retval=0; }

    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Builds and sends a 32-bit response data frame
**********************************************************************************/
static int __attribute__((used)) CommSendResponseFrame32(COMM_FRAME_t* frame, uint32_t data)
{
    int retval = 1;
    
    if (0 == frame->DataLen) // validating declared data length
    {
        // Swap data bytes
        if (true == Comm.Status.ByteSwap)
        { 
            uint32_t dataX = 0;
            dataX =  ((data & 0x000000FF) << 24);  // place value lower low byte
            dataX |= ((data & 0x0000FF00) <<  8);  // place value lower high byte
            dataX |= ((data & 0x00FF0000) >>  8);  // place value upper low byte
            dataX |= ((data & 0xFF000000) >> 24);  // place value upper high byte
            data  = dataX; // Override data with swapped order
        }
        
        // Build data frame to be sent
        uint8_t msg[9] = { 
            COMM_SOF,                               // add Start Of Frame (SOF)
            frame->Command,                         // set command ID
            0x04,                                   // set data length
            (uint8_t)((data & 0xFF000000) >> 24),   // place value high byte
            (uint8_t)((data & 0x00FF0000) >> 16),   // place value low byte
            (uint8_t)((data & 0x0000FF00) >>  8),   // place value high byte
            (uint8_t) (data & 0x000000FF),          // place value low byte
            COMM_EOM,                               // add End Of Message (EOM)
            COMM_EOF                                // add End Of Frame (EOF)
        };

        // Send LVReference
        //retval &= plib33c_Uart_SendData(&Comm, (uint8_t*)&msg, sizeof(msg));   
        //retval &= plib33c_Uart_SendBufferDma(&Comm, (uint8_t*)&msg, sizeof(msg));
        retval &= CommStageToTransmit((uint8_t*)&msg, sizeof(msg));
    
    } else { frame->DataError++; retval=0; }

    return(retval);
}


static inline int appComm_UpdateStatusFrame(void)
{
    int retval = 1;
    
    int16_t dat = psu.Data.VHighVoltage;
    if (false == sendRawData) { dat = (int16_t)(__builtin_mulss(dat, V48_COMM_FACTOR) >> (15 - V48_COMM_SCALER)); }
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.VHighVoltage.Mantissa = dat;
    
    dat = psu.Data.VLowVoltage;
    if (false == sendRawData) { dat = (int16_t)(__builtin_mulss(dat, V12_COMM_FACTOR) >> (15 - V12_COMM_SCALER)); }
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.VLowVoltage.Mantissa = dat;
    
    dat = psu.Data.ISense[0];
    if (false == sendRawData) { dat = (int16_t)(__builtin_mulss(dat, ISNS_COMM_FACTOR) >> (15 - ISNS_COMM_SCALER)); }
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.ISense.Mantissa = dat;

    dat = psu.Data.IAverage;
    if (false == sendRawData) { dat = (int16_t)(__builtin_mulss(dat, ISNS_COMM_FACTOR) >> (15 - ISNS_COMM_SCALER)); }
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.IAverage.Mantissa = dat;

    dat = psu.Data.Temperature;
    dat = (int16_t)(__builtin_mulss(dat, TEMP_COMM_FACTOR) >> (15 - TEMP_COMM_SCALER));
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.Temperature.Mantissa = dat;
    
    dat = psu.Status.value;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.Status.Mantissa = dat;
    
    dat = psu.StateId.OpStateId;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.OpStateId.Mantissa = dat;

    dat = (int16_t)(FaultMonitor.FaultStatusList & 0x0000FFFF);
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.FaultList.Mantissa = dat;

    // ---------------------------
    
    if (0 == psu.Properties.ConversionMode)
    {
        psu.Data.ControlError = (int16_t)*psu.Control.VLoop.Controller->Ports.Source.handle;
        psu.Data.ControlError -= (int16_t)psu.Control.VLoop.Controller->Ports.AltSource.Offset;
        psu.Data.ControlError -= (int16_t)*psu.Control.VLoop.Controller->Ports.ControlReference.handle;
    }
    else {
        psu.Data.ControlError = (int16_t)*psu.Control.VLoop.Controller->Ports.AltSource.handle;
        psu.Data.ControlError -= (int16_t)psu.Control.VLoop.Controller->Ports.AltSource.Offset;
        psu.Data.ControlError -= (int16_t)*psu.Control.VLoop.Controller->Ports.AltControlReference.handle;
    }
    
    psu.Data.ControlOutput = (uint16_t)*psu.Control.ILoop[0].Controller->Ports.Target.handle;
    // ---------------------------

    dat = psu.Data.ControlError;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.VoltageError.Mantissa = dat;

    dat = (int16_t)*psu.Control.VLoop.Controller->Ports.Target.handle;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.ReferenceCurrent.Mantissa = dat;

    dat = (int16_t)*psu.Control.ILoop[0].Controller->Ports.Source.handle;
    dat -= (int16_t)psu.Control.ILoop[0].Controller->Ports.Source.Offset;
    dat -= (int16_t)*psu.Control.ILoop[0].Controller->Ports.ControlReference.handle;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.CurrentError.Mantissa = dat;
    
    dat = (int16_t)psu.Data.ControlOutput;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.DutyCycle.Mantissa = dat;

    dat = (int16_t)psu.SwitchNode[0].Period;
    dat = ((true == Comm.Status.ByteSwap) ? (uint16_t)dat : SwapWordBytes((uint16_t)dat));
    statusData.Period.Mantissa = dat;
        
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   UART Transmit Routine 
**********************************************************************************/
int appComm_SendStatusFrame(void)
{
    int retval = 1;
    
    Nop();
    Nop();
    Nop();
    
    // Guarding conditions preventing status frame from being sent
    if ((true == skipStatusFrame) || (true == rxFrame.Ready))
    { return(retval); }
    
    // -----------------------flags----
    // Build status frame
    
    appComm_UpdateStatusFrame();
    
    // ---------------------------
    // Build status frame data set

    uint8_t* ptrData = (uint8_t*)&statusData;
    uint8_t dataLen=0;
    
    txBufferDma[TxDataBuffer.BufferPointer][dataLen++] = COMM_SOF;
    txBufferDma[TxDataBuffer.BufferPointer][dataLen++] = COMM_CMD;
    txBufferDma[TxDataBuffer.BufferPointer][dataLen++] = statusData_size;
    
    for (int i=0; i<statusData_size; i++)
    {
        txBufferDma[TxDataBuffer.BufferPointer][dataLen] = (uint8_t)*ptrData++;
        dataLen++;
    }

    txBufferDma[TxDataBuffer.BufferPointer][dataLen++] = COMM_EOM;
    txBufferDma[TxDataBuffer.BufferPointer][dataLen]   = COMM_EOF;
    
    // ---------------------------
    // Send status information
    
    retval &= CommStageToTransmit((uint8_t*)&txBufferDma[TxDataBuffer.BufferPointer][0], statusData_size + 5);
    
    return(retval);
}

// ______________________________________
// end of file

