/* 
 * @file    app_comm_protocol.c
 * @author  M91406
 * @date    February 27, 2024
 */

#include <xc_pral.h> // include device peripheral register abstraction layer
#include <stdio.h>   // include standard input/output library header file
#include "app_comm_protocol.h"

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   UART Protocol Execution Routine 
**********************************************************************************/
int appComm_ParseRxCommand(void)
{
    int retval = 1;
    
    // Guarding condition preventing parsing incomplete frames
    if (false == rxFrame.Ready) 
    { return(retval); }
    
    Nop();
    Nop();
    Nop();
    
    // Parse frame
    switch (rxFrame.Command)
    {
        // 'A' Sets/Gets new low voltage value
        case COMMCMD_SET_LVREF: retval &= CommSetLvReference(&rxFrame); break;
        case COMMCMD_GET_LVREF: retval &= CommGetLvReference(&rxFrame); break;
        
        // 'B' Sets/Gets new high voltage value
        case COMMCMD_SET_HVREF: retval &= CommSetHvReference(&rxFrame); break;
        case COMMCMD_GET_HVREF: retval &= CommGetHvReference(&rxFrame); break;
        
        // 'C' Sets/Gets new current limit value in Step-Down mode
        case COMMCMD_SET_DILIM: retval &= CommSetCurrentLimit(&rxFrame); break;
        case COMMCMD_GET_DILIM: retval &= CommGetCurrentLimit(&rxFrame); break;

        // 'D' Sets/Gets new current limit value in Step-Up mode
        case COMMCMD_SET_UILIM: retval &= CommSetAltCurrentLimit(&rxFrame); break;
        case COMMCMD_GET_UILIM: retval &= CommGetAltCurrentLimit(&rxFrame); break;
        
        // 'E' Sets/Gets ENABLE state of the power supply
        case COMMCMD_SET_PWREN: retval &= CommSetPowerSupplyEnable(&rxFrame); break;
        case COMMCMD_GET_PWREN: retval &= CommGetPowerSupplyEnable(&rxFrame); break;

        // 'F' Set power supply duty cycle
        case COMMCMD_SET_PDC: retval = CommSetDutyCycle(&rxFrame); break;
        case COMMCMD_GET_PDC: retval = CommSetDutyCycle(&rxFrame); break;
            
        // 'S' Swaps power transfer value
        case COMMCMD_SET_PWRSWP: retval &= CommSetConversionSwap(&rxFrame); break;
        case COMMCMD_GET_PWRSWP: retval &= CommGetConversionSwap(&rxFrame); break;
            
        // 'O' turns on/off the EnforcedOpenLoop mode
        case COMMCMD_SET_OLPEN: retval &= CommSetOpenLoopEnable(&rxFrame); break;
        case COMMCMD_GET_OLPEN: retval &= CommGetOpenLoopEnable(&rxFrame); break;
            
        // 'T' Request single status message
        case COMMCMD_GET_STATUS: retval &= CommGetStatusFrame(&rxFrame); break;
        
        // 'Z' Enable/disable Status Frame
        case COMMCMD_SET_STFEN: retval &= CommSetStatusFrameEnable(&rxFrame); break;
        case COMMCMD_GET_STFEN: retval &= CommGetStatusFrameEnable(&rxFrame); break;

        case COMMCMD_SET_RAWDAT: retval &= CommSetRawDataFrameEnable(&rxFrame); break;
        case COMMCMD_GET_RAWDAT: retval &= CommGetRawDataFrameEnable(&rxFrame); break;
        
        default:
            rxFrame.DataError++;
            break;
    }

    // Capture and handle error message
    if (0 == retval)
    {
        Nop(); // Capture errors with breakpoint here
        Nop();
        Nop();

        retval=1;
    }
    
    // Reset RxD Frame
    retval &= CommFrameReset();
    
    return(retval);
}

/* =============================== */
/* UART INTERRUPT SERVICE ROUTINES */
/* =============================== */

void __attribute__((__interrupt__, no_auto_psv)) VcpRxInterrupt(void)
{
    Nop();
    Nop();
    Nop();
    
    //CommReceiveFrame(&Comm); // Call RECEIVE Routine
    VCP_Rx_IF = 0; // Clear Interrupt Flag Bit
}

void __attribute__((__interrupt__, no_auto_psv)) VcpErrInterrupt(void)
{
    Nop(); // Place Breakpoint here
    Nop();
    Nop();

    P33C_UART_INSTANCE_t* uart = p33c_UartInstance_GetHandle(Comm.CommPort.UartInstance);
    uart->UxSTA.value = 0;
    VCP_ErrIF = 0; // Clear Interrupt Flag Bit
}

// ______________________________________
// end of file

