/* 
 * @file    svc_comm_engine.c
 * @author  M91406
 * @brief   Communication protocol driver
 * @version 1.0
 */

#include <xc_pral.h>
#include "svc_comm_engine.h"

// Declare UART data object defining the interface to use for communication
static int CommReceiveDataStream(uint8_t* dataArray, uint16_t dataSize);

// RxD Frame Tracker Data Object
COMM_FRAME_t rxFrame;
bool dataProcessing = false;
bool dataTransmission = false;

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   UART Receive Frame Reset Routine 
**********************************************************************************/
int CommFrameReset(void)
{
    rxFrame.Command = 0;
    rxFrame.DataLen = 0;
    rxFrame.ptrData = 0;
    rxFrame.State = 0;
    rxFrame.Ready = false; 
    rxFrame.Busy = false;
    
    for (int i=0; i<RxDataBuffer.Size; i++)
    { CmdBuffer[i] = 0; }
    
    return(1);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   UART Read Data from DMA Buffer
**********************************************************************************/
int CommDmaReceive(UART_t* comInterface)
{
    int retval = 1;
    static int receiveCounter=0;
    static bool ppPtr=0;
    static int receiveWait=0;

    // Capture DMA channel instance used by application
    P33C_DMA_INSTANCE_t* dma = p33c_DmaInstance_GetHandle(comInterface->CommPort.RxD.DmaInstance);
    
    // Guarding condition 
    if (0 == dma->DMAINTx.bits.DONEIF) { return(retval); }
    if (true == dataProcessing) { return(retval); }
    
    // Only ready data from DMA buffer when no more data has been received
    uint16_t dmaPtr = dma->DMADSTx.value; // Capture recent receive DMA memory address
    if (receiveCounter != dmaPtr)   // Check if address has changed since last time
    { receiveCounter = dmaPtr; receiveWait = 0; return(retval); }
    if (++receiveWait < 3) { return(retval); }
    
    // Check if there is data to read
    int dlen = dmaPtr - (uint16_t)(&rxBufferDma[ppPtr][0]);
    if (dlen == 0) { receiveCounter = dmaPtr; return(retval); }

    bool ppPtrX = ppPtr; // Capture most recent DMA buffer pointer
    ppPtr = !ppPtr; // switch to other DMA buffer
    dma->DMADSTx.value = (uint16_t)(&rxBufferDma[ppPtr][0]); // Set DMA to other buffer 
    dma->DMAINTx.bits.DONEIF = 0; // Clear DMA DONE flag bit

    dataProcessing = true;

    retval &= CommReceiveDataStream((uint8_t*)&rxBufferDma[ppPtrX][0], dlen);

    receiveWait = 0;
    receiveCounter = 0;
    dataProcessing = false;
    
    return(retval);
}

static int CommProcessData(uint8_t dataByte)
{
    int retval = 1;
    
    if ((COMMSTAT_SOF==rxFrame.State) && (COMM_SOF!=dataByte)) return(retval);
    bool doReset = false;

    // Do not receive new commands until previous command has been processed
    if ((false == rxFrame.Ready) && (false == doReset))
    {
        rxFrame.Busy = true;

        switch (rxFrame.State) 
        { 
            case COMMSTAT_SOF: // SOF byte received

                if (COMM_SOF == dataByte)
                {
                    rxFrame.ptrData = 0;
                    rxFrame.State = COMMSTAT_CMD; 
                    rxFrame.CRC = COMM_CRC_SEED;
                }
                else { doReset = true; }

                break;

            case COMMSTAT_CMD: // command byte received

                rxFrame.Command = dataByte;
                rxFrame.State = COMMSTAT_DLEN; 
                rxFrame.CRC ^= dataByte;

                break;

            case COMMSTAT_DLEN: // data length byte received

                rxFrame.DataLen = dataByte;
                rxFrame.State = ((rxFrame.DataLen != 0) ? COMMSTAT_DATA : COMMSTAT_EOM); 
                rxFrame.ptrData = 0;
                rxFrame.CRC ^= dataByte;

                break;

            case COMMSTAT_DATA: // receiving data content 

                if (rxFrame.ptrData < rxFrame.DataLen)
                { CmdBuffer[rxFrame.ptrData] = dataByte; }
                rxFrame.CRC ^= dataByte;

                if (++rxFrame.ptrData >= rxFrame.DataLen)
                { 
                    rxFrame.State = COMMSTAT_EOM; // BYPASSING CRC BYTE
                }

                break;

            case COMMSTAT_CRC: // checksum byte received

                if (rxFrame.CRC == dataByte) 
                {
                    rxFrame.State = COMMSTAT_DATA; 
                    rxFrame.ptrData = 0;
                }
                else { doReset = true; }
                break;

            case COMMSTAT_EOM: // end of message byte received

                if (COMM_EOM == dataByte)
                {
                    rxFrame.State = COMMSTAT_EOF; 
                    rxFrame.ptrData = 0;
                }
                else { doReset = true; }
                break;

            case COMMSTAT_EOF: // end of frame byte received

                if (COMM_EOF == dataByte)
                {
                    rxFrame.State = COMMSTAT_SOF;
                    rxFrame.Ready = true; 
                    rxFrame.Busy = false;
                }
                else { doReset = true; }

                break;

            default: // Frame format error
                doReset = true; 
                break;
        }

    }

    // Reset frame in case of a frame error
    if (true == doReset)
    {
        rxFrame.FrameErr++;
        retval &= CommFrameReset();
    }
    
    return(retval);
    
}

static int CommReceiveDataStream(uint8_t* dataArray, uint16_t dataSize)
{
    int retval = 1;
    
    for (int i=0; i<dataSize; i++)
    {
        uint8_t rxByte = *dataArray;
        *dataArray = 0x00;

        retval &= CommProcessData(rxByte);

        if ((rxFrame.Ready) && (rxFrame.CallRxCommandParser != NULL))
        { 
            Nop();
            Nop();
            Nop();

            retval &= rxFrame.CallRxCommandParser(); 
        }
        else if (false == retval) { break; }

        dataArray++;
    }
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-comm-functions-public
 * @brief   Puts messages into the transmit buffer
**********************************************************************************/
int CommStageToTransmit(uint8_t* dataArray, uint16_t dataSize)
{
    int retval = 1;
    
    // Guarding condition
    if (0 == dataSize) { return(retval); }
    if (NULL == dataArray) { return(retval); }
    
    for (int i=0; i<dataSize; i++)
    {
        if (TxDataBuffer.ArrayPointer >= txBufferSize) { retval = 0; break; }
        txBufferDma[TxDataBuffer.BufferPointer][TxDataBuffer.ArrayPointer++] = (*dataArray++);
    }
    
    TxDataBuffer.Ready = true;
    
    return(retval);
}


int CommDmaTransmit(UART_t* comInterface)
{
    int retval = 1;

    // Guarding condition 
    if (false == TxDataBuffer.Ready) { return(retval); }
    
    // Capture most recent buffer pointers
    int dlen = TxDataBuffer.ArrayPointer;   // Capture most recent DMA buffer index pointer
    int bufP = TxDataBuffer.BufferPointer;  // Capture most recent DMA buffer pointer
    if (0 == dlen) { return(retval); }      // Exit if there is no data to be sent

    // Set ping-pong buffer to next buffer
    if (++TxDataBuffer.BufferPointer >= TX_BUFFER_COUNT) // switch to next DMA buffer
    { TxDataBuffer.BufferPointer = 0; } // Manage overrun
    TxDataBuffer.ArrayPointer = 0; // Reset array index pointer
    TxDataBuffer.ptrData = (uint8_t*)&txBufferDma[TxDataBuffer.BufferPointer][0]; // Set pointer to most recent data buffer
    TxDataBuffer.Ready = false;
    
    // Initiate DMA data transfer 
    retval &= plib33c_Uart_SendBufferDma(comInterface, (uint8_t*)&txBufferDma[bufP][0], (uint16_t)dlen);
    
    if (0 == retval)
    {
        Nop();
        Nop();
        Nop();
        TxDataBuffer.Ready = true;
    }
    
    
    return(retval);
}

// ______________________________________
// end of file
