/**
 * @file    app_fault_config.c
 * @brief   Declaration of application specific fault objects
 * @author  M91406
 * @date    07/15/2021
 * @version 1.0.3
 */

#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

#include "config/hal.h" // include hardware abstraction layer header file
#include "config/apps.h" // include application layer header file
#include "apps/fault_monitor/svc/IFaultMonitor.h" // include common application task abstraction layer header file

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Under Voltage Lock Out Fault Object 
 * @details
 *   This fault object monitors the input voltage of the power supply
 *   scanning for conditions violating the minimum required input
 *   voltage level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BuckUVLO = {
    
    .SourceObject.ptrObject = (int16_t*)&psu.Data.VHighVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Clear pointer to "compare against" variable 
    .Status.bits.CompareType = FLTCMP_LESS_THAN,            // Select Compare-Type
    
    .TripResponse.CompareThreshold = V48_UVLO_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = UVLO_TDLY,               // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V48_UVLO_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = UVLO_RDLY,           // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = true,                        // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = true,                        // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = true,                       // If this fault is tripped, fault handler does not invoke global startup delay

    .Status.bits.Enable = true                              // Enable fault checks
            
};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Voltage Lock Out Fault Object
 * @details
 *   This fault object monitors the input voltage of the power supply
 *   scanning for conditions exceeding the maximum allowed input
 *   voltage level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BuckOVLO = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VHighVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V48_OVLO_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = OVLO_TDLY,               // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V48_OVLO_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = OVLO_RDLY,           // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = true,                        // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = true,                        // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = true,                       // If this fault is tripped, fault handler does not invoke global startup delay

    .Status.bits.Enable = true                              // Enable fault checks

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Current Protection Fault Object
 * @details
 *   This fault object monitors the average output current of the
 *   power supply scanning for conditions exceeding the maximum
 *   allowed output current level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BuckOCP = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.IAverage,     // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = ISNS_OCP_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = OCP_TDLY,                // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = ISNS_OCP_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = OCP_RDLY,            // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay
    
    .Status.bits.Enable = false                             // Enable fault checks    

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Output Over Voltage Protection Fault Object
 * @details
 *   This fault object monitors the average output current of the
 *   power supply scanning for conditions exceeding the maximum
 *   allowed output current level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BuckOVP = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VLowVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V12_OVP_TRIP,          // Set fault trip level
    .TripResponse.EventThreshold = OVP_TDLY,                // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V12_OVP_RELEASE,   // Set fault recovery level
    .RecoveryResponse.EventThreshold = OVP_RDLY,            // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay
    
    .Status.bits.Enable = false                             // Enable fault checks    

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief Regulation Error Fault Object in Step-Down/Buck mode
 * @details
 * This fault object monitors the deviation between voltage reference
 * and most recent output voltage of the power supply scanning for
 * conditions where a maximum allowed deviation is exceeded for the
 * specified amount of time.
 *******************************************************************/ 
FAULT_OBJECT_t fltobj_BuckRegErr = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VLowVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = (int16_t*)&psu.Properties.LVReference, // Set pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of Reference (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V12_DEV_TRIP,          // Set fault trip level
    .TripResponse.EventThreshold = REGERR_TDLY,             // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V12_DEV_RELEASE,   // Set fault recovery level
    .RecoveryResponse.EventThreshold = REGERR_RDLY,         // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay

    .Status.bits.Enable = false                             // Disable fault checks at startup

};

// ----------------------------------------------------------------------------------
/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Under Voltage Lock Out Fault Object in Step-Up/Boost mode
 * @details
 *   This fault object monitors the input voltage of the power supply
 *   scanning for conditions violating the minimum required input
 *   voltage level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BoostUVLO = {
    
    .SourceObject.ptrObject = (int16_t*)&psu.Data.VLowVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Clear pointer to "compare against" variable 
    .Status.bits.CompareType = FLTCMP_LESS_THAN,            // Select Compare-Type
    
    .TripResponse.CompareThreshold = V12_UVLO_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = UVLO_TDLY,               // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V12_UVLO_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = UVLO_RDLY,           // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = true,                        // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = true,                        // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = true,                       // If this fault is tripped, fault handler does not invoke global startup delay

    .Status.bits.Enable = true                              // Enable fault checks
            
};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Voltage Lock Out Fault Object in Step-Up/Boost mode
 * @details
 *   This fault object monitors the input voltage of the power supply
 *   scanning for conditions exceeding the maximum allowed input
 *   voltage level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BoostOVLO = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VLowVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V12_OVLO_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = OVLO_TDLY,               // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V12_OVLO_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = OVLO_RDLY,           // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = true,                        // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = true,                        // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = true,                       // If this fault is tripped, fault handler does not invoke global startup delay

    .Status.bits.Enable = true                              // Enable fault checks

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Current Protection Fault Object in Step-Up/Boost mode
 * @details
 *   This fault object monitors the average output current of the
 *   power supply scanning for conditions exceeding the maximum
 *   allowed output current level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BoostOCP = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.IAverage,     // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = ISNS_OCP_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = OCP_TDLY,                // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = ISNS_OCP_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = OCP_RDLY,            // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay
    
    .Status.bits.Enable = false                             // Enable fault checks    

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Current Protection Fault Object in Step-Up/Boost mode
 * @details
 *   This fault object monitors the average output current of the
 *   power supply scanning for conditions exceeding the maximum
 *   allowed output current level for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_BoostOVP = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VHighVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of SOURCE (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V48_OVP_TRIP,          // Set fault trip level
    .TripResponse.EventThreshold = OVP_TDLY,                // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V48_OVP_RELEASE,   // Set fault recovery level
    .RecoveryResponse.EventThreshold = OVP_RDLY,            // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay
    
    .Status.bits.Enable = false                             // Enable fault checks    

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Regulation Error Fault Object
 * @details
 *   This fault object monitors the deviation between voltage reference
 *   and most recent output voltage of the power supply scanning for
 *   conditions where a maximum allowed deviation is exceeded for the
 *   specified amount of time.
 *******************************************************************/ 
FAULT_OBJECT_t fltobj_BoostRegErr = {

    .SourceObject.ptrObject = (int16_t*)&psu.Data.VHighVoltage,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = (int16_t*)&psu.Properties.HVReference, // Set pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Compare all bits of Reference (no bit filter)
    .Status.bits.CompareType = FLTCMP_GREATER_THAN,         // Select Compare-Type

    .TripResponse.CompareThreshold = V48_DEV_TRIP,          // Set fault trip level
    .TripResponse.EventThreshold = REGERR_TDLY,             // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = V48_DEV_RELEASE,   // Set fault recovery level
    .RecoveryResponse.EventThreshold = REGERR_RDLY,         // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = false,                       // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = false,                       // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler invokes global startup delay

    .Status.bits.Enable = false                             // Disable fault checks at startup

};

/********************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Over Temperature Protection Fault Object 
 * @details
 *   This fault object monitors the board temperature of the power 
 *   supply scanning for conditions violating the maximum board 
 *   temperature for the specified amount of time.
 *******************************************************************/
FAULT_OBJECT_t fltobj_OTP = {
    
    .SourceObject.ptrObject = (int16_t*)&psu.Data.Temperature,       // Set pointer to variable to monitor
    .SourceObject.BitMask = 0xFFFF,                         // Compare all bits of SOURCE (no bit filter)
    .ReferenceObject.ptrObject = NULL,                      // Clear pointer to "compare against" variable 
    .ReferenceObject.BitMask = 0xFFFF,                      // Clear pointer to "compare against" variable 
    .Status.bits.CompareType = FLTCMP_LESS_THAN,            // Select Compare-Type
    
    .TripResponse.CompareThreshold = TEMP_OTP_TRIP,         // Set fault trip level
    .TripResponse.EventThreshold = OTP_TDLY,                // Set counter level at which a FAULT condition will be tripped
    .TripResponse.ptrResponseFunction = &appPowerSupply_Suspend, // Set pointer to user-function which should be called when a FAULT is tripped

    .RecoveryResponse.CompareThreshold = TEMP_OTP_RELEASE,  // Set fault recovery level
    .RecoveryResponse.EventThreshold = OTP_RDLY,            // Set counter level at which a FAULT condition will be cleared
    .RecoveryResponse.ptrResponseFunction = NULL,           // Clear recovery function pointer
    
    .Counter = 0,                                           // Clear fault event counter
    .Status.bits.FaultActive = true,                        // Set fault condition flag (must be cleared by fault check)
    .Status.bits.FaultStatus = true,                        // Set fault flag (must be cleared by fault check)
    .Status.bits.FastRecovery = false,                      // If this fault is tripped, fault handler does not invoke global startup delay

    .Status.bits.Enable = false                             // Enable fault checks
            
};


/*********************************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Fault Object List
 * @details
 *  This list is grouping all user defined fault objects in one list, which
 *  is used by the fault handler for initialization, disposal and scan for 
 *  fault conditions.
 **********************************************************************************/
FAULT_OBJECT_t* FaultObjectList[] = {
    
    &fltobj_BuckUVLO,       ///< #0: Buck mode Under Voltage Lock Out
    &fltobj_BuckOVLO,       ///< #1: Buck mode Over Voltage Lock Out
    &fltobj_BuckOVP,        ///< #2: Buck mode Output Over Voltage Protection
    &fltobj_BuckOCP,        ///< #3: Buck mode Over Current Protection
    &fltobj_BuckRegErr,     ///< #4: Buck mode Regulation Error
    
    &fltobj_BoostUVLO,      ///< #5: Boost mode Under Voltage Lock Out
    &fltobj_BoostOVLO,      ///< #6: Boost mode Over Voltage Lock Out
    &fltobj_BoostOVP,       ///< #7: Boost mode Output Over Voltage Protection
    &fltobj_BoostOCP,       ///< #8: Boost mode Over Current Protection
    &fltobj_BoostRegErr,    ///< #9: Boost mode Regulation Error

    &fltobj_OTP             ///< #10: OVer Temperature Protection
};

/*********************************************************************************
 * @ingroup apps-layer-fault-app-properties-public-variables
 * @brief   Fault Object List Size
 * @details
 *  This variable is used to determine the number of user specified fault objects.
 *  The size of the fault object list is required for all fault monitor function 
 *  calls.
 **********************************************************************************/

uint16_t FaultObjectList_size = (sizeof(FaultObjectList)/sizeof(FaultObjectList[0]));

// ________________________
// end of file
