/**
 * @file    app_fault_monitor.c
 * @brief   Fault monitor application layer source file
 * @author  M91406
 * @date    07/15/2021
 * @version 1.0.3
 */

#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

#include "config/hal.h" // include hardware abstraction layer header file
#include "config/apps.h" // include common application task abstraction layer header file

// User fault object declarations
extern FAULT_OBJECT_t* FaultObjectList[];
extern uint16_t FaultObjectList_size;

static int appFaultMonitor_SwitchPowerSupplyMonitor(bool reverseMode);

FAULT_MONITOR_t FaultMonitor;
static bool powerReverseMode = false; 

/*********************************************************************************
 * @ingroup apps-layer-fault-app-functions-public
 * @brief   Application wide fault object monitoring routine
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *   In this function all user-defined fault objects are monitored for 
 *   threshold violations. While fault responses are triggered by each 
 *   fault object individually, system recovery from a fault condition is 
 *   only allowed when all fault conditions have been cleared. Hence, 
 *   individual fault status bits are combined into a common fault bit, 
 *   which needs to be cleared to allow the power supply to start-up again.
 * 
 *********************************************************************************/

int appFaultMonitor_Execute(void) 
{
    int retval = 1;
    
    // Check for power transfer direction switch
    if (powerReverseMode != psu.Properties.ConversionMode)
    {
        retval &= appFaultMonitor_SwitchPowerSupplyMonitor(psu.Properties.ConversionMode);
        powerReverseMode = psu.Properties.ConversionMode;
    }
    
    // Scan all declared fault objects
    svc_FaultMonitor_Execute(&FaultMonitor);
    
    // Push global fault state to buck converter device driver status
    psu.Status.bits.FaultStatus = FaultMonitor.Status.bits.FaultStatus;
    
    // If power supply controller passes POWER GOOD and reaches state ONLINE,
    // reset restart cycle counter
    if (psu.StateId.OpStateId == BDBB_OPSTATE_ONLINE)
        FaultMonitor.FaultLatchCounter = 0;
    
    return (retval);
}

/*********************************************************************************
 * @ingroup apps-layer-fault-app-functions-public
 * @brief   Initialization of user-defined fault objects
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 * This function initializes the fault monitor data object user settings. 
 *********************************************************************************/

int appFaultMonitor_Initialize(void) 
{
    int retval = 1;
    
    // Clear Fault Latch conditions
    FaultMonitor.FaultLatchCounter = 0;                  // Clear restart attempt counter
    FaultMonitor.FaultLatchLimit = FAULT_RESTART_CYCLES; // Set maximum number of restart attempts
    
    // At startup, skip fault recovery delay
    FaultMonitor.FaultRecoveryCounter = 0;  // Clear recovery counter 
    FaultMonitor.FaultRecoveryPeriod = FLTRSTDLY;   // Set fault recovery delay period 
    FaultMonitor.FaultRecovery = &appPowerSupply_Resume; // Set pointer to recovery function

    // Set fault objects list
    FaultMonitor.fltObjectList = &FaultObjectList[0]; // Set pointer to fault object list
    FaultMonitor.fltObjectListSize = FaultObjectList_size;  // Set fault object list size

    // Clear status flags list
    FaultMonitor.FaultStatusList = 0; // Clear fault status flag bit list
    FaultMonitor.Status.bits.FastRecovery = true; // Set FastRecovery Flag at startup
    
    // Initialize fault object in accordance with conversion mode
    retval &= appFaultMonitor_SwitchPowerSupplyMonitor(psu.Properties.ConversionMode);
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-fault-app-functions-public
 * @brief   Function starting all pre-configured fault checks
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *   This function is used to set the fault status of all fault objects 
 *   to ACTIVE, enforcing the system monitor to clear them one-by-one as 
 *   default startup self-test procedure. Once cleared, the respective 
 *   recovery procedures will turn on the respective user-tasks being
 *   kept on-hold while faults have been active.
 * 
 *   Users can set the initial state of each fault object in accordance
 *   to their inclusion in the self-test period at firmware startup.
 * 
 *********************************************************************************/

int appFaultMonitor_Start(void) 
{
    int retval = 1;

    if(psu.Properties.ConversionMode == BDBB_PWRDIR_STEP_DOWN) 
    {
        fltobj_BuckUVLO.Status.bits.FaultStatus = true; // Set Under Voltage Lock Out fault
        fltobj_BuckUVLO.Status.bits.FaultActive = true; // Set Under Voltage Lock Out fault condition flag
        fltobj_BuckUVLO.Status.bits.Enable = true; // Enable Under Voltage Lock Out object

        fltobj_BuckOVLO.Status.bits.FaultStatus = true; // Set Over Voltage Lock Out fault
        fltobj_BuckOVLO.Status.bits.FaultActive = true; // Set Over Voltage Lock Out fault condition flag
        fltobj_BuckOVLO.Status.bits.Enable = true; // Enable Over Voltage Lock Out object

        fltobj_BuckOCP.Status.bits.FaultStatus = false; // Clear Over Current Protection fault
        fltobj_BuckOCP.Status.bits.FaultActive = false; // Clear Over Current Protection fault condition flag
        fltobj_BuckOCP.Status.bits.Enable = false; // Disable Over Current Protection object (will be enabled by power supply state machine)

        fltobj_BuckRegErr.Status.bits.FaultStatus = false; // Clear Regulation Error fault
        fltobj_BuckRegErr.Status.bits.FaultActive = false; // Clear Regulation Error fault condition flag
        fltobj_BuckRegErr.Status.bits.Enable = false; // Disable Regulation Error object (will be enabled by power supply state machine)

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        fltobj_BoostUVLO.Status.bits.FaultStatus = false; // Clear Under Voltage Lock Out fault
        fltobj_BoostUVLO.Status.bits.FaultActive = false; // Clear Under Voltage Lock Out fault condition flag
        fltobj_BoostUVLO.Status.bits.Enable = false; // Disable Under Voltage Lock Out object

        fltobj_BoostOVLO.Status.bits.FaultStatus = false; // Clear Over Voltage Lock Out fault
        fltobj_BoostOVLO.Status.bits.FaultActive = false; // Clear Over Voltage Lock Out fault condition flag
        fltobj_BoostOVLO.Status.bits.Enable = false; // Disable Over Voltage Lock Out object

        fltobj_BoostOCP.Status.bits.FaultStatus = false; // Clear Over Current Protection fault
        fltobj_BoostOCP.Status.bits.FaultActive = false; // Clear Over Current Protection fault condition flag
        fltobj_BoostOCP.Status.bits.Enable = false; // Disable Over Current Protection object (will be enabled by power supply state machine)

        fltobj_BoostRegErr.Status.bits.FaultStatus = false; // Clear Regulation Error fault
        fltobj_BoostRegErr.Status.bits.FaultActive = false; // Clear Regulation Error fault condition flag
        fltobj_BoostRegErr.Status.bits.Enable = false; // Disable Regulation Error object (will be enabled by power supply state machine)
    }
    else if (psu.Properties.ConversionMode == BDBB_PWRDIR_STEP_UP) 
    {
        fltobj_BoostUVLO.Status.bits.FaultStatus = true; // Set Under Voltage Lock Out fault
        fltobj_BoostUVLO.Status.bits.FaultActive = true; // Set Under Voltage Lock Out fault condition flag
        fltobj_BoostUVLO.Status.bits.Enable = true; // Enable Under Voltage Lock Out object

        fltobj_BoostOVLO.Status.bits.FaultStatus = true; // Set Over Voltage Lock Out fault
        fltobj_BoostOVLO.Status.bits.FaultActive = true; // Set Over Voltage Lock Out fault condition flag
        fltobj_BoostOVLO.Status.bits.Enable = true; // Enable Over Voltage Lock Out object

        fltobj_BoostOCP.Status.bits.FaultStatus = false; // Clear Over Current Protection fault
        fltobj_BoostOCP.Status.bits.FaultActive = false; // Clear Over Current Protection fault condition flag
        fltobj_BoostOCP.Status.bits.Enable = false; // Disable Over Current Protection object (will be enabled by power supply state machine)

        fltobj_BoostRegErr.Status.bits.FaultStatus = false; // Clear Regulation Error fault
        fltobj_BoostRegErr.Status.bits.FaultActive = false; // Clear Regulation Error fault condition flag
        fltobj_BoostRegErr.Status.bits.Enable = false; // Disable Regulation Error object (will be enabled by power supply state machine)

        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        fltobj_BuckUVLO.Status.bits.FaultStatus = false; // Clear Under Voltage Lock Out fault
        fltobj_BuckUVLO.Status.bits.FaultActive = false; // Clear Under Voltage Lock Out fault condition flag
        fltobj_BuckUVLO.Status.bits.Enable = false; // Disable Under Voltage Lock Out object

        fltobj_BuckOVLO.Status.bits.FaultStatus = false; // Clear Over Voltage Lock Out fault
        fltobj_BuckOVLO.Status.bits.FaultActive = false; // Clear Over Voltage Lock Out fault condition flag
        fltobj_BuckOVLO.Status.bits.Enable = false; // Disable Over Voltage Lock Out object

        fltobj_BuckOCP.Status.bits.FaultStatus = false; // Clear Over Current Protection fault
        fltobj_BuckOCP.Status.bits.FaultActive = false; // Clear Over Current Protection fault condition flag
        fltobj_BuckOCP.Status.bits.Enable = false; // Disable Over Current Protection object (will be enabled by power supply state machine)

        fltobj_BuckRegErr.Status.bits.FaultStatus = false; // Clear Regulation Error fault
        fltobj_BuckRegErr.Status.bits.FaultActive = false; // Clear Regulation Error fault condition flag
        fltobj_BuckRegErr.Status.bits.Enable = false; // Disable Regulation Error object (will be enabled by power supply state machine)
            
    }
    else
    {
        // Fatal Error: Latch Fault Conditions
        FaultMonitor.FaultLatchCounter = FaultMonitor.FaultLatchLimit;
        FaultMonitor.Status.bits.FaultLatch = true;
        retval = 0;
    }
    
    // Always enable Over Temperature Protection
    fltobj_OTP.Status.bits.FaultStatus = false;  // Clear Over Temperature fault
    fltobj_OTP.Status.bits.FaultActive = false;  // Clear Over Temperature fault condition flag
    fltobj_OTP.Status.bits.Enable = true; // Enable Over Temperature Protection object

    // Always enable LV-Port Over Voltage Protection
    fltobj_BuckOVP.Status.bits.FaultStatus = true; // Set Over Voltage Protection fault
    fltobj_BuckOVP.Status.bits.FaultActive = true; // Set Over Voltage Protection fault condition flag
    fltobj_BuckOVP.Status.bits.Enable = true; // Always enable Over Voltage Protection object 

    // Always enable HV-Port Over Voltage Protection
    fltobj_BoostOVP.Status.bits.FaultStatus = true; // Set Over Voltage Protection fault
    fltobj_BoostOVP.Status.bits.FaultActive = true; // Set Over Voltage Protection fault condition flag
    fltobj_BoostOVP.Status.bits.Enable = true; // Always enable Over Voltage Protection object 
    
    // Capture & store most recent conversion mode
    powerReverseMode = psu.Properties.ConversionMode;
    
    return(retval);
}

/*********************************************************************************
 * @ingroup apps-layer-fault-app-functions-public
 * @brief   Function clearing all fault object settings
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *   This function is used to clear all fault objects settings. Once cleared,
 *   the fault objects are detached from memory addresses and cannot be used
 *   for fault monitoring anymore until they have been re-initialized.
 * 
 *********************************************************************************/

int appFaultMonitor_Dispose(void) {
    
    // Clear all user defined fault objects
    svc_FaultMonitor_Dispose(&FaultMonitor);
    
    return(1);
}

/* ****************************************************************************** */
/* ****************************************************************************** */
/* ****************************************************************************** */

static int appFaultMonitor_SwitchPowerSupplyMonitor(bool reverseMode) {

    bool _boostEnable = reverseMode;
    bool _buckEnable = (!reverseMode);
    
    fltobj_BuckUVLO.Counter = 0;    // Reset UVLO error event counter
    fltobj_BuckOVLO.Counter = 0;    // Reset OVLO error event counter
    fltobj_BuckOCP.Counter = 0;     // Reset UCP error event counter
    fltobj_BuckRegErr.Counter = 0;  // Reset Regulation Error error event counter

    fltobj_BuckUVLO.Status.bits.Enable = _buckEnable; // Enable Under Voltage Lock Out object
    fltobj_BuckOVLO.Status.bits.Enable = _buckEnable; // Enable Over Voltage Lock Out object
    fltobj_BuckOCP.Status.bits.Enable = _buckEnable; // Enable Over Current Protection object (will be enabled by power supply state machine)
    fltobj_BuckRegErr.Status.bits.Enable = _buckEnable; // Disable Regulation Error object (will be enabled by power supply state machine)

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    fltobj_BoostUVLO.Counter = 0;   // Reset UVLO error event counter
    fltobj_BoostOVLO.Counter = 0;   // Reset OVLO error event counter
    fltobj_BoostOCP.Counter = 0;    // Reset UCP error event counter
    fltobj_BoostRegErr.Counter = 0; // Reset Regulation Error error event counter
    
    fltobj_BoostUVLO.Status.bits.Enable = _boostEnable; // Enable Under Voltage Lock Out object
    fltobj_BoostOVLO.Status.bits.Enable = _boostEnable; // Enable Over Voltage Lock Out object
    fltobj_BoostOCP.Status.bits.Enable = _boostEnable; // Enable Over Current Protection object (will be enabled by power supply state machine)
    fltobj_BoostRegErr.Status.bits.Enable = _boostEnable; // Disable Regulation Error object (will be enabled by power supply state machine)

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    FaultMonitor.FaultLatchCounter = 0;
    
    return(1);
}

// _________________________
// end of file
