# Fault Condition Monitor Service {#faultlibrary}

---

<span id="startDoc"><a name="startDoc"></a></span> <!-- start tag for internal references -->

## Class

PowerSmart™ Firmware Framework Component

## Summary

The generic, single-instance data monitoring service scans signal conditions declared in so-called fault objects for threshold and/or condition violations. 

## Table Of Contents

<div id="TOC">
<ul>
<li><a href="#intro">Introduction</a></li>
<li><a href="#lib-use">Library Usage Guidelines</a></li>
<li><a href="#api-guide">API Quick-Start Guide</a></li>
<li><a href="#api-objects">API Public Data Objects</a></li>
<li><a href="#history">History</a></li>
</ul>
</div>

<span id="intro"><a name="intro"></a></span>

## Introduction

#### Description

This generic fault handler driver declares a fault object data structure, which can be used in user code to define various different fault objects, which will get monitored and managed by  fault handler functions.

The main Execute function of this fault handler is restricted to detecting fault conditions, defined as filtered threshold violations by comparing the value of the defined fault object source variable against user-defined trip and release thresholds. Fault condition is triggered when the monitored value of a variable or register is exceeding a defined threshold n consecutive times. Consequently, a fault condition is cleared when the monitored value of the variable is found to be within a valid range m consecutive times.

The numbers n and m used for triggering/clearing a fault condition is set as parameter in user code and effectively provide filtering options influencing the sensitivity of the respective fault condition detection.

#### Fault Object Types

- Static: One 16-bit wide SFR or user variable compared against 16-bit wide constant
- Dynamic: One 16-bit wide SFR or user variable compared against another 16-bit wide SFR or user variable 
- Each variable is/can be supported by bit masks supporting range or bit-wise comparisons

#### Compare Functions

- None (no comparison takes place, fault object ignored)
- Greater Than
- Less Than
- Is Equal
- Is Not Equal
- Between Thresholds
- Outside Thresholds

#### Fault Indication/Fault Response

- Immediate indication of fault condition through status bit in fault object status word 
(set/cleared when change in fault condition is detected)
- Immediate indication of fault status through status bit in fault object status word 
(set/cleared when fault trips/resets)
- Automated trigger of user-defined software function when a fault condition trips
- Automated trigger of user-defined software function when a fault condition is released

<a href="#startDoc">back</a>

<span id="lib-use"><a name="lib-use"></a></span>

## Library Usage Guidance

This code library repository must be included in an independent directory within the target project. Changes to any of the files of this repository in the target project environment can be pushed back to this library repository from where these changes can be distributed to one, more or all target projects this library has been used in. The upgrade of library files in a particular target project needs to be performed individually, allowing to individually manage the device driver version used.

### Adding Subtree Repository

#### Step 1 - Adding Subtree Repository to Target Project

##### a) Using Sourcetree Application

Open *Sourcetree* Application

- Open menu *Repository* → *Repository Settings*
- Click *Add* to open the Add Repository dialog

Inside the Repository dialog

- Enter *Remote Name* `subrepo-fault`
- Enter clone URL/path https://bitbucket.microchip.com/scm/mcu16asmpsl/subrepo-fault-handler.git
- Under *Optional extended integration* → *Remote Account* select your user account
- Click *OK* to close the dialog

##### b) Using Git Bash

Open Git Bash at root of target project repository and use the following command for adding the new subtree remote:

- `$ git remote add subrepo-fault https://bitbucket.microchip.com/scm/mcu16asmpsl/subrepo-fault-handler.git`

#### Step 2) Cloning Subtree Repository

Further using Git Bash, use the `subtree add` command to clone the contents of this code library to the target project

- `$ git subtree add --prefix=<code_library_target> subrepo-fault main --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library should be cloned to (e.g. my_project.X/sources/fault_handler/drivers).

#### 3) Pulling latest version from Library Repository

When a new version of this code library is available, you can pull it by using the `subtree pull` command in the Git Bash:

`$ git subtree pull --prefix=<code_library_target> subtree-fault main --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library has been cloned to (e.g. my_project.X/sources/fault_handler/drivers).

#### 4) Pushing new version back to Library Repository

When changes have been made to files of this code library, the new version can be pushed back to the library repository by using the `subtree push` command in the Git Bash:

`$ git subtree push --prefix=<code_library_target> subtree-fault feature/version-update --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library has been cloned to (e.g. my_project.X/sources/fault_handler/drivers).

<span style="color:red">

    <u><b>Note:</b></u><br>
    Pushing directly to the library project 'main' or 'develop' branches may be prohibited. 
    Hence, <i><u><b>changes can only be pushed to feature branches</b></u></i>. A Pull Request is required to review and merge changes to 'develop'. Once changes have been approved and merged int 'develop', they may be merged into branch 'main' and thus released as new version through another pull request. This new version of 'main' can be tagged with a new version number and pulled into target projects.

</span>

<br>&nbsp;

<a href="#startDoc">back</a>

<span id="api-guide"><a name="api-guide"></a></span>

## API Quick-Start Guide

### API Public Functions

The API of this function driver provides the following global functions:

```c

volatile uint16_t svc_FaultHandler_CheckObject(volatile struct FAULT_OBJECT_s* fltobj);
volatile uint16_t svc_TrapHandler_SoftTrapsInitialize(
                bool accumulator_a_overflow_trap_enable, 
                bool accumulator_b_overflow_trap_enable, 
                bool accumulator_catastrophic_overflow_trap_enable);

```

- **`uint16_t svc_FaultHandler_CheckObject(volatile struct FAULT_OBJECT_s* fltobj)`**<br>&nbsp;
This function executes the comparison between the defined fault object variable or register and its thresholds, tracking the most recent fault condition of the object by incrementing or clearing fault threshold violation counters.
This function takes a function call parameter of type *struct FAULT_OBJECT_s* (see blow) on which the fault condition analysis is performed and returns an integer return value indicating success (= 1) or failure (= 0) of the function execution. The return value ***does not*** reflect the fault condition but only verifies if the ***execution of the fault check function*** was successful. The result of the fault check function can be read from the status bits of the fault object fltobj of type `FAULT_OBECT_t`.<br>&nbsp;<br>

- **`uint16_t svc_TrapHandler_SoftTrapsInitialize( bool, bool, bool)`**<br>&nbsp;
This function initializes the use of so-called *Soft Traps*. Soft traps define conditions which can be considered critical enough to be treated like other uninterruptible interrupts triggered by hardware errors (e.g. address error). 
In this version the trap handler allows to treat DSP accumulator saturation events be treated as uninterruptible interrupt event or be ignored. This function can be used to enable/disable trap-treatment of saturation event (32-bit overflow condition) of accumulator a and/or b and also provides an additional, independent trap option for super-saturation events of any accumulator (40-bit overflow condition) <br>&nbsp;<br>

<a href="#startDoc">back</a>

<span id="api-objects"><a name="api-objects"></a></span>

### API Public Data Objects

#### 1) Fault Object Data Structure

The fault monitor service monitors a collection of user-declared fault objects of type `FAULT_OBJECT_t`. This type is a collection of nested data structures covering parameter groups.

```c

struct FAULT_OBJECT_s {

    volatile FLT_OBJECT_STATUS_t  Status;           // Status word of this fault object
    volatile uint16_t Counter;                      // Fault event counter (controlled by FAULT MONITOR, do not use)
    volatile FLT_COMPARE_OBJECT_t SourceObject;     // Object which should be monitored
    volatile FLT_COMPARE_OBJECT_t ReferenceObject;  // Reference object the source should be compared with
    volatile FLT_EVENT_RESPONSE_t TripResponse;     // Settings defining the fault trip event
    volatile FLT_EVENT_RESPONSE_t RecoveryResponse; // Settings defining the fault recovery event

};

typedef struct FAULT_OBJECT_s FAULT_OBJECT_t; // Generic fault object

```

<br>&nbsp;

#### Sub Structure  `struct FLT_OBJECT_STATUS_s`

This data structure defines the fault object status.

```c
struct FLT_OBJECT_STATUS_s {

    volatile bool FaultStatus : 1;                      // Bit 0: Flag bit indicating if FAULT has been tripped
    volatile bool FaultActive : 1;                      // Bit 1: Flag bit indicating if fault condition has been detected but FAULT has not been tripped yet
    volatile unsigned : 6;                              // Bit <7:2>: (reserved)
    volatile enum FLT_COMPARE_TYPE_e CompareType: 3;    // Bit <10:8>: Fault check comparison type control bits
    volatile unsigned : 4;                              // Bit 14: (reserved)
    volatile bool Enable : 1;                           // Bit 15: Control bit enabling/disabling monitoring of the fault object
} __attribute__((packed));              

union FLT_OBJECT_STATUS_u {

    volatile struct FLT_OBJECT_STATUS_s bits; // Fault object status bit field for single bit access
    volatile uint16_t value; // Fault object status word  
};

typedef union FLT_OBJECT_STATUS_u FLT_OBJECT_STATUS_t; // Fault object status

```

<br>&nbsp;

#### Sub Structure  `struct FLT_COMPARE_OBJECT_s`

This data structure defines the data object which will be monitored by the fault monitor.

```c

struct FLT_COMPARE_OBJECT_s {
    
    volatile uint16_t* ptrObject;   // Pointer to register or variable which should be monitored 
    volatile uint16_t bitMask;      // Bit mask will be &-ed with source as value (use 0xFFFF for full value comparison)
    
}; 

typedef struct FLT_COMPARE_OBJECT_s FLT_COMPARE_OBJECT_t; // Fault compare object

```

<br>&nbsp;

#### Sub Structure  `struct FLT_EVENT_RESPONSE_s`

This data structure defines the fault object event response object.

```c

struct FLT_EVENT_RESPONSE_s {
    
    volatile uint16_t CompareThreshold;             // Signal level at which the fault condition will be detected
    volatile uint16_t EventThreshold;               // Bit mask will be &-ed with source as value (use 0xFFFF for full value comparison)
    volatile uint16_t (*ptrResponseFunction)(void); // pointer to a user-defined function called when a defined fault monitoring event is detected
    
}; // Fault monitor event response object

typedef struct FLT_EVENT_RESPONSE_s FLT_EVENT_RESPONSE_t; // Fault monitor event response object

```

<br>&nbsp;

---

<span id="history"><a name="history"></a></span>

#### History

- 03/13/2020 v1.0 (M91406) Initial release
- 11/02/2020 v2.0 (M91406) Enhanced version with generic trap handler
- 03/08/2021 v2.1 (M91406) Improved ENABLE bit handling keeping fault check executed but bypassing fault trip and recovery responses
- 07/16/2021 v2.2 (M91406) Introduced global FaultMonitor object and encapsulated scan through fault objects in fault driver
- 08/01/2022 v3.0 (M91406) Migrated the code to support multiple, user-defined fault monitor data object instances

---
&copy; 2022, Microchip Technology Inc.
