/**
 * @file    svc_fault_monitor.c
 * @brief   Fault monitor driver source file
 * @author  M91406
 * @date    12/27/2019
 * @version 2.3.2
 * 
 * @details
 * This file contains APIs to check current fault status of a user-defined fault object
 * 
 * @note
 * Microchip Technology Inc. has followed development methods required by
 * IEC-60730 and performed extensive validation and static testing to ensure
 * that the code operates as intended. Any modification to the code can
 * invalidate the results of Microchip's validation and testing.
 */

#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stdlib.h> // include standard library data types and macros
#include <stddef.h> // include standard definition data types

#include "../IFaultMonitor.h" // Include fault monitor interface header file

// Private Functions
extern int svc_FaultObject_CheckObject(FAULT_OBJECT_t* fltObject);

// Private data objects
extern const FAULT_OBJECT_t fltObjectClear;
static bool wakeUpBoot = true;

/*********************************************************************************
 * @ingroup apps-layer-fault-lib-functions-public
 * @brief   Scans a list of user-defined fault objects
 * @param   fltMonitorInstance Pointer to user defined fault monitoring data object of type FAULT_MONITOR_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  This function scans through a list of user defined fault objects of type 
 *  FAULT_OBJECT_t executing the check of declared fault conditions. Each fault
 *  object is capable of raising an individual fault response and fault reset 
 *  function call, which can be considered as the equivalent of a software 
 *  triggered interrupt.
 * 
 *  During the scan the status of each individual fault object is logged and 
 *  merged into one common fault condition flag bit fltMonitorInstance->Status.bits.FaultStatus.
 *  This common fault condition flag is also use to detect a Fault Recovery condition.
 *  Users can specify a global function used to reset the system wide fault condition
 *  triggering a system-wide recovery procedure.
 * 
 *  The fault scan also allows to specify if the system should be locked in a
 *  latched fault state if a given number of successive restart attempts failed.
 *  The maximum number of restart attempts can be specified by setting the fault 
 *  monitor property fltMonitorInstance->FaultLatchCount to a value greater than one.  
 *  If the property fltMonitorInstance->FaultLatchCount is set to zero, no latched 
 *  fault condition will be triggered then the system will initiate an unlimited  
 *  number of recovery attempts.
 * 
 *  This function requires users to declare a list (array) of pointers to 
 *  individually declared fault objects, like shown in the following code 
 *  example:
 *
 *  <p><b>Example:</b></p>
 *  @code{.c}
 *     FAULT_MONITOR_t FaultMonitor;           // Declare fault monitoring data object
 *     FAULT_OBJECT_t* FaultObjectList[] = {   // Declare fault objects collection
 *         &fltobj_Object_1,
 *         &fltobj_Object_2,
 *         (...),
 *         &fltobj_Object_n
 *     };
 *     uint16_t FaultObjectList_size = (sizeof(FaultObjectList)/sizeof(FaultObjectList[0]));
 *  @endcode
 *
 *  @note
 *   When initializing the fault monitoring data object, the fault objects collection
 *   must be referenced within the fault monitoring data object data structure like
 *   shown in the following code example:
 *
 *  @code{.c}
 * 
 *     FaultMonitor.fltObjectListSize = FaultObjectList_size; // initialize fault object collection size
 *     FaultMonitor.fltObjectList = &FaultObjectList[0];      // initialize fault object collection
 * 
 *  @endcode
 * 
 **********************************************************************************/
int svc_FaultMonitor_Execute(FAULT_MONITOR_t* fltMonitorInstance)
{
    int retval = 1;             // function return value
    uint32_t fltLog=0;          // buffer variable capturing the encoded fault list
    bool     fltState=false;    // buffer variable capturing the overall fault state
    
    static bool fltFastRecovery=true; // Static buffer variable for fault recovery tracking
    
    // Scan through all defined fault objects and log their state in common fault state flag
    for (int i=0; i<fltMonitorInstance->fltObjectListSize; i++)
    {
        // Check for fault conditions
        FAULT_OBJECT_t* _fltObject  = (FAULT_OBJECT_t*) fltMonitorInstance->fltObjectList[i];
        retval    &= svc_FaultObject_CheckObject(_fltObject);
        
        // Log overall fault status
        fltState |= _fltObject->Status.bits.FaultStatus;
        fltLog   |= (uint32_t)((_fltObject->Status.bits.FaultStatus)<<(i));
        
        // Detect if fast fault recovery is allowed
        if ((_fltObject->Status.bits.FaultStatus == true) && (_fltObject->Status.bits.Enable == true))
        { fltFastRecovery &= _fltObject->Status.bits.FastRecovery; }
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // Update common fault state flag status
    fltMonitorInstance->Status.bits.FaultStatus = fltState;
    fltMonitorInstance->FaultStatusList = fltLog;
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // If a latched fault condition has been set, 
    if (true == fltMonitorInstance->Status.bits.FaultLatch)
    {
        fltMonitorInstance->Status.bits.PreFaultStatus = true;
        fltMonitorInstance->Status.bits.FaultStatus = true;
    }
    else
    {
        // If a global fault conditions has been cleared 
        if ((fltMonitorInstance->Status.bits.PreFaultStatus) && (!fltState))
        { 
             fltFastRecovery &= fltMonitorInstance->Status.bits.FastRecovery;
            
            // Enforce Fast Startup?
            if (((fltFastRecovery) || (wakeUpBoot)) && (fltMonitorInstance->FaultLatchCounter == 0))
                fltMonitorInstance->FaultRecoveryCounter = fltMonitorInstance->FaultRecoveryPeriod;
            
            fltFastRecovery=false;
            wakeUpBoot=false;
            
            // Enforce recovery delay?
            if (++fltMonitorInstance->FaultRecoveryCounter > fltMonitorInstance->FaultRecoveryPeriod)
            {
                // Clamp counter at maximum value
                fltMonitorInstance->FaultRecoveryCounter = fltMonitorInstance->FaultRecoveryPeriod;

                // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                // Execute Recovery Function
                if (fltMonitorInstance->FaultRecovery != NULL)     // Check if recovery function has been assigned
                    retval &= fltMonitorInstance->FaultRecovery(); // Initiate fault recovery attempt
                // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

                // Track global fault bit transitions
                fltMonitorInstance->Status.bits.PreFaultStatus = fltState;

                // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                // Increment Fault Recovery Attempt Counter
                if (++fltMonitorInstance->FaultLatchCounter > fltMonitorInstance->FaultLatchLimit)
                { 
                    // Clamp counter at maximum value
                    fltMonitorInstance->FaultLatchCounter = fltMonitorInstance->FaultLatchLimit;

                    // Latch system-wide Fault Condition
                    fltMonitorInstance->Status.bits.FaultStatus = true; 
                    fltMonitorInstance->Status.bits.FaultLatch = true; 
                }
                // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            }

        }
        
        // If a global fault conditions has been set
        else if ((!fltMonitorInstance->Status.bits.PreFaultStatus) && (fltState)) 
        { // Track global fault bit transitions and clear Recovery counter
            fltMonitorInstance->Status.bits.PreFaultStatus = fltState;
            fltMonitorInstance->FaultRecoveryCounter = 0; 
            fltFastRecovery=true;
        }
        else { /* do nothing */ }
    }
    
	return(retval);

}

/*********************************************************************************
 * @ingroup apps-layer-fault-lib-functions-public
 * @brief   Disposes a list of user-defined fault objects
 * @param   fltMonitorInstance Pointer to user defined fault monitoring data object of type FAULT_MONITOR_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  This function disposes a list of user defined fault objects of type 
 *  FAULT_OBJECT_t by clearing all settings of all fault objects in the 
 *  fault object collection assigned within the fault monitor instance. 
 * 
 **********************************************************************************/
int svc_FaultMonitor_Dispose(FAULT_MONITOR_t* fltMonitorInstance)
{
    int retval = 1;
    
    for (int i=0; i<fltMonitorInstance->fltObjectListSize; i++)
    { *fltMonitorInstance->fltObjectList[i] = fltObjectClear; }
    
	return(retval);

}

/*********************************************************************************
 * @ingroup apps-layer-fault-lib-functions-public
 * @brief   Reset the status of all user-defined fault objects
 * @param   fltMonitorInstance Pointer to user defined fault monitoring data object of type FAULT_MONITOR_t
 * @return  0=failure of type unsigned integer
 * @return  1=success of type unsigned integer
 *
 * @details
 *  This function reset a previously initialized list of user defined 
 *  fault objects of type FAULT_OBJECT_t by clearing all status flags and
 *  clears the global reset and recovery status of the fault monitor 
 *  object itself.
 * 
 **********************************************************************************/
int svc_FaultMonitor_Reset(FAULT_MONITOR_t* fltMonitorInstance)
{
    int retval = 1;
    
    // Reset Fault Object Status Flags
    for (int i=0; i<fltMonitorInstance->fltObjectListSize; i++)
    { 
        FAULT_OBJECT_t* _flt = fltMonitorInstance->fltObjectList[i];
        
        _flt->Counter = 0;
        _flt->Status.bits.FaultActive = 1; 
        _flt->Status.bits.FaultStatus = 1; 
    }

    // Reset Fault Monitor Status Flags
    fltMonitorInstance->FaultRecoveryCounter = 0;
    fltMonitorInstance->FaultLatchCounter = 0;
    fltMonitorInstance->Status.bits.FaultLatch = false;
    fltMonitorInstance->Status.bits.PreFaultStatus = false;
    fltMonitorInstance->Status.bits.FaultStatus = true;
    
	return(retval);
}

// ____________________________
// end of file
