/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    svc_fault_object.h
 * @author  M91406
 * @brief   Fault object type declaration header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef FAULT_OBJECT_DECLARATION_H
#define	FAULT_OBJECT_DECLARATION_H

#include <stdint.h> // include standard integer types 
#include <stdbool.h> // include standard boolean types  

/**
 * @addtogroup apps-layer-fault-lib-properties-public-data-types
 * @{
 */
/***********************************************************************************
 * @brief 	This data structure is comprised of fault comparison types
 * @extends FLT_OBJECT_STATUS_s
 *********************************************************************************/
enum FLT_COMPARE_TYPE_e {
    
	FLTCMP_NONE             =  0,   ///<  No comparison type has been defined (fault object will be ignored)
	FLTCMP_GREATER_THAN     =  1,   ///<  Check for condition: SOURCE > trip level
	FLTCMP_GREATER_OR_EQUAL =  2,   ///<  Check for condition: SOURCE >= trip level
	FLTCMP_LESS_THAN        =  3,   ///<  Check for condition: SOURCE < trip level
	FLTCMP_LESS_OR_EQUAL    =  4,   ///<  Check for condition: SOURCE <= trip level
	FLTCMP_IS_EQUAL         =  5,   ///<  Check for condition: SOURCE = trip level
	FLTCMP_IS_NOT_EQUAL     =  6,   ///<  Check for condition: SOURCE != trip level
	FLTCMP_BETWEEN          =  7,   ///<  Check for condition: (reset_level < SOURCE) && (SOURCE < trip_level)
	FLTCMP_BETWEEN_INCLUDE  =  8,   ///<  Check for condition: (reset_level <= SOURCE) && (SOURCE <= trip_level)
	FLTCMP_OUTSIDE          =  9,   ///<  Check for condition: (SOURCE < reset_level) || (trip_level < SOURCE)
	FLTCMP_OUTSIDE_INCLUDE  = 10    ///<  Check for condition: (SOURCE <= reset_level) || (trip_level <= SOURCE)
        
}; ///< Fault handler compare type enumeration
typedef enum FLT_COMPARE_TYPE_e FLT_COMPARE_TYPE_t; ///< Fault handler compare type enumeration data type

/**********************************************************************************
 * @brief 	This data structure defines the fault object status
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
struct FLT_OBJECT_STATUS_s {

    bool FaultStatus : 1;               ///< Bit 0: Flag bit indicating if FAULT has been tripped
	bool FaultActive : 1;               ///< Bit 1: Flag bit indicating if FAULT CONDITION has been detected but FAULT has not been tripped yet
	unsigned : 6;                       ///< Bit <7:2>:   (reserved)
	FLT_COMPARE_TYPE_t CompareType: 4;  ///< Bit <11:8>:  Fault check comparison type control bits
	unsigned : 2;                       ///< Bit <13:12>: (reserved)
	bool FastRecovery : 1;              ///< Bit 14: Control bit requesting fast restart if fault is cleared
	bool Enable : 1;                    ///< Bit 15: Control bit enabling/disabling monitoring of the fault object
        
}__attribute__((packed)); // Fault object status bit field for single bit access

/**********************************************************************************
 * @brief 	This data structure defines the fault object status
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
union FLT_OBJECT_STATUS_u
{
    struct FLT_OBJECT_STATUS_s bits; // Fault object status bit-field
    uint16_t value;         // Fault object status word  
};
typedef union FLT_OBJECT_STATUS_u FLT_OBJECT_STATUS_t; ///< Fault object status data type

/**********************************************************************************
 * @brief 	This data structure defines the data object which will be monitored by the fault handler
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
struct FLT_COMPARE_OBJECT_s {
    
    int16_t* ptrObject;     ///< Pointer to register or variable which should be monitored 
    uint16_t BitMask;       ///< Bit mask will be &-ed with source as value (use 0xFFFF for full value comparison)
    
}; ///< Fault compare object
typedef struct FLT_COMPARE_OBJECT_s FLT_COMPARE_OBJECT_t; ///< Fault compare object data type

/**********************************************************************************
 * @brief   This data structure defines the fault monitor event response object
 * @extends FAULT_OBJECT_s
 *********************************************************************************/
struct FLT_RESPONSE_EVENT_s {
    
    int32_t  CompareThreshold;              ///< Signal level at which the fault condition will be detected
    uint16_t EventThreshold;                ///< Bit mask will be &-ed with source as value (use 0xFFFF for full value comparison)
    int (*ptrResponseFunction)(void);       ///< pointer to a user-defined function called when a defined fault monitoring event is detected
    
}; ///< Fault monitor event response object
typedef struct FLT_RESPONSE_EVENT_s FLT_RESPONSE_EVENT_t;	///< Fault object event response object data type

/**********************************************************************************
 * @brief 	This data structure is a collection of data structures for fault handling.
 *********************************************************************************/
struct FAULT_OBJECT_s {

	FLT_OBJECT_STATUS_t Status;             ///< Status word of this fault object
	uint16_t Counter;                       ///< Fault event counter (controlled by FAULT HANDLER)
	FLT_COMPARE_OBJECT_t SourceObject;      ///< Object which should be monitored
	FLT_COMPARE_OBJECT_t ReferenceObject;   ///< Reference object the source should be compared with
    FLT_RESPONSE_EVENT_t TripResponse;      ///< Settings defining the fault trip event
    FLT_RESPONSE_EVENT_t RecoveryResponse;  ///< Settings defining the fault recovery event

}; ///< Generic fault object 
typedef struct FAULT_OBJECT_s FAULT_OBJECT_t; ///< Generic fault object data type
typedef FAULT_OBJECT_t* FAULT_OBJECT_COLLECTION_t; ///< Type declaration of a collection of fault objects

/** @} */ // end of group

#endif	/* FAULT_OBJECT_DECLARATION_H */

// __________________________
// end of file
