/*
 * @file    svr_fault_object_handler.c
 * @author  M91406
 * @date    February 10, 2023, 6:14 PM
 */

#include <stdlib.h>
#include "svc_fault_object.h"

// Private function declarations
static inline int32_t svc_FaultObject_GetSource(volatile FAULT_OBJECT_t* fltObject);
static inline int svc_FaultObject_UpdateFaultFlag(volatile FAULT_OBJECT_t* fltObject, int32_t sourceData);

/*******************************************************************************
 * @ingroup apps-layer-fault-lib-functions-private
 * @param	fltObject Pointer to a Fault Monitoring Object of type FAULT_OBJECT_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @brief Check current fault status of a user-defined fault object
 *
 * @details
 *  This routine compares the most recent value of a global variable or SFR
 *  (SOURCE) against user-defined thresholds. If the value violates/exceeds
 *  one or more of the user defined conditions, a fault counter is incremented. 
 * 
 *  When this fault counter exceeds the user-defined maximum, a fault condition 
 *  is tripped and the fault_status flag bit is set.
 * 
 *  When a fault condition has been tripped and the fault status is set (=true), 
 *  the fault check monitors for the source value triggering a recovery event. 
 *  The recovery process works like the fault trip event by comparing the most 
 *  recent source value against the user-defined RECOVERY_LEVEL. Once the value
 *  has been RSTCNT_MAX times below the RECOVERY_LEVEL threshold, the fault 
 *  will automatically be cleared.
 * 
 *  a) Comparison Types
 * 
 *  The fault handler offers the following different comparison methods:
 *      - Greater Than:
 *          - performs comparison SOURCE > TRIP_LEVEL
 * 
 *          - TRIP_LEVEL is greater than RECOVERY_LEVEL. The difference between 
 *          TRIP_LEVEL and RECOVERY_LEVEL is the hysteresis of the defined
 *          threshold.
 * 
 *      - Less Than:
 *          - performs comparison SOURCE < TRIP_LEVEL
 * 
 *          - TRIP_LEVEL is less than RECOVERY_LEVEL. The difference between 
 *          TRIP_LEVEL and RECOVERY_LEVEL is the hysteresis of the defined
 *          threshold.
 *
 *      - Is Equal:
 *          - performs comparison SOURCE == TRIP_LEVEL
 *
 *          - RECOVERY_LEVEL is ignored.
 *
 *      - Is Not Equal:
 *          - performs comparison SOURCE != TRIP_LEVEL
 *
 *          - RECOVERY_LEVEL is ignored.
 *
 *      - Between:
 *          - performs comparison RECOVERY_LEVEL < SOURCE < TRIP_LEVEL
 *
 *          - min/max of the FAULT range is defined by the range between
 *          RECOVERY_LEVEL (min) and TRIP_LEVEL (max)
 *
 *      - Outside:
 *          - performs comparison (SOURCE < RECOVERY_LEVEL) or (TRIP_LEVEL < SOURCE)
 *
 *          - min/max of the allowed operating range is defined by the range 
 *          between RECOVERY_LEVEL (min) and TRIP_LEVEL (max)
 * 
 *  b) Value Filtering
 * 
 *    A bit-mask filter can be used to monitor individual register bits or filter
 *    on specific bits within a 16-bit wide value. The bit-mask is always applied 
 *    the source value. If no bit-mask is required, set bitmask = 0xFFFF; 
 * 
 *  c) Comparison of two dynamic values
 * 
 *    When dynamic values of two variables need to be compared, the fault object
 *    offers a second pointer to a so-called reference object "ref_obj". When this
 *    pointer is != NULL, the absolute value of the difference between SOURCE and 
 *    REFERENCE is compared against the user-defined TRIP and RECOVERY thresholds.
 *    When no second  value is used, set ref_obj = NULL (NULL-pointer). This 
 *    will force the fault check to ignore this value.
 * 
 *  d) User defined responses
 *    
 *    Function pointers "trip_response" and "reset_response" can be used 
 *    to assign user-defined functions which should be called when a fault
 *    condition has been tripped or reset.
 *    I no user-defined function should be called, these pointers can be 
 *    set = NULL (NULL-pointer)
 * 
 *  @note 
 *   If the value is within normal operating conditions, the fault 
 *   counter will be cleared. Thus fault events must occur successively 
 *   incrementing the fault event counter high enough to eventually 
 *   trip a fault event. Thus the fault event counter can be used to 
 *   adjust the sensitivity of the fault response.
 * 
 *****************************************************************************/
int svc_FaultObject_CheckObject(volatile FAULT_OBJECT_t* fltObject) 
{
    int retval = 1;
    
    // If the fault object is not initialized, exit here with error
    if (fltObject == NULL)
        return(0);

    // If the source object is not initialized, exit here with error
    if (fltObject->SourceObject.ptrObject == NULL)
        return(0);
    
    // Get FAULT ACTIVE flag 
    int32_t _source = (int32_t)svc_FaultObject_GetSource(fltObject);
    if (svc_FaultObject_UpdateFaultFlag(fltObject, _source) == false)
        return(0);

    // Allow fault status to be set only if fault object check is enabled
    if ((!fltObject->Status.bits.Enable) && (fltObject->Status.bits.FaultStatus))
    {
        fltObject->Status.bits.FaultStatus = false;
        fltObject->Counter = fltObject->RecoveryResponse.EventThreshold;
    }
    
    // If a fault condition has been detected while no FAULT has been tripped....
    if (((!fltObject->Status.bits.FaultStatus) && (fltObject->Status.bits.FaultActive)) && fltObject->Status.bits.Enable)
    {
        fltObject->Counter++;  // Increment fault event counter
        
        // Trigger on FAULT conditions
        if (fltObject->Counter >= fltObject->TripResponse.EventThreshold)
        {
            fltObject->Status.bits.FaultStatus = true;    // Set FAULT STATUS flag bit
            fltObject->Counter = fltObject->TripResponse.EventThreshold; // Set fault event counter to threshold level
            if (fltObject->TripResponse.ptrResponseFunction != NULL)    // If a user function has been defined,
                retval = fltObject->TripResponse.ptrResponseFunction();   // => call this function and capture return value
        }

    }
    // If a FAULT has been tripped but no fault condition has been detected anymore....
    else if ((fltObject->Status.bits.FaultStatus) && (!fltObject->Status.bits.FaultActive))
    {
        fltObject->Counter++;  // Increment fault event counter

        // Trigger on RECOVERY conditions
        if (fltObject->Counter >= fltObject->RecoveryResponse.EventThreshold)
        {
            fltObject->Status.bits.FaultStatus = false;   // Clear FAULT STATUS FLAG BIT
            fltObject->Counter = fltObject->RecoveryResponse.EventThreshold; // Set fault event counter to threshold level
            if (fltObject->RecoveryResponse.ptrResponseFunction != NULL)   // If a user function has been defined,
                retval = fltObject->RecoveryResponse.ptrResponseFunction();  // => call this function and capture return value
        }
    
    }
    // If everything is OK, reset counter
    else
    {
        fltObject->Counter = 0;  // clear fault event counter
    }
    
    return (retval); // Fault handler executed successfully
}

/**********************************************************************************/
/**********************************************************************************/
/**********************************************************************************/

/*********************************************************************************
 * @ingroup apps-layer-fault-lib-functions-public
 * @brief   Gets the most recent data input of the source of the given fault object
 * @param   fltObject:  Pointer to fault object of type FAULT_OBJECT_t*
 * @return  16-bit value of the fault object source data
 *
 **********************************************************************************/
static int32_t svc_FaultObject_GetSource(volatile FAULT_OBJECT_t* fltObject)
{
    volatile int32_t sourceData=0;

    // Read most recent fault object value with bit-mask
    sourceData  = (int32_t)(int16_t)((*fltObject->SourceObject.ptrObject) & (fltObject->SourceObject.BitMask));
    
    // If a reference object has been defined, read reference object value and override source with 
    // absolute value of difference between source and reference object values
    if(fltObject->ReferenceObject.ptrObject != NULL) {  
        
        int32_t reference = (int32_t)(*fltObject->ReferenceObject.ptrObject & fltObject->ReferenceObject.BitMask);
        sourceData = (volatile int32_t)
            abs((volatile int32_t)sourceData - (volatile int32_t)reference); // Load most recent value
    }

    return(sourceData);
    
}

/*********************************************************************************
 * @ingroup apps-layer-fault-lib-functions-public
 * @brief   Updates the immediate fault condition flag FaultActive of the given fault object
 * @param   fltObject:  Pointer to fault object of type FAULT_OBJECT_t*
 * @param   sourceData: Most recent source data of fault object of type FAULT_OBJECT_t
 * @return  0: Update of fault flag failed
 * @return  1: Update of fault flag was success
 *
 **********************************************************************************/
static int svc_FaultObject_UpdateFaultFlag(volatile FAULT_OBJECT_t* fltObject, int32_t sourceData)
{
    int retval = 1;
    bool _tripRes=false;
    bool _releaseRes=false;
    int32_t _tripComp=(int32_t)fltObject->TripResponse.CompareThreshold;
    int32_t _releaseComp=(int32_t)fltObject->RecoveryResponse.CompareThreshold;
            
    // ----------------------------------------------
    // Check declared fault condition
    switch(fltObject->Status.bits.CompareType) {

        case FLTCMP_GREATER_THAN:       // Check if SOURCE > TRIP_LEVEL
            _tripRes    = (sourceData > _tripComp); 
            _releaseRes = (sourceData < _releaseComp); 
            break; 
        case FLTCMP_GREATER_OR_EQUAL:   // Check if SOURCE >= TRIP_LEVEL
            _tripRes    = (sourceData >= _tripComp); 
            _releaseRes = (sourceData <= _releaseComp); 
            break; 
        case FLTCMP_LESS_THAN:          // Check if SOURCE < TRIP_LEVEL
            _tripRes    = (sourceData < _tripComp); 
            _releaseRes = (sourceData > _releaseComp); 
            break; 
        case FLTCMP_LESS_OR_EQUAL:      // Check if SOURCE < TRIP_LEVEL
            _tripRes    = (sourceData <= _tripComp); 
            _releaseRes = (sourceData >= _releaseComp); 
            break; 
        case FLTCMP_IS_EQUAL:           // Check if SOURCE = TRIP_LEVEL
            _tripRes    = (sourceData == _tripComp); 
            _releaseRes = (sourceData != _releaseComp); 
            break; 
        case FLTCMP_IS_NOT_EQUAL:       // Check if SOURCE != TRIP_LEVEL
            _tripRes    = (sourceData != _tripComp); 
            _releaseRes = (sourceData == _releaseComp); 
            break; 
        case FLTCMP_BETWEEN:            // Check if (RECOVERY_LEVEL < SOURCE) && (SOURCE < TRIP_LEVEL)
            _tripRes    = ((_releaseComp < sourceData) && (sourceData < _tripComp)); 
            _releaseRes = ((_releaseComp > sourceData) || (sourceData > _tripComp)); 
            break; 
        case FLTCMP_BETWEEN_INCLUDE:    // Check if (RECOVERY_LEVEL < SOURCE) && (SOURCE < TRIP_LEVEL)
            _tripRes    = ((_releaseComp <= sourceData) && 
                           (sourceData <= _tripComp)); 
            _releaseRes = ((_releaseComp >  sourceData) || 
                           (sourceData >  _tripComp)); 
            break; 
        case FLTCMP_OUTSIDE:            // Check if (SOURCE < RECOVERY_LEVEL) && (TRIP_LEVEL < SOURCE)
            _tripRes    = ((sourceData < _releaseComp) || (_tripComp < sourceData)); 
            _releaseRes = ((sourceData > _releaseComp) && (fltObject->TripResponse.CompareThreshold > sourceData)); 
            break; 
        case FLTCMP_OUTSIDE_INCLUDE:    // Check if (SOURCE <= RECOVERY_LEVEL) && (TRIP_LEVEL <= SOURCE)
            _tripRes    = ((sourceData <= _releaseComp) || (fltObject->TripResponse.CompareThreshold <= sourceData)); 
            _releaseRes = ((sourceData >  _releaseComp) && (fltObject->TripResponse.CompareThreshold >  sourceData)); 
            break; 

        default:
            _tripRes    = true;  // In case selected compare type is not supported, set fault ACTIVE
            _releaseRes = false; // In case selected compare type is not supported, set fault ACTIVE
            retval      = 0;     // Return=>Error (Ignore fault check if compare type is not defined)
            break;
    
    }
    
    // ----------------------------------------------
    // Capture compare result
    bool _bRes = true;
    
    // Trip level has been exceeded
    if (_tripRes) _bRes = true; 
    // Fault was active, but has not reached recovery level yet
    else if (fltObject->Status.bits.FaultActive && (!_releaseRes)) _bRes = true;
    // Fault condition has been cleared
    else _bRes = false;

    // Update FaultActive flag
    fltObject->Status.bits.FaultActive = _bRes;
    
    return(retval);
}

/**********************************************************************************/
/**********************************************************************************/
/**********************************************************************************/

// __________________________
// end of file
