# Buck Converter Service Component {#powerlibrary}

---

<span id="startDoc"> </span> <!-- start tag for internal references -->

## Class

PowerSmart™ Firmware Framework Middleware Component

## Dependencies

- [MPLAB&reg; PowerSmart&trade; Development Suite generated NPNZ16b Control Loop(s)](https://www.microchip.com/powersmart)
- [P33C Host Abstraction Layer / Peripheral Special Function Register Abstraction Layer (PRAL)](https://bitbucket.microchip.com/projects/MCU16ASMPSL/repos/p33c-peripheral-sfr-abstraction-layer/browse)

## Recommended Complementary Components

- [PowerSmart™ Firmware Framework Fault Handler Service](https://bitbucket.microchip.com/projects/MCU16ASMPSL/repos/subrepo-fault-handler/browse)

---

## Summary

Tunable **buck topology** step-down converter configuration and state machine service supporting single- and multiphase architectures with asynchronous and synchronous switch nodes.

---

## Table Of Contents

<div id="TOC">
<ul>
<li><a href="#intro">Introduction</a></li>
<li><a href="#lib-use">Library Usage Guidelines</a></li>
<li><a href="#api-guide">API Quick-Start Guide</a></li>
<li><a href="#api-objects">API Public Data Objects</a></li>
<li><a href="#state-machine">Buck Converter State Machine</a></li>
<li><a href="#history">History</a></li>
</ul>
</div>

<span id="intro"> </span>

### Introduction

#### Supported Features

- Single Phase Architecture
- Multiphase Architecture
- Synchronous Buck Converter
- Asynchronous Buck Converter
- Control Modes:
  - Voltage Mode Control (VMC)
  - Peak Current Mode Control (PCMC)
  - Average Current Mode Control (ACMC)
  - Constant Current Mode Control (CCMC)
- Multiphase Control
  - Individual Phase Control (PCMC, ACMC and CCMC only)
  - Phase Multiplexing (all control modes)

#### Special Features

- Tunable Startup Timing
  - Power On Delay (POD)
  - Ramp-Up Period / Slew Rate
  - Power Good Delay
- Current Ramp-Up State for Constant Current Sources (automatic)
- Slope controlled reference tuning for voltage and current limits during normal operation

#### Optional Dynamic RUntime Features

- Automatic Synchronous Rectifier Control (optional)
- Current Sense Feedback Offset Calibration (optional)
- GPIO extensions for Power Good Output and Enable Input (optional)

#### Description

This generic synchronous buck converter device driver provides a state machine, peripheral configuration templates and controller interface for driving one single- or multiphase synchronous or asynchronous buck converter instance in voltage, peak or average current mode control.

All settings of this code library are made in the Buck Converter Controller Object defined by the BUCK_POWER_CONTROLLER_s data structure. Each API function takes an pointer to a Buck Converter Controller Object of type `IBuckConverter_t` as parameter. This allows to define multiple Buck converter instances using the same library files to drive and control multiple, independent converters in parallel.

#### Tool Support

This device driver uses control loop code based on `NPNZ16b_t` data objects which is generated by the [MPLAB&reg; PowerSmart&trade; Development Suite](https://www.microchip.com/powersmart).

<a href="#startDoc">back</a>

<span id="lib-use"> </span>

### Library Usage Guidance

This code library repository must be included in an independent directory within the target project. Changes to any of the files of this repository in the target project environment can be pushed back to this library repository from where these changes can be distributed to one, more or all target projects this library has been used in. The upgrade of library files in a particular target project needs to be performed individually, allowing to individually manage the device driver version used.

#### Adding Subtree Repository

##### Step 1 - Adding Subtree Repository to Target Project

###### a) Using Sourcetree Application

Open *Sourcetree* Application

- Open menu *Repository* → *Repository Settings*
- Click *Add* to open the Add Repository dialog

Inside the Repository dialog

- Enter *Remote Name* `subrepo-buck`
- Enter clone URL/path https://bitbucket.microchip.com/scm/mcu16asmpsl/subrepo-buck-converter.git
- Under *Optional extended integration* → *Remote Account* select your user account
- Click *OK* to close the dialog

###### b) Using Git Bash

Open Git Bash at root of target project repository and use the following command for adding the new subtree remote:

- `$ git remote add subrepo-buck https://bitbucket.microchip.com/scm/mcu16asmpsl/subrepo-buck-converter.git`

##### Step 2) Cloning Subtree Repository

Further using Git Bash use the `subtree add` command to clone the contents of this code library into the target project

- `$ git subtree add --prefix=<code_library_target> subrepo-buck main --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library should be cloned to (e.g. my_project.X/sources/power_control/devices).

#### 3) Pulling latest version from Library Repository

When a new version of this code library is available, you can pull it by using the `subtree pull` command in the Git Bash:

`$ git subtree pull --prefix=<code_library_target> subtree-buck main --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library has been cloned to (e.g. my_project.X/sources/fault_handler/drivers).

#### 4) Pushing new version back to Library Repository

When changes have been made to files of this code library, the new version can be pushed back to the library repository by using the `subtree push` command in the Git Bash:

`$ git subtree push --prefix=<code_library_target> subtree-buck feature/version-update --squash`

with `<code_library_target>` = path to sub-folder within the target project directory structure this code library has been cloned to (e.g. my_project.X/sources/fault_handler/drivers).

<span style="color:red">
    <u><b>Note:</b></u><br>
    Pushing directly to the library project 'main' or 'develop' branches may be prohibited. 
    Hence, <i><u><b>changes can only be pushed to feature branches</b></u></i>. A Pull Request is required to review and merge changes to 'develop'. Once changes have been approved and merged int 'develop', they may be merged into branch 'main' and thus released as new version through another pull request. This new version of 'main' can be tagged with a new version number and pulled into target projects.
</span>


<br>&nbsp;

<a href="#startDoc">back</a>

<span id="api-guide"> </span>

### API Quick-Start Guide

#### API Public Functions

The API of this device driver provides the following standard power converter functions:

```c
volatile uint16_t BuckConverter_Initialize(volatile IBuckConverter_t* pcInstance);
volatile uint16_t BuckConverter_Execute(volatile IBuckConverter_t* pcInstance);
volatile uint16_t BuckConverter_Start(volatile IBuckConverter_t* pcInstance);
volatile uint16_t BuckConverter_Stop(volatile IBuckConverter_t* pcInstance);
volatile uint16_t BuckConverter_Suspend(volatile IBuckConverter_t* pcInstance);
volatile uint16_t BuckConverter_Resume(volatile IBuckConverter_t* pcInstance);
```

Each of these functions takes a function call parameter of type ```IBuckConverter_t``` (see blow) to be applied to a particular buck converter object and returns an integer return value indicating success (= 1) or failure (= 0) of the function execution.

- **`uint16_t BuckConverter_Initialize(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
Initializes the buck converter object by first loading the built-in peripheral configuration templates into the required peripheral special function registers, adding the user settings specified in the *struct BUCK_POWER_CONTROLLER_s* Buck Converter data object. After a successful peripheral initialization the specified control loop objects will be reset and their histories cleared. <br>&nbsp;<br>All peripherals and control loops will be initialized but kept disabled. The state machine will later enable the peripherals and controller objects used in their proper enable sequence during the startup procedure and no user code intervention is required.<br>&nbsp;<br>

- **`uint16_t BuckConverter_Start(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
The *BuckConverter Start* function is initiating the state machine from scratch, by enabling the fundamental peripherals PWM and ADC to start the acquisition of feedback data allowing the state machine to analyze the most recent operating conditions. <br>&nbsp;
Function *BuckConverter Execute* **should be** called after the execution of this function at a constant frequency.<br>&nbsp;

- **`uint16_t BuckConverter_Stop(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
The *BuckConverter Stop* function is terminating the state machine and disabling the fundamental peripherals PWM and ADC to stop the acquisition of feedback data stopping the analysis of the most recent operating conditions. <br>&nbsp;
Function *BuckConverter Execute* should **not** be called after the execution of this function.<br>&nbsp;

- **`uint16_t BuckConverter_Suspend(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
The *BuckConverter Suspend* function is terminating the state machine but keeps the fundamental peripherals PWM and ADC running to continue the acquisition of feedback data allowing the analysis of the most recent operating conditions. <br>&nbsp;
Function *BuckConverter Execute* **should continued to be** called after the execution of this function at a constant frequency.<br>&nbsp;

- **`uint16_t BuckConverter_Resume(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
The *BuckConverter Resume* function is re-initiating the state machine to recover from RESET during which the fundamental peripherals PWM and ADC are running to continue the acquisition of feedback data allowing the analysis of the most recent operating conditions. <br>&nbsp;
Function *BuckConverter Execute* **should be** called after the execution of this function at a constant frequency.<br>&nbsp;

- **`uint16_t BuckConverter_Execute(volatile IBuckConverter_t* pcInstance)`**<br>&nbsp;
The *BuckConverter Execute* function is executing the state machine managing the initialization, startup and online regulation in voltage and average current mode control. It provides mechanisms for fault handlers to shut down and recover the power supply control in controlled, deterministic procedures using the *drv_BuckConverter_Suspend* and *drv_BuckConverter_Resume* function calls described above.<br>&nbsp;

<a href="#startDoc">back</a>

<span id="api-objects"> </span>

#### API Public Data Objects

The Buck Converter Data Object holds all configuration data of the user-defined buck converter, covering

- Functional Declarations
  - Startup Timing Declarations
  - User Set Value Declarations

- Peripheral Configuration Declarations
  - Switch Node Declarations (PWM and ACMP Configuration)
  - Feedback Channel Declarations (ADC Configuration)
  - GPIO Declarations (GPIO Configuration)
  - Voltage Loop Controller Declarations
  - Current Loop Controller Declarations (ACMC only, one controller per phase required)

- Status Bit Declarations
  - Read-Only Data and Status Information
  - State Machine State Monitoring
  - Runtime Data

- Control Bit Declarations
  - Allowing fast runtime manipulation of features
  - Bit field allows simultaneous manipulation of features

```c
struct IBuckConverter_s 
{
    // State Machine Runtime Status Information
    volatile BUCK_STATE_ID_t StateId;               ///< BUCK state machine operating state ID
    volatile BUCK_CONVERTER_STATUS_t Status;        ///< BUCK operation status bits 
    volatile BUCK_CONVERTER_DATA_t Data;            ///< BUCK runtime data

    // State Machine Status Configuration 
    volatile BUCK_CONVERTER_CONTROL_t Control;      ///< BUCK operation control bits 
    volatile BUCK_CONVERTER_SETTINGS_t Settings;    ///< Control field for global access to references
    volatile BUCK_CONVERTER_STARTUP_t Startup;      ///< BUCK startup timing settings 

    // Power Supply Hardware Configuration 
    volatile BUCK_SWITCH_NODE_SETTINGS_t SwitchNode[BUCK_MPHASE_COUNT]; ///< BUCK converter switch node settings
    volatile BUCK_FEEDBACK_SETTINGS_t Feedback;     ///< BUCK converter feedback settings
    volatile BUCK_GPIO_SETTINGS_t Gpio;             ///< BUCK converter additional GPIO specification
    
    // Power Supply Control Configuration 
    volatile BUCK_LOOP_SETTINGS_t VLoop;            ///< BUCK voltage control loop object
    volatile BUCK_LOOP_SETTINGS_t ILoop[BUCK_MPHASE_COUNT]; ///< BUCK Current control loop objects
    
}; ///< BUCK control & monitoring data structure
typedef struct IBuckConverter_s IBuckConverter_t; ///< BUCK control & monitoring data structure data type
```

Before the state machine can be used, at least one buck converter data object has to be defined and initialized in the application layer of the user project.

##### Declaration Examples

**Example 1:** Declaration of single buck converter data object

```c
IBuckConverter_s myBuckConverter; // Declare single buck converter data object
```

**Example 2:** Declaration of array of four buck converter data objects

```c
IBuckConverter_s myBuckConverter[4]; // Declare array of four buck converter data objects
```


<br>&nbsp;

#### Sub Structure  `BUCK_CONVERTER_STATUS_t`

The buck converter status bit field contains read-only, auto-set/clear status bits:

```c
struct BUCK_CONVERTER_STATUS_s
{
    volatile bool Ready:1;              ///< Bit #0:  Status bit indicating if buck converter has been initialized and is ready to run
    volatile bool AdcActive:1;          ///< Bit #1:  Status bit indicating if ADC has been started and samples are taken
    volatile bool PwmActive:1;          ///< Bit #2:  Status bit indicating if PWM has been started and ADC triggers are generated
    volatile bool FaultActive :1;       ///< Bit #3:  Status bit indicating if system is in enforced shut down mode (usually due to a fault condition)
    volatile bool FaultLatch :1;        ///< Bit #4:  Status bit indicating if system is in latched enforced shut down mode (requires manual clearance)
    volatile bool CsCalibComplete :1;   ///< Bit #5:  Status bit indicating if current sense calibration process is complete
    volatile bool IsSuspended :1;       ///< Bit #6:  Status bit indicating if converter is in suspend mode and state machine is hold in reset
    volatile bool PwmOutputActive:1;    ///< Bit #7:  Status bit indicating if the PWM outputs are enabled or disabled

    volatile bool AsyncActive :1;       ///< Bit #8:  Status bit indicating if the switch node is driven in asynchronous mode (=true) or synchronous mode (=false)
    volatile unsigned :1;               ///< Bit #9:  (reserved)
    volatile unsigned :1;               ///< Bit #10: (reserved)
    volatile unsigned :1;               ///< Bit #11: (reserved)
    volatile unsigned :1;               ///< Bit #12: (reserved)
    volatile unsigned :1;               ///< Bit #13: (reserved)
    volatile bool Busy :1;              ///< Bit #14: Status bit indicating that the state machine is executing a process (e.g. startup-ramp)
    volatile bool Active :1;            ///< Bit #15: Status bit indicating if switch-node and conttol loops are active

}__attribute__((packed)); ///< Buck Converter Status & Control word

union BUCK_CONVERTER_STATUS_u 
{
    struct BUCK_CONVERTER_STATUS_s bits; ///< Buck Converter Status & Control bit-field
    volatile uint16_t value; ///< Buck Converter Status & Control word for 16-bit word read/write operations
};

typedef union BUCK_CONVERTER_STATUS_u BUCK_CONVERTER_STATUS_t; // Buck Converter Status & Control word data type

```

###### Please note

The data structure provides another control bit filed allowing to select and set features and functions at runtime

<br>&nbsp;

#### Sub Structure  `BUCK_STATE_ID_t`

The buck converter State ID is controlled by the state machine and holds the most recent operating state in a 16-bit wide variable. While the state machine is executing sequences of operating states, the busy flag (bit <14> in the status word) will be set. The second data
field provides the total number of states available helping to track state progress.

```c
struct BUCK_STATE_ID_s 
{
    volatile BUCK_OPSTATES_t  OpStateId;    ///< Most recent operating state of main state machine
    volatile uint16_t   OpStateListSize;    ///< Number of sub-states of main state machine
}; // Buck Converter State Machine operating state ID

typedef struct BUCK_STATE_ID_s BUCK_STATE_ID_t;  // Buck Converter State Machine operating state ID data type
```

<br>&nbsp;

#### Sub Structure  `struct BUCK_CONVERTER_DATA_s`

The basic converter data buffer `BUCK_CONVERTER_DATA_s` data structure is used to publish most recent runtime data across the firmware. During operation, control loops and state machine will continuously update this public data buffer. 

```c
struct BUCK_CONVERTER_DATA_s 
{
    volatile int16_t  Isns[BUCK_MPHASE_COUNT];      ///< BUCK phase current sense value
    volatile int16_t  Iout;                         ///< BUCK common output current (accumulated and averaged phase currents)
    volatile uint16_t Vin;                          ///< BUCK input voltage
    volatile uint16_t Vout;                         ///< BUCK output voltage
    volatile uint16_t Temperature;                  ///< BUCK board temperature
    volatile int16_t  ControlInput;                 ///< BUCK most recent control input value (raw Input)
    volatile int16_t  ControlError;                 ///< BUCK most recent control error value
    volatile int16_t  ControlOutput;                ///< BUCK most recent control output value

}; // BUCK runtime data buffer
typedef struct BUCK_CONVERTER_DATA_s BUCK_CONVERTER_DATA_t; // BUCK runtime data buffer data type
```

###### Please Note

When the buck converter is turned off using the  `BuckConverter_Stop(pcInstance)` function, the data buffer will be cleared and won't get updated anymore. If you need to suspend the power converter operation temporarily but have to keep the data buffer updating to allow system monitoring to continue, use the `BuckConverter_Suspend(pcInstance)` function.

<br>&nbsp;

#### Sub Structures  `BUCK_CONVERTER_STARTUP_t`

The buck converter startup declaration consists of multiple delay and ramp-up period declarations, which are defined in the nested `BUCK_REFERENCE_RAMP_HANDLER_t` data structures. This set of configurations also includes settings for open loop startup supporting the startup with limited feedback signal availability (e.g. caused by limited common mode voltage range of sense circuits).

```c
 struct BUCK_CONVERTER_STARTUP_s 
 {
    volatile BUCK_PERIOD_HANDLER_t PowerOnDelay; ///< Declaration of Power On Delay period and timeout
    volatile BUCK_PERIOD_HANDLER_t PowerGoodDelay; ///< Declaration of Power Good period and timeout
    volatile BUCK_REFERENCE_RAMP_HANDLER_t IRamp;  ///< Declaration of output current ramp reference increment step size and period
    volatile BUCK_REFERENCE_RAMP_HANDLER_t VRamp;  ///< Declaration of output voltage ramp reference increment step size and period
    volatile BUCK_OPEN_LOOP_STARTUP_t OpenLoopStart;  ///< Monitored Open Loop Startup Settings
    
}; // Power converter start-up settings and variables
typedef struct BUCK_CONVERTER_STARTUP_s  BUCK_CONVERTER_STARTUP_t; // Power converter start-up settings and variables data type

```

Each item is defined by a configuration data management data structure of type `BUCK_PERIOD_HANDLER_t`, `BUCK_REFERENCE_RAMP_HANDLER_t` or `BUCK_OPEN_LOOP_STARTUP_t`, each providing a dedicated, individual set of configurations.

##### Nested Sub-Data Structure `BUCK_PERIOD_HANDLER_t`

This data structure provides parameters for pure delay functions without further functional content. Delay period settings incorporate an independent counter variable allowing the state machine to count down the specified period before the state machine is allowed to progress to another state.

```c
struct BUCK_PERIOD_HANDLER_s 
{
    volatile uint16_t Counter;      ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    volatile uint16_t Period;       ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    
}; // Power converter soft-start auxiliary variables
typedef struct BUCK_PERIOD_HANDLER_s BUCK_PERIOD_HANDLER_t; // Power converter soft-start auxiliary variables data types

```

##### Nested Sub-Data Structure `BUCK_REFERENCE_RAMP_HANDLER_t`

For the Voltage and Current Ramp-Up periods the control reference will be incremented by the specified reference step size in each counter step until the nominal reference specified in `BUCK_REFERENCE_RAMP_HANDLER_t` is met. This data type uses the period setting for the declaration of the ramp up period, however, the value used to determine the end of the ramp is based on the sampled voltage or current value. Hence, the real ramp up time depends on the system response rather than the declared time period.

```c
struct BUCK_REFERENCE_RAMP_HANDLER_s 
{
    volatile uint16_t Counter;      ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    volatile uint16_t Period;       ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    volatile uint16_t StepSize;     ///< Size/value of one reference increment/decrement or this period
    volatile uint16_t Start;        ///< Reference value transition start point 
    volatile uint16_t Stop;         ///< Reference value transition end point 
    volatile uint16_t refBuffer;    ///< Internal reference value buffer manipulated by state machine during reference level transition
    
}; // Power converter soft-start auxiliary variables
typedef struct BUCK_REFERENCE_RAMP_HANDLER_s BUCK_REFERENCE_RAMP_HANDLER_t; // Power converter soft-start auxiliary variables data types

```

##### Nested Sub-Data Structure `BUCK_OPEN_LOOP_STARTUP_t`

In some cases power supplies must be started in open loop due to limited available of feedback signals (e.g. due to common mode voltage range limitations, for example, when feedback components are biased from the converted voltage). To overcome this obstacle, the startup engine of the state machine provides the declaration of an open loop startup period, which is defined by the control output value step size and an target cut-off voltage level at which the state machine will close the control loop and switch to a controlled ramp up period.

```c
struct BUCK_OPEN_LOOP_STARTUP_s 
{
    volatile bool     Enabled; ///< User option enabling the open loop startup feature, which will increment the control output until the output voltage is above the specified minimum cut-off voltage
    volatile uint16_t CutOffVoltage; ///< Monitored open loop startup cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    volatile uint16_t StepSize; ///< Value increment per execution step during monitored open loop startup. This value will be determined by the state machine during startup and is 'read only'.
    volatile uint16_t Counter; ///< internal open loop startup process counter
    
}; // Power converter start-up settings and variables
typedef struct BUCK_OPEN_LOOP_STARTUP_s  BUCK_OPEN_LOOP_STARTUP_t; // Power converter open-loop start-up settings and variables data type

```

<br>&nbsp;

#### Sub Structure  `BUCK_CONVERTER_SETTINGS_t`

The basic converter settings, such as control mode, number of converter phases, voltage and current reference, are specified in the `BUCK_CONVERTER_SETTINGS_t` data structure. The current reference will only be valid if the control mode is set to Average Current Mode Control (ACMC) and will be ignored in Voltage Mode Control (VMC) and Peak Current Mode Control (PCMC).

```c
struct BUCK_CONVERTER_SETTINGS_s 
{
    volatile BUCK_CONTROL_MODE_t ControlMode;   ///< Fundamental control mode 
    volatile BUCK_SYNC_RECTIFIER_CONTROL_t SyncRectifierControl; ///< Power converter automatic synchronous rectifier control settings
    volatile uint16_t PhaseCount;               ///< number of converter phases
    volatile uint16_t VReference;               ///< User voltage reference setting used to control the power converter controller
    volatile int16_t  IReference;               ///< User current reference setting used to control the power converter controller
};  ///> Buck converter main settings
typedef struct BUCK_CONVERTER_SETTINGS_s BUCK_CONVERTER_SETTINGS_t;  ///> Buck converter main settings
```

<br>&nbsp;

#### Sub Structure  `BUCK_LOOP_SETTINGS_t`

Buck converter control loop settings are split in properties and methods.

**Properties** specify default values for control reference, feedback offset, ADC trigger offset and anti-windup minimum and maximum values, which are used during initialization. Further a pointer to a control loop object of type `NPNZ16b_t` is declared, which holds all the detailed information of the specific control loop, such as filter coefficients, feedback input and control output scaling, pointers to data provider targets and further parameters supporting advanced control options.

**Methods** declare function pointers to default function calls of a control loop data object of type `NPNZ16b_t`. These include initialization, reset and value pre-charge functions as well as the default update function executing a control loop computation.

```c
struct BUCK_LOOP_SETTINGS_s 
{
    // Control Loop Object
    volatile NPNZ16b_t* Controller;         ///< pointer to control loop object data structure

    // Control Loop Default Settings & Methods
    volatile BUCK_LOOP_PROPERTIES_t Properties;  // Control loop properties store default configuration information
    volatile BUCK_LOOP_METHODS_t Methods; // Control loop methods hold function pointers to control loop default functions

}; ///< User defined settings for control loops; 
typedef struct BUCK_LOOP_SETTINGS_s BUCK_LOOP_SETTINGS_t; ///< User defined settings for control loops data type

```

###### Nested Sub Structure  `BUCK_LOOP_PROPERTIES_t`

```c
struct BUCK_LOOP_PROPERTIES_s
{   
    volatile int16_t FeedbackOffset;        ///< Feedback offset value for calibration or bi-directional feedback signals
    volatile int16_t TriggerOffset;         ///< ADC trigger offset value for ADC trigger placement fine-tuning

    volatile int16_t Reference;             ///< Nominal control loop voltage reference variable
    volatile int16_t Minimum;               ///< output clamping value (minimum)
    volatile int16_t Maximum;               ///< output clamping value (maximum)
    
    volatile int16_t AltReference;          ///< Nominal alternate control loop voltage reference variable in reverse mode
    volatile int16_t AltMinimum;            ///< output clamping value (minimum)
    volatile int16_t AltMaximum;            ///< output clamping value (maximum)

};
typedef struct BUCK_LOOP_PROPERTIES_s BUCK_LOOP_PROPERTIES_t; ///< User defined settings for control loops data type
```

###### Nested Sub Structure  `BUCK_LOOP_METHODS_t`

```c
struct BUCK_LOOP_METHODS_s
{
    // Methods (Function pointers to control loop object functions)
    volatile uint16_t (*ctrl_Initialize)(volatile NPNZ16b_t*); ///< Function pointer to INIT routine
    void (*ctrl_Reset)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine
    void (*ctrl_Update)(volatile NPNZ16b_t*); ///< Function pointer to UPDATE routine
    void (*ctrl_Precharge)(volatile NPNZ16b_t*, volatile fractional, volatile fractional); ///< Function pointer to PRECHARGE routine
    
};
typedef struct BUCK_LOOP_METHODS_s BUCK_LOOP_METHODS_t; ///< User defined settings for control loops data type
```

<br>&nbsp;

##### Sub Structure  `struct BUCK_SWITCH_NODE_SETTINGS_s`

The switch node settings of the buck converter data object specify the user parameters of the PWM configuration supporting a synchronous or asynchronous buck converter switching signal generator. Each converter phase requires its independent switch node definition. The synchronization of multiple PWM generators is automatically configured during the peripheral configuration by the initialization driver `dev_buck_pconfig.c` of the buck converter device driver. This driver will first load the default peripheral configuration from the built-in buck converter peripheral templates overwriting user parameters with the settings found in this data structure.

```c
struct BUCK_SWITCH_NODE_SETTINGS_s 
{
    volatile uint16_t PwmInstance;          ///< Index of the PWM channel instance used
    volatile uint16_t GpioInstance;         ///< GPIO instance of the selected PWM generator
    volatile uint16_t GpioHigh;             ///< GPIO port pin-number of PWMxH of the selected PWM generator
    volatile uint16_t GpioLow;              ///< GPIO port pin-number of PWMxL of the selected PWM generator
    volatile bool     SwapOutputs;          ///< Selecting if PWMxH (default) or PWMxL should be the leading default PWM output
    volatile bool     MasterPeriodEnable;   ///< Selecting MASTER or Individual period register
    volatile bool     HighResolutionEnable; ///< Selecting if PWM module should use high-resolution mode 
    volatile bool     SyncDrive;            ///< Selects if switch node supports synchronous or asynchronous commutation
    volatile uint16_t Period;               ///< Switching period
    volatile uint16_t Phase;                ///< Switching signal phase-shift
    volatile uint16_t DutyRatioMin;         ///< Absolute duty cycle minimum during normal operation
    volatile uint16_t DutyRatioMax;         ///< Absolute duty cycle maximum during normal operation
    volatile uint16_t DeadTimeRising;       ///< Dead time setting at rising edge of a half-bridge drive
    volatile uint16_t DeadTimeFalling;      ///< Dead time setting at falling edge of a half-bridge drive
    volatile uint16_t LebPeriod;            ///< Leading-Edge Blanking period
    volatile uint16_t AdcTriggerScaler;     ///< PWM triggers for ADC will be generated every n-th cycle  (only affects ADC Trigger #1, see register PGxEVTL for details)
    volatile uint16_t AdcTriggerOffset;     ///< PWM triggers for ADC will be offset by n cycles (only affects ADC Trigger #1, see register PGxEVTL for details)
    
    volatile BUCK_SLOPE_COMP_SETTINGS_t SlopeCompensation; ///< Peak current mode slope compensation settings (peak current mode control only)
    
}; ///< Switching signal timing settings
typedef struct BUCK_SWITCH_NODE_SETTINGS_s  BUCK_SWITCH_NODE_SETTINGS_t; ///< Switching signal timing settings data type

```

###### Nested Sub Structure  `BUCK_SLOPE_COMP_SETTINGS_t`

```c
struct BUCK_SLOPE_COMP_SETTINGS_s
{
    volatile uint16_t DacInstance; ///< Digital-to-analog converter instance used to generate the the slope compensation ramp
    volatile uint16_t CompInstance; ///< Analog comparator triggering on the peak current feedback signal
    volatile uint16_t CompInput; ///< Analog comparator input option A to D (0=A, 1=B, 2=C, 3=D)
    volatile uint16_t PwmTriggerInput; ///< PWM current limit PCI input source (must match comparator declared by cmp_instance)
    volatile uint16_t SlewRate; ///< Integer equivalent of compensation ramp slew rate in [V/µs]
    volatile uint16_t SlopeStartTrigger; ///< Period counter value at with the slope start is triggered
    volatile uint16_t SlopeStopTrigger; ///< Period counter value at with the slope stop is triggered
    volatile uint16_t DacValueInit;     ///< Initial Digital-to-Analog converter reference value (used when peripheral is enabled or being reset)
    volatile uint16_t* ptrDutyCycleRegister; ///< Pointer to switch node duty cycle register
    volatile uint16_t DutCycleLimit;    ///< Switch node duty cycle limit value
    
}; ///< Peak Current Mode Control Slope Compensation Settings
typedef struct BUCK_SLOPE_COMP_SETTINGS_s BUCK_SLOPE_COMP_SETTINGS_t; ///< Peak Current Mode Control Slope Compensation Settings data type

```

<br>&nbsp;

##### Sub Structure  `BUCK_FEEDBACK_SETTINGS_t`

Each converter offers the following standard feedback input channels in its `BUCK_FEEDBACK_SETTINGS_t` data structure:

- Output Voltage
- Input Voltage
- Inductor Current (one for each converter phase)
- Temperature

```c
struct BUCK_FEEDBACK_SETTINGS_s 
{
    volatile BUCK_FEEDBACK_CHANNEL_t adVin;                        ///< ADC input sampling input voltage
    volatile BUCK_FEEDBACK_CHANNEL_t adVout;                       ///< ADC input sampling output voltage
    volatile BUCK_FEEDBACK_CHANNEL_t adIsns[BUCK_MPHASE_COUNT];    ///< ADC input sampling phase current
    volatile BUCK_FEEDBACK_CHANNEL_t adTemperature;                ///< ADC input sampling temperature
    
}; ///< Buck converter feedback declarations
typedef struct BUCK_FEEDBACK_SETTINGS_s BUCK_FEEDBACK_SETTINGS_t; ///< Buck converter feedback declarations data type

```

Each feedback channel can be enabled or disabled. Disabled channels will be ignored during initialization and will remain inactive during operation. The peripheral configuration is performed by the initialization driver `dev_buck_pconfig.c` of the buck converter device driver. This driver will first load the default peripheral configuration from the built-in buck converter peripheral templates overwriting user parameters with the settings found in this data structure.

###### Nested Sub Structure  `BUCK_FEEDBACK_CHANNEL_t`

```c
struct BUCK_FEEDBACK_CHANNEL_s 
{
    volatile bool Enabled;                      ///< input channel enable bit
    volatile BUCK_ADC_INPUT_t Input;            ///< ADC input channel specifications
    volatile BUCK_ADC_TRIGGER_t Trigger;        ///< ADC input sample trigger configuration
    volatile BUCK_ADC_INTERRUPT_t Interrupt;    ///< ADC sample interrupt service routine call configuration
    volatile BUCK_ADC_SIGNAL_SCALING_t Scaling; ///< ADC result normalization settings

}; ///< ADC input channel configuration
typedef struct BUCK_FEEDBACK_CHANNEL_s BUCK_FEEDBACK_CHANNEL_t; ///< ADC input channel configuration data type

```

###### Nested Sub Structure  `BUCK_ADC_INPUT_t`

```c
struct BUCK_ADC_INPUT_s 
{
    volatile uint16_t* AdcBuffer;           ///< pointer to ADC result buffer
    volatile uint8_t   AdcInput;            ///< number of the ADC input channel used
    volatile uint8_t   AdcAltInput;         ///< selection of the alternate ADC input channel used (dedicated ADC cores only, set =0 for shared ADC inputs)
    volatile uint8_t   AdcCore;             ///< index of the ADC core the selected analog input channel is assigned to
    volatile bool      DifferentialInput;   ///< input channel differential mode enable bit
    volatile bool      SignedResult;        ///< input channel singed result mode enable bit
    volatile uint16_t  GpioInstance;        ///< GPIO instance of the selected PWM generator

}; ///< ADC input interrupt settings
typedef struct BUCK_ADC_INPUT_s BUCK_ADC_INPUT_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BUCK_ADC_TRIGGER_t`

```c
struct BUCK_ADC_TRIGGER_s 
{
    volatile uint8_t TriggerSource;         ///< input channel trigger source
    volatile bool LevelTrigger;             ///< input channel level trigger mode enable bit

}; ///< ADC input interrupt settings
typedef struct BUCK_ADC_TRIGGER_s BUCK_ADC_TRIGGER_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BUCK_ADC_INTERRUPT_t`

```c
struct BUCK_ADC_ISR_SETTING_s 
{
    volatile bool InterruptEnable;      ///< input channel interrupt enable bit (allows tripping an Interrupt Service Routine)
    volatile bool EarlyInterruptEnable; ///< input channel early interrupt enable bit (triggers the interrupt early before conversion is complete)

}; ///< ADC input interrupt settings
typedef struct BUCK_ADC_ISR_SETTING_s BUCK_ADC_INTERRUPT_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BUCK_ADC_SIGNAL_SCALING_t`

Supporting calculations of power, voltage ratios, gains or Root Mean Square (RMS) values feedback channels may have to be scaled/normalized. For this purpose the `BUCK_ADC_INPUT_SCALING_s` data structure was added allowing users to specify the feedback value scaling information in form of a fast floating point slope and integer offset value.

```c
struct BUCK_ADC_SIGNAL_SCALING_s 
{
    volatile int16_t Factor; ///< Fractional scaling factor (range -1 ... 0.99969)
    volatile int16_t Scaler; ///< Feedback bit-shift scaler used for number normalization
    volatile int16_t Offset; ///< Signal offset as signed integer to be subtracted from ADC input

}; ///< ADC input signal scaling = (ADCBUF - <offset>) * <factor> >> 2^<scaler>
typedef struct BUCK_ADC_SIGNAL_SCALING_s BUCK_ADC_SIGNAL_SCALING_t; ///< ADC input signal scaling data type

```

<br>&nbsp;

##### Sub Structure  `BUCK_GPIO_SETTINGS_t`

The buck converter device driver supports one Enable input (EN) and one Power Good output (PG) as two basic general purpose input/outputs (GPIO) for low-level hardware sequencing of converter stages.

```c
struct BUCK_GPIO_SETTINGS_s 
{
    volatile BUCK_GPIO_INSTANCE_t EnableInput; ///< External ENABLE input
    volatile BUCK_GPIO_INSTANCE_t PowerGood; ///< Power Good Output

}; ///< GPIO instance of the converter control GPIO
typedef struct BUCK_GPIO_SETTINGS_s BUCK_GPIO_SETTINGS_t; ///< GPIO instance of the converter control GPIO data type

```

###### Nested Sub Structure  `BUCK_GPIO_INSTANCE_t`

Each GPIO can be configured by the settings specified in the `BUCK_GPIO_INSTANCE_t` data structure. The Input/Output type can be set to

- Input (=1, max. input voltage = 3.3 V)
- Push-Pull Output (=0, max. sink-/source-current = 4 mA, max. output voltage = 3.3 V)
- Open-Drain Output (=0, max. sink-current = 4 mA, max. output voltage = 3.3/5.0 V)

```c
struct BUCK_GPIO_INSTANCE_s 
{
    volatile bool     Enabled;  ///< Specifies, if this IO is used or not
    volatile uint16_t Port;     ///< GPIO port instance number (0=Port RA, 0=Port RB, 0=Port RC, etc.)
    volatile uint16_t Pin;      ///< GPIO port pin number
    volatile uint16_t Polarity; ///< Output polarity, where 0=ACTIVE HIGH, 1=ACTIVE_LOW
    volatile uint16_t IoType;   ///< Input/Output definition (0=push-pull output, 1=input, 2=open-drain output)

}; ///< GPIO instance of the converter control GPIO
typedef struct BUCK_GPIO_INSTANCE_s BUCK_GPIO_INSTANCE_t; ///< GPIO instance of the converter control GPIO data type

```

<a href="#startDoc">back</a>

<span id="state-machine"> </span>
### Buck Converter State Machine

The Buck Converter State Machine is broken down into two major segments, Default Operating States (opstate) and Sub-States (substate)

#### Default Operating States

- **State Initialize**
  Initializes the pre-configured chip peripherals enabling the PWM and ADC to start the continuous acquisition of data allowing the analysis of the system status.

- **State Reset**
  Disables the PWM outputs while keeping the PWM itself running as trigger generator of the ADC module, and clears dynamic control parameters such as error and control histories to ensure a 
  deterministic state at startup.

- **State Standby**
  Waits for all status bits to be set, fault and suspend bits to be cleared and the GO bit to be set.

- **Startup State Sequence (see topology-specific startup process below)**
  The Start-Up state sequence splits up into converter-specific startup states, managing the specific startup procedure of a synchronous or asynchronous, single- or multiphase buck converter topology. This topology-specific startup procedure also includes the startup timing specified by the `BUCK_CONVERTER_STARTUP_s` data structure parameters.

- **State Online**
  Once the start procedure has been completed successfully, the converter state machine drops into constant regulation mode, where it will remain until the converter is shut-down or suspended. During constant regulation the control reference is monitored. If the value of the control reference is changed, the state machine will tune into the new reference by generating a defined defined transition ramp using the ramp slope specified in the startup voltage ramp timing.

- **State Error**
  Should any of the states ever return an error, the converter will be shut down and the state machine will fall back into an error state, from which is will recover automatically with the fault_active flag (bit <5> of the status word) being set.

#### Topology-Specific Startup States

The buck converter state machine also manages the power supply topology-specific startup procedure of a buck converter, tailored to its configuration, control and operating mode. It also incorporates the user definition of the startup timing specified in the `BUCK_CONVERTER_STARTUP_t` data structure.

- **State PowerOnDelay**
  Waiting period while the converter is kept in a disabled state until the Power-On Delay (POD) specified in the `BUCK_CONVERTER_STARTUP_t` data structure has expired.

- **State PrepareVRampUp**
  In this state the most recent voltage ratio between the inductor terminals is measured and the expected duty ratio at the startup frequency is calculated to ensure a smooth startup without accidental reverse current or forward current spikes at startup against a pre-biased output. The estimated startup duty ratio value is then programmed into PWM registers and the control loop histories get pre-charged and the startup control reference is set to match the output voltage.

- **State VRampUp**
  In this state the PWM outputs and control loops are enabled and the converter starts to operate. Due to the ramp-up pre-charge of the previous state, the expected output power is close to zero. With every execution cycle of the state machine the control reference is linearly incremented by the given amount of ticks until the ramp-up reference value matches desired value specified by the user by `v_ref` in the `BUCK_CONVERTER_SETTINGS_s` data structure.

- **State IRampUp**
  This state is only executed in Average Current Mode Control (ACMC) and only if the voltage loop hist an output saturation limit during the execution of the previous voltage ramp-up state. This additional output current ramp-up procedure has been introduced as controlled constant-current ramp-up control when ramping up the output against large capacitive loads or when this buck converter device driver is used in a constant current source application.

- **State PowerGoodDelay**
  Waiting period while the converter is kept in a enabled state at constant output regulation until the Power Good Delay (PGD) specified in the `BUCK_CONVERTER_STARTUP_s` data structure has expired.

<a href="#startDoc">back</a>

---

<span id="history"> </span>

##### History

- 03/13/2020 v1.0 (M91406) Initial release
- 11/02/2020 v2.0 (M91406) Modular state machine with startup sub-states
- 12/21/2020 v2.1 (M91406) Added current sense calibration function extensions to state machine
- 02/03/2021 v2.2 (M91406) Modified voltage reference management during startup, ensuring user setting is always loaded correctly before voltage ramp-up
- 02/22/2021 v2.3 (M91406) Control loop anti-windup limiter minimum and maxium are both set as unsigned integer allowing individual singed/unsigned operations in assembler control routine
- 03/01/2021 v2.4 (M91406) Added support for alternate ADC input of dedicated ADC cores of dsPIC33CK/CH
- 05/22/2022 v3.0 (M91406) Buck converter data structure data fields upgrade switching to name labels to combined capital notions, added auto synchronous rectifier control, optimized shut-down performance and restructured state function distribution across files

---
© 2022, Microchip Technology Inc.
