/*
 * @file    app_power_config.c
 * @author  M91406
 * @brief   Power control configuration source file
 * @date    Created on March 12, 2020, 12:18 PM
 * @details
 *  This file covers all configuration steps of the power supply 
 *  control instance, applying all user specific operation parameters
 *  to the generic power supply type device driver library.
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "config/hal.h" // include hardware abstraction layer declarations
#include "app_power_control.h" // include application layer power supply task header file
#include "control/IControlLoop.h" // include power supply control output multiplexing header file

/**************************************************************************************************
 * @ingroup app-layer-pcontrol-properties-public-variables
 * @brief Global data object for a BDBB CONVERTER 
 * 
 * @details
 *  The 'bdbb' data object holds all status, control and monitoring values of the 
 *  BDBB power controller. The IBdbbConverter_t data structure is defined in 
 *  IBdbbConverter_t. Please refer to the comments on top of this file for 
 *  further information.
 * 
 **************************************************************************************************/
IBdbbConverter_t psu;
#if (DBGDAC_ENABLE)
uint16_t debugOutput;
#endif

/* PRIVATE FUNCTION PROTOTYPES */
static int appPowerSupply_ConverterObjectInitialize(void);
static int appPowerSupply_SwitchNodeInitialize(void);
static int appPowerSupply_GpioInitialize(void);
static int appPowerSupply_FeedbackInitialize(void);
static int appPowerSupply_ControllerInitialize(void);
static int appPowerSupply_StartupInitialize(void);
static int appPowerSupply_PeripheralsInitialize(void);
static int appPowerSupply_UserPeripheralInitialize(void);
static int appPowerSupply_InterruptsInitialize(void);

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   Calls the application layer power controller initialization
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details 
 *  The power control application layer is the proprietary user code layer
 *  used to tailor the generic power converter device driver to the specific 
 *  hardware of this design. The initialization routine covers three levels
 * 
 *  - Converter Object Configuration
 *  - Controller Configuration
 *  - Peripheral Set Configuration
 *  - Interrupt Vector Configuration
 * 
 *  Once all modules have been configured successfully, the power converter 
 *  object is started with control loops and PWM outputs disabled. However, 
 *  the PWM module will start triggering the ADC to allow the standby monitoring
 *  of system conditions to allow the firmware to decide if it is safe to start
 *  up the power converter.
 * 
 *********************************************************************************/
int appPowerObjects_Initialize(void)
{ 
    int retval = 1;

    // Run initialization sequence
    retval &= appPowerSupply_ConverterObjectInitialize();
    retval &= appPowerSupply_SwitchNodeInitialize();
    retval &= appPowerSupply_GpioInitialize();
    retval &= appPowerSupply_FeedbackInitialize();
    retval &= appPowerSupply_ControllerInitialize();
    retval &= appPowerSupply_StartupInitialize();
    retval &= appPowerSupply_PeripheralsInitialize();
    retval &= appPowerSupply_UserPeripheralInitialize();
    retval &= appPowerSupply_InterruptsInitialize();

    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the bdbb converter device driver instance
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initialize the bdbb converter object status, reset the bdbb state 
 *  machine, set reference values, clear the runtime data, initialize the switch node,
 *  and setup the feedback channels and start-up settings.
 *********************************************************************************/
static int appPowerSupply_ConverterObjectInitialize(void)
{
    int retval = 1;
    
    // Set BDBB Converter Default Properties
    psu.Properties.CsCalibEnable    = ISNS_OFFSET_CALIBRATION_ENABLE; // Disable current sense feedback calibration
    psu.Properties.AsyncMode        = SYNCHRONOUS_RECTIFIER_CONTROL; // Start up BDBB converter in synchronous/asynchronous mode (requires SWNODE_SYNCHRONOUS = true to take effect)
    psu.Properties.AutoAsyncMode    = AUTO_SYNCH_RECTIFIER_CONTROL; // Enable/disable automatic synchronous/asynchronous mode control (requires SWNODE_SYNCHRONOUS = true to take effect)
    psu.Properties.Autorun          = AUTO_START; // Allow the BDBB converter to start automatically when all start conditions are met (e.g. faults are cleared)
    psu.Properties.Enable           = false; // Disable BDBB converter (will be enabled after configuration is complete)
    psu.Properties.ConversionMode   = DEFAULT_OPERATING_MODE; // Determines the default startup operating mode (buck or boost)
    psu.Properties.HotSwapEnable    = DEFAULT_SWAP_MODE; // Set Swap Mode to HOT (on-the-fly) or SOFT (through psu reset)
    
    // Set Reference values
    psu.Properties.PhaseCount       = NO_OF_PHASES; // Set number of power train phases
    psu.Properties.IReference       = ISNS_MAX; // Set current loop reference to nominal reference value
    psu.Properties.LVReference      = V12_NOM; // Set nominal voltage as default loop reference
    psu.Properties.HVReference      = V48_NOM; // Set nominal voltage as default loop reference
    
    // Set Initial State Machine State
    psu.StateId.OpStateId = BDBB_OPSTATE_INITIALIZE; // Reset Boost State Machine
    
    // Set main control mode
    psu.Control.ControlMode = BDBB_CONTROL_MODE_ACMC; // Set Control Mode
    
    return(retval);
    
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the switch node of the buck converter device driver instance 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the switch node of the buck converter object by 
 *  assigning it to a specific PWM generator instance and setting the associated 
 *  GPIO pins. Further, users need to specify basic parameters like commutation 
 *  mode (synchronous or asynchronous drive), switching frequency, phase shift,
 *  duty ratio clamping values, dead times, leading edge blanking and ADC 
 *  trigger offsets to compensate propagation delays of FET drivers and feedback
 *  bandwidth limitations.
 *********************************************************************************/
static int appPowerSupply_SwitchNodeInitialize(void)
{
    int retval = 1;
    
    // ~~~ SWITCH NODE CONFIGURATION #1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 0)
    // Initialize Switch Node
    psu.SwitchNode[PHASE_1].PwmInstance = PWM1H.Objects.PG;
    psu.SwitchNode[PHASE_1].GpioHigh = PWM1H;
    psu.SwitchNode[PHASE_1].GpioLow = PWM1L;
    psu.SwitchNode[PHASE_1].SwapOutputs = PWM1_OUTPUT_SWAP; // Swap PWM outputs in Boost Converter mode
    psu.SwitchNode[PHASE_1].SyncDrive = true; 
    psu.SwitchNode[PHASE_1].Period = PWM_PERIOD;
    psu.SwitchNode[PHASE_1].PhaseShift = 0;
    psu.SwitchNode[PHASE_1].DutyRatioMinimum = PWM_DC_MIN;
    psu.SwitchNode[PHASE_1].DutyRatioMaximum = PWM_DC_MAX;
    psu.SwitchNode[PHASE_1].DeadTimeRising = PWM_DEAD_TIME_LE;
    psu.SwitchNode[PHASE_1].DeadTimeFalling = PWM_DEAD_TIME_FE;
    psu.SwitchNode[PHASE_1].ControlTriggerOffset = SAMPLING_OFFSET;
    psu.SwitchNode[PHASE_1].ControlTriggerDivider = SAMPLING_DIVIDER;
    psu.SwitchNode[PHASE_1].HighResolutionEnable = PWM_CLOCK_HIGH_RESOLUTION;
    #endif
    // ~~~ SWITCH NODE CONFIGURATION #1 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // ~~~ SWITCH NODE CONFIGURATION #2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 1)
    // Initialize Switch Node
    psu.SwitchNode[PHASE_2].PwmInstance = PWM2PPS;  // PWM4 is routed to these output through PPS
    psu.SwitchNode[PHASE_2].GpioHigh = PWM2H;
    psu.SwitchNode[PHASE_2].GpioLow = PWM2L;
    psu.SwitchNode[PHASE_2].SwapOutputs = PWM2_OUTPUT_SWAP;
    psu.SwitchNode[PHASE_2].SyncDrive = true; 
    psu.SwitchNode[PHASE_2].Period = PWM_PERIOD;
    psu.SwitchNode[PHASE_2].PhaseShift = PWM_PHASE_SHIFT;
    psu.SwitchNode[PHASE_2].DutyRatioMinimum = PWM_DC_MIN;
    psu.SwitchNode[PHASE_2].DutyRatioMaximum = PWM_DC_MAX;
    psu.SwitchNode[PHASE_2].DeadTimeRising = PWM_DEAD_TIME_LE;
    psu.SwitchNode[PHASE_2].DeadTimeFalling = PWM_DEAD_TIME_FE;
    psu.SwitchNode[PHASE_2].ControlTriggerOffset = SAMPLING_OFFSET;
    psu.SwitchNode[PHASE_2].ControlTriggerDivider = SAMPLING_DIVIDER;
    psu.SwitchNode[PHASE_2].HighResolutionEnable = PWM_CLOCK_HIGH_RESOLUTION;
    #endif
    // ~~~ SWITCH NODE CONFIGURATION #2 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

    // ~~~ SWITCH NODE CONFIGURATION #3 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 2)
    // Initialize Switch Node
    psu.SwitchNode[PHASE_3].PwmInstance = PWM3H.Objects.PG;
    psu.SwitchNode[PHASE_3].GpioHigh = PWM3H;
    psu.SwitchNode[PHASE_3].GpioLow = PWM3L;
    psu.SwitchNode[PHASE_3].SwapOutputs = PWM3_OUTPUT_SWAP;
    psu.SwitchNode[PHASE_3].SyncDrive = true; 
    psu.SwitchNode[PHASE_3].Period = PWM_PERIOD;
    psu.SwitchNode[PHASE_3].PhaseShift = PWM_PHASE_SHIFT;
    psu.SwitchNode[PHASE_3].DutyRatioMinimum = PWM_DC_MIN;
    psu.SwitchNode[PHASE_3].DutyRatioMaximum = PWM_DC_MAX;
    psu.SwitchNode[PHASE_3].DeadTimeRising = PWM_DEAD_TIME_LE;
    psu.SwitchNode[PHASE_3].DeadTimeFalling = PWM_DEAD_TIME_FE;
    psu.SwitchNode[PHASE_3].ControlTriggerOffset = SAMPLING_OFFSET;
    psu.SwitchNode[PHASE_3].ControlTriggerDivider = SAMPLING_DIVIDER;
    psu.SwitchNode[PHASE_3].HighResolutionEnable = PWM_CLOCK_HIGH_RESOLUTION;
    #endif
    // ~~~ SWITCH NODE CONFIGURATION #3 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
    
    // ~~~ SWITCH NODE CONFIGURATION #4 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 3)
    // Initialize Switch Node
    psu.SwitchNode[PHASE_4].PwmInstance = PWM4H.Objects.PG;
    psu.SwitchNode[PHASE_4].GpioHigh = PWM4H;
    psu.SwitchNode[PHASE_4].GpioLow = PWM4L;
    psu.SwitchNode[PHASE_4].SwapOutputs = PWM4_OUTPUT_SWAP;
    psu.SwitchNode[PHASE_4].SyncDrive = true; 
    psu.SwitchNode[PHASE_4].Period = PWM_PERIOD;
    psu.SwitchNode[PHASE_4].PhaseShift = PWM_PHASE_SHIFT;
    psu.SwitchNode[PHASE_4].DutyRatioMinimum = PWM_DC_MIN;
    psu.SwitchNode[PHASE_4].DutyRatioMaximum = PWM_DC_MAX;
    psu.SwitchNode[PHASE_4].DeadTimeRising = PWM_DEAD_TIME_LE;
    psu.SwitchNode[PHASE_4].DeadTimeFalling = PWM_DEAD_TIME_FE;
    psu.SwitchNode[PHASE_4].ControlTriggerOffset = SAMPLING_OFFSET;
    psu.SwitchNode[PHASE_4].ControlTriggerDivider = SAMPLING_DIVIDER;
    psu.SwitchNode[PHASE_4].HighResolutionEnable = PWM_CLOCK_HIGH_RESOLUTION;
    #endif
    // ~~~ SWITCH NODE CONFIGURATION #4 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
    
    
    return(retval);

}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the IO functions of the buck converter device driver instance 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the optional Input/Output functions of the buck
 *  converter device driver instance used to provide external sequencing control,
 *  such as ENABLE Input and Power Good Output. These optional features can be 
 *  individually enabled and mapped to any available device GPIO.
 *********************************************************************************/
static int appPowerSupply_GpioInitialize(void)
{
    int retval = 1;

    // Initialize additional GPIOs 
    
    // ~~~ EXTERNAL ENABLE INPUT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    psu.Gpio.EnableInput.Enabled = PWREN_ENABLE; // this converter doesn't support external enable control
    #if (true == PWREN_ENABLE)
    psu.Gpio.EnableInput.Port = PWREN_PORT; // Number of the GPIO port (0=A, 1=B, 2=C, etc.)
    psu.Gpio.EnableInput.Pin = PWREN_PIN; // Number of the GPIO port pin
    psu.Gpio.EnableInput.Polarity = 0;   // This pin is ACTIVE HIGH (only required if io_type = OUTPUT)
    psu.Gpio.EnableInput.IoType = 0;    // This pin is configured as OUTPUT
    #endif
    // ~~~ EXTERNAL ENABLE INPUT END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // ~~~ POWER GOOD OUTPUT ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    psu.Gpio.PowerGood.Enabled = PWRGOOD_ENABLE; // This converter supports an additional POWER GOOD output
    #if (true == PWRGOOD_ENABLE)
    psu.Gpio.PowerGood.Port = PWRGOOD_PORT; // Number of the GPIO port (0=A, 1=B, 2=C, etc.)
    psu.Gpio.PowerGood.Pin = PWRGOOD_PIN; // Number of the GPIO port pin
    psu.Gpio.PowerGood.Polarity = 0;   // This pin is ACTIVE HIGH (only required if io_type = OUTPUT)
    psu.Gpio.PowerGood.IoType = 0;    // This pin is configured as OUTPUT
    #endif
    
    // ~~~ POWER GOOD OUTPUT END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    return(retval);
    
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the feedback channels of the buck converter device driver instance 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the supported feedback channels of the buck
 *  converter device driver instance used for system monitoring and control.
 *  A buck converter instance supports the following standard feedback 
 *  channels:
 * 
 * - Input Voltage:    (used for protection and advanced linearization control)
 * - Output Voltage:   (used for output voltage control)
 * - Inductor Current
 * - Temperature
 * 
 *********************************************************************************/
static int appPowerSupply_FeedbackInitialize(void)
{
    int retval = 1;

    // Initialize Feedback Channels
    
    // ~~~ LOW VOLTAGE PORT FEEDBACK ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    psu.Feedback.AdVLowVoltage.Enabled = true;   // Use this channel

    psu.Feedback.AdVLowVoltage.Input.AdcInput = V12.Properties.ADCAN;
    psu.Feedback.AdVLowVoltage.Input.AdcCore = V12_ADCCORE;
    psu.Feedback.AdVLowVoltage.Input.DifferentialInput = V12_ADCDIFF;
    psu.Feedback.AdVLowVoltage.Input.SignedResult = V12_ADCSIGN;

    psu.Feedback.AdVLowVoltage.Trigger.ptrTriggerRegister = (uint16_t*)&V12_ADCTRIG;
    psu.Feedback.AdVLowVoltage.Trigger.TriggerSource = V12_TRGSRC;
    psu.Feedback.AdVLowVoltage.Trigger.TriggerDelay = (psu.SwitchNode[0].Period >> 1) + V12_ADC_TRGDLY;
    psu.Feedback.AdVLowVoltage.Trigger.LevelTrigger = V12_LVLTRG;

    psu.Feedback.AdVLowVoltage.Interrupt.InterruptEnable = true;
    psu.Feedback.AdVLowVoltage.Interrupt.EarlyInterruptEnable = true;
    
    psu.Feedback.AdVLowVoltage.Scaling.Factor = V12_NORM_FACTOR;
    psu.Feedback.AdVLowVoltage.Scaling.Scaler = V12_NORM_SCALER;
    psu.Feedback.AdVLowVoltage.Scaling.Offset = V12_OFFSET;
    psu.Feedback.AdVLowVoltage.Scaling.Polarity = V12_POLARITY;

    psu.Feedback.AdVLowVoltage.Range.Minimum = V12_MIN;
    psu.Feedback.AdVLowVoltage.Range.Nominal = V12_NOM;
    psu.Feedback.AdVLowVoltage.Range.Maximum = V12_MAX;
    
    // ~~~ LOW VOLTAGE PORT FEEDBACK END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // ~~~ HIGH VOLTAGE PORT FEEDBACK ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    psu.Feedback.AdVHighVoltage.Enabled = true;   // Use this channel
    
    psu.Feedback.AdVHighVoltage.Input.AdcInput = V48.Properties.ADCAN;
    psu.Feedback.AdVHighVoltage.Input.AdcCore = V48_ADCCORE;
    psu.Feedback.AdVHighVoltage.Input.DifferentialInput = V48_ADCDIFF;
    psu.Feedback.AdVHighVoltage.Input.SignedResult = V48_ADCSIGN;

    psu.Feedback.AdVHighVoltage.Trigger.ptrTriggerRegister = (uint16_t*)&V48_ADCTRIG;
    psu.Feedback.AdVHighVoltage.Trigger.TriggerSource = V48_TRGSRC;
    psu.Feedback.AdVHighVoltage.Trigger.TriggerDelay = V48_ADC_TRGDLY;
    psu.Feedback.AdVHighVoltage.Trigger.LevelTrigger = V48_LVLTRG;

    psu.Feedback.AdVHighVoltage.Interrupt.InterruptEnable = true;
    psu.Feedback.AdVHighVoltage.Interrupt.EarlyInterruptEnable = true;
    
    psu.Feedback.AdVHighVoltage.Scaling.Factor = V48_NORM_FACTOR;
    psu.Feedback.AdVHighVoltage.Scaling.Scaler = V48_NORM_SCALER;
    psu.Feedback.AdVHighVoltage.Scaling.Offset = V48_OFFSET;
    psu.Feedback.AdVHighVoltage.Scaling.Polarity = V48_POLARITY;

    psu.Feedback.AdVHighVoltage.Range.Minimum = V48_MIN;
    psu.Feedback.AdVHighVoltage.Range.Nominal = V48_NOM;
    psu.Feedback.AdVHighVoltage.Range.Maximum = V48_MAX;
    
    // ~~~ HIGH VOLTAGE PORT FEEDBACK END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~ PHASE CURRENT FEEDBACK #1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 0)
    psu.Feedback.AdISense[PHASE_1].Enabled = true;   // Use this channel

    psu.Feedback.AdISense[PHASE_1].Input.AdcInput = ISNS1.Properties.ADCAN;
    psu.Feedback.AdISense[PHASE_1].Input.AdcCore = ISNS1_ADCCORE;
    psu.Feedback.AdISense[PHASE_1].Input.DifferentialInput = ISNS1_ADCDIFF;
    psu.Feedback.AdISense[PHASE_1].Input.SignedResult = ISNS1_ADCSIGN;

    psu.Feedback.AdISense[PHASE_1].Trigger.ptrTriggerRegister = (uint16_t*)&ISNS1_ADCTRIG;
    psu.Feedback.AdISense[PHASE_1].Trigger.TriggerSource = ISNS1_TRGSRC;
    psu.Feedback.AdISense[PHASE_1].Trigger.TriggerDelay = ISNS1_ADC_TRGDLY;
    psu.Feedback.AdISense[PHASE_1].Trigger.LevelTrigger = ISNS1_LVLTRG;

    psu.Feedback.AdISense[PHASE_1].Interrupt.InterruptEnable = false;
    psu.Feedback.AdISense[PHASE_1].Interrupt.EarlyInterruptEnable = false;

    psu.Feedback.AdISense[PHASE_1].Scaling.Factor = ISNS_NORM_FACTOR;
    psu.Feedback.AdISense[PHASE_1].Scaling.Scaler = ISNS_NORM_SCALER;
    psu.Feedback.AdISense[PHASE_1].Scaling.Offset = ISNS1_OFFSET;
    psu.Feedback.AdISense[PHASE_1].Scaling.Polarity = ISNS1_POLARITY;

    psu.Feedback.AdISense[PHASE_1].Range.Minimum = ISNS_MIN;
    psu.Feedback.AdISense[PHASE_1].Range.Nominal = ISNS_NOM;
    psu.Feedback.AdISense[PHASE_1].Range.Maximum = ISNS_MAX;
    #endif
    // ~~~ PHASE CURRENT FEEDBACK #1 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    
    
    // ~~~ PHASE CURRENT FEEDBACK #2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 1)
    psu.Feedback.AdISense[PHASE_2].Enabled = true;   // Use this channel

    psu.Feedback.AdISense[PHASE_2].Input.AdcInput = ISNS2.Properties.ADCAN;
    psu.Feedback.AdISense[PHASE_2].Input.AdcCore = ISNS2_ADCCORE;
    psu.Feedback.AdISense[PHASE_2].Input.DifferentialInput = ISNS2_ADCDIFF;
    psu.Feedback.AdISense[PHASE_2].Input.SignedResult = ISNS2_ADCSIGN;

    psu.Feedback.AdISense[PHASE_2].Trigger.ptrTriggerRegister = (uint16_t*)&ISNS2_ADCTRIG;
    psu.Feedback.AdISense[PHASE_2].Trigger.TriggerSource = ISNS2_TRGSRC;
    psu.Feedback.AdISense[PHASE_2].Trigger.TriggerDelay = ISNS2_ADC_TRGDLY;
    psu.Feedback.AdISense[PHASE_2].Trigger.LevelTrigger = ISNS2_LVLTRG;

    psu.Feedback.AdISense[PHASE_2].Interrupt.InterruptEnable = false;
    psu.Feedback.AdISense[PHASE_2].Interrupt.EarlyInterruptEnable = false;

    psu.Feedback.AdISense[PHASE_2].Scaling.Factor = ISNS_NORM_FACTOR;
    psu.Feedback.AdISense[PHASE_2].Scaling.Scaler = ISNS_NORM_SCALER;
    psu.Feedback.AdISense[PHASE_2].Scaling.Offset = ISNS2_OFFSET;
    psu.Feedback.AdISense[PHASE_2].Scaling.Polarity = ISNS2_POLARITY;
    
    psu.Feedback.AdISense[PHASE_2].Range.Minimum = ISNS_MIN;
    psu.Feedback.AdISense[PHASE_2].Range.Nominal = ISNS_NOM;
    psu.Feedback.AdISense[PHASE_2].Range.Maximum = ISNS_MAX;
    #endif
    // ~~~ PHASE CURRENT FEEDBACK #2 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~ PHASE CURRENT FEEDBACK #3 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 2)
    psu.Feedback.AdISense[PHASE_3].Enabled = true;   // Use this channel

    psu.Feedback.AdISense[PHASE_3].Input.AdcInput = ISNS3.Properties.ADCAN;
    psu.Feedback.AdISense[PHASE_3].Input.AdcCore = ISNS3_ADCCORE;
    psu.Feedback.AdISense[PHASE_3].Input.DifferentialInput = ISNS3_ADCDIFF;
    psu.Feedback.AdISense[PHASE_3].Input.SignedResult = ISNS3_ADCSIGN;

    psu.Feedback.AdISense[PHASE_3].Trigger.ptrTriggerRegister = (uint16_t*)&ISNS3_ADCTRIG;
    psu.Feedback.AdISense[PHASE_3].Trigger.TriggerSource = ISNS3_TRGSRC;
    psu.Feedback.AdISense[PHASE_3].Trigger.TriggerDelay = ISNS3_ADC_TRGDLY;
    psu.Feedback.AdISense[PHASE_3].Trigger.LevelTrigger = ISNS3_LVLTRG;

    psu.Feedback.AdISense[PHASE_3].Interrupt.InterruptEnable = false;
    psu.Feedback.AdISense[PHASE_3].Interrupt.EarlyInterruptEnable = false;

    psu.Feedback.AdISense[PHASE_3].Scaling.Factor = ISNS_NORM_FACTOR;
    psu.Feedback.AdISense[PHASE_3].Scaling.Scaler = ISNS_NORM_SCALER;
    psu.Feedback.AdISense[PHASE_3].Scaling.Offset = ISNS3_OFFSET;
    psu.Feedback.AdISense[PHASE_3].Scaling.Polarity = ISNS3_POLARITY;

    psu.Feedback.AdISense[PHASE_3].Range.Minimum = ISNS_MIN;
    psu.Feedback.AdISense[PHASE_3].Range.Nominal = ISNS_NOM;
    psu.Feedback.AdISense[PHASE_3].Range.Maximum = ISNS_MAX;
    #endif
    // ~~~ PHASE CURRENT FEEDBACK #3 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // ~~~ PHASE CURRENT FEEDBACK #4 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #if (NO_OF_PHASES > 3)
    psu.Feedback.AdISense[PHASE_4].Enabled = true;   // Use this channel

    psu.Feedback.AdISense[PHASE_4].Input.AdcInput = ISNS4.Properties.ADCAN;
    psu.Feedback.AdISense[PHASE_4].Input.AdcCore = ISNS4_ADCCORE;
    psu.Feedback.AdISense[PHASE_4].Input.DifferentialInput = ISNS4_ADCDIFF;
    psu.Feedback.AdISense[PHASE_4].Input.SignedResult = ISNS4_ADCSIGN;

    psu.Feedback.AdISense[PHASE_4].Trigger.ptrTriggerRegister = (uint16_t*)&ISNS4_ADCTRIG;
    psu.Feedback.AdISense[PHASE_4].Trigger.TriggerSource = ISNS4_TRGSRC;
    psu.Feedback.AdISense[PHASE_4].Trigger.TriggerDelay = ISNS4_ADC_TRGDLY;
    psu.Feedback.AdISense[PHASE_4].Trigger.LevelTrigger = ISNS4_LVLTRG;

    psu.Feedback.AdISense[PHASE_4].Interrupt.InterruptEnable = false;
    psu.Feedback.AdISense[PHASE_4].Interrupt.EarlyInterruptEnable = false;

    psu.Feedback.AdISense[PHASE_4].Scaling.Factor = ISNS_NORM_FACTOR;
    psu.Feedback.AdISense[PHASE_4].Scaling.Scaler = ISNS_NORM_SCALER;
    psu.Feedback.AdISense[PHASE_4].Scaling.Offset = ISNS4_OFFSET;
    psu.Feedback.AdISense[PHASE_4].Scaling.Polarity = ISNS4_POLARITY;

    psu.Feedback.AdISense[PHASE_4].Range.Minimum = ISNS_MIN;
    psu.Feedback.AdISense[PHASE_4].Range.Nominal = ISNS_NOM;
    psu.Feedback.AdISense[PHASE_4].Range.Maximum = ISNS_MAX;
    #endif
    // ~~~ PHASE CURRENT FEEDBACK #4 END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    
    // ~~~ TEMPERATURE FEEDBACK ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    psu.Feedback.AdTemperature.Enabled = true;   // Use this channel

    psu.Feedback.AdTemperature.Input.AdcInput = TEMP1.Properties.ADCAN;
    psu.Feedback.AdTemperature.Input.AdcCore = TEMP1_ADCCORE;
    psu.Feedback.AdTemperature.Input.DifferentialInput = TEMP1_ADCDIFF;
    psu.Feedback.AdTemperature.Input.SignedResult = TEMP1_ADCSIGN;
    
    psu.Feedback.AdTemperature.Trigger.ptrTriggerRegister = (uint16_t*)&TEMP1_ADCTRIG;
    psu.Feedback.AdTemperature.Trigger.TriggerSource = TEMP1_TRGSRC;
    psu.Feedback.AdTemperature.Trigger.TriggerDelay = TEMP_ADC_TRGDLY;
    psu.Feedback.AdTemperature.Trigger.LevelTrigger = TEMP1_LVLTRG;

    psu.Feedback.AdTemperature.Interrupt.InterruptEnable = false;
    psu.Feedback.AdTemperature.Interrupt.EarlyInterruptEnable = false;

    psu.Feedback.AdTemperature.Scaling.Factor = TEMP_NORM_FACTOR;
    psu.Feedback.AdTemperature.Scaling.Scaler = TEMP_NORM_SCALER;
    psu.Feedback.AdTemperature.Scaling.Offset = TEMP_OFFSET;
    psu.Feedback.AdTemperature.Scaling.Polarity = TEMP1_POLARITY;
    
    // ~~~ TEMPERATURE FEEDBACK END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    return(retval);
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the control system feedback loop objects
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function allows the user to set up and initialize the loop configuration.
 *  This includes the following setup.
 *     - Initialize Default Loop Configuration
 *     - Set Controller Object of Voltage Loop
 *     - Configure Voltage Loop Controller Object
 *     - Configure controller input/output ports
 *     - Data Input/Output Limit Configuration
 *     - ADC Trigger Control Configuration
 *     - Data Provider Configuration
 *     - Cascaded Function Configuration of multi-loop feedback loops
 *     - Initialize Advanced Control Settings 
 *     - Custom Advanced Control Settings
 *********************************************************************************/
static int appPowerSupply_ControllerInitialize(void)
{
    int retval = 1;

    // Assign feedback loop controller objects in state machine
    // control interface
    
    // Set Controller Object of Voltage Loop
    psu.Control.VLoop.Controller = &vComp;
    psu.Control.VLoop.Methods.Initialize = &vComp_Initialize;
    psu.Control.VLoop.Methods.Update = &vComp_Update;
    psu.Control.VLoop.Methods.Reset = &vComp_Reset;
    psu.Control.VLoop.Methods.ResetControlHistory = &vComp_ResetControlHistory;
    psu.Control.VLoop.Methods.ResetErrorHistory = &vComp_ResetErrorHistory;
    psu.Control.VLoop.Methods.Precharge = &vComp_Precharge;
    psu.Control.VLoop.Methods.PrechargeControlHistory = &vComp_PrechargeControlHistory;
    psu.Control.VLoop.Methods.PrechargeErrorHistory = &vComp_PrechargeErrorHistory;
    psu.Control.VLoop.Methods.PTermUpdate = &vComp_PTermUpdate;
    psu.Control.VLoop.Methods.OutputMultiplexing = NULL;
    
    #ifdef CONTROL_LOOP_ICOMP
    
    if (psu.Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        // Initialize all current loop controllers with the same configuration
        for (int i=0; i<psu.Properties.PhaseCount; i++)
        {
            // Set Controller Object of Current Loop
            psu.Control.ILoop[i].Controller = &iComp[i];
            
            // Assign feedback loop functions to power supply data object method function pointers
            psu.Control.ILoop[i].Methods.Initialize = &iComp_ArrayInitialize;
            psu.Control.ILoop[i].Methods.Update = &iComp_Update;
            psu.Control.ILoop[i].Methods.Reset = &iComp_Reset;
            psu.Control.ILoop[i].Methods.ResetControlHistory = &iComp_ResetControlHistory;
            psu.Control.ILoop[i].Methods.ResetErrorHistory = &iComp_ResetErrorHistory;
            psu.Control.ILoop[i].Methods.Precharge = &iComp_Precharge;
            psu.Control.ILoop[i].Methods.PrechargeControlHistory = &iComp_PrechargeControlHistory;
            psu.Control.ILoop[i].Methods.PrechargeErrorHistory = &iComp_PrechargeErrorHistory;
            psu.Control.ILoop[i].Methods.PTermUpdate = &vComp_PTermUpdate;
            psu.Control.ILoop[i].Methods.OutputMultiplexing = NULL;
            psu.Control.ILoop[i].Methods.CurrentBalancing = NULL;
        }
    }
    
    // Set phase multiplexing with and configure current balancing
    psu.Control.ILoop[0].Methods.OutputMultiplexing = &iComp_CtrlMux;
    psu.Control.ILoop[0].Methods.CurrentBalancing = &iComp_IBalance;
    
    #endif /* CONTROL_LOOP_ICOMP */
    
    Nop();
    Nop();
    Nop();

    // Initialize state machine control interface with 
    // static user settings
    retval &= BdbbLoopConfig_ControllerInitialize(&psu);
    
    // Initialize current balancing limits
    #ifdef CONTROL_LOOP_ICOMP

    if (psu.Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        // Set Controller Object of Current Loop
        for (int i=1; i<psu.Properties.PhaseCount; i++)
        {
            psu.Control.ILoop[i].Controller->Limits.MinTargetOffset = (-ISNS_BLNC_LIMIT);
            psu.Control.ILoop[i].Controller->Limits.MaxTargetOffset = ISNS_BLNC_LIMIT;
        }
    }
    
    #endif /* CONTROL_LOOP_ICOMP */
    
    Nop();
    Nop();
    Nop();

    return(retval);
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function initializes the startup configuration of the buck converter device driver instance 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the startup configuration of the buck converter 
 *  device driver instance. A buck converter instance supports the following 
 *  startup options:
 * 
 * - Power-On Delay:   
 *   User-specified delay before the output voltage is ramping up
 * 
 * - VRamp-Up Period:  
 *   User specified output voltage ramp-up period in which the output voltage 
 *   is ramped up linearly from 0 to nominal output voltage
 * 
 * - IRamp-Up Period:  
 *   User specified output current ramp-up period; This function is only 
 *   available in average current mode control and only if a current limit 
 *   is hit during a voltage ramp-up and the output voltage cannot reach the 
 *   nominal level. under these conditions the state machine will switch over
 *   to state IRamp-Up and linearly increase the current limit up to the 
 *   allowed absolute maximum current. 
 *   This mode has been introduced especially for battery chargers or when
 *   converters are powering up against large capacitive loads.
 * 
 * - Power Good Delay: 
 *   User-specified period during which the output must stand stable before 
 *   the output is considered to be reliable
 * 
 * optional:
 * - Open Loop Startup: 
 *   Specific startup mode in average current mode control, where the PWM 
 *   output is increased manually in open loop while the output is monitored.
 *   This mode was introduced to bridge limited common mode voltage ranges
 *   of certain current sense circuits, which require a certain, minimum 
 *   voltage to be applied across the sense points before they start to 
 *   produce reliable feedback signals.
 * 
 *********************************************************************************/
static int appPowerSupply_StartupInitialize(void)
{
    int retval = 1;

    // Initialize Startup Settings
    psu.Startup.PowerOnDelay.Period = START_POD;

    psu.Startup.VCapMonitor.DelayPeriod = START_CUD;
    psu.Startup.VCapMonitor.TimeoutPeriod = START_CUD_TO;
    psu.Startup.VCapMonitor.Threshold = V48_VFWD_DROP;

    psu.Startup.OpenLoopStart.CutOffVoltage = ISNS_CMOD_LIMIT;
    psu.Startup.OpenLoopStart.CutOffHysteresis = ISNS_CMOD_HYST;
    psu.Startup.OpenLoopStart.Enabled = OPEN_LOOP_STARTUP;

    psu.Startup.LVRamp.Period = START_LVRAMP_PER;
    psu.Startup.LVRamp.StepSize = START_LVREF_STEP;
    psu.Startup.LVRamp.Start = V12_MIN;
    psu.Startup.LVRamp.Stop = V12_NOM;

    psu.Startup.HVRamp.Period = START_HVRAMP_PER;
    psu.Startup.HVRamp.StepSize = START_HVREF_STEP;
    psu.Startup.HVRamp.Start = V12_MIN;
    psu.Startup.HVRamp.Stop = V12_NOM;
    
    if (psu.Control.ControlMode == BDBB_CONTROL_MODE_ACMC) 
    {
        psu.Startup.IRamp.Period = START_IRAMP_PER;
        psu.Startup.IRamp.StepSize = START_IREF_STEP;
        psu.Startup.IRamp.Start  = ISNS_START;
        psu.Startup.IRamp.Stop   = ISNS_MAX;
    }
    
    psu.Startup.PowerGoodDelay.Period = START_PGD;
    
    return(retval);
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function is used to load peripheral configuration templates from the power controller device driver
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function hand over the peripheral configuration to the bdbb converter driver
 *********************************************************************************/
static int appPowerSupply_PeripheralsInitialize(void)
{
    int retval = 1;
    
    retval &= BdbbConverter_Initialize(&psu);
    
    return(retval);
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function is used to load user-specific peripheral configurations not included by the power controller device driver
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
static int appPowerSupply_UserPeripheralInitialize(void)
{
    int retval = 1;
    
    // Mapping of PWM generator #4 to output pins of PWM6
    #ifdef PWM2H
    p33c_Pps_UnlockIO();
    p33c_Pps_MapOutput(PWM2H.Properties.RPid, PWM2H_PPSO_ID);
    p33c_Pps_MapOutput(PWM2L.Properties.RPid, PWM2L_PPSO_ID);
    p33c_Pps_LockIO();
    #endif
    
    return(retval);
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   This function is used to initialize the interrupt service events of the control loop
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function initializes and enables the control loop interrupt(s) of the
 *  cycle-by-cycle real-time control. These interrupts are commonly synchronous
 *  to the switching process and therefore triggered by ADC conversion or PWM 
 *  event interrupts. The interrupt service routines are called on high levels 
 *  overriding the recently executed processes to maintain real-time accuracy
 *  of the feedback loop response.
 * 
 *********************************************************************************/
static int appPowerSupply_InterruptsInitialize(void)
{
    int retval = 1;

    #if (VLOOP_TRIGGER_MODE != TRIGGER_MODE_NONE)
    // Initialize Control Interrupt
    VLOOP_ISR_IP = VLOOP_ISR_PRIORITY;
    VLOOP_ISR_IF = 0;
    VLOOP_ISR_IE = true;
    
    retval &= (VLOOP_ISR_IE); // Return INTERRUPT_ENABLE flag bit
    #endif

    #if (ILOOP_TRIGGER_MODE != TRIGGER_MODE_NONE)
    // Initialize Control Interrupt
    ILOOP1_ISR_IP = ISNS1_ISR_PRIORITY;
    ILOOP1_ISR_IF = 0;
    ILOOP1_ISR_IE = true;

    retval &= (ILOOP1_ISR_IE); // Return INTERRUPT_ENABLE flag bit
    #endif
    
    return(retval);
}

// ________________________
// end of file
