/*
 * @file    app_power_control.c
 * @author  M91406
 * @brief   Power control main application task source file
 * @date    Created on March 12, 2020, 11:55 AM
 */

#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

#include "config/hal.h" // include hardware abstraction layer header file
#include "svc/IBdbbConverter.h" // include bidirectional buck-boost converter interface header file
#include "app_power_control.h" // include power control header file

// ------------------------------------------------------------------------------------------------
/* PRIVATE DATA OBJECT AND VARIABLES DECLARATIONS */
uint32_t buckMonitorState;

// ------------------------------------------------------------------------------------------------
/* PRIVATE FUNCTION PROTOTYPES */
extern int appPowerObjects_Initialize(void);

// ------------------------------------------------------------------------------------------------
/* PRIVATE INTERNAL FUNCTION PROTOTYPES */
extern int appPowerSupply_InitializeAverageFilters(void);
extern int appPowerSupply_UpdateDataFilter(void);
extern int appPowerSupply_UpdateFaultObjects(void);
// ------------------------------------------------------------------------------------------------

/* *************************************************************************************************
 * PRIVATE VARIABLE DECLARATIONS
 * ************************************************************************************************/
// (none)

/* *************************************************************************************************
 * PUBLIC FUNCTIONS
 * ************************************************************************************************/
// (none)

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   This is the top-level function call triggering the most recent state machine of all associated power supply controllers
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details 
 *  After initialization, the proprietary user code has to call this function 
 *  on a deterministic, constant time base. In each execution step this function
 *  will call the power control state machines of each supported/included power
 *  supply unit. 
 * 
 *********************************************************************************/
int appPowerSupply_Execute(void)
{ 
    int retval = 1;

    // Execute BDBB converter state machine
    retval &= BdbbConverter_Execute(&psu);

    // Update Fault Monitoring Object Settings
    retval &= appPowerSupply_UpdateFaultObjects();

    // Update private data averaging filters
    retval &= appPowerSupply_UpdateDataFilter();
    
    // Track most recent state machine state
    buckMonitorState = (uint32_t)(0x0001 << psu.StateId.OpStateId);
    
    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   Calls the application layer power controller initialization
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details 
 *  The power control application layer is the proprietary user code layer
 *  used to tailor the generic power converter device driver to the specific 
 *  hardware of this design. The initialization routine covers three levels
 * 
 *    - Converter Object Configuration
 *    - Controller Configuration
 *    - Peripheral Set Configuration
 *    - Interrupt Vector Configuration
 * 
 *  Once all modules have been configured successfully, the power converter 
 *  object is started with control loops and PWM outputs disabled. However, 
 *  the PWM module will start triggering the ADC to allow the standby monitoring
 *  of system conditions to allow the firmware to decide if it is safe to start
 *  up the power converter.
 * 
 *********************************************************************************/
int appPowerSupply_Initialize(void)
{ 
    int retval = 1;

    // Run initialization sequence of power supply data objects
    retval &= appPowerObjects_Initialize();
    
    // Initialize data monitoring settings
    retval &= appPowerSupply_InitializeAverageFilters();
    
    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   This function calls the bdbb converter device driver function starting the power supply 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details 
 *  This function exposes the Power Converter Start function of the device driver.
 *********************************************************************************/
int appPowerSupply_Start(void)
{
    int retval = 1;

    retval &= BdbbConverter_Start(&psu); // Start PWM with outputs disabled to start ADC triggering

    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   This function calls the bdbb converter device driver function stopping the power supply 
 * @return  unsinged integer (0=failure, 1=success)
 *  
 * @details
 *  This function exposes the Power Converter Stop function of the device driver.
 * 
 * @note
 *  The STOP function terminates the state machine and all peripherals used by
 *  the power controller. This includes the PWM and ADC peripheral modules and 
 *  will therefore also stop all data acquisition. 
 *  If you are trying to stop the power supply but keep its state machine and
 *  data acquisition running, use the SUSPEND function instead
 * 
 *********************************************************************************/
int appPowerSupply_Stop(void)
{
    int retval = 1;

    retval &= BdbbConverter_Stop(&psu); // Shut down all power supply peripherals and data objects

    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   This function stops the power supply operation
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The SUSPEND function stops the power supply operation but keep its state machine
 *  and data acquisition running.
 *********************************************************************************/
int appPowerSupply_Suspend(void)
{ 
    int retval = 1;

    retval &= BdbbConverter_Suspend(&psu); // Shut down PWM immediately
    
    return(retval); 
}

/*******************************************************************************
 * @ingroup app-layer-pcontrol-functions-public
 * @brief   This function resumes the power supply operation
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function calls the bdbb converter device driver function recovering
 *  the power supply operation from a previously initiated shut-down.
 * 
 *********************************************************************************/
int appPowerSupply_Resume(void)
{ 
    int retval=0;

    retval &= BdbbConverter_Resume(&psu); // Shut down PWM immediately
    
    return(retval); 
}

// ____________________
// end of file
