/*
 * File:   app_power_control_isr.c
 * Author: M91406
 *
 * Created on May 26, 2020, 7:32 PM
 */


#include "config/hal.h"
#include "app_power_control.h"

#define CAPTURE_CURRENT_BALANCING   false

#if (DBGDAC_ENABLE)
  extern uint16_t debugOutput;
#endif

#if CAPTURE_CURRENT_BALANCING
  #if __DEBUG
    #define DEBUG_CAPTURE_BUFFER_SIZE 2048
    int16_t debugCaptureBuffer[DEBUG_CAPTURE_BUFFER_SIZE];
    uint16_t ptrDbgCapBuf;
  #endif
#endif
    
volatile bool pTermControlEnable = PLANT_MEASUREMENT;

/*********************************************************************************
 * @ingroup app-layer-pcontrol-functions-interrupts
 * @brief   Main Control Interrupt
 *   
 * @details
 *  The control interrupt is calling the control loop and serves the ADC Activity
 *  semaphore of the power supply state machine.
 * 
 *  The interrupt trigger source is selected by the xxx_TRIGGER_MODE option 
 *  and fine tuned by macro xxx_ADC_TRG_DELAY of the respective control
 *  target.
 * 
 *  Available trigger options are 
 * 
 *   - PWM Interrupt of ADC Trigger:
 *     Interrupt is invoked synchronously to triggering the ADC, allowing 
 *     to process data while the ADC is still converting the most recent 
 *     signal. (recommended for short response time)
 * 
 *   - ADC Conversion Complete:
 *     Interrupt is invoked by the ADC module, allowing 
 *     to process data while the ADC is still converting the most recent 
 *     signal. 
 * 
 *********************************************************************************/ 
void __attribute__((__interrupt__, auto_psv, context))VLOOP_Interrupt(void)
{
    #if (DBGPIN2_ENABLE)
    DBGPIN2.Methods.Set();  // Set control loop execution time indicator output
    #endif
    
    // Set flag bit indication of feedback loop interrupt activity
    psu.Status.bits.LoopCtrlActive = true;
    
    // Call feedback loop
    if (pTermControlEnable == false)
    {
        psu.Control.VLoop.Methods.Update(psu.Control.VLoop.Controller);
    }
    else {
        psu.Control.VLoop.Methods.PTermUpdate(psu.Control.VLoop.Controller);
        PG1STATbits.UPDREQ = 1;
    }

    #if (DBGPIN2_ENABLE)
    DBGPIN2.Methods.Clear();  // Clear control loop execution time indicator output
    #endif

    #if (CAPTURE_CURRENT_BALANCING == true)
    #if __DEBUG
    uint16_t _mm = ((ptrDbgCapBuf++) & 0x07FF);
    debugCaptureBuffer[_mm] = psu.Control.ILoop[1].Controller->Ports.Target.Offset;
    #endif
    #endif
    
    // Copy most recent control output to DAC output for debugging
    #if (DBGDAC_ENABLE)
    debugOutput = 
        (debugOutput < DAC_OUTPUT_MAX_VALUE ? 
            debugOutput : DAC_OUTPUT_MAX_VALUE);
    
    P33C_DAC_INSTANCE_t* dac;
    dac = p33c_DacInstance_GetHandle(DAC_OUTPUT.DACx);
    dac->DACxDATH.value = debugOutput;

    #endif
    
    // Clear the interrupt flag bit allowing the next interrupt to trip
    VLOOP_ISR_IF = 0;

}

/*********************************************************************************
 * @ingroup app-layer-pcontrol-functions-interrupts
 * @brief   Main Control Interrupt
 * @param   void
 * @return  void
 *   
 * @details
 * The control interrupt is calling the current control loop. The point in time where
 * this interrupt is thrown is determined by selecting the ISNS_TRIGGER_MODE
 * option and fine tuned by macro ISNS_ADC_TRGDLY.
 * 
 * @note
 * This interrupt service routine is not used in this code example
 * 
 *********************************************************************************/

#if (ILOOP_TRIGGER_MODE != TRIGGER_MODE_NONE)
uint16_t dummy = 0;

void __attribute__((__interrupt__, auto_psv, context))ILOOP1_Interrupt(void)
{

    #if (DBGPIN2_ENABLE)
    DBGPIN2_Set();  // Set control loop execution time indicator output
    #endif
    
    dummy = ISNS1_ADCBUF;
    
    #if (DBGPIN2_ENABLE)
    DBGPIN2_Clear();  // Clear control loop execution time indicator output
    #endif
    
    // Clear the interrupt flag bit allowing the next interrupt to trip
    ILOOP1_ISR_IF = 0;

}
#endif


// end of file
