/*
 * @file    app_power_extensions.c
 * @author  M91406
 * @brief   Provides additional, proprietary functions called by 
 *          power supply main execute function.
 */

#include "app_power_control.h"
#include "common/p33c_flib/moving_average.h"
#include "apps/fault_monitor/app_fault_monitor.h"

/* *************************************************************************************************
 * PRIVATE VARIABLE DECLARATIONS
 * ************************************************************************************************/
#define  isnsAverageArraySize   32U
static int16_t isnsArray[NO_OF_PHASES][isnsAverageArraySize];

/* *************************************************************************************************
 * PUBLIC VARIABLE DECLARATIONS
 * ************************************************************************************************/
static AVERAGE_DATA_SOURCE_t isnsData[NO_OF_PHASES];

/*********************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   Initializes the data averaging module averaging phase currents
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  The data capture module is used to capture feedback signals from the board,
 *  which are not directly part of the dedicated power supply control scheme. 
 *  For data shown on displays or sent via communication ports are averaged 
 *  to provide stable, observable data points. Each of these data channels
 *  are filtered by a moving average filter, which are initialized in this 
 *  function.
 *
 **********************************************************************************/
int appPowerSupply_InitializeAverageFilters(void)
{
    int retval = 1;
    
    for (int _i=0; _i<psu.Properties.PhaseCount; _i++)
    {
        isnsData[_i].ptrSource = (int16_t*)&psu.Data.ISense[_i];
        isnsData[_i].ptrArray = &isnsArray[_i][0];
        isnsData[_i].ArraySize = isnsAverageArraySize;
        isnsData[_i].minInput = ISNS_MIN;
        isnsData[_i].maxInput = ISNS_MAX;
        isnsData[_i].Average = 0;

        ClearAverage(&isnsData[_i]);
    }
    
    return(retval);
    
}

/*********************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   Averages runtime data captured by the state machine object
 * @return  unsigned integer (0=failure, 1=success)
 * @details
 *  Function appPowerSupply_DataCapture is used to average runtime data from
 *  the power supply control object, required for system analysis functions 
 *  such as fault management, system monitoring and/or communication. 
 * 
 **********************************************************************************/
int appPowerSupply_UpdateDataFilter(void) 
{
    int retval = 1;

    // Skip data capture if power supply is disabled
    if (psu.Properties.Enable == false)
        return(retval);
    
    // Update declared averaging filters
    for (int _i=0; _i<psu.Properties.PhaseCount; _i++)
    { UpdateAverage(&isnsData[_i]); }
    
    return(retval); 
}

/*********************************************************************************
 * @ingroup app-layer-pcontrol-functions-private
 * @brief   Modifies power supply related fault monitoring objects during state machine execution
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
int appPowerSupply_UpdateFaultObjects(void)
{
    int retval = 1;

    // Regulation error detection is only active while controller is running
    // and being tied to a valid reference
    if((psu.StateId.OpStateId  >= BDBB_OPSTATE_V_CL_RAMP_UP) ||
       (psu.StateId.OpStateId  == BDBB_OPSTATE_ONLINE))
    {
        if(false == psu.Properties.ConversionMode) // Downstream (Buck) mode 
        {
        	fltobj_BuckRegErr.ReferenceObject.ptrObject = 
            (int16_t*)psu.Control.VLoop.Controller->Ports.ControlReference.handle;
        
            #if ((false == PLANT_MEASUREMENT) && (CURRENT_LIMIT_MODE == OUTPUT_OCP_MODE))
            fltobj_BuckRegErr.Status.bits.Enable = psu.Control.VLoop.Controller->Status.bits.Enable;
            #else
            fltobj_BuckRegErr.Status.bits.Enable = false;
            #endif
		}
        else  // Upstream (Boost) mode 
        {
            fltobj_BoostRegErr.ReferenceObject.ptrObject = 
            (int16_t*)psu.Control.VLoop.Controller->Ports.AltControlReference.handle;
         
            #if ((false == PLANT_MEASUREMENT) && (CURRENT_LIMIT_MODE == OUTPUT_OCP_MODE))
            fltobj_BoostRegErr.Status.bits.Enable = psu.Control.VLoop.Controller->Status.bits.Enable;
            #else
            fltobj_BoostRegErr.Status.bits.Enable = false;
            #endif
        }
    }
    else 
    {
        if(false == psu.Properties.ConversionMode) // Downstream (Buck) mode 
        {
            fltobj_BuckRegErr.Status.bits.Enable = false;
            fltobj_BuckOCP.Status.bits.Enable = false;
        }
        else // Upstream (Boost) mode 
        {
            fltobj_BoostRegErr.Status.bits.Enable = false;
            fltobj_BoostOCP.Status.bits.Enable = false;    
        }
    }
    
    return(retval);
}

// ____________________
// end of file

