/* *********************************************************************************
 * PowerSmart Digital Control Library Designer, Version 2.3.0.1007
 * *********************************************************************************
 * 2p2z controller function declarations and compensation filter coefficients
 * derived for following operating conditions:
 * *********************************************************************************
 *
 *  Controller Type:    2P2Z - Basic Current Mode Compensator
 *  Sampling Frequency: 250000 Hz
 *  Fixed Point Format: Q15
 *  Scaling Mode:       4 - Fast Floating Point Coefficient Scaling
 *  Input Gain:         0.01
 *
 * *********************************************************************************
 * CGS Version:         4.0.7
 * CGS Date:            11/08/23
 * *********************************************************************************
 * User:                M91406
 * Date/Time:           01/12/2024 15:21:07
 * ********************************************************************************/

#include "config/hal.h"
#include "iComp.h"                                  // include individual controller header file

/* *********************************************************************************
 * Data Arrays:
 * ============
 *
 * This source file declares the default parameters of the z-domain compensation
 * filter. The NPNZ16b_t data structure contains two pointers to A- and B-
 * coefficient arrays and two pointers to control and error history arrays.
 *
 * For optimized data processing during DSP computations, these arrays must be
 * located in specific memory locations (X-space for coefficient arrays and
 * Y-space for control and error history arrays).
 *
 * The following declarations are used to define the array data contents, their
 * length and memory location. These declarations are made publicly accessible
 * through extern declarations in header file iComp.h
 * ********************************************************************************/

#ifdef P33C_EDS                                     // Devices with extended data space require alternate array declaration attributes
volatile ICOMP_CONTROL_LOOP_COEFFICIENTS_t __attribute__((space(xmemory), eds, aligned)) iComp_coefficients; // A/B-Coefficients
volatile ICOMP_CONTROL_LOOP_HISTORIES_t __attribute__((space(ymemory), eds, aligned)) iComp_histories; // Control/Error Histories
#else                                               // Devices without extended data space
volatile ICOMP_CONTROL_LOOP_COEFFICIENTS_t __attribute__((space(xmemory), near, aligned)) iComp_coefficients; // A/B-Coefficients
volatile ICOMP_CONTROL_LOOP_HISTORIES_t __attribute__((space(ymemory), far, aligned)) iComp_histories; // Control/Error Histories
#endif

volatile uint16_t iComp_ACoefficients_size = (sizeof(iComp_coefficients.ACoefficients)/sizeof(iComp_coefficients.ACoefficients[0])); // A-coefficient array size
volatile uint16_t iComp_BCoefficients_size = (sizeof(iComp_coefficients.BCoefficients)/sizeof(iComp_coefficients.BCoefficients[0])); // B-coefficient array size
volatile uint16_t iComp_ControlHistory_size = (sizeof(iComp_histories.ControlHistory)/sizeof(iComp_histories.ControlHistory[0])); // Control history array size
volatile uint16_t iComp_ErrorHistory_size = (sizeof(iComp_histories.ErrorHistory)/sizeof(iComp_histories.ErrorHistory[0])); // Error history array size

/* *********************************************************************************
 * Pole&Zero Placement:
 * *********************************************************************************
 *
 *    fP0:    0.5 Hz
 *    fP1:    66400 Hz
 *    fZ1:    1720 Hz
 *
 * *********************************************************************************
 * Filter Coefficients and Parameters:
 * ********************************************************************************/

volatile int32_t iComp_ACoefficients [2] =
{
    0x45C7FFFF, // Coefficient A1 will be multiplied with controller output u(n-1)
    0xA3910003  // Coefficient A2 will be multiplied with controller output u(n-2)
};

volatile int32_t iComp_BCoefficients [3] =
{
    0x6EB00006, // Coefficient B0 will be multiplied with error input e(n-0)
    0x4AF0000A, // Coefficient B1 will be multiplied with error input e(n-1)
    0x96000006  // Coefficient B2 will be multiplied with error input e(n-2)
};

// Coefficient normalization factors
volatile int16_t iComp_pre_shift = 3;               // Bit-shift value used to perform input value normalization
volatile int16_t iComp_post_shift_A = 0;            // Bit-shift value A used to perform control output value backward normalization
volatile int16_t iComp_post_shift_B = 0;            // Bit-shift value B used to perform control output value backward normalization
volatile fractional iComp_post_scaler = 0x0000;     // Q15 fractional factor used to perform control output value backward normalization

// P-Term Coefficient for Plant Measurements
volatile int16_t iComp_PTermFactor = 0x63AB;        // Q15 fractional of the P-Term factor
volatile int16_t iComp_PTermScaler = 0xFFFA;        // Bit-shift scaler of the P-Term factor


// User-defined NPNZ16b_t controller data object
volatile NPNZ16b_t iComp[NO_OF_PHASES]; // user-controller data object
volatile uint16_t iComp_size = sizeof(iComp)/(sizeof(iComp[0])); // number of controller data objects in user-controller array

/* ********************************************************************************/

/* *********************************************************************************
 * Controller Initialization:
 * ==========================
  *
 * Public controller initialization function loading known default settings
 * into the NPNZ16b data structure.
 *
 * ********************************************************************************/

volatile uint16_t iComp_Initialize(volatile NPNZ16b_t* controller)
{
    // Initialize controller data structure at runtime with pre-defined default values
    controller->Status.bits.Enable = false;         // turn off feedback loop
    controller->Status.value  &= ~NPNZ_STATUS_SATURATION_MSK; // clear status flag bits

    controller->Filter.ptrACoefficients = &iComp_coefficients.ACoefficients[0]; // initialize pointer to A-coefficients array
    controller->Filter.ptrBCoefficients = &iComp_coefficients.BCoefficients[0]; // initialize pointer to B-coefficients array
    controller->Filter.ptrControlHistory = &iComp_histories.ControlHistory[0]; // initialize pointer to control history array
    controller->Filter.ptrErrorHistory = &iComp_histories.ErrorHistory[0]; // initialize pointer to error history array
    controller->Filter.normPostShiftA = iComp_post_shift_A; // initialize A-coefficients/single bit-shift scaler
    controller->Filter.normPostShiftB = iComp_post_shift_B; // initialize B-coefficients/dual/post scale factor bit-shift scaler
    controller->Filter.normPostScaler = iComp_post_scaler; // initialize control output value normalization scaling factor
    controller->Filter.normPreShift = iComp_pre_shift; // initialize A-coefficients/single bit-shift scaler
    
    controller->Filter.ACoefficientsArraySize = iComp_ACoefficients_size; // initialize A-coefficients array size
    controller->Filter.BCoefficientsArraySize = iComp_BCoefficients_size; // initialize B-coefficients array size
    controller->Filter.ControlHistoryArraySize = iComp_ControlHistory_size; // initialize control history array size
    controller->Filter.ErrorHistoryArraySize = iComp_ErrorHistory_size; // initialize error history array size

    // Load default set of A-coefficients from user RAM into controller A-array located in X-Space
    for (int i=0; i<controller->Filter.ACoefficientsArraySize; i++)
    {
        iComp_coefficients.ACoefficients[i] = iComp_ACoefficients[i]; // Load coefficient A(n) value into iComp coefficient(i) data space
    }

    // Load default set of B-coefficients from user RAM into controller B-array located in X-Space
    for (int i=0; i<controller->Filter.BCoefficientsArraySize; i++)
    {
        iComp_coefficients.BCoefficients[i] = iComp_BCoefficients[i]; // Load coefficient B(n) value into iComp coefficient(i) data space
    }
    
    // Capture EDS page where data arrays are located
    #ifdef P33C_EDS
    controller->Filter.EdsDataPage = __builtin_edspage(&iComp_histories.ControlHistory);
    controller->Filter.EdsDataPage = __builtin_edspage(&iComp_histories.ControlHistory);
    #endif
    
    // Clear error and control histories of the 2P2Z controller
    iComp_Reset(controller);
    
    // Load P-Term factor and scaler into data structure
    controller->Filter.PTermFactor = iComp_PTermFactor;
    controller->Filter.PTermScaler = iComp_PTermScaler;
    
    return(1);
}


//**********************************************************************************
// Download latest version of this tool here: https://www.microchip.com/powersmart
//**********************************************************************************

