/* *********************************************************************************
 * PowerSmart(TM) Digital Control Library Designer, Version 0.9.14.683
 * *********************************************************************************
 * 2p2z controller function declarations and compensation filter coefficients
 * derived for following operating conditions:
 * *********************************************************************************
 *
 *  Controller Type:    2P2Z - Basic Current Mode Compensator
 *  Sampling Frequency: 250000 Hz
 *  Fixed Point Format: Q15
 *  Scaling Mode:       4 - Fast Floating Point Coefficient Scaling
 *  Input Gain:         0.01
 *
 * *********************************************************************************
 * CGS Version:         4.0.3
 * CGS Date:            07/27/22
 * *********************************************************************************
 * User:                M91406
 * Date/Time:           03/02/2023 15:13:50
 * ********************************************************************************/

// This is a guard condition so that contents of this file are not included
// more than once.
#ifndef __SPECIAL_FUNCTION_LAYER_ICOMP_MIRROR_H__
#define __SPECIAL_FUNCTION_LAYER_ICOMP_MIRROR_H__

//#include <xc.h>                                   // include processor files - each processor file is guarded
#include <dsp.h>                                  // include DSP data types (e.g. fractional)
#include <stdint.h>                               // include standard integer number data types
#include <stdbool.h>                              // include standard boolean data types (true/false)

#include "npnz16b.h"                              // include NPNZ library header file
#include "iComp.h"                                // include current control loop object

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-objects
 * @brief   External reference to user-defined NPNZ16b controller data object 'iComp'
 ********************************************************************************/

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler feedback control loop mirroring routine 
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  Output mirroring of control loops is used in multiphase converters using the
 *  phase multiplexing scheme. In this scheme only one phase is properly 
 *  controlled by a dedicated control algorithm (primary phase). The most recent 
 *  control output of this algorithm handed over to this mirroring function, where
 *  the value gets distributed across all synchronized, secondary phases with 
 *  every control cycle.
 * 
 *  Each secondary phase target incorporates a singed offset value allowing the 
 *  compensation of deviations in component tolerances and/or layout achieving 
 *  well balanced phase currents across the converter. These offsets are determined
 *  and set by function '_iComp_Balance', which is executed at lower priority
 *  and execution rate.
 * 
 * @note
 *  Available control options depend on the controller feature configuration.
 *  Please refer to the user guide for more detailed information.
 ********************************************************************************/
extern void __attribute__((near)) iComp_CtrlMux( ///< Calls the control loop output multiplexing routine
        volatile NPNZ16b_t* controller          ///< Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler phase current balancing routine 
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  Phase current balancing in phase-multiplexed multiphase converters is
 *  performed by applying offsets to the most recent control output of the 
 *  only, fully controlled leading primary phase before the individual 
 *  control output values are written to the individual phase control 
 *  registers. These offsets are determined by comparing the individual 
 *  phase currents with the primary phase current. Higher phase currents 
 *  are compensated by decrementing the individual control output, lower
 *  phase currents are compensated by incrementing it. 
 * 
 *  To prevent internal oscillations while providing fast response, the current 
 *  balancing is based on incremental iteration using the most recent result
 *  of the moving average of each individual phase current.
 * 
 * @note
 *  Available control options depend on the controller feature configuration.
 *  Please refer to the user guide for more detailed information.
 ********************************************************************************/
extern void __attribute__((near)) iComp_IBalance( ///< Calls the multiphase current balancing routine
        volatile NPNZ16b_t* controller          ///< Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Initializes the control output multiplexing data interface
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This routine uses the lists multiplexed current loop controllers in
 *  the main current loop controller in the correct way that the control
 *  output multiplexer routine is distributing the data in the correct 
 *  order.
 * 
 **********************************************************************************/
static inline uint16_t iComp_CtrlMux_Initialize()
{
    int retval = 1;

    iComp[0].Advanced.usrParam0 = (uint16_t)(iComp_size-1); // Store number of multiplexed phases

    volatile uint16_t* _ptr = &iComp[0].Advanced.usrParam1;
    
    for (int i=1; i<iComp_size; i++)
    {
        *_ptr = (uint16_t)&iComp[i]; // Pointer to second current loop data object
        _ptr ++;
    }
    
    // Set hook to call phase multiplexer from main current loop controller
    iComp[0].ExtensionHooks.ptrExtHookPreAdcUpdateFunction = (uint16_t)&iComp_CtrlMux;

    return(retval);
}

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Initializes a collection of current loop controllers
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This routine uses the compensation filter settings of the main current loop 
 *  controller object to configure each of the secondary current loop controllers
 *  ensuring all of them are using the exact same filter configuration.
 * 
 **********************************************************************************/
static volatile inline uint16_t iComp_ArrayInitialize(volatile NPNZ16b_t* controller)
{
    int retval = 1;

    for (int i=0; i<iComp_size; i++)
    { retval &= iComp_Initialize(&iComp[i]); }
    retval &= iComp_CtrlMux_Initialize();
    
    return(retval);
}

#endif  // end of __SPECIAL_FUNCTION_LAYER_ICOMP_MIRROR_H__


//**********************************************************************************
// Download latest version of this tool here: https://www.microchip.com/powersmart/
//**********************************************************************************

