/* *********************************************************************************
 * PowerSmart Digital Control Library Designer, Version 2.3.0.1007
 * *********************************************************************************
 * 2p2z controller function declarations and compensation filter coefficients
 * derived for following operating conditions:
 * *********************************************************************************
 *
 *  Controller Type:    2P2Z - Basic Current Mode Compensator
 *  Sampling Frequency: 250000 Hz
 *  Fixed Point Format: Q15
 *  Scaling Mode:       4 - Fast Floating Point Coefficient Scaling
 *  Input Gain:         0.0508731966590736
 *
 * *********************************************************************************
 * CGS Version:         4.0.7
 * CGS Date:            11/08/23
 * *********************************************************************************
 * User:                M91406
 * Date/Time:           01/11/2024 16:21:29
 * ********************************************************************************/

#include "vComp.h"                                  // include individual controller header file

/* *********************************************************************************
 * Data Arrays:
 * ============
 *
 * This source file declares the default parameters of the z-domain compensation
 * filter. The NPNZ16b_t data structure contains two pointers to A- and B-
 * coefficient arrays and two pointers to control and error history arrays.
 *
 * For optimized data processing during DSP computations, these arrays must be
 * located in specific memory locations (X-space for coefficient arrays and
 * Y-space for control and error history arrays).
 *
 * The following declarations are used to define the array data contents, their
 * length and memory location. These declarations are made publicly accessible
 * through extern declarations in header file vComp.h
 * ********************************************************************************/

#ifdef P33C_EDS                                     // Devices with extended data space require alternate array declaration attributes
volatile VCOMP_CONTROL_LOOP_COEFFICIENTS_t __attribute__((space(xmemory), eds, aligned)) vComp_coefficients; // A/B-Coefficients
volatile VCOMP_CONTROL_LOOP_HISTORIES_t __attribute__((space(ymemory), eds, aligned)) vComp_histories; // Control/Error Histories
#else                                               // Devices without extended data space
volatile VCOMP_CONTROL_LOOP_COEFFICIENTS_t __attribute__((space(xmemory), near, aligned)) vComp_coefficients; // A/B-Coefficients
volatile VCOMP_CONTROL_LOOP_HISTORIES_t __attribute__((space(ymemory), far, aligned)) vComp_histories; // Control/Error Histories
#endif

volatile uint16_t vComp_ACoefficients_size = (sizeof(vComp_coefficients.ACoefficients)/sizeof(vComp_coefficients.ACoefficients[0])); // A-coefficient array size
volatile uint16_t vComp_BCoefficients_size = (sizeof(vComp_coefficients.BCoefficients)/sizeof(vComp_coefficients.BCoefficients[0])); // B-coefficient array size
volatile uint16_t vComp_ControlHistory_size = (sizeof(vComp_histories.ControlHistory)/sizeof(vComp_histories.ControlHistory[0])); // Control history array size
volatile uint16_t vComp_ErrorHistory_size = (sizeof(vComp_histories.ErrorHistory)/sizeof(vComp_histories.ErrorHistory[0])); // Error history array size

/* *********************************************************************************
 * Pole&Zero Placement:
 * *********************************************************************************
 *
 *    fP0:    400 Hz
 *    fP1:    77300 Hz
 *    fZ1:    5061 Hz
 * 
 * *********************************************************************************
 * Filter Coefficients and Parameters:
 * ********************************************************************************/

volatile int32_t vComp_ACoefficients [2] =
{
    0x40EEFFFF, // Coefficient A1 will be multiplied with controller output u(n-1)
    0x89130006  // Coefficient A2 will be multiplied with controller output u(n-2)
};

volatile int32_t vComp_BCoefficients [3] =
{
    0x683E0000, // Coefficient B0 will be multiplied with error input e(n-0)
    0x63BB0003, // Coefficient B1 will be multiplied with error input e(n-1)
    0xA43A0000  // Coefficient B2 will be multiplied with error input e(n-2)
};

#define HRLINK  false

// Coefficient normalization factors
volatile int16_t vComp_pre_shift = 3;               // Bit-shift value used to perform input value normalization
volatile int16_t vComp_post_shift_A = 0;            // Bit-shift value A used to perform control output value backward normalization
volatile int16_t vComp_post_shift_B = 0;            // Bit-shift value B used to perform control output value backward normalization
volatile fractional vComp_post_scaler = 0x0000;     // Q15 fractional factor used to perform control output value backward normalization

// P-Term Coefficient for Plant Measurements
volatile int16_t vComp_PTermFactor = 0x4C15;        // Q15 fractional of the P-Term factor
volatile int16_t vComp_PTermScaler = 0x0003;        // Bit-shift scaler of the P-Term factor
//volatile int16_t vComp_PTermFactor = 0x5267;        // Q15 fractional of the P-Term factor
//volatile int16_t vComp_PTermScaler = 0x0003;        // Bit-shift scaler of the P-Term factor


// User-defined NPNZ16b_t controller data object
volatile NPNZ16b_t vComp;                           // user-controller data object

/* ********************************************************************************/

/* *********************************************************************************
 * Controller Initialization:
 * ==========================
  *
 * Public controller initialization function loading known default settings
 * into the NPNZ16b data structure.
 *
 * ********************************************************************************/

volatile uint16_t vComp_Initialize(volatile NPNZ16b_t* controller)
{
    // Initialize controller data structure at runtime with pre-defined default values
    controller->Status.bits.Enable = false;         // turn off feedback loop
    controller->Status.value  &= ~NPNZ_STATUS_SATURATION_MSK; // clear status flag bits

    controller->Filter.ptrACoefficients = &vComp_coefficients.ACoefficients[0]; // initialize pointer to A-coefficients array
    controller->Filter.ptrBCoefficients = &vComp_coefficients.BCoefficients[0]; // initialize pointer to B-coefficients array
    controller->Filter.ptrControlHistory = &vComp_histories.ControlHistory[0]; // initialize pointer to control history array
    controller->Filter.ptrErrorHistory = &vComp_histories.ErrorHistory[0]; // initialize pointer to error history array
    controller->Filter.normPostShiftA = vComp_post_shift_A; // initialize A-coefficients/single bit-shift scaler
    controller->Filter.normPostShiftB = vComp_post_shift_B; // initialize B-coefficients/dual/post scale factor bit-shift scaler
    controller->Filter.normPostScaler = vComp_post_scaler; // initialize control output value normalization scaling factor
    controller->Filter.normPreShift = vComp_pre_shift; // initialize A-coefficients/single bit-shift scaler
    
    controller->Filter.ACoefficientsArraySize = vComp_ACoefficients_size; // initialize A-coefficients array size
    controller->Filter.BCoefficientsArraySize = vComp_BCoefficients_size; // initialize B-coefficients array size
    controller->Filter.ControlHistoryArraySize = vComp_ControlHistory_size; // initialize control history array size
    controller->Filter.ErrorHistoryArraySize = vComp_ErrorHistory_size; // initialize error history array size

    // Load default set of A-coefficients from user RAM into controller A-array located in X-Space
    for (int i=0; i<controller->Filter.ACoefficientsArraySize; i++)
    {
        #if (true == HRLINK)
        vComp_coefficients.ACoefficients[i] = vComp_ACoefficients[i] + vComp_post_shift_A; // Load coefficient A(n) value into vComp coefficient(i) data space
        #else
        vComp_coefficients.ACoefficients[i] = vComp_ACoefficients[i]; // Load coefficient A(n) value into vComp coefficient(i) data space        
        #endif
    }

    // Load default set of B-coefficients from user RAM into controller B-array located in X-Space
    for (int i=0; i<controller->Filter.BCoefficientsArraySize; i++)
    {
        #if (true == HRLINK)
        vComp_coefficients.BCoefficients[i] = vComp_BCoefficients[i] + vComp_post_shift_B; // Load coefficient B(n) value into vComp coefficient(i) data space
        #else
        vComp_coefficients.BCoefficients[i] = vComp_BCoefficients[i]; // Load coefficient B(n) value into vComp coefficient(i) data space
        #endif
    }
    
    // Capture EDS page where data arrays are located
    #ifdef P33C_EDS
    controller->Filter.EdsDataPage = __builtin_edspage(&vComp_histories.ControlHistory);
    controller->Filter.EdsDataPage = __builtin_edspage(&vComp_histories.ControlHistory);
    #endif
    
    // Clear error and control histories of the 2P2Z controller
    vComp_Reset(controller);
    
    // Load P-Term factor and scaler into data structure
    controller->Filter.PTermFactor = vComp_PTermFactor;
    controller->Filter.PTermScaler = vComp_PTermScaler;
    
    return(1);
}


//**********************************************************************************
// Download latest version of this tool here: https://www.microchip.com/powersmart
//**********************************************************************************

