/* *********************************************************************************
 * PowerSmart Digital Control Library Designer, Version 2.3.0.1007
 * *********************************************************************************
 * 2p2z controller function declarations and compensation filter coefficients
 * derived for following operating conditions:
 * *********************************************************************************
 *
 *  Controller Type:    2P2Z - Basic Current Mode Compensator
 *  Sampling Frequency: 250000 Hz
 *  Fixed Point Format: Q15
 *  Scaling Mode:       4 - Fast Floating Point Coefficient Scaling
 *  Input Gain:         0.0508731966590736
 *
 * *********************************************************************************
 * CGS Version:         4.0.7
 * CGS Date:            11/08/23
 * *********************************************************************************
 * User:                M91406
 * Date/Time:           01/11/2024 16:21:29
 * ********************************************************************************/

// This is a guard condition so that contents of this file are not included
// more than once.
#ifndef __SPECIAL_FUNCTION_LAYER_VCOMP_H__
#define __SPECIAL_FUNCTION_LAYER_VCOMP_H__

#include <dsp.h>                                    // include DSP data types (e.g. fractional)
#include <stdint.h>                                 // include standard integer number data types
#include <stdbool.h>                                // include standard boolean data types (true/false)

#include "npnz16b.h"                                // include NPNZ library header file

#ifndef CONTROL_LOOP_VCOMP
#define CONTROL_LOOP_VCOMP                          // Public macro to be used to verify presence on this control loop
#endif

/* *******************************************************************************
 * Data Arrays:
 * The NPNZ16b_t data structure contains pointers to coefficient, control and error
 * history arrays. The pointer target objects (variables and arrays) are defined
 * in controller source file vComp.c
 *
 * Type definitions for A- and B- coefficient arrays as well as error- and control
 * history arrays are aligned in memory using the 'packed' attribute for optimized
 * addressing during DSP computations. These aligned data structures need to be
 * placed in specific memory locations to allow direct X/Y-access from the DSP.
 * This X/Y-memory placement is covered by the declarations used in controller
 * source file vcomp.c
 * ******************************************************************************/

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-data-objects
 * @struct  VCOMP_CONTROL_LOOP_COEFFICIENTS_s
 * @brief   Data structure packing A- and B- coefficient arrays in a linear memory space for optimized DSP code execution
 ********************************************************************************/
struct VCOMP_CONTROL_LOOP_COEFFICIENTS_s
{
    volatile int32_t ACoefficients[2];              // A-Coefficients
    volatile int32_t BCoefficients[3];              // B-Coefficients
} __attribute__((packed));                          // Data structure packing A- and B- coefficient arrays in a linear memory space for optimized DSP code execution
typedef struct VCOMP_CONTROL_LOOP_COEFFICIENTS_s VCOMP_CONTROL_LOOP_COEFFICIENTS_t;

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-data-objects
 * @struct  VCOMP_CONTROL_LOOP_HISTORIES_s
 * @brief   Data structure packing A- and B- coefficient arrays in a linear memory space for optimized DSP code execution
 ********************************************************************************/
struct VCOMP_CONTROL_LOOP_HISTORIES_s
{
    volatile fractional ControlHistory[2];          // Control History Array
    volatile fractional ErrorHistory[3];            // Error History Array
} __attribute__((packed));                          // Data structure packing control and error histories arrays in a linear memory space for optimized DSP code execution
typedef struct VCOMP_CONTROL_LOOP_HISTORIES_s VCOMP_CONTROL_LOOP_HISTORIES_t; // Data type of data structure packing control and error histories arrays

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-variables
 * @brief   Q15 fractional of the P-Term Coefficient for Plant Measurements
 ********************************************************************************/
extern volatile int16_t vComp_PTermFactor;          // Q15 fractional of the P-Term factor

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-variables
 * @brief   Bit-shift scaler of the P-Term Coefficient for Plant Measurements
 ********************************************************************************/
extern volatile int16_t vComp_PTermScaler;          // Bit-shift scaler of the P-Term factor


/*********************************************************************************
 * @ingroup special-function-layer-npnz16-objects
 * @brief   External reference to user-defined NPNZ16b controller data object 'vComp'
 ********************************************************************************/
extern volatile NPNZ16b_t vComp;                    // user-controller data object


/* *******************************************************************************
 * Function call prototypes for initialization routines and control loop handling
 * ******************************************************************************/

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Initializes controller coefficient arrays and normalization factors
 * @param   controller: Pointer to NPNZ Controller Data Object of type NPNZ16b_t
 *
 * @details
 *  This function needs to be called from user code at startup once to initialize
 *  coefficient arrays and number normalization settings of the vComp controller
 *  object.
 *
 * @attention
 *  This routine DOES NOT initialize the complete controller object.
 *  User-defined settings such as pointers to the control reference, source and
 *  target registers, output minima and maxima and further, design-dependent
 *  settings, need to be specified in user code.
 ********************************************************************************/
extern volatile uint16_t vComp_Initialize(          // vComp initialization function call
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_Reset' clearing the NPNZ16b controller output and error histories
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  This Assembler function clears the NPNZ16b controller output and
 *  error histories by settings all elements of the delay lines to zero. This
 *  resets the controller to its default state. This function should be called
 *  every time before the control loop is started from a disabled, unbiased output.
 * @note
 *  Use function 'vComp_Precharge' to start the feedback loop controller when
 *  the output is pre-biased.
 ********************************************************************************/
extern void vComp_Reset(                            // vComp reset function call (Assembler)
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_ResetControlHistory' clearing the NPNZ16b control output history delay line array
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  This Assembler function clears the NPNZ16b control output history delay line array
 *  by settings all elements of the delay line to zero. This effectively resets the controller
 *  to its default state. This function should be called every time before the control loop is
 *  started from a disabled, unbiased output.
 * @note
 *  Use function 'vComp_Precharge' to start the feedback loop controller when
 *  the control loop must start with an initial non-zero output value.
 *  (e.g. Converter output is pre-biased)
 ********************************************************************************/
extern void vComp_ResetControlHistory(              // vComp reset control history function call (Assembler)
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_ResetErrorHistory' clearing the NPNZ16b control output history delay line array
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  This Assembler function clears the NPNZ16b error input history delay line array
 *  by settings all elements of the delay line to zero. This effectively resets the controller
 *  to its default state. This function should be called every time before the control loop is
 *  started from a disabled state.
 ********************************************************************************/
extern void vComp_ResetErrorHistory(                // vComp reset error history function call (Assembler)
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_Precharge' loading user-defined values into the NPNZ16b  controller output and error histories
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 * @param   controlInput: user-defined, constant error history value of type fractional
 * @param   controlOutput: user-defined, constant control output history value of type fractional
 *
 * @details
 *  This function loads user-defined values into NPNZ16b controller
 *  output and error histories where the parameters controlInput and controlOutput
 *  will written to the entire delay line of the filter emulating a pre-existing
 *  steady state operation under the user defined conditions.
 ********************************************************************************/
extern void vComp_Precharge(                        // vComp history pre-charge function call (Assembler)
        volatile NPNZ16b_t* controller,             // Pointer to NPNZ16b data object
        volatile fractional controlInput,           // user-defined, constant error history value
        volatile fractional controlOutput           // user-defined, constant control output history value
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_PrechargeControlHistory' loading user-defined values into the NPNZ16b  control output history delay line array
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 * @param   controlOutput: user-defined, constant control output history value of type fractional
 *
 * @details
 *  This function loads the user-defined value controlOutput into each cell of the
 *  NPNZ16b control output delay line array of the compensation filter
 *  emulating a pre-existing steady state operation under the user defined conditions.
 ********************************************************************************/
extern void vComp_PrechargeControlHistory(          // vComp control history pre-charge function call (Assembler)
        volatile NPNZ16b_t* controller,             // Pointer to NPNZ16b data object
        volatile fractional controlOutput           // user-defined, constant control output history value
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler routine '_vComp_PrechargeControlHistory' loading user-defined values into the NPNZ16b  control output history delay line array
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 * @param   controlInput: user-defined, constant control input (error) history value of type fractional
 *
 * @details
 *  This function loads the user-defined value controlInput into each cell of the
 *  NPNZ16b error input delay line array of the compensation filter
 *  emulating a pre-existing steady state operation under the user defined conditions.
 ********************************************************************************/
extern void vComp_PrechargeErrorHistory(            // vComp error history pre-charge function call (Assembler)
        volatile NPNZ16b_t* controller,             // Pointer to NPNZ16b data object
        volatile fractional controlInput            // user-defined, constant error input history value
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the Assembler feedback control loop routine helping to call the vComp controller from C-code
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  This function is the main controller function which must be called at
 *  the control frequency from the control interrupt service routine. It
 *  calculates the most recent control error and processes it in the compensation
 *  filter computation after which the new result is written to the control
 *  output target.
 *  Runtime control is provided through the NPNZ16b data object status & control word.
 * @note
 *  Available control options depend on the controller feature configuration.
 *  Please refer to the user guide for more detailed information.
 ********************************************************************************/
extern void vComp_Update(                           // Calls the 2P2Z controller (Assembler)
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

/*********************************************************************************
 * @ingroup special-function-layer-npnz16-functions
 * @brief   Prototype of the alternate Assembler P-Term control loop helping to call the vComp P-Term controller from C-code
 * @param   controller: Pointer to NPNZ16b data object of type NPNZ16b_t*
 *
 * @details
 *  This function has been added as extension function supporting the development
 *  process, introducing an alternative control loop able to replace the conventional
 *  compensation feedback loop with a simple P-Term controller, allowing users to
 *  perform measurements of the plant transfer function.
 *  To use this function, users may replace the default compensation feedback loop
 *  function call vComp_Update by vComp_PTermUpdate to perform the measurement.
 *
 *  This additional controller seamlessly fits into the controller data
 *  interface by using the same NPNZ16b_t data structure and does not need
 *  additional configuration or initialization.
 *
 * @attention
 *  THIS CONTROLLER IS USED FOR MEASUREMENTS OF THE PLANT TRANSFER FUNCTION ONLY.
 *  THIS LOOP IS BY DEFAULT UNSTABLE AND ONLY WORKS UNDER STABLE TEST CONDITIONS.
 *  DO NOT USE THIS CONTROLLER TYPE FOR NORMAL OPERATION
 ********************************************************************************/
extern void vComp_PTermUpdate(                      // Calls the P-Term controller (Assembler)
        volatile NPNZ16b_t* controller              // Pointer to NPNZ16b data object
    );

#endif                                              // end of __SPECIAL_FUNCTION_LAYER_VCOMP_H__


//**********************************************************************************
// Download latest version of this tool here: https://www.microchip.com/powersmart
//**********************************************************************************

