/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    IBdbbConverter.h
 * @author  M91406
 * @brief   Bidirectional Buck-Boost Converter interface header file
 * @details
 * This is the main bidirectional buck-boost converter device driver API 
 * interface header file. It grants access to all required data objects and 
 * functions of the bidirectional buck-boost converter object.
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_INTERFACE_H
#define	BDBB_CONVERTER_INTERFACE_H

#include "./sm/svc_bdbb_typedef.h" // Include data types

/*****************************************************************************************************
 * @ingroup lib-layer-bdbb-api-properties
 * @brief   Enumeration of power transfer direction flags
 *****************************************************************************************************/
enum BDBB_PWRDIR_e {
    
    BDBB_PWRDIR_STEP_DOWN,
    BDBB_PWRDIR_STEP_UP
};
typedef enum BDBB_PWRDIR_e BDBB_PWRDIR_t;

/*****************************************************************************************************
 * @ingroup lib-layer-bdbb-api-properties
 * @brief   Power Converter API Properties
 *****************************************************************************************************/
struct IBdbbConverter_s 
{
    // State Machine Runtime Status Information
    BDBB_STATE_ID_t StateId;               ///< BDBB state machine operating state ID
    BDBB_CONVERTER_STATUS_t Status;        ///< BDBB operation status bits 
    BDBB_CONVERTER_DATA_t Data;            ///< BDBB runtime data

    // State Machine Runtime Control Interface
    BDBB_CONVERTER_PROPERTIES_t Properties; ///< Control field for global access to references
    BDBB_CONVERTER_STARTUP_t Startup;      ///< BDBB startup configuration 

    // Power Supply Hardware Configuration 
    BDBB_SWITCH_NODE_SETTINGS_t SwitchNode[BDBB_MPHASE_COUNT]; ///< BDBB converter switch node settings
    BDBB_FEEDBACK_SETTINGS_t Feedback;     ///< BDBB converter feedback channel settings
    BDBB_GPIO_SETTINGS_t Gpio;             ///< BDBB converter control pin specification
    
    // Power Supply Feedback Loop and Control Configuration 
    BDBB_CONVERSION_CONTROL_t Control;     ///< BDBB control loop system objects
    
}; ///< BDBB control & monitoring data structure
typedef struct IBdbbConverter_s IBdbbConverter_t; ///< BDBB control & monitoring data structure data type

// ==============================================================================================
// BDBB CONVERTER PUBLIC API FUNCTION CALL PROTOTYPES
// ==============================================================================================

/****************************************************************************************************
 * @addtogroup  lib-layer-bdbb-api-methods
 * @brief       Power Converter API Methods
 * @{
 *****************************************************************************************************/

extern int BdbbConverter_Initialize(void* handle);
extern int BdbbConverter_Execute(void* handle);
extern int BdbbConverter_Start(void* handle);
extern int BdbbConverter_Stop(void* handle);
extern int BdbbConverter_Suspend(void* handle);
extern int BdbbConverter_Resume(void* handle);

/** @} */  // end of group lib-layer-bdbb-api-methods

#endif  /* BDBB_CONVERTER_INTERFACE_H */

// ______________________________________
// end of file
