# Bidirectional Buck-Boost (BDBB) Converter Device Driver {#powerlibrary}

## State Machine Middleware Layer with Peripheral Configuration Templates

<span id="startDoc"> </span> <!-- start tag for internal references -->

### Class

PowerSmart™ Firmware Framework Component

### Summary

Voltage, peak current and average current mode controlled single- and multiphase asynchronous or synchronous BDBB converter device driver.

### Table Of Contents

<div id="TOC">
<ul>
<li><a href="#intro">Introduction</a></li>
<li><a href="#api-guide">API Quick-Start Guide</a></li>
<li><a href="#api-objects">API Public Data Object</a></li>
<li><a href="#state-machine">BDBB Converter State Machine</a></li>
<li><a href="#history">History</a></li>
</ul>
</div>

<span id="intro"> </span>

### Introduction

#### Supported Features

* Single Phase Converter
* Multiphase Converter
* Synchronous Buck Converter
* Asynchronous Buck Converter
* Synchronous Boost Converter
* Asynchronous Boost Converter
* Control Modes:
  * Voltage Mode Control (VMC)
  * Peak Current Mode Control (PCMC)
  * Average Current Mode Control (ACMC)
* Multiphase Control
  * Individual Phase Control (PCMC and ACMC only)
  * Phase Multiplexing (all control modes)

#### Special Features

* Tunable Startup Timing
  * Power On Delay (POD)
  * Ramp-Up Period
  * Power Good Delay
* Current Ramp-Up State for Constant Current Sources for battery chargers (automatic)
* Slope controlled reference tuning for voltage and current limits during normal operation
* Conversion Mode Swap
  * Soft conversion mode swap from down-stream BDBB to up-stream boost and vice versa (shut-down, reset, restart)
  * Hot conversion mode swap from down-stream BDBB to up-stream boost and vice versa (on-the-fly)
* Sustained reverse operation

#### Optional Features

* Automatic Synchronous Rectifier Control (optional)
* Current Sense Feedback Offset Calibration (optional)
* GPIO extensions for Power Good Output and Enable Input (optional)

#### Description

This generic synchronous BDBB converter device driver provides a state machine, peripheral configuration templates and controller interface for driving one single- or multiphase synchronous or asynchronous BDBB converter instance in voltage, peak or average current mode control.

All settings of this code library are made in the BDBB Converter Controller Object defined by the BDBB_POWER_CONTROLLER_s data structure. Each API function takes an pointer to a BDBB Converter Controller Object of type `IBdbbConverter_t` as parameter. This allows to define multiple BDBB converter instances using the same library files to drive and control multiple, independent converters in parallel.

#### Tool Support

This device driver uses control loop code based on `NPNZ16b_t` data objects which is generated by the [MPLAB&reg; PowerSmart&trade; Development Suite](https://www.microchip.com/powersmart). 


<a href="#startDoc">back</a>

<span id="api-guide"> </span>

### API Quick-Start Guide

#### API Public Functions

The API of this device driver provides the following standard power converter functions:

```c
volatile uint16_t BdbbConverter_Initialize(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Execute(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Start(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Stop(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Suspend(volatile IBdbbConverter_t* pcInstance);
volatile uint16_t BdbbConverter_Resume(volatile IBdbbConverter_t* pcInstance);
```

Each of these functions takes a function call parameter of type ```IBdbbConverter_t``` (see blow) to be applied to a particular BDBB converter object and returns an integer return value indicating success (= 1) or failure (= 0) of the function execution.

* **`uint16_t BdbbConverter_Initialize(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
Initializes the BDBB converter object by first loading the built-in peripheral configuration templates into the required peripheral special function registers, adding the user settings specified in the *struct BDBB_POWER_CONTROLLER_s* BDBB Converter data object. After a successful peripheral initialization the specified control loop objects will be reset and their histories cleared. <br>&nbsp;<br>All peripherals and control loops will be initialized but kept disabled. The state machine will later enable the peripherals and controller objects used in their proper enable sequence during the startup procedure and no user code intervention is required.<br>&nbsp;<br>

* **`uint16_t BdbbConverter_Start(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
The *BdbbConverter Start* function is initiating the state machine from scratch, by enabling the fundamental peripherals PWM and ADC to start the acquisition of feedback data allowing the state machine to analyze the most recent operating conditions. <br>&nbsp;
Function *BdbbConverter Execute* **should be** called after the execution of this function at a constant frequency.<br>&nbsp;

* **`uint16_t BdbbConverter_Stop(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
The *BdbbConverter Stop* function is terminating the state machine and disabling the fundamental peripherals PWM and ADC to stop the acquisition of feedback data stopping the analysis of the most recent operating conditions. <br>&nbsp;
Function *BdbbConverter Execute* should **not** be called after the execution of this function.<br>&nbsp;

* **`uint16_t BdbbConverter_Suspend(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
The *BdbbConverter Suspend* function is terminating the state machine but keeps the fundamental peripherals PWM and ADC running to continue the acquisition of feedback data allowing the analysis of the most recent operating conditions. <br>&nbsp;
Function *BdbbConverter Execute* **should continued to be** called after the execution of this function at a constant frequency.<br>&nbsp;

* **`uint16_t BdbbConverter_Resume(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
The *BdbbConverter Resume* function is re-initiating the state machine to recover from RESET during which the fundamental peripherals PWM and ADC are running to continue the acquisition of feedback data allowing the analysis of the most recent operating conditions. <br>&nbsp;
Function *BdbbConverter Execute* **should be** called after the execution of this function at a constant frequency.<br>&nbsp;

* **`uint16_t BdbbConverter_Execute(volatile IBdbbConverter_t* pcInstance)`**<br>&nbsp;
The *BdbbConverter Execute* function is executing the state machine managing the initialization, startup and online regulation in voltage and average current mode control. It provides mechanisms for fault handlers to shut down and recover the power supply control in controlled, deterministic procedures using the *drv_BdbbConverter_Suspend* and *drv_BdbbConverter_Resume* function calls described above.<br>&nbsp;

<a href="#startDoc">back</a>

<span id="api-objects"> </span>

#### API Public Data Objects

The BDBB Converter Data Object holds all configuration data of the user-defined BDBB converter, covering

* Functional Declarations
  * Startup Timing Declarations
  * User Set Value Declarations

* Peripheral Configuration Declarations
  * Switch Node Declarations (PWM and ACMP Configuration)
  * Feedback Channel Declarations (ADC Configuration)
  * GPIO Declarations (GPIO Configuration)
  * Voltage Loop Controller Declarations
  * Current Loop Controller Declarations (ACMC only, one controller per phase required)

* Status Bit Declarations
  * Read-Only Data and Status Information
  * State Machine State Monitoring
  * Runtime Data

* Control Bit Declarations
  * Allowing fast runtime manipulation of features
  * Bit field allows simultaneous manipulation of features

```c
struct IBdbbConverter_s 
{
    // State Machine Runtime Status Information
    BDBB_STATE_ID_t StateId;               ///< BDBB state machine operating state ID
    BDBB_CONVERTER_STATUS_t Status;        ///< BDBB operation status bits 
    BDBB_CONVERTER_DATA_t Data;            ///< BDBB runtime data

    // State Machine Runtime Control Interface
    BDBB_CONVERTER_PROPERTIES_t Properties; ///< Control field for global access to references
    BDBB_CONVERTER_STARTUP_t Startup;      ///< BDBB startup configuration 

    // Power Supply Hardware Configuration 
    BDBB_SWITCH_NODE_SETTINGS_t SwitchNode[BDBB_MPHASE_COUNT]; ///< BDBB converter switch node settings
    BDBB_FEEDBACK_SETTINGS_t Feedback;     ///< BDBB converter feedback channel settings
    BDBB_GPIO_SETTINGS_t Gpio;             ///< BDBB converter control pin specification
    
    // Power Supply Feedback Loop and Control Configuration 
    BDBB_CONVERSION_CONTROL_t Control;     ///< BDBB control loop system objects
    
}; ///< BDBB control & monitoring data structure
typedef struct IBdbbConverter_s IBdbbConverter_t; ///< BDBB control & monitoring data structure data type
```

Before the state machine can be used, at least one BDBB converter data object has to be defined and initialized in the application layer of the user project.

##### Declaration Examples

```c
IBdbbConverter_t myBdbbConverter; // Declare single BDBB converter data object
```

```c
IBdbbConverter_t myBdbbConverter[4]; // Declare array of four BDBB converter data objects
```


<br>&nbsp;

##### Sub Structure  `BDBB_CONVERTER_STATUS_t`

The BDBB converter status bit field contains read-only, auto-set/clear status bits:

```c
struct BDBB_CONVERTER_STATUS_s
{
    bool IsInitialized:1;       ///< Bit #0:  Status bit indicating if power converter has been initialized
    bool Ready:1;               ///< Bit #1:  Status bit indicating if power converter settings have been reset and is ready to run
    bool PodComplete:1;         ///< Bit #2:  Status bit indicating if the Power On Delay has expired
    bool AdcActive:1;           ///< Bit #3:  Status bit indicating if ADC has been started and samples are taken
    bool PwmActive:1;           ///< Bit #4:  Status bit indicating if PWM has been started and ADC triggers are generated
    bool FaultStatus :1;        ///< Bit #5:  Status bit indicating if system is in enforced shut down mode (usually due to a fault condition)
    bool FaultLatch :1;         ///< Bit #6:  Status bit indicating if system is in latched enforced shut down mode (requires manual clearance)
    bool CsCalibComplete :1;    ///< Bit #7:  Status bit indicating if current sense calibration process is complete

    bool IsSuspended :1;        ///< Bit #8:  Status bit indicating if converter is in suspend mode and state machine is hold in reset
    bool PwmOutputActive:1;     ///< Bit #9:  Status bit indicating if the PWM outputs are enabled or disabled
    bool AsyncActive :1;        ///< Bit #10: Status bit indicating if the switch node is driven in asynchronous mode (=true) or synchronous mode (=false)
    bool SwapDetect :1;         ///< Bit #11: Hot swap event detection status flag (mirrors SwapEnable control bit when a switch over has been performed)
    unsigned :1;                ///< Bit #12: (reserved)
    bool Busy :1;               ///< Bit #13: Status bit indicating that the state machine is executing a process (e.g. startup-ramp)
    bool Active :1;             ///< Bit #14: Status bit indicating if switch-node and control loops are active
    bool Online :1;             ///< Bit #15: Status bit indicating if the state machine has reached continuous operation in state ONLINE

}__attribute__((packed)); ///< BDBB Converter Status & Control word

union BDBB_CONVERTER_STATUS_u 
{
    struct BDBB_CONVERTER_STATUS_s bits; ///< BDBB Converter Status & Control bit-field
    volatile uint16_t value; ///< BDBB Converter Status & Control word for 16-bit word read/write operations
};

typedef union BDBB_CONVERTER_STATUS_u BDBB_CONVERTER_STATUS_t; // BDBB Converter Status & Control word data type

```

###### Please note

The data structure provides another control bit filed allowing to select and set features and functions at runtime

<br>&nbsp;

##### Sub Structure  `BDBB_STATE_ID_t`

The BDBB converter State ID is controlled by the state machine and holds the most recent operating state in a 16-bit wide variable. While the state machine is executing sequences of operating states, the busy flag (bit <14> in the status word) will be set. The second data
field provides the total number of states available helping to track state progress.

```c
struct BDBB_STATE_ID_s 
{
    BDBB_OPSTATES_t  OpStateId;    ///< Most recent operating state of main state machine
    uint16_t   OpStateListSize;    ///< Number of sub-states of main state machine
}; // BDBB Converter State Machine operating state ID

typedef struct BDBB_STATE_ID_s BDBB_STATE_ID_t;  // BDBB Converter State Machine operating state ID data type

```

<br>&nbsp;

##### Sub Structure  `struct BDBB_CONVERTER_DATA_s`

The basic converter data buffer `BDBB_CONVERTER_DATA_s` data structure is used to publish most recent runtime data across the firmware. During operation, control loops and state machine will continuously update this public data buffer. 

```c
struct BDBB_CONVERTER_DATA_s 
{
    int16_t ISense[BDBB_MPHASE_COUNT];  ///< BDBB phase current sense value (rectified)
    int16_t IAverage;                   ///< BDBB accumulated, averaged, rectified sum of phase currents
    int16_t VHighVoltage;               ///< BDBB high voltage port
    int16_t VLowVoltage;                ///< BDBB low voltage port
    int16_t Temperature;                ///< BDBB board temperature
    
    int16_t ControlInput;               ///< BDBB most recent control input value
    int16_t ControlError;               ///< BDBB most recent control error value
    int16_t ControlOutput;              ///< BDBB most recent control output value

}; // BDBB runtime data buffer
typedef struct BDBB_CONVERTER_DATA_s BDBB_CONVERTER_DATA_t; // BDBB runtime data buffer data type
```

###### Please Note

When the BDBB converter is turned off using the  `BdbbConverter_Stop(pcInstance)` function, the data buffer will be cleared and won't get updated anymore. If you need to suspend the power converter operation temporarily but have to keep the data buffer updating to allow system monitoring to continue, use the `BdbbConverter_Suspend(pcInstance)` function.

<br>&nbsp;

##### Sub Structures  `BDBB_CONVERTER_STARTUP_t`

The BDBB converter startup declaration consists of multiple delay and ramp-up period declarations, which are defined in the nested `BDBB_REFERENCE_RAMP_HANDLER_t` data structures. This set of configurations also includes settings for open loop startup supporting the startup with limited feedback signal availability (e.g. caused by limited common mode voltage range of sense circuits).

```c
struct BDBB_CONVERTER_STARTUP_s 
{
    BDBB_PERIOD_HANDLER_t PowerOnDelay;        ///< Declaration of Power On Delay period and timeout
    BDBB_PERIOD_HANDLER_t PowerGoodDelay;      ///< Declaration of Power Good period and timeout
    BDBB_REFERENCE_RAMP_HANDLER_t IRamp;       ///< Declaration of output current ramp reference increment step size and period
    BDBB_REFERENCE_RAMP_HANDLER_t LVRamp;      ///< Declaration of output voltage ramp reference increment step size and period in buck mode
    BDBB_REFERENCE_RAMP_HANDLER_t HVRamp;      ///< Declaration of output voltage ramp reference increment step size and period in boost mode
    BDBB_OPEN_LOOP_STARTUP_t OpenLoopStart;    ///< Monitored Open Loop Startup Settings
    BDBB_PARAM_MONITOR_HANDLER_t VCapMonitor;  ///< Output capacitor charge monitor (in Boost mode only)
    
}; // Power converter start-up settings and variables
typedef struct BDBB_CONVERTER_STARTUP_s  BDBB_CONVERTER_STARTUP_t; // Power converter start-up settings and variables data type

```

Each item is defined by a configuration data management data structure of type `BDBB_PERIOD_HANDLER_t`, `BDBB_REFERENCE_RAMP_HANDLER_t` or `BDBB_OPEN_LOOP_STARTUP_t`, each providing a dedicated, individual set of configurations.

###### Nested Sub-Data Structure `BDBB_PERIOD_HANDLER_t`

This data structure provides parameters for pure delay functions without further functional content. Delay period settings incorporate an independent counter variable allowing the state machine to count down the specified period before the state machine is allowed to progress to another state.

```c
struct BDBB_PERIOD_HANDLER_s 
{
    volatile uint16_t Counter;      ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    volatile uint16_t Period;       ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    
}; // Power converter soft-start auxiliary variables
typedef struct BDBB_PERIOD_HANDLER_s BDBB_PERIOD_HANDLER_t; // Power converter soft-start auxiliary variables data types

```

###### Nested Sub-Data Structure `BDBB_REFERENCE_RAMP_HANDLER_t`

For the Voltage and Current Ramp-Up periods the control reference will be incremented by the specified reference step size in each counter step until the nominal reference specified in `BDBB_REFERENCE_RAMP_HANDLER_t` is met. This data type uses the period setting for the declaration of the ramp up period, however, the value used to determine the end of the ramp is based on the sampled voltage or current value. Hence, the real ramp up time depends on the system response rather than the declared time period.

```c
struct BDBB_REFERENCE_RAMP_HANDLER_s 
{
    uint16_t Counter;          ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t Period;           ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    uint16_t StepSize;         ///< Size/value of one reference increment/decrement or this period
    uint16_t Start;            ///< Reference value transition start point 
    uint16_t Stop;             ///< Reference value transition end point 
    uint16_t refBuffer;        ///< READ ONLY: \n Internal reference value buffer manipulated by state machine during reference level transition
    uint16_t preDelayBuffer;   ///< READ ONLY: \n Internal value buffer used to inject a startup delay compensating for variations of the voltage ramp start level caused by pre-biased output voltages \n This buffer is only used during the startup process and does not affect runtime changes of output references.
    
}; // Power converter soft-start auxiliary variables
typedef struct BDBB_REFERENCE_RAMP_HANDLER_s BDBB_REFERENCE_RAMP_HANDLER_t; // Power converter soft-start auxiliary variables data types

```

###### Nested Sub-Data Structure `BDBB_OPEN_LOOP_STARTUP_t`

In some cases power supplies must be started in open loop due to limited available of feedback signals (e.g. due to common mode voltage range limitations, for example, when feedback components are biased from the converted voltage). To overcome this obstacle, the startup engine of the state machine provides the declaration of an open loop startup period, which is defined by the control output value step size and an target cut-off voltage level at which the state machine will close the control loop and switch to a controlled ramp up period.

```c
struct BDBB_OPEN_LOOP_STARTUP_s 
{
     bool     Enabled; ///< User option enabling the open loop startup feature, which will increment the control output until the output voltage is above the specified minimum cut-off voltage
    uint16_t CutOffVoltage; ///< Monitored open loop startup cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t CutOffHysteresis; ///< Hysteresis above cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t StepSize; ///< Value increment per execution step during monitored open loop startup. This value will be determined by the state machine during startup and is 'read only'.
    uint16_t Counter; ///< internal open loop startup process counter
    
}; // Power converter start-up settings and variables
typedef struct BDBB_OPEN_LOOP_STARTUP_s  BDBB_OPEN_LOOP_STARTUP_t; // Power converter open-loop start-up settings and variables data type

```

<br>&nbsp;

##### Sub Structure  `BDBB_CONVERTER_PROPERTIES_t`

This is the central control interface allowing external firmware modules to manipulate the power supply state machine asynchronously across process priorities.

```c
struct BDBB_CONVERTER_PROPERTIES_s 
{
    BDBB_SYNC_RECTIFIER_CONTROL_t SyncRectifier; ///< Power converter automatic synchronous rectifier control settings

    uint16_t PhaseCount;       ///< number of converter phases
    uint16_t LVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in buck mode
    uint16_t HVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in boost mode
    int16_t  IReference;       ///< User current reference setting used to control the power converter controller

    bool Go;                   ///< Control Flag: While GO = false, the power supply is kept in standby, the power supply will start, when GO is set to 'true'. GO is set automatically, when option 'Autostart' is enabled.
    bool Swap;                 ///< Control Flag: When SWAP = true, triggers the switch over of power transfer direction at runtime without state change. This flag is reset when the switch-over has been performed.
    bool Suspend;              ///< Control Flag: When set, state machine is pushed to and hold in RESET

    bool EnforcedOpenLoop;     ///< Control Flag: When set, enables PWM output while keeping control loop disabled; Skips closed loop ramp up states and directly jumps to state Online 
    bool PowerOnDelayByPass;   ///< Control Flag: When set, it bypasses the power on delay state in state machine once and will be auto cleared 
    bool HotSwapEnable;        ///< Control Flag: When set, allows switching of power transfer direction immediately at runtime without state change. When cleared, a change of power transfer direction forces the power converter through a shut-down and restart cycle
    bool AsyncMode;            ///< Control Flag: When set, suspending the synchronous rectifier switch drive PWM channel(s)
    bool AutoAsyncMode;        ///< Control Flag: When set, enables the automatic synchronous rectifier control using threholds declared in 'Settings.SyncRectifierControl'
    bool CsCalibEnable;        ///< Control Flag: When set, enforcing current sense calibration
    bool ConversionMode;       ///< Control Flag: When set, power transfer direction is set to up-stream (boost) mode, when cleared, power transfer direction is set to down-stream (buck) mode)
    bool Autorun;              ///< Control Flag: When set, converter starts up automatically as soonas all start conditions are met. \nWhen cleared, converter will be held in standby until the GO bit is set
    bool Enable;               ///< Control Flag: When set, enables the converter triggering a startup sequence; When cleared, forces teh power supply to reset and turn off

};  ///> BDBB converter main settings
typedef struct BDBB_CONVERTER_PROPERTIES_s BDBB_CONVERTER_PROPERTIES_t;  ///> BDBB converter main settings

```

<br>&nbsp;

##### Sub Structure  `BDBB_LOOP_SETTINGS_t`

BDBB converter control loop settings are split in properties and methods.

**Properties** specify default values for control reference, feedback offset, ADC trigger offset and anti-windup minimum and maximum values, which are used during initialization. Further a pointer to a control loop object of type `NPNZ16b_t` is declared, which holds all the detailed information of the specific control loop, such as filter coefficients, feedback input and control output scaling, pointers to data provider targets and further parameters supporting advanced control options.

**Methods** declare function pointers to default function calls of a control loop data object of type `NPNZ16b_t`. These include initialization, reset and value pre-charge functions as well as the default update function executing a control loop computation.

```c
struct BDBB_LOOP_SETTINGS_s 
{
    // Control Loop Object
    volatile NPNZ16b_t* Controller; ///< pointer to control loop object data structure

    // Control Loop Default Settings & Methods
    BDBB_LOOP_PARAMETER_BUFFER_t Properties; // Control loop properties store default configuration information
    BDBB_LOOP_METHODS_t Methods; // Control loop methods hold function pointers to control loop default functions

}; ///< User defined settings for control loops; 
typedef struct BDBB_LOOP_SETTINGS_s BDBB_LOOP_SETTINGS_t; ///< User defined settings for control loops data type

```

###### Nested Sub Structure  `BDBB_LOOP_PARAMETER_BUFFER_t`

```c
struct BDBB_LOOP_PARAMETER_BUFFER_s
{   // Properties (user settings)
    
    int16_t Reference;             ///< Nominal control loop voltage reference variable
    int16_t Minimum;               ///< output clamping value (minimum)
    int16_t Maximum;               ///< output clamping value (maximum)

    int16_t AltReference;          ///< Alternate nominal control loop voltage reference variable in reverse mode
    int16_t AltMinimum;            ///< Alternate output clamping value (minimum)
    int16_t AltMaximum;            ///< Alternate output clamping value (maximum)

};
typedef struct BDBB_LOOP_PARAMETER_BUFFER_s BDBB_LOOP_PARAMETER_BUFFER_t; ///< User defined settings for control loops data type

```

###### Nested Sub Structure  `BDBB_LOOP_METHODS_t`

```c
struct BDBB_LOOP_METHODS_s
{
    volatile uint16_t (*Initialize)(volatile NPNZ16b_t*); ///< Function pointer to INIT routine
    void (*Precharge)(volatile NPNZ16b_t*, volatile fractional, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control output and error histories
    void (*PrechargeControlHistory)(volatile NPNZ16b_t*, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control output history
    void (*PrechargeErrorHistory)(volatile NPNZ16b_t*, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control error history
    void (*Reset)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control output and error histories
    void (*ResetControlHistory)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control output history
    void (*ResetErrorHistory)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control error history
    void (*Update)(volatile NPNZ16b_t*); ///< Function pointer to UPDATE routine, calling a regular loop control step
    void (*PTermUpdate)(volatile NPNZ16b_t*); ///< Function pointer to PTerm UPDATE routine, calling a unity gain loop control step
    
};
typedef struct BDBB_LOOP_METHODS_s BDBB_LOOP_METHODS_t; ///< User defined settings for control loops data type
```

<br>&nbsp;

##### Sub Structure  `struct BDBB_SWITCH_NODE_SETTINGS_s`

The switch node settings of the BDBB converter data object specify the user parameters of the PWM configuration supporting a synchronous or asynchronous BDBB converter switching signal generator. Each converter phase requires its independent switch node definition. The synchronization of multiple PWM generators is automatically configured during the peripheral configuration by the initialization driver `dev_buck_pconfig.c` of the BDBB converter device driver. This driver will first load the default peripheral configuration from the built-in BDBB converter peripheral templates overwriting user parameters with the settings found in this data structure.

```c
struct BDBB_SWITCH_NODE_SETTINGS_s 
{
    P33C_PWM_INSTANCE_t* PwmInstance;   ///< Pointer to PWM generator instance used
    P33C_GPIO_t GpioHigh;               ///< GPIO port pin-number of PWMxH of the selected PWM generator
    P33C_GPIO_t GpioLow;                ///< GPIO port pin-number of PWMxL of the selected PWM generator
    bool        SwapOutputs;            ///< Selecting if PWMxH (default) or PWMxL should be the leading default PWM output
    bool        MasterPeriodEnable;     ///< Selecting MASTER or Individual period register
    bool        HighResolutionEnable;   ///< Selecting if PWM module should use high-resolution mode 
    bool        SyncDrive;              ///< Selects if switch node supports synchronous or asynchronous commutation
    uint16_t    Period;                 ///< Switching period
    uint16_t    PhaseShift;             ///< Switching signal phase-shiftbetween multiple phases
    uint16_t    DutyRatioMinimum;       ///< Absolute duty cycle minimum during normal operation
    uint16_t    DutyRatioMaximum;       ///< Absolute duty cycle maximum during normal operation
    uint16_t    DeadTimeRising;         ///< Dead time setting at rising edge of a half-bridge drive
    uint16_t    DeadTimeFalling;        ///< Dead time setting at falling edge of a half-bridge drive
    uint16_t    ControlTriggerDivider;  ///< PWM triggers for ADC will be generated every n-th cycle  (only affects ADC Trigger #1, see register PGxEVTL for details)
    uint16_t    ControlTriggerOffset;   ///< PWM triggers for ADC will be offset by n cycles (only affects ADC Trigger #1, see register PGxEVTL for details)
    
    BDBB_SLOPE_COMP_SETTINGS_t SlopeCompensation; ///< Peak current mode slope compensation settings (peak current mode control only)
    
}; ///< Switching signal timing settings
typedef struct BDBB_SWITCH_NODE_SETTINGS_s  BDBB_SWITCH_NODE_SETTINGS_t; ///< Switching signal timing settings data type

```

###### Nested Sub Structure  `BDBB_SLOPE_COMP_SETTINGS_t`

```c
struct BDBB_SLOPE_COMP_SETTINGS_s
{
    uint16_t DacInstance; ///< Digital-to-analog converter / Analog comparator instance used to triggering the the slope compensation ramp generator
    uint16_t CompInput; ///< Analog comparator input option A to D (0=A, 1=B, 2=C, 3=D)
    uint16_t LebPeriod; ///< Leading-Edge Blanking period
    uint16_t PwmTriggerInput; ///< PWM current limit PCI input source (must match comparator declared by cmp_instance)
    uint16_t SlewRate; ///< Integer equivalent of compensation ramp slew rate in [V/ï¿½s]
    uint16_t SlopeStartTrigger; ///< Period counter value at with the slope start is triggered
    uint16_t SlopeStopTrigger; ///< Period counter value at with the slope stop is triggered
    uint16_t DacValueInit;     ///< Initial Digital-to-Analog converter reference value (used when peripheral is enabled or being reset)
    uint16_t DutCycleLimit;    ///< Switch node duty cycle limit value
    
}; ///< Peak Current Mode Control Slope Compensation Settings
typedef struct BDBB_SLOPE_COMP_SETTINGS_s BDBB_SLOPE_COMP_SETTINGS_t; ///< Peak Current Mode Control Slope Compensation Settings data type

```

<br>&nbsp;

##### Sub Structure  `BDBB_FEEDBACK_SETTINGS_t`

Each converter offers the following standard feedback input channels in its `BDBB_FEEDBACK_SETTINGS_t` data structure:

* Output Voltage
* Input Voltage
* Inductor Current (one for each converter phase)
* Temperature

```c
struct BDBB_FEEDBACK_SETTINGS_s 
{
    BDBB_FEEDBACK_CHANNEL_t AdVHighVoltage;             ///< ADC input sampling input voltage
    BDBB_FEEDBACK_CHANNEL_t AdVLowVoltage;              ///< ADC input sampling output voltage
    BDBB_FEEDBACK_CHANNEL_t AdISense[BDBB_MPHASE_COUNT]; ///< ADC input sampling phase current
    BDBB_FEEDBACK_CHANNEL_t AdTemperature;              ///< ADC input sampling temperature
    
}; ///< BDBB converter feedback declarations
typedef struct BDBB_FEEDBACK_SETTINGS_s BDBB_FEEDBACK_SETTINGS_t; ///< BDBB converter feedback declarations data type

```

Each feedback channel can be enabled or disabled. Disabled channels will be ignored during initialization and will remain inactive during operation. The peripheral configuration is performed by the initialization driver `dev_bdbb_pconfig.c` of the BDBB converter device driver. This driver will first load the default peripheral configuration from the built-in BDBB converter peripheral templates overwriting user parameters with the settings found in this data structure.

###### Nested Sub Structure  `BDBB_FEEDBACK_CHANNEL_t`

```c
struct BDBB_FEEDBACK_CHANNEL_s 
{
    bool Enabled;                      ///< input channel enable bit
    BDBB_ADC_INPUT_t Input;            ///< ADC input channel specifications
    BDBB_ADC_TRIGGER_t Trigger;        ///< ADC input sample trigger configuration
    BDBB_ADC_INTERRUPT_t Interrupt;    ///< ADC sample interrupt service routine call configuration
    BDBB_ADC_SIGNAL_SCALING_t Scaling; ///< ADC result normalization settings
    BDBB_ADC_SIGNAL_RANGE_t Range;     ///< ADC result valid data range specification

}; ///< ADC input channel configuration
typedef struct BDBB_FEEDBACK_CHANNEL_s BDBB_FEEDBACK_CHANNEL_t; ///< ADC input channel configuration data type

```

###### Nested Sub Structure  `BDBB_ADC_INPUT_t`

```c
struct BDBB_ADC_INPUT_s 
{
    uint8_t   AdcInput;            ///< number of the ADC input channel used
    uint8_t   AdcAltInput;         ///< selection of the alternate ADC input channel used (dedicated ADC cores only, set =0 for shared ADC inputs)
    uint8_t   AdcCore;             ///< index of the ADC core the selected analog input channel is assigned to
    bool      DifferentialInput;   ///< input channel differential mode enable bit
    bool      SignedResult;        ///< input channel singed result mode enable bit
    uint16_t  GpioInstance;        ///< GPIO instance of the selected PWM generator

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_INPUT_s BDBB_ADC_INPUT_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BDBB_ADC_TRIGGER_t`

```c
struct BDBB_ADC_TRIGGER_s 
{
    volatile uint16_t* ptrTriggerRegister;  ///< pointer to trigger register of this 
    uint8_t   TriggerSource;       ///< input channel trigger source
    uint16_t  TriggerDelay;        ///< input channel trigger delay (fixed, will be added to trigger delay set in PWM generator register PGxTRIGy)
    bool      LevelTrigger;        ///< input channel level trigger mode enable bit

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_TRIGGER_s BDBB_ADC_TRIGGER_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BDBB_ADC_INTERRUPT_t`

```c
struct BDBB_ADC_INTERRUPT_s 
{
    bool InterruptEnable;      ///< input channel interrupt enable bit (allows tripping an Interrupt Service Routine)
    bool EarlyInterruptEnable; ///< input channel early interrupt enable bit (triggers the interrupt early before conversion is complete)

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_INTERRUPT_s BDBB_ADC_INTERRUPT_t; ///< ADC input interrupt settings data type

```

###### Nested Sub Structure  `BDBB_ADC_SIGNAL_SCALING_t`

Supporting calculations of power, voltage ratios, gains or Root Mean Square (RMS) values feedback channels may have to be scaled/normalized. For this purpose the `BDBB_ADC_INPUT_SCALING_s` data structure was added allowing users to specify the feedback value scaling information in form of a fast floating point slope and integer offset value.

```c
struct BDBB_ADC_SIGNAL_SCALING_s 
{
    int16_t Factor; ///< Fractional scaling factor (range -1 ... 0.99969)
    int16_t Scaler; ///< Feedback bit-shift scaler used for number normalization
    int16_t Offset; ///< Signal offset as signed integer to be subtracted from ADC input
    bool  Polarity; ///< Signal polarity declarations, where (0 = positive, 1 = negative)

}; ///< ADC input signal scaling = (ADCBUF - <offset>) * <factor> >> 2^<scaler>
typedef struct BDBB_ADC_SIGNAL_SCALING_s BDBB_ADC_SIGNAL_SCALING_t; ///< ADC input signal scaling data type

```

```c
struct BDBB_ADC_SIGNAL_RANGE_s 
{
    int16_t Minimum;    ///< Minimum valid signal level value
    int16_t Nominal;    ///< Nominal expected signal level value
    int16_t Maximum;    ///< Maximum valid signal level value

}; ///< ADC input signal range
typedef struct BDBB_ADC_SIGNAL_RANGE_s BDBB_ADC_SIGNAL_RANGE_t; ///< ADC input signal range data type

```

<br>&nbsp;

##### Sub Structure  `BDBB_GPIO_SETTINGS_t`

The BDBB converter device driver supports one Enable input (EN) and one Power Good output (PG) as two basic general purpose input/outputs (GPIO) for low-level hardware sequencing of converter stages.

```c
struct BDBB_GPIO_SETTINGS_s 
{
    BDBB_GPIO_INSTANCE_t EnableInput; ///< External ENABLE input
    BDBB_GPIO_INSTANCE_t PowerGood; ///< Power Good Output

}; ///< GPIO instance of the converter control GPIO
typedef struct BDBB_GPIO_SETTINGS_s BDBB_GPIO_SETTINGS_t; ///< GPIO instance of the converter control GPIO data type

```

###### Nested Sub Structure  `BDBB_GPIO_INSTANCE_t`

Each GPIO can be configured by the settings specified in the `BDBB_GPIO_INSTANCE_t` data structure. The Input/Output type can be set to

* Input (=1, max. input voltage = 3.3 V)
* Push-Pull Output (=0, max. sink-/source-current = 4 mA, max. output voltage = 3.3 V)
* Open-Drain Output (=0, max. sink-current = 4 mA, max. output voltage = 3.3/5.0 V)

```c
struct BDBB_GPIO_INSTANCE_s 
{
    bool        Enabled;    ///< Specifies, if this IO is used or not
    P33C_GPIO_t IoPin;      ///< Assigns a device pin to be controlled by the library
    uint16_t    Polarity;   ///< Output polarity, where 0=ACTIVE HIGH, 1=ACTIVE_LOW
    uint16_t    IoType;     ///< Input/Output definition (0=push-pull output, 1=input, 2=open-drain output)

}; ///< GPIO instance of the converter control GPIO
typedef struct BDBB_GPIO_INSTANCE_s BDBB_GPIO_INSTANCE_t; ///< GPIO instance of the converter control GPIO data type

```

<a href="#startDoc">back</a>

<span id="state-machine"> </span>
### BDBB Converter State Machine

The BDBB Converter State Machine is broken down into two major segments, Default Operating States (opstate) and Sub-States (substate)

#### Default Operating States

* **State_Initialize**
  Initializes the pre-configured chip peripherals enabling the PWM and ADC to start the continuous acquisition of data allowing the analysis of the system status.

* **State_Reset**
  Disables the PWM outputs while keeping the PWM itself running as trigger generator of the ADC module, and clears dynamic control parameters such as error and control histories to ensure a
  deterministic state at startup.

* **State_Standby**
  Waits for all status bits to be set, fault and suspend bits to be cleared and the GO bit to be set.

* **RampUp State Sequence**
  The Ramp-Up state sequence splits up into converter-specific startup sub-states, managing the specific startup procedure of a synchronous or asynchronous single- or multiphase BDBB converter topology. This topology-specific startup procedure also includes the startup timing specified by the `BDBB_CONVERTER_STARTUP_t` data structure parameters.

* **State_Online**
  Once the start procedure has been completed successfully, the converter state machine drops into constant regulation mode, where it will remain until the converter is shut-down or suspended. During constant regulation the control reference is monitored. If the value of the control reference is changed, the state machine will tune into the new reference by generating a defined defined transition ramp using the ramp slope specified in the startup voltage ramp timing.

* **State_Error**
  Should any of the states ever return an error, the converter will be shut down and the state machine will fall back into an error state, from which is will recover automatically with the fault_active flag (bit <5> of the status word) being set.


#### Topology-Specific Startup Sub-States

The BDBB converter startup sub-state machine manages the power topology-specific startup procedure of a BDBB converter, tailored to its configuration, control and operating mode. It also incorporates the user definition of the startup timing specified in the `BDBB_CONVERTER_STARTUP_t` data structure.

* **SubState_PowerOnDelay**
  Waiting period while the converter is kept in a disabled state until the Power-On Delay (POD) specified in the `BDBB_CONVERTER_STARTUP_t` data structure has expired.

* **SubState_PrepareVRampUp**
  In this state the most recent voltage ratio between the inductor terminals is measured and the expected duty ratio at the startup frequency is calculated to ensure a smooth startup without accidental reverse current or forward current spikes at startup against a pre-biased output. The estimated startup duty ratio value is then programmed into PWM registers and the control loop histories get pre-charged and the startup control reference is set to match the output voltage.

* **SubState_VRampUp**
  In this state the PWM outputs and control loops are enabled and the converter starts to operate. Due to the ramp-up pre-charge of the previous state, the expected output power is close to zero. With every execution cycle of the state machine the control reference is linearly incremented by the given amount of ticks until the ramp-up reference value matches desired value specified by the user by `v_ref` in the `BDBB_CONVERTER_SETTINGS_t` data structure.

* **SubState_IRampUp**
  This state is only executed in Average Current Mode Control (ACMC) and only if the voltage loop hist an output saturation limit during the execution of the previous voltage ramp-up state. This additional output current ramp-up procedure has been introduced as controlled constant-current ramp-up control when ramping up the output against large capacitive loads or when this BDBB converter device driver is used in a constant current source application.

* **SubState_PowerGoodDelay**
  Waiting period while the converter is kept in a enabled state at constant output regulation until the Power Good Delay (PGD) specified in the `BDBB_CONVERTER_STARTUP_t` data structure has expired.

<a href="#startDoc">back</a>

---
<span id="history"> </span>

##### History

* 03/13/2020 v1.0 (M91406) Initial release
* 11/02/2020 v2.0 (M91406) Modular state machine with startup sub-states
* 12/21/2020 v2.1 (M91406) Added current sense calibration function extensions to state machine
* 02/03/2021 v2.2 (M91406) Modified voltage reference management during startup, ensuring user setting is always loaded correctly before voltage ramp-up
* 02/22/2021 v2.3 (M91406) Control loop anti-windup limiter minimum and maximum are both set as unsigned integer allowing individual singed/unsigned operations in assembler control routine
* 03/01/2021 v2.4 (M91406) Added support for alternate ADC input of dedicated ADC cores of dsPIC33CK/CH
* 05/22/2022 v3.0 (M91406) Bdbb converter data structure data fields upgrade switching to name labels to combined capital notions, added auto synchronous rectifier control, optimized shut-down performance and restructured state function distribution across files
* 08/07/20223 v3.2 (M91406) Bdbb converter data structure data fields upgrade introducing new device abstraction layer device pin object class, simplifying the BDBB data object and its configuration

---
© 2023, Microchip Technology Inc.
