/**
 * @file    svc_bdbb_functions.c
 * @brief   Group of auxiliary functions dedicated to bidirectional buck-boost converter configurations
 * @author  M91406
 * @date    05/11/2022
 */

#include "../IBdbbConverter.h"      // include bdbb converter interface header file
#include "svc_bdbb_pcontrol_pwm.h"  // include bdbb converter PWM driver header file
#include "svc_bdbb_loop_control.h"  // include bdbb converter control loop driver header file

/***********************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Determines the expected, ideal, no-load duty ratio by looking at most recent input/output voltage levels
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @param   highVoltage  Power converter high voltage terminal value of type 16-bit unsigned integer
 * @param   lowVoltage   Power converter low voltage terminal value of type 16-bit unsigned integer
 * @return  expected duty ratio of type unsigned integer
 * 
 * @details
 *  This function calculates the ideal on-time in PWM ticks by looking
 *  at the most recent input and output voltage, considering the most recent 
 *  period value setting of the PWM logic. Input and output voltage values
 *  must represent the most recent voltage levels without signal offsets.
 *  The signal scaling settings specified in the feedback configuration 
 *  is used to normalize the values before computing the ideal duty ratio.
 *
 **********************************************************************************/
uint16_t BdbbControl_GetDutyRatio(void* handle, int16_t highVoltage, int16_t lowVoltage)
{
    uint16_t _retval=0;    // 16-bit return value
    int32_t _dc=0;         // PWM duty cycle
    int32_t _period=0;     // PWM Period value buffer

    // Cast void pointer to power supply data structure
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Protect against negative duty cycle results
    if (lowVoltage > highVoltage) { lowVoltage = highVoltage; }
    
    // If converter is in asynchronous mode, always return minimum duty cycle
    if ((false == pcInstance->SwitchNode[0].SyncDrive) || 
        ((true == pcInstance->SwitchNode[0].SyncDrive) && (true == pcInstance->Properties.AsyncMode))
       )
    { return(pcInstance->SwitchNode[0].DutyRatioMinimum); }
    
    // Capture most recent parameters
    _period = (int32_t)pcInstance->SwitchNode[0].Period; // capture switching period
    
    int32_t _vlv = __builtin_mulss(lowVoltage, pcInstance->Feedback.AdVLowVoltage.Scaling.Factor);
    int32_t _vhv = __builtin_mulss(highVoltage, pcInstance->Feedback.AdVHighVoltage.Scaling.Factor);

    int16_t _vlv16 = (int16_t)(_vlv >> (16 - pcInstance->Feedback.AdVLowVoltage.Scaling.Scaler));
    int16_t _vhv16 = (int16_t)(_vhv >> (16 - pcInstance->Feedback.AdVHighVoltage.Scaling.Scaler));
    
    // if high-voltage terminal value is greater zero, accept it for duty cycle calculation
    if(_vhv16 > 0)
    {
        _dc = __builtin_muluu((uint16_t)_vlv16, (uint16_t)_period);
        _dc = __builtin_divud((uint32_t)_dc, (uint16_t)_vhv16);
    }
    // or set specified minimum duty cycle
    else
    { _dc = (uint16_t)pcInstance->SwitchNode[0].DutyRatioMinimum; }

    // IF result is out of 16-bit number range, return error
    if (_dc > 0xFFFF) { return(0); }
    
    // In up-stream-mode, calculate duty cycle in boost mode
    if (pcInstance->Properties.ConversionMode)
    { _dc = (uint16_t)_period - (uint16_t)_dc; }

    // Clamp duty ratio result to PWM duty ratio range
    if (_dc > pcInstance->SwitchNode[0].DutyRatioMaximum)
    { _dc = pcInstance->SwitchNode[0].DutyRatioMaximum; }
    else if (_dc < pcInstance->SwitchNode[0].DutyRatioMinimum)
    { _dc = pcInstance->SwitchNode[0].DutyRatioMinimum; }
    
    _retval = (uint16_t)_dc; // Convert result to 16-bit
    
    return(_retval);
}

/***********************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Determines the expected, nominal duty ratio by looking at volt-seconds under nominal operating conditions
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  expected duty ratio of type unsigned integer
 * 
 * @details
 *  This function calculates the ideal on-time in PWM ticks by looking
 *  at the most recent input voltage and the desired output voltage, 
 *  considering the most recent period value setting of the PWM logic.
 *
 **********************************************************************************/
uint16_t BdbbControl_GetOpenLoopDutyRatioStep(IBdbbConverter_t *pcInstance)
{
    uint16_t _retval=0;    // 16-bit return value
    uint32_t _dv=0;        // output voltage ramp range data buffer
    uint32_t _vstep=0;     // output voltage ramp step data buffer
    uint16_t _nom_dc=0;    // Nominal Duty Cycle
    uint16_t _start_dc=0;  // Startup Duty Cycle
    uint32_t _ol_step=0;   // Control output step size in open loop

    // Capture most recent parameters
    _start_dc = BdbbControl_GetDutyRatio(pcInstance, pcInstance->Data.VHighVoltage, pcInstance->Data.VLowVoltage);
    
    if (false == pcInstance->Properties.ConversionMode)
    { _nom_dc = BdbbControl_GetDutyRatio(pcInstance, pcInstance->Data.VHighVoltage, pcInstance->Properties.LVReference); }
    else
    { _nom_dc = BdbbControl_GetDutyRatio(pcInstance, pcInstance->Properties.HVReference, pcInstance->Data.VLowVoltage); }
    
    // Calculate desired output voltage step range per ramp-up step to get from 
    // most recent output voltage to common mode voltage minimum level
    if ((false == pcInstance->Properties.ConversionMode) && (pcInstance->Startup.OpenLoopStart.CutOffVoltage > pcInstance->Data.VLowVoltage))
    {
        _dv = (pcInstance->Startup.OpenLoopStart.CutOffVoltage - pcInstance->Data.VLowVoltage);
        if (_dv < 0) { _dv = 0; }
    }
    else 
    { _dv = 0; }

    _vstep = __builtin_divud((uint16_t)(_dv), pcInstance->Startup.LVRamp.StepSize); 
    if (_vstep == 0) { _vstep = 1; }

    // Calculate estimated open-loop control output range to get from most recent
    // control output to common mode voltage minimum level
    _ol_step = (_nom_dc - _start_dc);
    if(_ol_step <= 0) { _ol_step = 1; }
    _ol_step = __builtin_divud((uint16_t)(_ol_step), (uint16_t)_vstep); 
    _ol_step >>= 1;
    if(_ol_step > _dv) { _ol_step = _dv; }
    if(_ol_step <= 0) { _ol_step = 1; }
    
    // Limit return value
    _retval = (uint16_t)_ol_step;

    return(_retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Gets the most recent output voltage value of the recently selected converter port
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  output voltage ADC value of type unsigned integer
 *********************************************************************************/
int16_t BdbbControl_GetActiveOutputVoltage(IBdbbConverter_t* pcInstance)
{
    int16_t portVal=0;

    // Get most recent output voltage value
    if(false == pcInstance->Properties.ConversionMode)
    { portVal = pcInstance->Data.VLowVoltage; }
    else
    { portVal = pcInstance->Data.VHighVoltage; }
    
    return(portVal);  // return value of voltage level
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Gets the nominal output voltage value of the most recent converter port
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  output voltage ADC value of type unsigned integer 
 *********************************************************************************/
int16_t BdbbControl_GetNominalOutputVoltage(IBdbbConverter_t* pcInstance)
{
    int16_t portVal=0;
    
    // Get nominal output voltage value
    if(false == pcInstance->Properties.ConversionMode)
    { portVal = pcInstance->Feedback.AdVLowVoltage.Range.Nominal; }
    else
    { portVal = pcInstance->Feedback.AdVHighVoltage.Range.Nominal; }
    
    return(portVal);  // return value of voltage level
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Gets the average value of the phase current across all phases
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int16_t BdbbControl_GetActivePhaseCurrent(IBdbbConverter_t *pcInstance)
{
    int16_t portVal=0;

    // Capture average of most recent phase current
    portVal = __builtin_divud(
                (uint32_t)pcInstance->Data.IAverage, 
                (uint16_t)pcInstance->Properties.PhaseCount
            );

    return(portVal);  // return value of current level
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   This function gets the active user reference value from the power supply data object
 * @param   pcInstance  Pointer to a Buck Converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
uint16_t BdbbControl_GetUserVoltageReference(IBdbbConverter_t *pcInstance) 
{
    // decrement reference until new reference level is reached
    if(false == pcInstance->Properties.ConversionMode)
    { return(pcInstance->Properties.LVReference); }
    else
    { return(pcInstance->Properties.HVReference); }
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Loads the most recent, user-defined default values of high-and low-voltage references into the control engine
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbControl_UpdateUserReferences(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Update active reference property values
    pcInstance->Control.VLoop.Properties.Reference = pcInstance->Properties.LVReference;
    pcInstance->Control.VLoop.Properties.AltReference = pcInstance->Properties.HVReference;
    
    return(retval);  // return success/failure
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Sets the control loop reference pointer to the startup voltage ramp reference buffer
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbControl_SetVRampReferencePointers(IBdbbConverter_t *pcInstance)
{
    int retval = 1;
    uint16_t _rampStop=0;
    volatile BDBB_REFERENCE_RAMP_HANDLER_t* _refRamp;

    // Set reference pointer of control loop to startup reference buffer
    if (false == pcInstance->Properties.ConversionMode)
    {
        _refRamp = &pcInstance->Startup.LVRamp;
        _rampStop = pcInstance->Properties.LVReference;
    }
    else
    {
        _refRamp = &pcInstance->Startup.HVRamp;
        _rampStop = pcInstance->Properties.HVReference;
    }
    
    // Clear/reset process counter and buffer
    pcInstance->Startup.LVRamp.Counter = 0;
    pcInstance->Startup.HVRamp.Counter = 0;
    
    // Set reference pointer of control loop to startup reference buffer
    BdbbLoopControl_SetLoopReferencePointer(pcInstance->Control.VLoop.Controller, 
                (uint16_t*)&_refRamp->refBuffer);
        
    _refRamp->preDelayBuffer = 0;
    _refRamp->refBuffer = BdbbControl_GetActiveOutputVoltage(pcInstance);
    _refRamp->Start = _refRamp->refBuffer;
    _refRamp->Stop = _rampStop;

    // Sanity-Check ramp increment values specified by user
    if (_refRamp->StepSize == 0) { _refRamp->StepSize = 1; }
    
    return(retval);  // return success/failure
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Sets the control reference pointers to the default reference buffers
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbControl_SetDefaultReference(IBdbbConverter_t *pcInstance)
{
    int retval = 1;
    
    // Reconnect API reference to controller
    if (false == pcInstance->Properties.ConversionMode)
    {
        retval &= BdbbLoopControl_SetLoopReferencePointer(
            pcInstance->Control.VLoop.Controller,
            (uint16_t*)&pcInstance->Control.VLoop.Properties.Reference);
    }
    else
    {
        retval &= BdbbLoopControl_SetLoopReferencePointer(
            pcInstance->Control.VLoop.Controller,
            (uint16_t*)&pcInstance->Control.VLoop.Properties.AltReference);
    }
    
    return(retval);  // return success/failure
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-control-functions
 * @brief   Increments the most recent control output by the given step size while loop controllers are disabled
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbControl_IncrementOpenLoopOutput(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Open Loop Startup Variables
    uint32_t _olOutput=0; // open loop startup output value auxiliary variable
    uint16_t _minLimit=0;  // buffer variable capturing most recent maximum control loop output limit
    uint16_t _maxLimit=0;  // buffer variable capturing most recent maximum control loop output limit
    int16_t  _phaseCurrent=0; // buffer variable capturing most recent average phase current value
    volatile BDBB_LOOP_SETTINGS_t* _activeLoopSettings; // Pointer to control loop settings data object 
    volatile BDBB_REFERENCE_RAMP_HANDLER_t* _refRamp; // Pointer to startup reference control data object

    // Keep control loops disabled
    if(BdbbLoopControl_IsOn(pcInstance))
    { retval &= BdbbLoopControl_Off(pcInstance); }

    // Set reference pointer of control loop to startup reference buffer
    if (false == pcInstance->Properties.ConversionMode)
    { _refRamp = &pcInstance->Startup.LVRamp; }
    else
    { _refRamp = &pcInstance->Startup.HVRamp; }
    
    // In open loop ramp-up, the control output across all phases is incremented
    // with the converter output being monitored while the actual feedback loops 
    // are kept disabled
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // Capture active control loop data object for this phase
        if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
        { _activeLoopSettings = &pcInstance->Control.ILoop[i]; }
        else if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_VMC)
        { _activeLoopSettings = &pcInstance->Control.VLoop; }
        else
        { return(0); }

        // Reference tracking: Update voltage loop reference to most recent voltage level
        _refRamp->refBuffer = BdbbControl_GetActiveOutputVoltage(pcInstance);
        _refRamp->refBuffer += _refRamp->StepSize;

        // In ACMC, continuously update voltage loop output history with tracked phase current
        if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
        {
            // Pre-charge voltage loop control history with most recent output
            // current to ensure smooth transition into regulated control
            _phaseCurrent = BdbbControl_GetActivePhaseCurrent(pcInstance);
            pcInstance->Control.VLoop.Methods.Precharge(
                        pcInstance->Control.VLoop.Controller, 0, _phaseCurrent
                    );
        }
        
        // Get most recent control output value from control loop target pointer
        _olOutput = BdbbLoopControl_GetLoopOutput(_activeLoopSettings->Controller);
        _minLimit = BdbbLoopControl_GetLoopMinLimit(_activeLoopSettings->Controller);
        _maxLimit = BdbbLoopControl_GetLoopMaxLimit(_activeLoopSettings->Controller);
        
        // Increment value
        _olOutput += pcInstance->Startup.OpenLoopStart.StepSize;

        // Clamp value to user-defined min/max thresholds
        if(_olOutput < _minLimit) { _olOutput = _minLimit; }
        else if(_olOutput > _maxLimit) { _olOutput = _maxLimit; }
        else { /* continue */ }

        // Write value back to loop output target memory address
        BdbbLoopControl_SetLoopOutput(_activeLoopSettings->Controller, (uint16_t)_olOutput);
        
        // Update control history of target controller
        _activeLoopSettings->Methods.Precharge((void*)_activeLoopSettings->Controller, 0, (uint16_t)_olOutput);
    }

    // Enforce PWM timing registers to update
    retval &= BdbbPWM_Update(pcInstance); 
    
    return(retval);   // return success/failure
    
}

// ______________________________________
// end of file
