/*
 * File:   svc_bdbb_loop_config.c
 * Author: M91406
 *
 * Created on February 17, 2023, 12:19 PM
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "../IBdbbPControl.h" // include power converter peripheral control
#include "../IBdbbConverter.h" // include power converter data object declarations

#include "svc_bdbb_loop_control.h" // include power converter control loop manipulation interface header file
#include "svc_bdbb_iloop_mirror.h" // include phase multiplexer iLoop_Mirror

// Private functions
static int BdbbLoopConfig_VLoopInitialize(IBdbbConverter_t *pcInstance);
static int BdbbLoopConfig_ILoopInitialize(IBdbbConverter_t *pcInstance);

/*********************************************************************************
 * @ingroup lib-layer-bdbb-loop-config
 * @brief   Maps power supply specific parameters to the NPNZ control loop data object
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
int BdbbLoopConfig_ControllerInitialize(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Call initialization of voltage loop controller
    // Voltage loop initialization must be called after current loop initialization
    // to allow correct pointer sharing in data interfaces.
    retval &= BdbbLoopConfig_VLoopInitialize(pcInstance);

    // In average current mode control, call initialization of 
    // primary and secondary current loop controllers
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        retval &= BdbbLoopConfig_ILoopInitialize(pcInstance);
    }
    
    return(retval);

}

/**********************************************************************************/
/**********************************************************************************/
/**********************************************************************************/

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-config-private
 * @brief   This function initializes the voltage feedback loop object
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function initializes the voltage loop controller of the power converter
 *  using the user configuration entered in the switch-node and feedback parameter
 *  data objects of the power converter data object of type \ref IBdbbConverter_t.
 *********************************************************************************/
static int BdbbLoopConfig_VLoopInitialize(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

   // ~~~ VOLTAGE LOOP CONFIGURATION ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    // Initialize Default Loop Configuration
    volatile BDBB_LOOP_PARAMETER_BUFFER_t* props = &pcInstance->Control.VLoop.Properties;

    props->Reference = pcInstance->Feedback.AdVLowVoltage.Range.Nominal;
    props->AltReference  = pcInstance->Feedback.AdVHighVoltage.Range.Nominal;
    
    if ((pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) || 
        (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC)) 
    {
        props->Minimum = pcInstance->Feedback.AdISense[0].Range.Minimum; // ISNS_MIN
        props->Maximum = pcInstance->Feedback.AdISense[0].Range.Maximum; // ISNS_MAX
        props->AltMinimum = pcInstance->Feedback.AdISense[0].Range.Minimum;
        props->AltMaximum = pcInstance->Feedback.AdISense[0].Range.Maximum;
    }
    else if ((pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_VMC))
    {
        props->Minimum = pcInstance->SwitchNode[0].DutyRatioMinimum;
        props->Maximum = pcInstance->SwitchNode[0].DutyRatioMaximum;
        props->AltMinimum = pcInstance->SwitchNode[0].DutyRatioMinimum;
        props->AltMaximum = pcInstance->SwitchNode[0].DutyRatioMaximum;
        #if (BDBB_MPHASE_COUNT > 1)
        props->AltMinimum = pcInstance->SwitchNode[1].DutyRatioMinimum;
        props->AltMaximum = pcInstance->SwitchNode[1].DutyRatioMaximum;
        #endif
    }
    
    // Configure Voltage Loop Controller Object
    pcInstance->Control.VLoop.Methods.Initialize((void*)pcInstance->Control.VLoop.Controller);   // Call Initialization Routine setting histories and scaling
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Configure controller input ports
    
    // Primary source points to low-voltage port being output in step-down/buck mode
    volatile NPNZ_PORTS_t* ports = &pcInstance->Control.VLoop.Controller->Ports;
    volatile P33C_ADCAN_BUFFER_INSTANCE_t* adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdVLowVoltage.Input.AdcInput);
    ports->Source.handle = (uint16_t*)&(adc->ADCBUFx.value); // Output Voltage is Common Source
    ports->Source.Offset = pcInstance->Feedback.AdVLowVoltage.Scaling.Offset; // Output Voltage feedback signal offset 
    ports->Source.NormScaler = pcInstance->Feedback.AdVLowVoltage.Scaling.Scaler; // Output voltage normalization factor bit-shift scaler 
    ports->Source.NormFactor = pcInstance->Feedback.AdVLowVoltage.Scaling.Factor; // Output voltage normalization factor fractional
    
    // Secondary source points to high-voltage port being output in step-up/boost mode
    adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdVHighVoltage.Input.AdcInput);
    ports->AltSource.handle = (uint16_t*)&(adc->ADCBUFx.value); // Output Voltage is Alternate Source
    ports->AltSource.Offset = pcInstance->Feedback.AdVHighVoltage.Scaling.Offset;; // Input Voltage feedback signal offset 
    ports->AltSource.NormScaler = pcInstance->Feedback.AdVHighVoltage.Scaling.Scaler; // Input voltage normalization factor bit-shift scaler 
    ports->AltSource.NormFactor = pcInstance->Feedback.AdVHighVoltage.Scaling.Factor; // Input voltage normalization factor fractional

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Configure controller reference ports

    // Primary reference points to low-voltage output reference in step-down/buck mode
    ports->ControlReference.handle = (uint16_t*)&pcInstance->Control.VLoop.Properties.Reference; // Set pointer to Reference
    ports->ControlReference.Offset = pcInstance->Feedback.AdVLowVoltage.Scaling.Offset; // Static primary control reference value offset
    ports->ControlReference.NormScaler = pcInstance->Feedback.AdVLowVoltage.Scaling.Scaler;  // Static primary control reference normalization factor bit-shift scaler
    ports->ControlReference.NormFactor = pcInstance->Feedback.AdVLowVoltage.Scaling.Factor;  // Static primary control reference normalization factor fractional 
    
    // Secondary reference points to low-voltage output reference in step-down/buck mode
    ports->AltControlReference.handle = (uint16_t*)&pcInstance->Control.VLoop.Properties.AltReference; // Set pointer to Reference
    ports->AltControlReference.Offset = pcInstance->Feedback.AdVHighVoltage.Scaling.Offset; // Static primary control reference value offset
    ports->AltControlReference.NormScaler = pcInstance->Feedback.AdVHighVoltage.Scaling.Scaler;  // Static primary control reference normalization factor bit-shift scaler
    ports->AltControlReference.NormFactor = pcInstance->Feedback.AdVHighVoltage.Scaling.Factor;  // Static primary control reference normalization factor fractional 
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Configure controller output ports
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) 
    {
        ports->Target.handle = (uint16_t*)&pcInstance->Control.ILoop[0].Properties.Reference; // Current loop reference is Control Target
        ports->Target.Offset = 0; // Static primary output value offset
        ports->Target.NormScaler = pcInstance->Feedback.AdISense[0].Scaling.Scaler; // Primary control output normalization factor bit-shift scaler 
        ports->Target.NormFactor = pcInstance->Feedback.AdISense[0].Scaling.Factor; // Primary control output normalization factor fractional 

        ports->AltTarget.handle = (uint16_t*)&pcInstance->Control.ILoop[0].Properties.AltReference;; // No alternate target used
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = pcInstance->Feedback.AdISense[0].Scaling.Scaler; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = pcInstance->Feedback.AdISense[0].Scaling.Factor; // Secondary control output normalization factor fractional 
    }
    else if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_VMC) 
    {
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[0].PwmInstance;
        ports->Target.handle = (uint16_t*)&(pg->PGxDC.value); // Voltage loop Primary Control Output Destination
        ports->Target.Offset = 0; // Static primary output value offset
        ports->Target.NormScaler = 0; // Primary control output normalization factor bit-shift scaler 
        ports->Target.NormFactor = 0x7FFF; // Primary control output normalization factor fractional 

        #if (BDBB_MPHASE_COUNT > 1)
        pg = pcInstance->SwitchNode[1].PwmInstance;
        ports->AltTarget.handle = (uint16_t*)&(pg->PGxDC.value); // Voltage loop Secondary Control Output Destination
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = 0; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = 0x7FFF; // Secondary control output normalization factor fractional 
        #else
        ports->AltTarget.handle = NULL; // no Secondary Control Output Destination available
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = 0; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = 0x7FFF; // Secondary control output normalization factor fractional 
        #endif
    }
    else if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC) 
    {
        volatile P33C_DAC_INSTANCE_t* dac = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[0].SlopeCompensation.DacInstance);
        ports->Target.handle = (uint16_t*)&(dac->DACxDATH.value); // Voltage loop Primary Control Output Destination
        ports->Target.Offset = 0; // Static primary output value offset
        ports->Target.NormScaler = 0; // Primary control output normalization factor bit-shift scaler 
        ports->Target.NormFactor = 0x7FFF; // Primary control output normalization factor fractional 

        #if (BDBB_MPHASE_COUNT > 1)
        dac = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[1].SlopeCompensation.DacInstance);
        ports->AltTarget.handle = (uint16_t*)&(dac->DACxDATH.value); // Voltage loop Secondary Control Output Destination
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = 0; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = 0x7FFF; // Secondary control output normalization factor fractional 
        #else
        ports->AltTarget.handle = NULL; // no Secondary Control Output Destination available
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = 0; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = 0x7FFF; // Secondary control output normalization factor fractional 
        #endif
    }
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Data Input/Output Limit Configuration
    volatile NPNZ_LIMITS_t* limits = &pcInstance->Control.VLoop.Controller->Limits;
    limits->MinOutput = pcInstance->Control.VLoop.Properties.Minimum;
    limits->MaxOutput = pcInstance->Control.VLoop.Properties.Maximum;
    limits->AltMinOutput = pcInstance->Control.VLoop.Properties.AltMinimum;
    limits->AltMaxOutput = pcInstance->Control.VLoop.Properties.AltMaximum;

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ADC Trigger Control Configuration (ADC trigger controlled by current loop)
    volatile NPNZ_ADC_TRGCTRL_t* trgctrl = &pcInstance->Control.VLoop.Controller->AdcTriggerControl;
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) 
    {
        trgctrl->ptrAdcTriggerARegister = NULL;
        trgctrl->AdcTriggerAOffset = 0;
        trgctrl->ptrAdcTriggerBRegister = NULL;
        trgctrl->AdcTriggerBOffset = 0; 
    }
    else if ((pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_VMC) || 
             (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC)) 
    {
        trgctrl->ptrAdcTriggerARegister = pcInstance->Feedback.AdVLowVoltage.Trigger.ptrTriggerRegister;
        trgctrl->AdcTriggerAOffset = pcInstance->Feedback.AdVLowVoltage.Trigger.TriggerDelay;
        trgctrl->ptrAdcTriggerBRegister = pcInstance->Feedback.AdISense[0].Trigger.ptrTriggerRegister;
        trgctrl->AdcTriggerBOffset = pcInstance->Feedback.AdISense[0].Trigger.TriggerDelay;; 
    }
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // ToDo: Data Provider must be swapped with I/Os (review and fix))
    // Note: Data Provider Configuration (buck configuration is default at this point. 
    //       Destination is swapped to Data.VHighVolatge later by state machine)
    volatile NPNZ_DATA_PROVIDERS_t* dprov = &pcInstance->Control.VLoop.Controller->DataProviders;
    dprov->ptrDProvControlInput = &pcInstance->Data.ControlInput; 
    dprov->ptrDProvControlInputCompensated = (int16_t*)&pcInstance->Data.VLowVoltage;  
    dprov->ptrDProvControlError = &pcInstance->Data.ControlError; 
    dprov->ptrDProvControlOutput = (int16_t*)&pcInstance->Data.ControlOutput;
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    /* User Extension Function Configuration
     * 
     * PowerSmart DCLD allows users to create and call user extension 
     * functions from specific locations of the main control loop to 
     * cover design-specific requirements and features which are not
     * supported by the main controller by default.
     * 
     * Control Loop User Extension Declaration Example:
     *    bdbb.v_loop.controller->ExtensionHooks.ptrExtHookStartFunction = (uint16_t)&my_function; 
     * 
     * Control Loop User Extension Parameter Declaration Example (optional):
     *    bdbb.v_loop.controller->ExtensionHooks.ExtHookStartFunctionParam = 512;
     * 
     * Please refer to the PowerSmart DCLD User Guide for more details about
     * how to use this feature, its requirements and limitations.
    */
    volatile NPNZ_EXTENSION_HOOKS_t* hooks = &pcInstance->Control.VLoop.Controller->ExtensionHooks;
    hooks->ptrExtHookStartFunction = (uint16_t)NULL;
    hooks->ExtHookStartFunctionParam = 0;
    hooks->ptrExtHookSourceFunction = (uint16_t)NULL;
    hooks->ExtHookSourceFunctionParam = 0;
    hooks->ptrExtHookPreAntiWindupFunction = (uint16_t)NULL;
    hooks->ExtHookPreAntiWindupFunctionParam = 0;
    hooks->ptrExtHookPreTargetWriteFunction = (uint16_t)NULL;
    hooks->ExtHookPreTargetWriteFunctionParam = 0;
    hooks->ptrExtHookPreAdcUpdateFunction = (uint16_t)NULL;
    hooks->ExtHookPreAdcUpdateFunctionParam = 0;
    hooks->ptrExtHookPostWritebackFunction = (uint16_t)NULL;
    hooks->ExtHookPostWritebackFunctionParam = 0;
    hooks->ptrExtHookEndOfLoopFunction = (uint16_t)NULL;
    hooks->ExtHookEndOfLoopFunctionParam = 0;
    
    // ToDo: URGENT: pointer to current loop is not initialized at this point 
    hooks->ptrExtHookExitFunction = (uint16_t)pcInstance->Control.ILoop[0].Methods.Update;
    hooks->ExtHookExitFunctionParam = (uint16_t)pcInstance->Control.ILoop[0].Controller;
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Initialize Advanced Control Settings (not used in this code example)
    volatile NPNZ_GAIN_CONTROL_t* gainctrl = &pcInstance->Control.VLoop.Controller->GainControl;
    gainctrl->AgcFactor = 0x7FFF; // Adaptive Gain Control factor fractional
    gainctrl->AgcScaler = 0x0000; // Adaptive Gain Control factor bit-shift scaler
    gainctrl->AgcMedian = 0x0000; // Q15 number representing normalized Nominal Operating Point

    // Custom Advanced Control Settings
    volatile NPNZ_USER_DATA_BUFFER_t* buffers = &pcInstance->Control.VLoop.Controller->Advanced;
    buffers->usrParam0 = 0; // No additional advanced control options used
    buffers->usrParam1 = 0; // No additional advanced control options used
    buffers->usrParam2 = 0; // No additional advanced control options used
    buffers->usrParam3 = 0; // No additional advanced control options used
    buffers->usrParam4 = 0; // No additional advanced control options used
    buffers->usrParam5 = 0; // No additional advanced control options used
    buffers->usrParam6 = 0; // No additional advanced control options used
    buffers->usrParam7 = 0; // No additional advanced control options used
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Reset Controller Status
    pcInstance->Control.VLoop.Controller->Status.bits.Enable                = false; // Keep controller disabled
    pcInstance->Control.VLoop.Controller->Status.bits.AgcEnable             = false; // Disable Adaptive Gain Modulation by default
    pcInstance->Control.VLoop.Controller->Status.bits.InvertInput           = pcInstance->Feedback.AdVLowVoltage.Scaling.Polarity; // Set low-voltage port voltage feedback signal polarity in buck mode
    pcInstance->Control.VLoop.Controller->Status.bits.MirrorTarget          = false; // Use single control TARGET (do not copy to other targets)
    pcInstance->Control.VLoop.Controller->Status.bits.SwapSource            = false; // Use PRIMARY SOURCE as control input
    pcInstance->Control.VLoop.Controller->Status.bits.SwapTarget            = false; // Use PRIMARY TARGET as control output
    pcInstance->Control.VLoop.Controller->Status.bits.SwapReference         = false; // Use PRIMARY REFERENCE as control reference

    pcInstance->Control.VLoop.Controller->Status.bits.LowerSaturationEvent  = false; // Clear Anti-Windup Minimum Status bit
    pcInstance->Control.VLoop.Controller->Status.bits.UpperSaturationEvent  = false; // Clear Anti-Windup Minimum Status bits
    
    
    // ~~~ VOLTAGE LOOP CONFIGURATION END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    

    return(retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-config-private
 * @brief   This function initializes the main current feedback loop object
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function initializes the main current loop controller of the power converter
 *  using the user configuration entered in the switch-node and feedback parameter
 *  data objects of the power converter data object of type \ref IBdbbConverter_t.
 *********************************************************************************/
static int BdbbLoopConfig_ILoopInitialize(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // ~~~ CURRENT LOOP CONFIGURATION ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Nop();
    Nop();
    Nop();
    
    // Set advanced options based on user configuration
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
    {
        // Initialize Default Loop Configuration
        // pcInstance->Control.ILoop[i].Properties.FeedbackOffset = ISNS1_OFFSET;
        pcInstance->Control.ILoop[i].Properties.Reference = pcInstance->Feedback.AdISense[i].Range.Nominal;
        pcInstance->Control.ILoop[i].Properties.Minimum = pcInstance->SwitchNode[i].DutyRatioMinimum;
        pcInstance->Control.ILoop[i].Properties.Maximum = pcInstance->SwitchNode[i].DutyRatioMaximum;

        // Configure controller input ports
        volatile NPNZ_PORTS_t* ports = &pcInstance->Control.ILoop[i].Controller->Ports;
        volatile P33C_ADCAN_BUFFER_INSTANCE_t* adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdISense[i].Input.AdcInput);
        ports->Source.handle = (uint16_t*)&(adc->ADCBUFx.value); // Inductor Current is Common Source
        ports->Source.Offset = pcInstance->Feedback.AdISense[i].Scaling.Offset; // Inductor Current feedback signal offset  
        ports->Source.NormScaler = pcInstance->Feedback.AdISense[i].Scaling.Scaler; // Inductor Current normalization factor bit-shift scaler  
        ports->Source.NormFactor = pcInstance->Feedback.AdISense[i].Scaling.Factor; // Inductor Current normalization factor fractional

        ports->AltSource.handle = NULL; // // No alternate source used 
        ports->AltSource.Offset = 0; // Static secondary input value offset
        ports->AltSource.NormScaler = 0; // Secondary control input normalization factor bit-shift scaler
        ports->AltSource.NormFactor = 0; // Secondary control input normalization factor fractional

        // Configure controller output ports
        volatile P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;

        ports->Target.handle = (uint16_t*)&(pg->PGxDC.value); // Current loop Primary Control Output Destination
        ports->Target.Offset = 0; // Static primary output value offset
        ports->Target.NormScaler = 0; // Primary control output normalization factor bit-shift scaler 
        ports->Target.NormFactor = 0x7FFF; // Primary control output normalization factor fractional 

        ports->AltTarget.handle = NULL; // no Secondary Control Output Destination available
        ports->AltTarget.Offset = 0; // Static secondary output value offset
        ports->AltTarget.NormScaler = 0; // Secondary control output normalization factor bit-shift scaler
        ports->AltTarget.NormFactor = 0x7FFF; // Secondary control output normalization factor fractional 

        // Configure controller control reference ports
        // => Set pointer to SAME Reference for all current controllers (!!!)
        ports->ControlReference.handle = (uint16_t*)&pcInstance->Control.ILoop[0].Properties.Reference; 
        ports->ControlReference.Offset = 0; // Set static offset value
        ports->ControlReference.NormScaler = 0; // Primary control reference normalization factor bit-shift scaler
        ports->ControlReference.NormFactor = 0x7FFF; // Primary control reference normalization factor fractional 

        ports->AltControlReference.handle = NULL; // Set pointer to Alternate Reference
        ports->AltControlReference.Offset = 0; // Set static offset value
        ports->AltControlReference.NormScaler = 0; // Secondary control reference normalization factor bit-shift scaler
        ports->AltControlReference.NormFactor = 0x7FFF; // Secondary control reference normalization factor fractional 

        // Data Input/Output Limit Configuration
        // => Set limits to SAME values for all current controllers (!!!)
        volatile NPNZ_LIMITS_t* limits = &pcInstance->Control.ILoop[i].Controller->Limits;
        limits->MinOutput = pcInstance->Control.ILoop[i].Properties.Minimum;
        limits->MaxOutput = pcInstance->Control.ILoop[i].Properties.Maximum;
        limits->AltMinOutput = pcInstance->Control.ILoop[i].Properties.AltMinimum;
        limits->AltMaxOutput = pcInstance->Control.ILoop[i].Properties.AltMaximum;

        // ADC Trigger Control Configuration 
        volatile NPNZ_ADC_TRGCTRL_t* trgctrl = &pcInstance->Control.ILoop[i].Controller->AdcTriggerControl;

        // Set current loop ADC trigger first => assigned to Loop ADC Trigger A
        trgctrl->ptrAdcTriggerARegister = pcInstance->Feedback.AdISense[i].Trigger.ptrTriggerRegister;
        trgctrl->AdcTriggerAOffset = pcInstance->Feedback.AdISense[i].Trigger.TriggerDelay; 

        if (i==0) // Leading loop also triggers output voltage ADC input
        {

            // Set voltage loop ADC trigger => assigned to Loop ADC Trigger B
            if (0 == pcInstance->Properties.ConversionMode) // Downstream Buck Configuration
            {
                trgctrl->ptrAdcTriggerBRegister = pcInstance->Feedback.AdVLowVoltage.Trigger.ptrTriggerRegister;
                trgctrl->AdcTriggerBOffset = pcInstance->Feedback.AdVLowVoltage.Trigger.TriggerDelay;
            }
            else // Downstream Boost Configuration
            {
                trgctrl->ptrAdcTriggerBRegister = pcInstance->Feedback.AdVHighVoltage.Trigger.ptrTriggerRegister;
                trgctrl->AdcTriggerBOffset = pcInstance->Feedback.AdVHighVoltage.Trigger.TriggerDelay;
            }
            
        }
        else
        {
            trgctrl->ptrAdcTriggerBRegister = NULL;
            trgctrl->AdcTriggerBOffset = 0; 
        }

        // Data Provider Configuration
        volatile NPNZ_DATA_PROVIDERS_t* dprov = &pcInstance->Control.ILoop[i].Controller->DataProviders;
        dprov->ptrDProvControlInput = &pcInstance->Data.ControlInput; 
        dprov->ptrDProvControlInputCompensated = &pcInstance->Data.ISense[i]; 
        dprov->ptrDProvControlError = &pcInstance->Data.ControlError; 
        dprov->ptrDProvControlOutput = &pcInstance->Data.ControlOutput; 

        // User Extension Function Configuration
        /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         * 
         * PowerSmart DCLD allows users to create and call user extension 
         * functions from specific locations of the main control loop to 
         * cover design-specific requirements and features which are not
         * supported by the main controller by default.
         * 
         * Control Loop User Extension Declaration Example:
         * 
         *  bdbb.ILoop[0].controller->ExtensionHooks.ptrExtHookStartFunction = (uint16_t)&my_function; 
         * 
         * Control Loop User Extension Parameter Declaration Example (optional):
         * 
         *  bdbb.ILoop[0].controller->ExtensionHooks.ExtHookStartFunctionParam = 512;
         * 
         * Please refer to the PowerSmart DCLD User Guide for more details about
         * how to use this feature, its requirements and limitations.
         * 
         * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        */
        volatile NPNZ_EXTENSION_HOOKS_t* hooks = &pcInstance->Control.ILoop[i].Controller->ExtensionHooks;
        hooks->ptrExtHookStartFunction = (uint16_t)NULL;
        hooks->ExtHookStartFunctionParam = 0;
        hooks->ptrExtHookSourceFunction = (uint16_t)NULL;
        hooks->ExtHookSourceFunctionParam = 0;
        hooks->ptrExtHookPreAntiWindupFunction = (uint16_t)NULL;
        hooks->ExtHookPreAntiWindupFunctionParam = 0;
        hooks->ptrExtHookPreTargetWriteFunction = (uint16_t)NULL;
        hooks->ExtHookPreTargetWriteFunctionParam = 0;
        hooks->ptrExtHookPreAdcUpdateFunction = (uint16_t)NULL;
        hooks->ExtHookPreAdcUpdateFunctionParam = 0;
        hooks->ptrExtHookPostWritebackFunction = (uint16_t)NULL;
        hooks->ExtHookPostWritebackFunctionParam = 0;
        hooks->ptrExtHookEndOfLoopFunction = (uint16_t)NULL;
        hooks->ExtHookEndOfLoopFunctionParam = 0;
        hooks->ptrExtHookExitFunction = (uint16_t)NULL;
        hooks->ExtHookExitFunctionParam = 0;

        // Initialize Advanced Control Settings (not used in this code example)
        volatile NPNZ_GAIN_CONTROL_t* gainctrl = &pcInstance->Control.ILoop[i].Controller->GainControl;
        gainctrl->AgcFactor = 0x7FFF; // Adaptive Gain Control factor fractional
        gainctrl->AgcScaler = 0x0000; // Adaptive Gain Control factor bit-shift scaler
        gainctrl->AgcMedian = 0x0000; // Q15 number representing normalized Nominal Operating Point

        // Custom Advanced Control Settings
        volatile NPNZ_USER_DATA_BUFFER_t* buffers = &pcInstance->Control.ILoop[i].Controller->Advanced;
        buffers->usrParam0 = 0; // Store number of multiplexed phases
        buffers->usrParam1 = 0; // No additional advanced control options used
        buffers->usrParam2 = 0; // No additional advanced control options used
        buffers->usrParam3 = 0; // No additional advanced control options used
        buffers->usrParam4 = 0; // No additional advanced control options used
        buffers->usrParam5 = 0; // No additional advanced control options used
        buffers->usrParam6 = 0; // No additional advanced control options used
        buffers->usrParam7 = 0; // No additional advanced control options used

        // Reset Controller Status
        pcInstance->Control.ILoop[i].Controller->Status.bits.Enable         = false; // Keep controller disabled
        pcInstance->Control.ILoop[i].Controller->Status.bits.AgcEnable      = false; // Disable Adaptive Gain Modulation by default
        pcInstance->Control.ILoop[i].Controller->Status.bits.InvertInput    = pcInstance->Feedback.AdISense[i].Scaling.Polarity; // Set default feedback polarity of current sense input signal in buck mode
        pcInstance->Control.ILoop[i].Controller->Status.bits.MirrorTarget   = false; // Use single control TARGET (do not copy to other targets)
        pcInstance->Control.ILoop[i].Controller->Status.bits.SwapSource     = false; // Use PRIMARY SOURCE as control input
        pcInstance->Control.ILoop[i].Controller->Status.bits.SwapTarget     = false; // Use PRIMARY TARGET as control output
        pcInstance->Control.ILoop[i].Controller->Status.bits.SwapReference  = false; // Use PRIMARY REFERENCE as control reference

        pcInstance->Control.ILoop[i].Controller->Status.bits.LowerSaturationEvent = false; // Clear Anti-Windup Minimum Status bit
        pcInstance->Control.ILoop[i].Controller->Status.bits.UpperSaturationEvent = false; // Clear Anti-Windup Minimum Status bits
    
    }
    
    Nop();
    Nop();
    Nop();
    
    // Initialize compensation filter parameters of Current Loop Controller Object(s)
    //  => Call Initialization Routine setting histories and scaling
    //  => Setting up control loop output multiplexing
    pcInstance->Control.ILoop[0].Methods.Initialize(pcInstance->Control.ILoop[0].Controller);   
    
	// ~~~ CURRENT LOOP CONFIGURATION END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    return(retval);    
}

// _____________________________________
// end of file
