/**
 * @file    svc_bdbb_loop_control.c
 * @brief   Bidirectional Buck-Boost converter state machine operating sub-states
 * @author  M91406
 */

#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types

#include "svc_bdbb_loop_control.h" // include power converter control loop manipulation interface header file

#include "../IBdbbPControl.h" // include power converter peripheral control
#include "../IBdbbConverter.h" // include power converter data object declarations

/* PRIVATE FUNCTION CALL PROTOTYPES */

/***********************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Resets the control loop data objects to their default state
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function resets the control loop data objects being executed in
 *  the control interrupt service routines to their default state.
 **********************************************************************************/
int BdbbLoopControl_Reset(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Disable voltage loop controller and reset control loop histories
    pcInstance->Control.VLoop.Controller->Status.bits.Enable = false; // disable voltage control loop
    pcInstance->Control.VLoop.Methods.Reset((void*)pcInstance->Control.VLoop.Controller); // Reset control histories of outer voltage controller
    
    // Restore user presets
    volatile BDBB_LOOP_PARAMETER_BUFFER_t* props = &pcInstance->Control.VLoop.Properties;
    volatile NPNZ_PORTS_t* ports = &pcInstance->Control.VLoop.Controller->Ports;
    volatile NPNZ_LIMITS_t* limits = &pcInstance->Control.VLoop.Controller->Limits;

    props->Reference = pcInstance->Properties.LVReference;
    props->AltReference = pcInstance->Properties.HVReference;
    ports->ControlReference.handle = (uint16_t*)&props->Reference;
    ports->AltControlReference.handle = (uint16_t*)&props->AltReference;
    limits->MinOutput = props->Minimum;
    limits->MaxOutput = props->Maximum;
    limits->AltMinOutput = props->AltMinimum;
    limits->AltMaxOutput = props->AltMaximum;
    
    // Reset controller output values to minimum
    *ports->Target.handle = 
        limits->MinOutput;

    // Set options for alternate target
    if (ports->AltTarget.handle != NULL)
    {
        if (pcInstance->Control.VLoop.Controller->Status.bits.MirrorTarget)
            *ports->AltTarget.handle = 
                limits->MinOutput;
        else 
            *ports->AltTarget.handle = 
                limits->AltMinOutput;
    }
    
    // Disable current loop controller and reset control loop histories
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) 
    {   
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
        {
            // Disable all current control loops and reset control loop histories
            pcInstance->Control.ILoop[i].Controller->Status.bits.Enable = false; 
            pcInstance->Control.ILoop[i].Methods.Reset((void*)pcInstance->Control.ILoop[i].Controller); 
            
            *pcInstance->Control.ILoop[i].Controller->Ports.Target.handle = 
                pcInstance->Control.ILoop[i].Controller->Limits.MinOutput;

            // Set options for alternate target
            if (pcInstance->Control.ILoop[i].Controller->Ports.AltTarget.handle != NULL)
            {
                if (pcInstance->Control.ILoop[i].Controller->Status.bits.MirrorTarget)
                    *pcInstance->Control.ILoop[i].Controller->Ports.AltTarget.handle = 
                        pcInstance->Control.ILoop[i].Controller->Limits.MinOutput;
                else 
                    *pcInstance->Control.ILoop[i].Controller->Ports.AltTarget.handle = 
                        pcInstance->Control.ILoop[i].Controller->Limits.AltMinOutput;
            }
            
        }
    }

    return(retval);
}

/***********************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Turns on the control loop
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function turns on the control loop data objects being executed in
 *  the control interrupt service routines.
 **********************************************************************************/
int BdbbLoopControl_On(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Enable control loops
    pcInstance->Control.VLoop.Controller->Status.bits.Enable = true; // Enable voltage loop
    
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) // In current mode...
    {
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++) // Enable all current loops
        { pcInstance->Control.ILoop[i].Controller->Status.bits.Enable = true; }
    }

    return(retval);
}

/***********************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Turns off the control loop
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function turns off the control loop data objects being executed in
 *  the control interrupt service routines.
 **********************************************************************************/
int BdbbLoopControl_Off(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Disable control loops
    pcInstance->Control.VLoop.Controller->Status.bits.Enable = false; // Disable voltage loop
    
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) // In average current mode... 
    {
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++) // Disable all current loops
        { pcInstance->Control.ILoop[i].Controller->Status.bits.Enable = false; }
    }

    return(retval);
}

/***********************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Returns, if any of the control loops are enabled
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  true        At least one of the control loops is enabled
 * @return  false       All control loops are disabled
 * 
 * @details
 *  This function returns the Enable state of the control loop data object(s)
 *  being called the control interrupt service routine(s). If any of the loops
 *  is enabled, this function returns True, otherwise False.
 **********************************************************************************/
bool BdbbLoopControl_IsOn(IBdbbConverter_t *pcInstance)
{
    // Disable control loops
    bool retval = pcInstance->Control.VLoop.Controller->Status.bits.Enable; // Get Enable state of voltage loop
    
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) // In average current mode... 
    {
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++) // Get Enable state of current loops
        { retval |= pcInstance->Control.ILoop[i].Controller->Status.bits.Enable; }
    }

    return(retval);
}

/***********************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Swaps control input and output of bidirectional control loops
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function swaps the inputs and references of the control loop data objects 
 *  being executed in the control interrupt service routines using declarations
 *  of alternate input ports and reference as well as invert the current feedback
 *  signal of current loops. 
 * 
 * @note
 *  This function does not change the control loop output port and does not change
 *  the switch node output signal target.
 **********************************************************************************/
int BdbbLoopControl_SetControlSwap(IBdbbConverter_t *pcInstance)
{
    int retval = 1;

    // Set voltage control loop conversion direction from HV->LV to LV->HV or vice versa
    pcInstance->Control.VLoop.Controller->Status.bits.SwapSource = pcInstance->Properties.ConversionMode;
    pcInstance->Control.VLoop.Controller->Status.bits.SwapReference = pcInstance->Properties.ConversionMode;
    
    // Invert current feedback signals of all current loop controllers
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
        { 
            pcInstance->Control.ILoop[i].Controller->Status.bits.InvertInput = 
                    (pcInstance->Feedback.AdISense[i].Scaling.Polarity ^ pcInstance->Properties.ConversionMode); 
        }
    }
    
    // ToDo: review this case switch if it is working as intended (not tested yet)
    switch (pcInstance->Properties.ConversionMode)
    {
        case false: // forward buck mode
            pcInstance->Control.VLoop.Controller->DataProviders.ptrDProvControlInputCompensated = (int16_t*)&pcInstance->Data.VLowVoltage; 
            pcInstance->Control.VLoop.Controller->Status.bits.InvertInput = pcInstance->Feedback.AdVLowVoltage.Scaling.Polarity;
            break;
        case true: // reverse boost mode
            pcInstance->Control.VLoop.Controller->DataProviders.ptrDProvControlInputCompensated = (int16_t*)&pcInstance->Data.VHighVoltage; 
            pcInstance->Control.VLoop.Controller->Status.bits.InvertInput = pcInstance->Feedback.AdVHighVoltage.Scaling.Polarity;
            break;
    }
    
    return(retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function returns the value stored in output target memory address
 * @param   loopInstance  Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  Pointer to loop controller output target memory address
 *  
 * @details
 *  This function returns the value stored at the memory address the loop 
 *  controller output pointer is pointing to.
 *
 *********************************************************************************/
uint16_t BdbbLoopControl_GetLoopOutput(volatile NPNZ16b_t* loopInstance)
{
    volatile uint16_t* _ptrTarget=NULL;
    
    // Get memory address stored in the control loop output target data field
    if (!loopInstance->Status.bits.SwapTarget)
        _ptrTarget = loopInstance->Ports.Target.handle;
    else if (loopInstance->Status.bits.SwapTarget)
        _ptrTarget = loopInstance->Ports.AltTarget.handle;

    if (_ptrTarget != NULL)
        return((uint16_t)*_ptrTarget);  // return value from memory address
    else
        return(0); // Return empty value

}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function writes a new value to the output target memory address
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function writes a new value to the the memory address the loop 
 *  controller output pointer is pointing to.
 *
 *********************************************************************************/
int BdbbLoopControl_SetLoopOutput(volatile NPNZ16b_t* loopInstance, uint16_t value)
{
    int retval = 1;

    // Get memory address stored in the control loop output target data field
    if (loopInstance->Status.bits.MirrorTarget)
    {
        if (loopInstance->Ports.Target.handle != NULL)
            *loopInstance->Ports.Target.handle = value;
        if (loopInstance->Ports.AltTarget.handle != NULL)
            *loopInstance->Ports.AltTarget.handle = value;
    }
    else if (true == loopInstance->Status.bits.SwapTarget)
    {
        if (loopInstance->Ports.AltTarget.handle != NULL)
            *loopInstance->Ports.AltTarget.handle = value;
    }
    else if (false == loopInstance->Status.bits.SwapTarget)
    {
        if (loopInstance->Ports.Target.handle != NULL)
            *loopInstance->Ports.Target.handle = value;
    }
    else
        return(0); // Return error
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function pre-charges the last control loop(s) controlling the switch-node outputs
 * @param   pcInstance   Pointer to a power converter data object of type IBdbbConverter_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *   This function pre-charges the last control loop(s) controlling the 
 *   switch-node outputs with the given new value. This pre-charge procedure
 *   includes the control loop(s) output target memory address.
 *********************************************************************************/
int BdbbLoopControl_SetOutputLoopHistory(IBdbbConverter_t *pcInstance, uint16_t value)
{
    int retval = 1;

    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // pre-charge current loop control histories with ideal duty cycle
        pcInstance->Control.ILoop[i].Methods.Precharge(
                    pcInstance->Control.ILoop[i].Controller, 0, value
                );

        // Set initial PWM duty ratio
        retval &= BdbbLoopControl_SetLoopOutput(pcInstance->Control.ILoop[i].Controller, value);
    }    

    return(retval);  
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function reads the loop maximum limit from the target memory address
 * @param   loopInstance  Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int16_t BdbbLoopControl_GetLoopMaxLimit(volatile NPNZ16b_t* loopInstance)
{
    int retval = 1;
    
    if (!loopInstance->Status.bits.SwapTarget)
        retval = loopInstance->Limits.MaxOutput; // Read value from target
    else 
        retval = loopInstance->Limits.AltMaxOutput; // Read value from alternate target
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function writes a new value of the loop maximum limit to the target memory address
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbLoopControl_SetLoopMaxLimit(volatile NPNZ16b_t* loopInstance, uint16_t value)
{
    int retval = 1;
    
    if (!loopInstance->Status.bits.SwapTarget)
        loopInstance->Limits.MaxOutput = value; // Write new value to target
    else 
        loopInstance->Limits.AltMaxOutput = value; // Write new value to alternate target
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function reads the loop minimum limit from the target memory address
 * @param   loopInstance  Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int16_t BdbbLoopControl_GetLoopMinLimit(volatile NPNZ16b_t* loopInstance)
{
    int retval = 1;
    
    if (!loopInstance->Status.bits.SwapTarget)
        retval = loopInstance->Limits.MinOutput; // Read value from target
    else if (loopInstance->Status.bits.SwapTarget)
        retval = loopInstance->Limits.AltMinOutput; // Read value from alternate target
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function writes a new value of the loop minimum limit to the target memory address
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbLoopControl_SetLoopMinLimit(volatile NPNZ16b_t* loopInstance, uint16_t value)
{
    int retval = 1;
    
    if (!loopInstance->Status.bits.SwapTarget)
        loopInstance->Limits.MinOutput = value; // Write new value to target
    else if (loopInstance->Status.bits.SwapTarget)
        loopInstance->Limits.AltMinOutput = value; // Write new value to alternate target
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function returns the value stored in loop reference target memory address
 * @param   loopInstance  Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  Pointer to loop controller reference target memory address
 *********************************************************************************/
uint16_t BdbbLoopControl_GetLoopReference(volatile NPNZ16b_t* loopInstance)
{
    volatile uint16_t* _ptrTarget;

    if (!loopInstance->Status.bits.SwapReference)
        _ptrTarget = loopInstance->Ports.ControlReference.handle; // Read value from target
    else 
        _ptrTarget = loopInstance->Ports.AltControlReference.handle; // Read value from alternate target
    
    if (_ptrTarget != NULL)
        return((uint16_t)*_ptrTarget);  // return value from memory address
    else
        return(0); // Return empty value
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function writes a new value for the loop reference to the target memory address
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbLoopControl_SetLoopReference(volatile NPNZ16b_t* loopInstance, uint16_t value)
{
    int retval = 1;
    volatile uint16_t* _ptrTarget;
    
    if (false == loopInstance->Status.bits.SwapReference)
        _ptrTarget = loopInstance->Ports.ControlReference.handle; // Read value from target
    else 
        _ptrTarget = loopInstance->Ports.AltControlReference.handle; // Read value from alternate target

    if (_ptrTarget != NULL)
        *_ptrTarget = value;  // Write new value to target
    else
        return(0); // Return error
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function sets pointer to the new loop reference value
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   ptrReference Pointer to a data object of type uint16_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbLoopControl_SetLoopReferencePointer(volatile NPNZ16b_t* loopInstance, volatile uint16_t* ptrReference)
{
    int retval = 1;

    if (ptrReference != NULL)
    {
        if (!loopInstance->Status.bits.SwapReference)
            loopInstance->Ports.ControlReference.handle = ptrReference; // Set pointer to new reference value
        else 
            loopInstance->Ports.AltControlReference.handle = ptrReference; // Set pointer to new reference value
    }
    else
        return(0); // Return error
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   This function gets the source channel to be used for loop control
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
uint16_t BdbbLoopControl_GetSource(volatile NPNZ16b_t* loopInstance)
{
    volatile uint16_t* _ptrTarget;
    
    if (!loopInstance->Status.bits.SwapSource)
        _ptrTarget = loopInstance->Ports.Source.handle; // Read value from target
    else 
        _ptrTarget = loopInstance->Ports.AltSource.handle; // Read value from alternate target
    
    if (_ptrTarget != NULL)
        return((uint16_t)*_ptrTarget);  // return value from memory address
    else
        return(0); // Return empty value

}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Gets offset value of the most recent source channel
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int16_t BdbbLoopControl_GetSourceOffset(volatile NPNZ16b_t* loopInstance)
{
    uint16_t offVal=0;
    
    if (!loopInstance->Status.bits.SwapSource)
        offVal = loopInstance->Ports.Source.Offset; // Write value to target
    else 
        offVal = loopInstance->Ports.AltSource.Offset; // Write value to target

    return(offVal);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-loop-control
 * @brief   Writes the offset value to the most recent source channel
 * @param   loopInstance Pointer to a NPNZ16 controller data object of type NPNZ16b_t
 * @param   value        New value to be written to target address of type 16-bit unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbLoopControl_SetSourceOffset(volatile NPNZ16b_t* loopInstance, uint16_t value)
{
    int retval = 1;
    
    if (!loopInstance->Status.bits.SwapSource)
        loopInstance->Ports.Source.Offset = value; // Write value to target
    else 
        loopInstance->Ports.AltSource.Offset = value; // Write value to target

    return(retval);  
    
}

// _____________________________
// end of file
