/**
 * @file    svc_bdbb_pcontrol_adc.c
 * @brief   Bidirectional Buck-Boost Converter peripheral configuration and management functions
 * @author  M91406
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "svc_bdbb_pcontrol_adc.h"
#include "templates/svc_bdbb_ptemp_adc.h"
#include "../IBdbbConverter.h" // include data object interface header file

/* PRIVATE VARIABLES */
/*******************************************************************************
 * @ingroup lib-layer-bdbb-adc-properties-private
 * @brief This variable is use to set the ADC core mask
 *********************************************************************************/
static uint16_t adcore_mask=0;        

/*******************************************************************************
 * @ingroup lib-layer-bdbb-adc-properties-private
 * @brief This variable is use to set the ADC core mask
 *********************************************************************************/
static uint16_t adcore_diff_mask=0;   

/* PRIVATE FUNCTION CALL PROTOTYPES */

// (none)

/*******************************************************************************
 * @ingroup lib-layer-bdbb-adc-functions
 * @brief   Initializes the ADC module by resetting all its special function registers to their recommended default settings
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The ADC initialization covers basic configurations like data format, clock sources and dividers
 *  as well as specific configurations for ADC cores. These settings are general, basic settings
 *  and not related to specific analog inputs. The standard configuration set here sets up the 
 *  ADC module and ADC cores for maximum performance.
 *
 *********************************************************************************/
int BdbbADC_ModuleInitialize(void) 
{
    int retval = 1;

    // Turn on power to ADC module
    retval &= p33c_AdcModule_PowerOn();
    
    // Set default configuration of ADC module for maximum ADC performance
    retval &= p33c_AdcModule_SetConfig(adcModuleConfigDefault);

    // Set default configuration of dedicated ADC cores
    for (int i=0; i<p33c_AdcCoreInstances; i++)
    { retval &= p33c_AdcCoreInstance_SetConfig(i, adcDedicatedCoreConfigDefault); }
    
    // Set default configuration of shared ADC core
    retval &= p33c_AdcCoreInstance_SetConfig(p33c_AdcCoreSharedCoreInstance, adcSharedCoreConfigDefault);

    return(retval);   
    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-adc-functions
 * @brief   Initializes the default settings of the specified ADC channel and sets the user-defined options
 * @param   feedbackChannel Pointer to an ADC Input Configuration data object of type BDBB_ADC_INPUT_SETTINGS_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the ADC input registers based on the selected ADC channel. 
 *  This function sets the input channel trigger source, input mode, and the ADC core 
 *  connected to the selected channel. 
 *
 *********************************************************************************/
int BdbbADC_ChannelInitialize(BDBB_FEEDBACK_CHANNEL_t* feedbackChannel) 
{
    int retval = 1;
    volatile uint8_t* ptrADCRegister;
    uint8_t bit_offset;
    
    // Initialize ADC input registers
    if (feedbackChannel->Enabled) {

        // Write level trigger setting
        if (feedbackChannel->Input.AdcInput < 16) {
            ADLVLTRGL |= ((uint16_t)(feedbackChannel->Trigger.LevelTrigger) << feedbackChannel->Input.AdcInput);
            ADEIEL |= ((uint16_t)(feedbackChannel->Interrupt.EarlyInterruptEnable) << feedbackChannel->Input.AdcInput);
            ADIEL |= ((uint16_t)(feedbackChannel->Interrupt.InterruptEnable) << feedbackChannel->Input.AdcInput);
        }
        else if (feedbackChannel->Input.AdcInput < 32) {
            ADLVLTRGH |= ((uint16_t)(feedbackChannel->Trigger.LevelTrigger) << (feedbackChannel->Input.AdcInput - 16));
            ADEIEH |= ((uint16_t)(feedbackChannel->Interrupt.EarlyInterruptEnable) << (feedbackChannel->Input.AdcInput - 16));
            ADIEH |= ((uint16_t)(feedbackChannel->Interrupt.InterruptEnable) << (feedbackChannel->Input.AdcInput - 16));
        }
        else {
            return(0); // ADC input number out of range
        }

        // write input mode setting
        if (feedbackChannel->Input.AdcInput < 8)
            bit_offset = (2 * feedbackChannel->Input.AdcInput);
        else if (feedbackChannel->Input.AdcInput < 16)
            bit_offset = (2 * (feedbackChannel->Input.AdcInput-8));
        else if (feedbackChannel->Input.AdcInput < 24)
            bit_offset = (2 * (feedbackChannel->Input.AdcInput-16));
        else if (feedbackChannel->Input.AdcInput < 32)
            bit_offset = (2 * (feedbackChannel->Input.AdcInput-24));
        else
            return(0); // ADC input number out of range

        ptrADCRegister = (volatile uint8_t*)
            ((uint8_t*)&ADMOD0L + (uint8_t)(feedbackChannel->Input.AdcInput >> 8));
        
        *ptrADCRegister |= ((unsigned int)feedbackChannel->Input.SignedResult << bit_offset);
        *ptrADCRegister |= ((unsigned int)feedbackChannel->Input.DifferentialInput << (bit_offset + 1));
       
        // Write ADC trigger source setting
        ptrADCRegister = (volatile uint8_t *)
            ((uint8_t *)&ADTRIG0L + (uint8_t)feedbackChannel->Input.AdcInput);
        
        *ptrADCRegister = (uint8_t)feedbackChannel->Trigger.TriggerSource;
        
        // Write dedicated ADC core alternate input selection
        // dsPIC33CK and dsPIC33CH secondary core families of devices have register 
        // space for up to six dedicated ADC cores
        #if defined (__P33SMPS_CK__) || defined (__P33SMPS_CH_SEC__)
        if (feedbackChannel->Input.AdcAltInput != 0) {
            if (feedbackChannel->Input.AdcCore < 7) { 
                ADCON4H |= (feedbackChannel->Input.AdcAltInput << (2 * feedbackChannel->Input.AdcCore));
            }
        }
        #endif 
        
        // Register ADC core to be activated
        switch (feedbackChannel->Input.AdcCore) {
            case 0:
                adcore_mask |= ADC_CORE0_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE0_MASK_INDEX;
                break;
            case 1:
                adcore_mask |= ADC_CORE1_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE1_MASK_INDEX;
                break;
            case 2:
                adcore_mask |= ADC_CORE2_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE2_MASK_INDEX;
                break;
            case 3:
                adcore_mask |= ADC_CORE3_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE3_MASK_INDEX;
                break;
            case 4:
                adcore_mask |= ADC_CORE4_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE4_MASK_INDEX;
                break;
            case 5:
                adcore_mask |= ADC_CORE5_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE5_MASK_INDEX;
                break;
            case 6:
                adcore_mask |= ADC_CORE6_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_CORE6_MASK_INDEX;
                break;
            default:
                adcore_mask |= ADC_SHRCORE_MASK_INDEX;
                if (feedbackChannel->Input.DifferentialInput)
                    adcore_diff_mask |= ADC_SHRCORE_MASK_INDEX;
                break;
        }
        
    }
    
    // Set the analog GPIO configuration
    retval &= p33c_AdcAnInput_Enable(feedbackChannel->Input.AdcInput, feedbackChannel->Enabled);
    
    return(retval);
    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-adc-functions
 * @brief   Enables the ADC module, starts the ADC cores and all analog inputs specified by the user
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function enables the ADC module, powers-up and enables the ADC cores used and waits 
 *  until ADC cores are ready.
 *
 *********************************************************************************/
int BdbbADC_Start(void) 
{
    int retval = 1;
    uint16_t timeout=0;
    uint16_t adcore_mask_compare=0;
    
    // Turn on ADC module
    ADCON1Lbits.ADON = 1;

    ADCON5L = (adcore_mask & ADC_CORE_ENABLE_MASK);    // Enable power to all used ADC cores
    adcore_mask_compare = ((adcore_mask << 8) | adcore_mask); // Set ADC Core Ready Verification Bit Mask
    
    // Wait until ADC cores are ready
    while ((ADCON5L != adcore_mask_compare) && (timeout++ < ADC_POWERUP_TIMEOUT)); 
    retval = (uint16_t)(bool)(timeout < ADC_POWERUP_TIMEOUT);

    // Clear all ADC Core Enable bits preserving clock settings
    ADCON3H &= ADC_CLK_PRESERVE_MASK; 
    
    // If no timeout error occurred, enable ADC cores preserving clock settings
    if (true == retval)
    { ADCON3H |= (adcore_mask & ADC_CORE_ENABLE_MASK); }
    
    // In case of a timeout error, disable power to all ADC cores
    else
    { ADCON5L = ADC_CORE_DISABLE_MASK; }
                
    return(retval);    
    
} 

// ______________________________________
// end of file
