/**
 * @file    svc_bdbb_pcontrol_dac.c
 * @brief   Bidirectional Buck-Boost Converter peripheral configuration and management functions
 * @author  M91406
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "svc_bdbb_pcontrol_dac.h"
#include "templates/svc_bdbb_ptemp_dac.h"

/* PRIVATE VARIABLES */

// (none)

/* PRIVATE FUNCTION CALL PROTOTYPES */

// (none)

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function initializes the Digital-To-Analog converter peripheral module
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbDAC_ModuleInitialize(void)
{
    int retval = 1;
    volatile P33C_DAC_MODULE_t* dacModule;     // user-defined DAC object 

    dacModule = p33c_DacModule_GetHandle();                    // Get pointer address to DAC module
    retval &= p33c_DacModule_SetConfig(bdbbDacModuleConfig);  // Initialize DAC module with default values

    dacModule->DACCTRL1L.bits.DACON = 0;        // Make sure DAC module is turned off
    
    return(retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function initializes the Digital-To-Analog converter, slope compensation and analog comparator used in peak current mode control 
 * @param   pcInstance  Pointer to a Bidirectional Buck-Boost Converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function captures the user selected pin then set the pin to inactive
 *  and set it as digital input or output pin. 
 *********************************************************************************/
int BdbbDAC_ChannelInitialize(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        volatile P33C_DAC_INSTANCE_t* dacInstance = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[i].SlopeCompensation.DacInstance); // user-defined DAC 1 object
        uint16_t bit_mask = (dacInstance->DACxCONL.value & REG_DACOEN_EN); // Capture DAC Output Enable bit setting

        retval &=p33c_DacInstance_SetConfig(pcInstance->SwitchNode[i].SlopeCompensation.DacInstance, bdbbDacInstanceConfig);

        dacInstance->DACxCONL.value = (REG_DACxCONL_PC | bit_mask); // Load PCMC default configuration, including previous DAC output setting
        dacInstance->DACxCONL.bits.INSEL = pcInstance->SwitchNode[i].SlopeCompensation.CompInput;
         
        dacInstance->DACxCONH.value = REG_DACxCONH; // Load PCMC default configuration
        dacInstance->DACxCONH.bits.TMCB = pcInstance->SwitchNode[i].SlopeCompensation.LebPeriod; // Load leading edge blanking value

        dacInstance->SLPxDAT.value  = pcInstance->SwitchNode[i].SlopeCompensation.SlewRate; // Set DAC slope compensation slew rate
        dacInstance->DACxDATH.value = pcInstance->SwitchNode[i].SlopeCompensation.DacValueInit; // Set initial DAC reference
        dacInstance->DACxDATL.value = 0; // Clear DAC LOW register
        
        uint16_t pgInstance = p33c_PwmInstance_GetInstance(pcInstance->SwitchNode[i].PwmInstance);
        
        dacInstance->SLPxCONL.value = REG_SLPxCONL; // Enabling negative slope mode
        dacInstance->SLPxCONL.bits.SLPSTRT  = pgInstance; // Set PWM trigger starting slope
        dacInstance->SLPxCONL.bits.SLPSTOPA = pgInstance; // Set PWM trigger stopping slope
        dacInstance->SLPxCONL.bits.SLPSTOPB = pcInstance->SwitchNode[i].SlopeCompensation.DacInstance;  // Set comparator trigger stopping slope
    }

    return (retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function starts the Digital-To-Analog converter peripheral module
 * @param   pcInstance  Pointer to a Bidirectional Buck Converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbDAC_Start(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    volatile P33C_DAC_MODULE_t* dacModule;     // user-defined DAC object 
    volatile P33C_DAC_INSTANCE_t* dacInstance; // user-defined DAC instance
    
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        dacInstance = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[i].SlopeCompensation.DacInstance); // Get pointer address to DAC module
        dacInstance->SLPxCONH.bits.SLOPEN = true;   // Turn on slope generator
        dacInstance->DACxCONL.bits.DACEN = true;    // Turn on DAC instance
    }

    dacModule = p33c_DacModule_GetHandle();         // Get pointer address to DAC module
    dacModule->DACCTRL1L.bits.DACON = true;   // turn on DAC module
    
    return(retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function stops the Digital-To-Analog converter peripheral module
 * @param   pcInstance  Pointer to a Bidirectional Buck Converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbDAC_Stop(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    volatile P33C_DAC_INSTANCE_t* dacInstance; // user-defined DAC instance

    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        dacInstance = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[i].SlopeCompensation.DacInstance); // Get pointer address to DAC module
        dacInstance->DACxCONL.bits.DACEN = false;       // Turn on DAC instance
    }
        
    return(retval);
}

/*******************************************************************************
 * @ingroup llib-layer-bdbb-dac-functions
 * @brief   This function resets the DAC registers to their initial default values
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function resets the registers of all DAC instances assigned to the 
 *  switch node PWM channels to their initial default values declared in the 
 *  power converter settings data structure.
 *********************************************************************************/
int BdbbDAC_Reset(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;

    // Reset all DAC reference values to their initial default value
    retval &= BdbbDAC_SetReference(pcInstance, pcInstance->SwitchNode[0].SlopeCompensation.DacValueInit);

    return(retval);
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function writes the given reference value to all DAC instances used by switch nodes
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @param   refValue    New reference value to be written to target address of type 16-bit unsigned integer 
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The switch node configuration supports single and multiphase drives. This 
 *  function writes the given reference value to all switch node slope compensation 
 *  DAC instances.
 *
 *********************************************************************************/
int BdbbDAC_SetReference(IBdbbConverter_t* pcInstance, uint16_t refValue)
{
    int retval = 1;
    volatile P33C_DAC_INSTANCE_t* dac;
    
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN DAC INSTANCE
        dac = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[i].SlopeCompensation.DacInstance);    
        dac->DACxDATH.value = refValue;
    }
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-dac-functions
 * @brief   This function gets the reference value from the first DAC instance of the first switch node
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The switch node configuration supports single and multiphase drives. This 
 *  function gets the reference value from the first switch node slope compensation 
 *  DAC instance, assuming that in  multiphase converters all phases are operating
 *  with the same reference value.
 *
 *********************************************************************************/
uint16_t BdbbDAC_GetReference(IBdbbConverter_t* pcInstance)
{
    uint16_t refValue=0;
    volatile P33C_DAC_INSTANCE_t* dac;
    
    // CAPTURE MEMORY ADDRESS OF GIVEN DAC INSTANCE
    dac = p33c_DacInstance_GetHandle(pcInstance->SwitchNode[0].SlopeCompensation.DacInstance);    
    refValue = dac->DACxDATH.value;
    
    return(refValue);  
}

// ______________________________________
// end of file
