/**
 * @file    svc_bdbb_pcontrol_io.c
 * @brief   Bidirectional Buck-Boost Converter peripheral configuration and management functions
 * @author  M91406
 */

#include <xc_pral.h> // include device peripheral register abstraction layer
#include "svc_bdbb_pcontrol_io.h"

/* PRIVATE VARIABLES */

// (none)

/* PRIVATE FUNCTION CALL PROTOTYPES */


/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function clears the Relay output to its inactive state
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbGPIO_ClearRelay(IBdbbConverter_t* pcInstance)
{
    // Guarding condition
    if (!pcInstance->Gpio.EnableRelay.Enabled) return(1);
    if (!p33c_Gpio_IsInitialized(&pcInstance->Gpio.EnableRelay.IoPin)) return(0);
    
    // Set GPIO of Relay output
    bool gpioState = !pcInstance->Gpio.EnableRelay.Polarity;
    pcInstance->Gpio.EnableRelay.IoPin.Methods.SetState(gpioState);
    
    // Read signal level of Relay output
    gpioState = pcInstance->Gpio.EnableRelay.IoPin.Methods.Get();
    bool result = (bool)(gpioState != pcInstance->Gpio.EnableRelay.Polarity);
    
    return((uint16_t)result);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function sets the Relay output to its active state
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbGPIO_SetRelay(IBdbbConverter_t* pcInstance)
{
    // Guarding conditions
    if (!pcInstance->Gpio.EnableRelay.Enabled) return(1);
    if (!p33c_Gpio_IsInitialized(&pcInstance->Gpio.EnableRelay.IoPin)) return(0);
    
    // Set GPIO of Relay output
    bool gpioState = pcInstance->Gpio.EnableRelay.Polarity;
    pcInstance->Gpio.EnableRelay.IoPin.Methods.SetState(gpioState);
    
    // Read signal level of Relay output
    gpioState = pcInstance->Gpio.EnableRelay.IoPin.Methods.Get();
    bool result = (bool)(gpioState == pcInstance->Gpio.EnableRelay.Polarity);
    
    return((uint16_t)result);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function sets the Power Good output to its inactive state
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbGPIO_ClearPowerGood(IBdbbConverter_t* pcInstance)
{
    // Guarding condition
    if (!pcInstance->Gpio.PowerGood.Enabled) return(1);
    if (!p33c_Gpio_IsInitialized(&pcInstance->Gpio.PowerGood.IoPin)) return(0);
    
    // Set GPIO of PowerGood output
    bool gpioState = !pcInstance->Gpio.PowerGood.Polarity;
    pcInstance->Gpio.PowerGood.IoPin.Methods.SetState(gpioState);
    
    // Read signal level of Power Good output
    gpioState = pcInstance->Gpio.PowerGood.IoPin.Methods.Get();
    bool result = (bool)(gpioState != pcInstance->Gpio.PowerGood.Polarity);
    
    return((uint16_t)result);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function sets the Power Good output to its active state
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbGPIO_SetPowerGood(IBdbbConverter_t* pcInstance)
{
    // Guarding conditions
    if (!pcInstance->Gpio.PowerGood.Enabled) return(1);
    if (!p33c_Gpio_IsInitialized(&pcInstance->Gpio.PowerGood.IoPin)) return(0);
    
    // Set GPIO of PowerGood output
    bool gpioState = pcInstance->Gpio.PowerGood.Polarity;
    pcInstance->Gpio.PowerGood.IoPin.Methods.SetState(gpioState);
    
    // Read signal level of Power Good output
    gpioState = pcInstance->Gpio.PowerGood.IoPin.Methods.Get();
    bool result = (bool)(gpioState == pcInstance->Gpio.PowerGood.Polarity);
    
    return((uint16_t)result);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function gets the signal level of the Power Enable pin
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  ACTIVE of INACTIVE as Boolean (false = INACTIVE, true = ACTIVE)
 *********************************************************************************/
bool BdbbGPIO_GetEnableInputState(IBdbbConverter_t* pcInstance)
{
    // Guarding condition
    if (!pcInstance->Gpio.EnableInput.Enabled) return(1);
    if (!p33c_Gpio_IsInitialized(&pcInstance->Gpio.EnableInput.IoPin)) return(0);

    //  active state signal level of ENABLE pin
    bool gpioState = pcInstance->Gpio.EnableInput.IoPin.Methods.Get();
    bool result = (bool)(gpioState == pcInstance->Gpio.EnableInput.Polarity);
    
    // Return ACTIVE of INACTIVE state
    return(result);
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function initializes a BDBB device pin
 * @param   pcInstance  Pointer to a power converter device pin data object of type BDBB_GPIO_INSTANCE_t
 * @return  unsigned integer (0=failure, 1=success)
 * @details
 *  This function initializes a power supply-related device pin in accordance 
 *  with the user configuration of that pin. This function is private.
 * 
 * @see
 *  BdbbGPIO_Initialize
 *********************************************************************************/
static int BdbbGpio_IoInitialize(BDBB_GPIO_INSTANCE_t* gpio)
{
    int retval = 1;

    if (p33c_Gpio_IsInitialized(&gpio->IoPin))
    {
        gpio->IoPin.Methods.SetDigital(); // Disable analog functions

        switch (gpio->IoType)
        {
            case 0: // Configure device pin as standard push-pull output

                gpio->IoPin.Methods.SetPullDown(0); // Disable pull-down resistor
                gpio->IoPin.Methods.SetPullUp(0); // Disable pull-up resistor
                gpio->IoPin.Methods.SetOpenDrain(0); // Set Push-Pull Mode (disables open-drain mode)
                gpio->IoPin.Methods.Init(0, gpio->Polarity); // Set device pin as output

                break;

            case 2: // Configure device pin as open-drain output

                if (gpio->Polarity == 0) // Signal is Active Low
                {
                    gpio->IoPin.Methods.SetPullDown(0); // Disable pull-down resistor
                    gpio->IoPin.Methods.SetPullUp(1); // Enable pull-up resistor
                }
                else  // Signal is Active High
                {
                    gpio->IoPin.Methods.SetPullUp(0); // Disable pull-up resistor
                    gpio->IoPin.Methods.SetPullDown(1); // Enable pull-down resistor
                }

                gpio->IoPin.Methods.SetOpenDrain(1); // Set Open-Drain Mode
                gpio->IoPin.Methods.Init(0, gpio->Polarity); // Set device pin as output

                break;

            default:

                gpio->IoPin.Methods.SetPullDown(0); // Disable pull-down resistor
                gpio->IoPin.Methods.SetPullUp(0); // Disable pull-up resistor
                gpio->IoPin.Methods.SetOpenDrain(0); // Set Push-Pull Mode (disables open-drain mode)
                gpio->IoPin.Methods.Init(1, gpio->Polarity); // Set device pin as input
                break;
        }

    }
    else { retval = 0; }

    return(retval);
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-io-functions
 * @brief   This function initializes the BDBB GPIO pins
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * @details
 *  This function initializes the ENABLE input pin and POWER GOOD output pin.
 * @see
 *  BdbbGpio_IoInitialize
 *********************************************************************************/
int BdbbGPIO_Initialize(IBdbbConverter_t* pcInstance)
{
    int retval = 1;

    // Initialize ENABLE input pin
    if (pcInstance->Gpio.EnableInput.Enabled) 
    { 
        if (pcInstance->Gpio.PowerGood.IoType != 1) pcInstance->Gpio.EnableInput.IoType = 1; // Device pin is logic input
        retval &= BdbbGpio_IoInitialize(&pcInstance->Gpio.EnableInput); 
    }
    
    // Initialize POWER GOOD output pin
    if (pcInstance->Gpio.PowerGood.Enabled) 
    { 
        if (pcInstance->Gpio.PowerGood.IoType == 1) pcInstance->Gpio.PowerGood.IoType = 1; // Device pin is logic output
        retval &= BdbbGpio_IoInitialize(&pcInstance->Gpio.PowerGood); 
    }

	// Initialize Relay output pin
    if (pcInstance->Gpio.EnableRelay.Enabled) 
    { 
        if (pcInstance->Gpio.EnableRelay.IoType == 1) pcInstance->Gpio.EnableRelay.IoType = 1; // Device pin is logic output
        retval &= BdbbGpio_IoInitialize(&pcInstance->Gpio.EnableRelay); 
    }

    return(retval);
} 

// ______________________________________
// end of file
