/**
 * @file    svc_bdbb_pcontrol_pwm.c
 * @brief   Bidirectional Buck-Boost Converter peripheral configuration and management functions
 * @author  M91406
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "svc_bdbb_pcontrol_dac.h"
#include "svc_bdbb_pcontrol_io.h"
#include "svc_bdbb_pcontrol_pwm.h"
#include "templates/svc_bdbb_ptemp_pwm.h"

/* PRIVATE VARIABLES */

// (none)

/* PRIVATE FUNCTION CALL PROTOTYPES */

// (none)

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   Initializes the bidirectional buck-boost PWM module by resetting its registers to default
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the PWM module base registers with default
 *  values for maximum performance. 
 * 
 *  Default configuration:
 *     - all PWM peripheral power is enabled
 *     - all PWM generators are disabled
 *     - default PWM Module configurations are written in bidirectional buck-boost PWM module base registers 
 *
 *********************************************************************************/
int BdbbPWM_ModuleInitialize(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    volatile P33C_PWM_MODULE_t* pwm;
    
    // Make sure power to peripheral is enabled
    retval &= p33c_PwmModule_PowerOn();
    
    // DISABLE ALL PWM GENERATORS
    for (int i=1; i<p33c_PwmInstances; i++)
    { retval &= p33c_PwmInstance_Disable(i); }
    
    // Write default PWM Module configuration to PWM module base registers
    pwm = p33c_PwmModule_GetHandle();
    retval &= p33c_PwmModule_SetConfig(bdbbPwmModuleConfig);

    // If bidirectional buck-boost converter has been configured in MASTER PERIOD mode
    if (pcInstance->SwitchNode[0].MasterPeriodEnable) // If master period mode is enabled...
        pwm->MPER.value = pcInstance->SwitchNode[0].Period; // Set Period of phase #1

    return(retval);    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   Initializes the PWM channels declared in the switch node configuration
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the PWM channels of all converter phases as 
 *  declared in the switch node configuration. The base configuration of
 *  each PWM channel, including additional commutation components, such as
 *  comparators and logic inputs, are loaded from configuration templates 
 *  before loading the user-specific switch-node configuration.
 * 
 *  The templates used are:
 *    - @ref bdbbPwmModuleConfig        Basic PWM Module Configuration Template
 *    - @ref bdbbPwmGeneratorConfig     Basic PWM Channel Configuration Template
 *    - @ref bdbbDacModuleConfig        Basic PDM DAC/CMP Module Configuration Template
 *    - @ref bdbbDacInstanceConfig      Basic PDM DAC/CMP Channel Configuration Template
 *
 *********************************************************************************/
int BdbbPWM_ChannelInitialize(IBdbbConverter_t* pcInstance)
{
    int retval = 1;

    // Guarding conditions
    if (pcInstance == NULL) return(0);
    
    // Configure switch node peripheral sets
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // Guarding conditions
        if (pcInstance->SwitchNode[i].PwmInstance == NULL) return(0);
        if (!p33c_Gpio_IsInitialized(&pcInstance->SwitchNode[i].GpioHigh)) return(0);
        if (!p33c_Gpio_IsInitialized(&pcInstance->SwitchNode[i].GpioLow)) return(0);

        // WRITE GPIO CONFIGURATION OF PWM OUTPUT(S)
        pcInstance->SwitchNode[i].GpioHigh.Methods.SetPullDown(1); // Enable weak pull-down resistor at pin PWMxH
        pcInstance->SwitchNode[i].GpioHigh.Methods.Init(0, 0); // Set PWMxH pin to OUTPUT/LOW
        pcInstance->SwitchNode[i].GpioLow.Methods.SetPullDown(1); // Enable weak pull-down resistor at pin PWMxH
        pcInstance->SwitchNode[i].GpioLow.Methods.Init(0, 0); // Set PWMxH pin to OUTPUT/LOW

        // COPY CONFIGURATION FROM TEMPLATE TO PWM GENERATOR x CONTROL REGISTERS
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;
        
        if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC)
        { *pg = bdbbPwmGeneratorPcmcConfig; }
        else { *pg = bdbbPwmGeneratorConfig; }

        // DERIVING SYNCHRONIZATION SETTINGS ACROSS PWM GENERATORS
        P33C_PWM_INSTANCE_t* pgPre = NULL;
        uint16_t pgPreInstance = 0;
        uint16_t phaseShift = 0;
        
        if ((0 < i) && (i <= (pcInstance->Properties.PhaseCount-1)))
        { // Last phase does not provide any master trigger
            pgPre = pcInstance->SwitchNode[i-1].PwmInstance;
            pgPreInstance = p33c_PwmInstance_GetInstance(pgPre);
        }
        if ((0 <= i) && (i <= (pcInstance->Properties.PhaseCount-2)))
        { phaseShift = pcInstance->SwitchNode[i+1].PhaseShift; }
        
        // LOAD PWM GENERATOR TIMING SETTINGS FROM BUCK CONVERTER OBJECT
        pg->PGxCONH.bits.MPERSEL = (uint16_t)pcInstance->SwitchNode[i].MasterPeriodEnable;
        pg->PGxCONL.bits.HREN = (uint16_t)pcInstance->SwitchNode[i].HighResolutionEnable;
        
        pg->PGxDC.value = pcInstance->SwitchNode[i].DutyRatioMinimum; // PGxDC: PWM GENERATOR x DUTY CYCLE REGISTER
        pg->PGxPER.value = pcInstance->SwitchNode[i].Period; // PGxPER: PWM GENERATOR x PERIOD REGISTER
        pg->PGxDTL.value = pcInstance->SwitchNode[i].DeadTimeFalling; // PGxDTL: PWM GENERATOR x DEAD-TIME REGISTER LOW
        pg->PGxDTH.value = pcInstance->SwitchNode[i].DeadTimeRising; // PGxDTH: PWM GENERATOR x DEAD-TIME REGISTER HIGH
        
        // Select if PWMxH/PWMxL pins should be swapped
        pg->PGxIOCONL.bits.SWAP = pcInstance->SwitchNode[i].SwapOutputs; // PGxIOCONL: PWM GENERATOR x I/O CONTROL REGISTER LOW -> SWAP: Swap PWM Signals to PWMxH and PWMxL Device Pins bit
        
        // Set ADC Trigger A Post Scaler and Trigger Offset for default control interrupt generation
        pg->PGxEVTL.bits.ADTR1PS = pcInstance->SwitchNode[i].ControlTriggerDivider;
        pg->PGxEVTH.bits.ADTR1OFS = pcInstance->SwitchNode[i].ControlTriggerOffset;
        pg->PGxEVTH.bits.IEVTSEL = REG_PGxEVTH_IEVTSEL_ADTR1; // Always set ADC trigger 1 as PWM interrupt trigger event
        
        // ToDo: PWM Synchronization needs to be more universaL
        // Description: 
        //   PWM synchronization only works within groups of 4 (PG1-PG4 or PG5-PG8)
        //   Synchronization across groups requires additional phantom PWM and introduces
        //   synchronization phase lag of up to 20ns. Make decision if cross-group 
        //   synchronization should even be allowed.
        if ((i == 0) && (pcInstance->Properties.PhaseCount > 1U))
        { // First phase is always master phase
            pg->PGxCONH.bits.MSTEN = 1; // Enable Master synchronization mode
            pg->PGxCONH.bits.SOCS = 0b0000; // Master PWM always triggers itself
            pg->PGxEVTL.bits.PGTRGSEL = 0b011; // PGxTRIGC is always used as PWM synchronization trigger output
            pg->PGxTRIGC.value = phaseShift; // Set phase shift between master phase and first synchronized phase
            pg->PGxCONH.bits.UPDMOD = 0b001; // Immediate update
            pg->PGxEVTL.bits.UPDTRG = 0b11; // A write of the PGxTRIGA register automatically sets the UPDATE bit
        }
        else if ((0 < i) && (i < (pcInstance->Properties.PhaseCount-1)))
        { // Every synchronized phase is synchronized to previous phase while being master to following
            pg->PGxCONH.bits.MSTEN = 1; // Enable Master synchronization mode
            pg->PGxCONH.bits.SOCS = pgPreInstance; // synchronized PWM is always triggered by previous generator while providing trigger for following
            pg->PGxEVTL.bits.PGTRGSEL = 0b011; // PGxTRIGC is always used as PWM synchronization trigger output
            pg->PGxTRIGC.value = phaseShift; // Set phase shift between master phase and first synchronized phase
            pg->PGxCONH.bits.UPDMOD = 0b011; // Sync immediate update
            pg->PGxEVTL.bits.UPDTRG = 0; // User must set the UPDREQ bit (PGxSTAT[3]) manually
        }
        else if ((0 < i) && (i == (pcInstance->Properties.PhaseCount-1)))
        { // Last phase does not provide any master trigger
            pg->PGxCONH.bits.MSTEN = 0; // Disable Master synchronization mode
            pg->PGxCONH.bits.SOCS = pgPreInstance; // synchronized PWM is always triggered by previous generator while providing trigger for following
            pg->PGxEVTL.bits.PGTRGSEL = 0b000; // EOC is the general trigger output
            pg->PGxTRIGC.value = 0x0000; // Clear phase shift value 
            pg->PGxCONH.bits.UPDMOD = 0b011; // Sync immediate update
            pg->PGxEVTL.bits.UPDTRG = 0; // User must set the UPDREQ bit (PGxSTAT[3]) manually
        }
        else if (pcInstance->Properties.PhaseCount == 1U)
        { // This is only a single phase system (no PWM dependencies)
            pg->PGxCONH.bits.MSTEN = 0; // Disable Master synchronization mode
            pg->PGxCONH.bits.SOCS = 0b0000; // Master PWM always triggers itself
            pg->PGxEVTL.bits.PGTRGSEL = 0b000; // EOC is the general trigger output
            pg->PGxTRIGC.value = 0x0000; // Clear phase shift between master phase and first synchronized phase
            pg->PGxCONH.bits.UPDMOD = 0b001; // Immediate update
            pg->PGxEVTL.bits.UPDTRG = 0b11; // A write of the PGxTRIGA register automatically sets the UPDATE bit
        }
        else { /* continue */ }
        
		// Set PWM ADC trigger #1 cycle post scaler and offset
		pg->PGxEVTL.bits.ADTR1PS = pcInstance->SwitchNode[i].ControlTriggerDivider;
		pg->PGxEVTH.bits.ADTR1OFS = pcInstance->SwitchNode[i].ControlTriggerOffset;
		
        // Update PWM generator timing registers
        pg->PGxSTAT.bits.UPDREQ = 1; // Manually set the Update Request bit 
        pg->PGxCONH.bits.TRGMOD = 1; // all PWM generators are in 'retriggerable' mode
        
        // In Peak Current Mode Control, initialize PWM channel current limit PCI interface
        if(pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC) {
            
            pg->PGxEVTH.bits.IEVTSEL = REG_PGxEVTH_IEVTSEL_TRIGA;
            pg->PGxEVTL.bits.ADTR1EN1 = true; // Use PGxTRIGA as PWM trigger #1
            pg->PGxEVTH.bits.ADTR2EN2 = true; // Use PGxTRIGB as PWM trigger #2
            pg->PGxTRIGA.value  = pcInstance->SwitchNode[i].SlopeCompensation.SlopeStartTrigger;
            pg->PGxTRIGB.value  = pcInstance->SwitchNode[i].SlopeCompensation.SlopeStopTrigger;
            pg->PGxFFPCIL.value = REG_PGxyPCILPC;  // Load PCI configuration template of Current Limit PCI Low Register
            pg->PGxFFPCIH.value = REG_PGxyPCIHPC;  // Load PCI configuration template of Current Limit PCI High Register
            pg->PGxFFPCIL.bits.PSS = pcInstance->SwitchNode[i].SlopeCompensation.PwmTriggerInput;
            
            pg->PGxIOCONL.value = REG_PGxIOCONLPC; // Load IO configuration template for peak current mode control 
            
        }
        
    }

    // In Peak Current Mode Control, initialize comparator and PDM DAC instances
    if(pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC) {
        retval &= BdbbDAC_ModuleInitialize();
        retval &= BdbbDAC_ChannelInitialize(pcInstance);
    }    
    
    return(retval);    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function enables the PWM operation
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function starts the operation of the PWM by enabling the PWM 
 *  generator and its output pins.
 *
 *********************************************************************************/
int BdbbPWM_Start(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;
    uint16_t timeout=0;
    uint16_t sync_sw_mask=0;

    // Enable all PWM channels of the recent bidirectional buck-boost converter configuration
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;

        pg->PGxIOCONL.value |= P33C_PGxIOCONL_OVREN_SYNC; // PWMxH/L Output Override Enable: PWM generator controls the PWMxH output pin
        pg->PGxIOCONH.value &= ~(P33C_PGxIOCONH_PEN_SYNC); // PWMxH/L Output Port Disable: PWM generator controls the PWMxH output pin

        pg->PGxCONL.bits.ON = 1;  // PWM Generator Enable: PWM Generator is enabled
        pg->PGxSTAT.bits.UPDREQ = 1;  // Update all PWM timing registers
        
        if(pg->PGxCONL.value & P33C_PGxCONL_HRES_EN) // If high resolution is enabled
        {
            while((!PCLKCONbits.HRRDY) && (timeout++ < 5000));  // wait for high resolution to get ready
            if ((timeout >= 10000) || (PCLKCONbits.HRERR))       // if there is an error ...
                return(0);                                      // ... exit returning ERROR
        }

        // Select the control bits for either synchronous or asynchronous PWM drive
        // note: swapping PWMs changes H/L assignments and the 'active' pin has to 
        //       selected in asynchronous mode
        if (pcInstance->SwitchNode[i].SyncDrive)
            sync_sw_mask = P33C_PGxIOCONH_PEN_SYNC;
        else
        {   
            /* By default BDBB assumes 
             * 
             * Buck Mode:
             * - PWMxH pin being routed to high side FET 
             * - PWMxL pin being routed to low side FET  (disable when SyncDrive=false)
             *  
             * Boost Mode:
             * - PWMxL pin being routed to high side FET (disable when SyncDrive=false)
             * - PWMxH pin being routed to low side FET
             * 
             * !!! PGxIOCONLbits.SWAP only swaps the signal, not the pins !!!
             * 
             * ==============================================================
             * When SwapOutputs = true, it is assumed the device pins 
             * are swapped.
             * 
             * Buck Mode: 
             * - PWMxH signal => PWMxL pin => high side FET
             * - PWMxL signal => PWMxH pin => low side FET  (disable when SyncDrive=false)
             * 
             * Boost Mode: 
             * - PWMxL signal => PWMxL pin => high side FET (disable when SyncDrive=false)
             * - PWMxH signal => PWMxH pin => low side FET 
             * 
             */

            if (true == pcInstance->SwitchNode[i].SwapOutputs) 
            {
                if (false == pcInstance->Properties.ConversionMode) // In Buck mode DC => PWMxL Signal 
                { sync_sw_mask = P33C_PGxIOCONH_PEN_ASYNC_BUCK_SWAP; } 
                else // Boost mode
                { sync_sw_mask = P33C_PGxIOCONH_PEN_ASYNC_BOOST_SWAP; }
            }
            else
            {
                if (false == pcInstance->Properties.ConversionMode) // Buck mode
                { sync_sw_mask = P33C_PGxIOCONH_PEN_ASYNC_BUCK; }
                else // Boost mode
                { sync_sw_mask = P33C_PGxIOCONH_PEN_ASYNC_BOOST; }
            }
            
        }
        
        // PWMxH/L Output Port Enable: PWM generator controls the PWMxH output pin
        pg->PGxIOCONH.value |= sync_sw_mask; 

        // Turn on PWM generator
        retval &= (uint16_t)(pg->PGxCONL.bits.ON);

    }
    
    return(retval);    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function stops the PWM output
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function shuts down the PWM output by disabling all PWM channels of the recent 
 *  power converter object, overriding the output pins and resetting the PWM duty cycle 
 *  to its minimum duty ratio. 
 * 
 *  If the Power Good output is enabled, this output pin will also be reset.
 *
 *********************************************************************************/
int BdbbPWM_Stop(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;

    // Disable all PWM channels of the recent bidirectional buck-boost converter configuration
    for (int i=0; i< pcInstance->Properties.PhaseCount; i++) 
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;

        pg->PGxIOCONL.value |= P33C_PGxIOCONL_OVREN_SYNC;  // PWMxH/L Output Override Enable
        pg->PGxIOCONH.value &= ~(P33C_PGxIOCONH_PEN_SYNC); // PWMxH/L Output Pint Control Disable
        pg->PGxCONL.value &= ~(P33C_PGxCONL_PWM_ON);  // PWM Generator Disable
        pg->PGxDC.value = pcInstance->SwitchNode[i].DutyRatioMinimum; // Reset Duty Cycle
        pg->PGxSTAT.value |= P33C_PGxSTAT_UPDREQ;     // Set the Update Request bit to update PWM timing
        
        retval &= (uint16_t)((bool)(pg->PGxCONL.bits.ON == 0));

    }

    return(retval);    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function disables the PWM generator IOs
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function suspends the PWM operation by disabling all PWM outputs of the recent 
 *  power converter configuration, overriding the PWM output pins and setting the 
 *  duty cycle to 0.
 *
 *********************************************************************************/
int BdbbPWM_Suspend(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;

    // Disable all PWM outputs of the recent bidirectional buck-boost converter configuration
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;
        
        // PWMxH/L Output Override Enable
        pg->PGxIOCONL.value |= P33C_PGxIOCONL_OVREN_SYNC; 

        retval &= (bool)(pg->PGxIOCONL.value & P33C_PGxIOCONL_OVREN_SYNC);

    }
    
    // Clear PWM output enable status bit
    pcInstance->Status.bits.PwmOutputActive = false;
    
    return(retval);
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function resumes the PWM operation
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function updates the PWM timing bit and the PWM output pins are enabled.
 *
 *********************************************************************************/
int BdbbPWM_Resume(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;
    uint16_t sync_sw_mask=0;

    // Disable all PWM channels of the recent bidirectional buck-boost converter configuration
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;

        // Select the control bits for either synchronous or asynchronous PWM drive
        bool sync_mode = pcInstance->SwitchNode[i].SyncDrive; // Select if synchronous mode is supported by switch node 
        
        // If automatic switch-over between synchronous and asynchronous mode is enabled,
        // always set PWM outputs to asynchronous mode when re-enabling the PWM
        if (true == pcInstance->Properties.AutoAsyncMode)
            pcInstance->Properties.AsyncMode = true;

        // Clear selected override bits
        pg->PGxSTAT.bits.UPDREQ = 1; // Set the Update Request bit to update PWM timing
            
        // Capture status of asynchronous/synchronous mode control bit
        sync_mode &= (!pcInstance->Properties.AsyncMode);
        
        // Select the control bits for either synchronous or asynchronous PWM drive
        // note: swapping PWMs changes H/L assignments and the 'active' pin has to 
        //       selected in asynchronous mode
        if (sync_mode)
        {   
            pg->PGxIOCONL.value &= (uint16_t)~P33C_PGxIOCONL_OVREN_SYNC; // clear both override bits in register
            retval &= (uint16_t)((bool)(0 == (pg->PGxIOCONL.value & P33C_PGxIOCONL_OVREN_SYNC)));
        }
        else
        {   
            /* By default BDBB assumes 
             * 
             * Buck Mode:
             * - PWMxH pin being routed to high side FET 
             * - PWMxL pin being routed to low side FET  (override when async=true)
             *  
             * Boost Mode:
             * - PWMxL pin being routed to high side FET (override when async=true)
             * - PWMxH pin being routed to low side FET
             * 
             * !!! PGxIOCONLbits.SWAP only swaps the signal, not the pins !!!
             * 
             * ==============================================================
             * When SwapOutputs = true, it is assumed the device pins 
             * are swapped.
             * 
             * Buck Mode: 
             * - PWMxH signal => PWMxL pin => high side FET
             * - PWMxL signal => PWMxH pin => low side FET  (override when async=true)
             * 
             * Boost Mode: 
             * - PWMxL signal => PWMxL pin => high side FET (override when async=true)
             * - PWMxH signal => PWMxH pin => low side FET 
             * 
             */
            
            if (true == pcInstance->SwitchNode[i].SwapOutputs) 
            {
                if (false == pcInstance->Properties.ConversionMode) // In Buck mode DC => PWMxL Signal 
                { sync_sw_mask = P33C_PGxIOCONL_OVREN_ASYNC_BUCK_SWAP; } 
                else // Boost mode
                { sync_sw_mask = P33C_PGxIOCONL_OVREN_ASYNC_BOOST_SWAP; }
            }
            else
            {
                if (false == pcInstance->Properties.ConversionMode) // Buck mode
                { sync_sw_mask = P33C_PGxIOCONL_OVREN_ASYNC_BUCK; }
                else // Boost mode
                { sync_sw_mask = P33C_PGxIOCONL_OVREN_ASYNC_BOOST; }
            }

            uint16_t bitmsk = pg->PGxIOCONL.value; // capture status of register
            sync_sw_mask |= (uint16_t)(~P33C_PGxIOCONL_OVREN_SYNC); // Create clearance bit mask of override bits
            bitmsk &= (uint16_t)(sync_sw_mask); // set/clear PWMxH/L output override according to synch drive result
            pg->PGxIOCONL.value &= bitmsk; // set override bits in register
            
            // Build return value
            retval &= (uint16_t)((bool)((P33C_PGxIOCONL_OVREN_ASYNC_BUCK  == (pg->PGxIOCONL.value & P33C_PGxIOCONL_OVREN_SYNC)) || 
                                        (P33C_PGxIOCONL_OVREN_ASYNC_BOOST == (pg->PGxIOCONL.value & P33C_PGxIOCONL_OVREN_SYNC))));
        }
    }
    
    // Set PWM output enable status bit
    pcInstance->Status.bits.PwmOutputActive = true;
    
    return(retval);  
    
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function resets the PWM generators to their initial default values
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function resets the PWM generator registers to their initial default 
 *  values declared in the power converter settings data structure.
 *
 *********************************************************************************/
int BdbbPWM_Reset(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;

    // Reset all PWM duty cycles to their initial default value
    retval &= BdbbPWM_SetDutyCycle(pcInstance, pcInstance->SwitchNode[0].DutyRatioMinimum);

    return(retval);
    
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function updates the PWM timing registers of all PWM modules across all phases
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function sets the update request of each PWM channel specified in the 
 *  switch node configuration, enforcing the update of all timing registers of 
 *  each PWM generator.
 *
 *********************************************************************************/
int BdbbPWM_Update(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    
    // Set update bit across all PWM channels of the recent switch-node configuration
    for (int i=0; i< pcInstance->Properties.PhaseCount; i++) 
    {
        // Set UPDATE Request bit
        pcInstance->SwitchNode[i].PwmInstance->PGxSTAT.bits.UPDREQ = 1;
    }
    
    return(retval);  
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function swaps between PWMxH or PWMxL dominant signal generation
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function sets the PWM generator outputs to be either PWMxH or PWMxL 
 *  dominant. PWMxH dominant is used in down-stream (buck) mode, where the 
 *  control loop sets the on-time of the high-side switch channel. PWMxL 
 *  dominant is used in up-stream (boost) mode, where the control loop sets 
 *  the on-time of the low-side switch channel.
 *********************************************************************************/
int BdbbPWM_SetOutputSwap(IBdbbConverter_t* pcInstance) 
{
    int retval = 1;
    
    // Disable all PWM outputs of the recent bidirectional buck-boost converter configuration
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;
           
        // Check if the PWM configuration is swapped by default 
        if (false == pcInstance->SwitchNode[i].SwapOutputs)
        { pg->PGxIOCONL.bits.SWAP = pcInstance->Properties.ConversionMode; } // PWMxH/L Output Override Enable
        else
        { pg->PGxIOCONL.bits.SWAP = (!pcInstance->Properties.ConversionMode); } // PWMxH/L Output Override Enable
        
        // In Peak Current Mode, also flip the Feed Forward Data bits of the PCI I/O Control register
        if(pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC)
        { pg->PGxIOCONL.bits.FFDAT = (0x0001 << pg->PGxIOCONL.bits.SWAP); }
    }
    
    return(retval);
} 

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function enables/disables the synchronous rectifier PWM outputs of all PWM modules across all phases
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The switch node configuration supports synchronous and asynchronous commutation
 *  as well as swapping of PWM outputs. This function enables/disables the recently
 *  active PWM outputs in accordance with the most recent switch node drive settings.
 *  This function may be called during basic configuration or at runtime.
 * 
 *********************************************************************************/
int BdbbPWM_SetSynchronousMode(IBdbbConverter_t* pcInstance)
{
    int retval = 1;
    PGxIOCONL_t iocon;
    
    // Check if switch node drive is currently enabled, leave if not
    if (pcInstance->Status.bits.IsSuspended)
        return(retval);
    
    // SELECT BIT MASK IN ACCORDANCE WITH SWITCH NODE CONDITIONS
    if (pcInstance->SwitchNode[0].SyncDrive == false)
    { // Check if switch nodes are supporting synchronous rectification, leave if not
        pcInstance->Properties.AsyncMode = true;
    }
    else { /* do nothing */ } // if sync_drive = true, allow user setting of async_mode pass through
    
    if (pcInstance->Properties.AsyncMode == false)
    { // Select synchronous mode, ignoring swapped mode
        iocon.bits.OVRENH = false; // Enable signal output
        iocon.bits.OVRENL = false; // Enable signal output
    }
    else if (pcInstance->Properties.AsyncMode == true)
    { // Select non-swapped asynchronous mode
        iocon.bits.OVRENH = false; // Enable signal output
        iocon.bits.OVRENL = true;  // Disable signal output
    }
    else { return(0); } // Something is off
    
    // Set ASYNC_ACTIVE status flag bit
    pcInstance->Status.bits.AsyncActive = pcInstance->Properties.AsyncMode;
    
    // Apply settings to PWM module only if PWM outputs are HOT
    if (pcInstance->Status.bits.PwmOutputActive)
    {
        // Set I/O configuration across all PWM channels of the recent switch-node configuration
        for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
        {
            // If user has swapped PWM outputs, also swap override bits
            iocon.bits.OVRENH ^= (uint16_t)(pcInstance->SwitchNode[i].SwapOutputs);
            iocon.bits.OVRENL ^= (uint16_t)(pcInstance->SwitchNode[i].SwapOutputs);
            
            // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
            P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;
            pg->PGxIOCONL.bits.OVRENH = iocon.bits.OVRENH; // Set IO configuration of PWMxH
            pg->PGxIOCONL.bits.OVRENL = iocon.bits.OVRENL; // Set IO configuration of PWMxL

            retval &= (pg->PGxIOCONL.bits.OVRENH == iocon.bits.OVRENH); // Check PWMxH state
            retval &= (pg->PGxIOCONL.bits.OVRENH == iocon.bits.OVRENL); // Check PWMxL state

        }
    }
    else { retval = true; }

    return(retval);  
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function writes the given duty ratio value to all PWM channels
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @param   dcValue     Duty cycle value of type unsigned integer
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  The switch node configuration supports single and multiphase drives. This 
 *  function writes the duty cycle values declared in the switch node configuration
 *  to all PWM channels.
 *
 *********************************************************************************/
int BdbbPWM_SetDutyCycle(IBdbbConverter_t* pcInstance, uint16_t dcValue)
{
    int retval = 1;
    
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
    {
        // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
        P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[i].PwmInstance;    
        pg->PGxDC.value = dcValue;
        pg->PGxSTAT.bits.UPDREQ = 1;
    }
    
    return(retval);  
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-pwm-functions
 * @brief   This function sets the duty cycle off-time complement
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  Instantaneous duty ratio off-time complement of type unsigned integer
 *  
 * @details
 *   When performing a hot-swap of the power transfer direction, the control 
 *   loop output must be pre-charged with the duty ratio off-time complement
 *   value to prevent artificial control transient glitches. This function 
 *   returns the duty ratio off-time complement of the leading control PWM 
 *   signal. 
 * 
 *   <b>Example:</b><br>
 *   When operating in forward bidirectional buck-boost converter mode at a duty ratio of 30%,
 *   the off-time complement would be 70%. At a period of 10,000 ticks, the
 *   duty ratio would be 3,000 ticks, at which this function would return
 *   the value of 7,000 ticks.
 *   
 *********************************************************************************/
uint16_t BdbbPWM_GetDutyCycleComplement(IBdbbConverter_t* pcInstance) 
{
    // CAPTURE MEMORY ADDRESS OF GIVEN PWM GENERATOR INSTANCE
    P33C_PWM_INSTANCE_t* pg = pcInstance->SwitchNode[0].PwmInstance;
    uint16_t _pgRet=0;
    
    if (pg->PGxPER.value > pg->PGxDC.value)
        _pgRet = (pg->PGxPER.value - pg->PGxDC.value);
    else { _pgRet = 0; }

    return(_pgRet);
} 


// ______________________________________
// end of file
