/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_ptemp_adc.h
 * @author  M91406
 * @brief   ADC Peripheral Special Function Register Configuration Template
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_PERIPHERAL_CONFIGURATION_ADC_H
#define	BDBB_CONVERTER_PERIPHERAL_CONFIGURATION_ADC_H

#include <xc_pral.h> // include device peripheral register abstraction layer

/* ********************************************************************************
 * ADC BASE MODULE BASE REGISTERS CONFIGURATION
 * *******************************************************************************/
/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   Digital-To-Analog Converter and High Speed Comparator Special Function Register Set
 */  
#define ADC_POWERUP_TIMEOUT  30000
/* SLPxDAT: DACx SLOPE DATA REGISTER => Timing Register: User value will be set in code */

// ADC module base register configurations
/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON1L ADC module register setting
 * @details
 *  Register ADCON1L covers the enable bit and ADc operation in Idle mode of the ADC module
 *   
 *  - Enable: off by default
 *  - IdleMode: continues operation in Idle Mode
 */
#define ADC_MODULE_ADCON1L      0b0000000000000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON1H ADC module register setting
 * @details
 *  Register ADCON1H covers the output data format and data resolution of the ADC module
 *   
 *  - Output = Integer Format
 *  - Resolution = 12 bit
 */
#define ADC_MODULE_ADCON1H      0b0000000001100000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON2L ADC module register setting
 * @details
 *   Register ADCON2L covers general interrupt settings and shared ADC core input clock configuration 
 *   of the ADC module.
 *   
 *   - Shared ADC Core Sample Time Selection: 3 TADCORE
 */
#define ADC_MODULE_ADCON2L      0b0001011100000001

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON2H ADC module register setting
 * @details
 *  Register ADCON2H covers the shared ADC core sampling time and bandgap reference status 
 *  bits of the ADC module
 *   
 *  - Band Gap and Reference Voltage Ready Flag (Status bit, set and cleared by hardware)
 *  - Band Gap or Reference Voltage Error Flag (Status bit, set and cleared by hardware)
 *  - Shared ADC Core Sample Time Selection: 10x Tad (=160ns @ 16ns Tad)
 */
#define ADC_MODULE_ADCON2H      0b0000000000001010

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON3L ADC module register setting
 * @details
 *  Register ADCON3L covers the shared ADC core sampling time and bandgap reference status 
 *  bits of the ADC module
 *   
 *  - Band Gap and Reference Voltage Ready Flag (Status bit, set and cleared by hardware)
 *  - Band Gap or Reference Voltage Error Flag (Status bit, set and cleared by hardware)
 *  - Shared ADC Core Sample Time Selection: 8x Tad (=128ns @ 16ns Tad) (= binary value +2)
 */
#define ADC_MODULE_ADCON3L      0b0000000000000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON3H ADC module register setting
 * @details
 *  The upper 8 bits of the Special Function Register (SFR) ADCON3H of dsPIC33C devices is used 
 *  to set the input clock to the ADC module and the clock divider. This clock is provided to each 
 *  ADC core (dedicated and shared), where it can be individually scaled by independent core clock 
 *  dividers (ADCS) in register ADCON2L for the shared ADC core or the respective ADCORExH registers.
 *  The maximum Tad of each core is limited to 14.3ns (=70 MHz). Due to dependencies across clock
 *  domains of various peripherals, the Tad is set to 16ns (=62.5 MHz) for all ADC cores in this 
 *  default configuration.
 *  
 *  Default configuration:
 * 
 *  - Source Clock Selection: Auxiliary PLL VCO Divider = 250 MHz (expecting AFVCO = 1000 MHz, AFVCODIV = 4)
 *  - ADC Module Clock Divider of 1:2 (output frequency = 125 MHz)
 *  - ADC core clock dividers of 1:2 (Fad = 62.5 MHz; Tad = 16 ns)
 
 *  - Effective conversion time:
 *    - Each channel of approx. 295 ns (sampling time not included)
 *    - Sampling Time Dedicated Cores: 2 TAD  (= 32 ns)
 *    - Sampling Time Shared Core: 8 TAD (= 128 ns sampling delay)
 *    - Early interrupts of all ADC inputs is enabled and set to 8x TADs, raising interrupts 128 ns prior to conversion is complete
 * 
 *  Dedicated and shared cores are disabled by default and will be enabled by the initialization
 *  routine.
 */
#define ADC_MODULE_ADCON3H      0b1000000100000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON4L ADC module register setting
 * @details
 *   Register ADCON4L covers the Conversion Delay Enable of all dedicated ADC Cores 
 *   of the ADC module
 *   
 *   - Conversion Delay of all ADC cores is disabled by default (immediate conversion)
 */
#define ADC_MODULE_ADCON4L      0b0000000000001111

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON4H ADC module register setting
 * @details
 *   Register ADCON4H covers the Input Channel Selection of all dedicated ADC Cores 
 *   of the ADC module
 *   
 *   - All dedicated ADC core inputs are reset to their default input channels
 */
#define ADC_MODULE_ADCON4H      0b0000000000000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON5L ADC module register setting
 * @details
 *   Register ADCON5L covers the Power Enable and ADC Core Ready bits of all ADC cores 
 *   of the ADC module
 *   
 *   - Shared/Dedicated ADC Core Ready Flags are status bits set/cleared by hardware
 *   - Shared/Dedicated ADC Core Power Enable: Power to all ADC cores is off by default
 */
#define ADC_MODULE_ADCON5L      0b0000000000000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCON5H ADC module register setting
 * @details
 *   Register ADCON5H covers the the warmup time and common interrupt configuration
 *   of all ADC cores of the ADC module. Warm-up time sets the number of TCORESRC cycles.
 *   At default clock configuration TCORESRC = 8 ns (125 MHz after ADC clock divider)
 *   
 *   - ADC Dedicated Core x Power-up Delay: Warmup time is set to 2048 clock periods = 16.4 �s
 *   - All dedicated ADC core inputs are reset to their default input channels
 */
#define ADC_MODULE_ADCON5H      0b0000101100000000


// ADC specific macro declarations
/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks all cores of ADC
 */
#define ADC_CORE_MASK           0b0000000010001111

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 0 of ADC
 */
#define ADC_CORE0_MASK_INDEX    0b0000000000000001

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 1 of ADC
 */
#define ADC_CORE1_MASK_INDEX    0b0000000000000010

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 2 of ADC
 */
#define ADC_CORE2_MASK_INDEX    0b0000000000000100

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 3 of ADC
 */
#define ADC_CORE3_MASK_INDEX    0b0000000000001000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 4 of ADC
 */
#define ADC_CORE4_MASK_INDEX    0b0000000000010000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 5 of ADC
 */
#define ADC_CORE5_MASK_INDEX    0b0000000000100000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks core 6 of ADC
 */
#define ADC_CORE6_MASK_INDEX    0b0000000001000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define masks shared core of ADC
 */
#define ADC_SHRCORE_MASK_INDEX  0b0000000010000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This macro masks out clock settings of ADC configuration register 3 high (ADCON3H)
 */
#define ADC_CORE_ENABLE_MASK    0b0000000011111111

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief This macro masks out clock settings and the shared core of ADC configuration register 3 high (ADCON3H)
 */
#define ADC_CORE_DISABLE_MASK   0b1111111110000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This macro masks out ADC core enable settings of ADC configuration register 3 high (ADCON3H)
 */
#define ADC_CLK_PRESERVE_MASK   0b1111111100000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This macro provides a bit mask used to check if all ADC cores are disabled (ADCON5L)
 */
#define ADC_CORE_OFF_MASK       0b0000000000000000

/**
 * @brief   This define holds the base register configuration of ADCORExL ADC core register setting
 * @details
 *   Register ADCORExL covers the dedicated ADC core sampling time 
 *   of the ADC module.
 *   
 *   - Dedicated ADC Core Sample Time Selection: 2 TADCORE
 */
#define ADC_CORE_ADCORExL       0b0000000000000000

/**
 * @ingroup lib-layer-bdbb-adc-defines
 * @brief   This define holds the base register configuration of ADCORExH ADC core register setting
 * @details
 *   Register ADCORExH covers the dedicated ADC core input clock divider, resolution and
 *   early interrupt configuration of the ADC module
 *   
 *   - ADC Core x Early Interrupt Time Selection: interrupt will be triggered 8 Tad clocks prior to conversion being complete
 *   - ADC Core x Resolution Selection: 12-bit
 *   - ADC Core x Input Clock Divider: 1:2 (FCORE = 62.5 MHz, Tad = 16ns)
 */
#define ADC_CORE_ADCORExH       0b0001111100000001

/* ********************************************************************************
 * ADC INPUT INSTANCE CONFIGURATION
 * *******************************************************************************/

/* REGISTER VALUE DECLARATION TEMPLATE */
/* PCLKCON: PWM CLOCK CONTROL REGISTER

                           ________________ BIT 15: HRRDY: High-Resolution Ready bit
                          | _______________ BIT 14: HRERR: High-Resolution Error bit
                          || ______________ BIT 13: (unimplemented) 
                          ||| _____________ BIT 12: (unimplemented) 
                          |||| ____________ BIT 11: (unimplemented) 
                          ||||| ___________ BIT 10: (unimplemented)
                          |||||| __________ BIT  9: (unimplemented)
                          ||||||| _________ BIT  8: LOCK: Lock bit
                          |||||||| ________ BIT  7: (unimplemented)
                          ||||||||| _______ BIT  6: (unimplemented)
                          |||||||||| ______ BIT  5: DIVSEL[1:0]: PWM Clock Divider Selection bits
                          ||||||||||| _____ BIT  4: 
                          |||||||||||| ____ BIT  3: (unimplemented)
                          ||||||||||||| ___ BIT  2: (unimplemented)
                          |||||||||||||| __ BIT  1: MCLKSEL[1:0]: PWM Master Clock Selection bits
                          ||||||||||||||| _ BIT  0: 
                          ||||||||||||||||  */
        /*
#define REG_PCLKCON     0b0000000000000011   */


/****************************************************************************************************
 * @ingroup lib-layer-bdbb-adc-templates
 * @brief   ADC module register set default configuration
 *****************************************************************************************************/
static const P33C_ADC_t bdbbAdcConfigDefault = {
    
    .ADCON1L.value = ADC_MODULE_ADCON1L,
    .ADCON1H.value = ADC_MODULE_ADCON1H,
    .ADCON2L.value = ADC_MODULE_ADCON2L,
    .ADCON2H.value = ADC_MODULE_ADCON2H,
    .ADCON3L.value = ADC_MODULE_ADCON3L,
    .ADCON3H.value = ADC_MODULE_ADCON3H,
    #ifdef ADCON4L
    .ADCON4L.value = ADC_MODULE_ADCON4L,
    #endif
    .ADCON4H.value = ADC_MODULE_ADCON4H,
    .ADCON5L.value = ADC_MODULE_ADCON5L,
    .ADCON5H.value = ADC_MODULE_ADCON5H,

    .ADCORE0L.value = ADC_CORE_ADCORExL,
    .ADCORE0H.value = ADC_CORE_ADCORExH,
    .ADCORE1L.value = ADC_CORE_ADCORExL,
    .ADCORE1H.value = ADC_CORE_ADCORExH

};

/****************************************************************************************************
 * @ingroup lib-layer-bdbb-adc-templates
 * @brief   ADC module default configuration
 *****************************************************************************************************/
static const P33C_ADC_MODULE_t bdbbAdcModuleConfig = {
    
    .ADCON.ADON = false,        // = ADC turned off by default
    .ADCON.ADSIDL = false,      // = Continue operation in Idle Mode
    .ADCON.CLKSEL = 0b11,       // = APLLVCODIV
    .ADCON.CLKDIV = 2,          // = 1:2
    .ADCON.EIEN = true,         // = Enable Early Interrupts
    .ADCON.FORM = 0,            // = Result format = Integer (right-justified)
    .ADCON.WARMTIME = 0b1011,   // = 2048 TCORESRC periods
    
    .REFCON.REFCIE = false,     // = Band Gap and Reference Voltage Ready Common Interrupt disabled 
    .REFCON.REFERCIE = false,   // = Band Gap or Reference Voltage Error Common Interrupt disabled 
    .REFCON.REFSEL = 0b000,     // = ADC Reference Voltage Selection = default
    .REFCON.REFERR = 0,         // = Clear ADC Reference Voltage Error flag bit
    .REFCON.REFRDY = 0,         // = Clear ADC Reference Voltage Ready flag bit 
    
    .TRGCON.CNVCHSEL = 0,       // = AN0 is default Channel Number Selection for Software Individual Channel Conversion Trigger
    .TRGCON.SUSPCIE = false,    // = Common interrupts of all ADC cores are disabled
    .TRGCON.SUSPEND = false,    // = Clear All ADC Core Triggers Disable bit
    .TRGCON.SUSPRDY = false,    // = Clear All ADC Cores Suspended Flag bit
    .TRGCON.SWCTRG = false,     // = Clear Software Common Trigger bit
    .TRGCON.SWLCTRG = false,    // = Clear Software Level-Sensitive Common Trigger bit
    .TRGCON.SHRSAMP = false,    // = Clear Shared ADC Core Sampling Direct Control bit
    .TRGCON.CNVRTCH = false,    // = Clear Software Individual Channel Conversion Trigger bit
    .TRGCON.PTGEN = false       // = Clear External Conversion Request Interface bit
};

/****************************************************************************************************
 * @ingroup lib-layer-bdbb-adc-templates
 * @brief   ADC input instance default configuration
 *****************************************************************************************************/
static const P33C_ADCAN_INSTANCE_t bdbbAdcAnConfigDefault = {

    .Enable = false, // ADC input channel disabled by default
    .IE = false,    // ADC input interrupt disabled by default
    .EIE = true,    // Enable Early Interrupts
    .TRGSRC = 0,    // Clear trigger source (will be set in software)
    .DIFF = false,  // Single-Ended input channel
    .SIGN = false,  // Unsigned result
    .LVLEN = false, // ADC input level trigger mode disabled
    .ANxRDY = 0     // Clear ADC input READY bit
};

#endif  /* BDBB_CONVERTER_PERIPHERAL_CONFIGURATION_ADC_H */


// ______________________________________
// end of file
