/**
 * @file    svc_bdbb_opstates.c
 * @brief   Bidirectional Buck-Boost Converter common state machine operating states
 * @author  M91406
 */

#include "svc_bdbb_subtasks.h" // include bidirectional buck-boost converter special function declarations
#include "../../IBdbbPControl.h" // include bidirectional buck-boost converter peripheral configuration declarations
#include "../../IBdbbConverter.h" // include bidirectional buck-boost converter data object declarations

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   This function resets the counters and conditional flag bits.
 * @param   pcInstance  Pointer to a bidirectional buck-boost data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 *
 * @details
 *   If the controller has not been run yet, the POWER ON and POWER GOOD delay
 *   counters are reset and all conditional flag bits are cleared. Status of 
 *   power source, ADC and current sensor calibration have to be set during
 *   runtime by system check routines. 
 *********************************************************************************/
int State_Initialize(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Disable control loops
    retval &= BdbbLoopControl_Off(pcInstance);

    // Reset all status bits
    pcInstance->Status.value &= (BDBB_STAT_FAULT_LATCH | BDBB_STAT_ADC_ACTIVE | BDBB_STAT_PWM_ACTIVE);  // Clear all status flags except FAULT:LATCH and PWM_ACTIVE
    pcInstance->Status.bits.IsSuspended = true; // Set IS_SUSPENDED flag bit to force power supply into safe off-state
    pcInstance->Status.bits.FaultStatus = true; // Set FAULT_ACTIVE flag bit expecting it to be cleared by fault handler
    
    // Initiate current sensor calibration status flag bit
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_VMC)
        pcInstance->Status.bits.CsCalibComplete = true; 
    else if (pcInstance->Properties.CsCalibEnable)
        pcInstance->Status.bits.CsCalibComplete = false; 
    else 
        pcInstance->Status.bits.CsCalibComplete = true; 
    
    // Set INITIALIZED flag bit indicating state machine has passed INITIALIZE state
    pcInstance->Status.bits.IsInitialized = true; 
    
    // If any sub-function call went unsuccessful, reset state machine
    // else, trigger transition to next state
    if (retval) return(BDBB_OPSRET_COMPLETE); // Trigger transition to next state
    else return(BDBB_OPSRET_ERROR);  // Invoke state machine reset if error occurred
    
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   This function resets the bidirectional buck-boost converter control 
 *          operation by re-initiating the control mode, references and status bits.
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *   After successful initialization or after an externally triggered state machine reset,
 *   the state machine returns to this RESET mode, re-initiating control mode, references 
 *   and status bits before switching further into STANDBY mode. 
 *********************************************************************************/
int State_Reset(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Disable all PWM outputs & control loops (immediate power cut-off)
    retval &= BdbbPWM_Suspend(pcInstance); // Disable PWM outputs

    // Disable control loops
    retval &= BdbbLoopControl_Off(pcInstance);
    
	// Reset Relay output
    retval &= BdbbGPIO_ClearRelay(pcInstance);
	
    // Reset Power Good output
    retval &= BdbbGPIO_ClearPowerGood(pcInstance);
    
    // Clear DAC or PWM target registers
    switch (pcInstance->Control.ControlMode)
    {
        case BDBB_CONTROL_MODE_PCMC:
            retval &= BdbbDAC_Reset(pcInstance); // Reset switch-node DAC/CMP registers
            break;
        
        default:
            retval &= BdbbPWM_Reset(pcInstance); // Reset PWM registers
            break;
    }
    
    // Disable control loops reset control loop histories
    retval &= BdbbLoopControl_Reset(pcInstance);
    
    // Reset runtime data
    retval &= BdbbConverter_ResetRuntimeData(pcInstance);
    
    // Reset all startup counters
    retval &= BdbbConverter_ResetProcessCounters(pcInstance);
            
    // Set initial power conversion transfer direction
    retval &= BdbbConverter_SetDirectionSwap(pcInstance);

    // Reset all process monitoring status bits
    pcInstance->Status.bits.LoopCtrlActive = false; // Clear LOOP_CTRL_ACTIVE bit to verify feedback loop ISR operation
    pcInstance->Status.bits.Busy = false; // Clear BUSY bit
    pcInstance->Status.bits.Active = false; // Clear ACTIVE bit
    pcInstance->Status.bits.Ready = true; // Set READY flag bit indicating state machine has passed RESET state
    pcInstance->Status.bits.Online = false;  // Reset ONLINE bit
            
    // If any sub-function call went unsuccessful, reset state machine
    // else, trigger transition to next state
    if (retval) return(BDBB_OPSRET_COMPLETE); // Trigger transition to next state
    else return(BDBB_OPSRET_ERROR);  // Invoke state machine reset if error occurred
    
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   This function delays the startup until the Power-on Delay has expired 
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  After the converter has been cleared to get started, the power-on
 *  delay counter is incremented until the defined power-on delay period
 *  has expired.
 **********************************************************************************/
int State_PowerOnDelay(void* handle)
{
    BDBB_OPSTATE_RETURNS_t retval = BDBB_OPSRET_ERROR;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    if(pcInstance->Properties.PowerOnDelayByPass){
        pcInstance->Startup.PowerOnDelay.Counter = 0xFFFE;
        pcInstance->Properties.PowerOnDelayByPass = 0;
    }
    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit
    
    // Delay startup until POWER ON DELAY PERIOD has expired
    if(pcInstance->Startup.PowerOnDelay.Counter++ > pcInstance->Startup.PowerOnDelay.Period)
    {
        // Power On Delay has expired
        pcInstance->Status.bits.PodComplete = true;
        retval = BDBB_OPSRET_COMPLETE; 
    }
    else
    // When the period has not expired yet, return to this function
    {
        // Power On Delay is being enforced
        pcInstance->Status.bits.PodComplete = false;
        retval = BDBB_OPSRET_REPEAT;
    }    

    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   This function waits until all start-up conditions are met.
 * @param   pcInstance  Pointer to a bi-directioanl buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  After a successful state machine reset, the state machine waits in  
 *  STANDBY mode until all conditional flag bits are set/cleared allowing  
 *  the converter to run.
 *
 *********************************************************************************/
int State_Standby(void* handle)
{
    BDBB_OPSTATE_RETURNS_t retval = BDBB_OPSRET_ERROR;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    pcInstance->Status.bits.Busy = false; 
    
    // if the 'Autorun' option is set, the GO bit is automatically set when
    // the converter is enabled
    if ((pcInstance->Properties.Enable) && (pcInstance->Properties.Autorun))
    { pcInstance->Properties.Go = true;}

    // Wait for all startup conditions to be met
    if ((pcInstance->Properties.Enable          == true)  &&  // State machine must be enabled
        (pcInstance->Properties.Go              == true)  &&  // GO-bit must be set
        (pcInstance->Status.bits.LoopCtrlActive == true)  &&  // Feedback Loop ISR must be running
        (pcInstance->Status.bits.AdcActive      == true)  &&  // ADC must be running
        (pcInstance->Status.bits.PwmActive      == true)  &&  // PWM must be running 
        (pcInstance->Status.bits.FaultStatus    == false) &&  // No active fault must be present
        (pcInstance->Status.bits.IsSuspended    == false)     // Power supply is not held in suspend mode
        )
    {
        pcInstance->Properties.Go = false; // Clear GO bit
        pcInstance->Status.bits.Active = true; // Set ACTIVE bit
        pcInstance->Status.bits.Busy = true; 
        
        retval = BDBB_OPSRET_COMPLETE;
    }
    else
    // Remain in current state until bit-test above becomes true
    { retval = BDBB_OPSRET_REPEAT; }

   return(retval);
    
}

// STARTUP STATES ========================================

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   In Boost mode only, this function monitors the output capacitor voltage waiting until the voltage has settled
 * @param	pcInstance  Pointer to a boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  The bdbb converter output capacitor is by default automatically charged 
 *  through the bdbb rectifier resp. the high-side MOSFET body diode. Before 
 *  the power supply is started, the state machine waits for the output
 *  capacitor voltage to settle to reduce component stress during startup.
 *
 **********************************************************************************/
int State_VCapMonitor(void* handle)
{
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Set BUSY bit until process is complete
    pcInstance->Status.bits.Busy = true;
    
    // Skip this state in step-down/buck mode, 
    if (false == pcInstance->Properties.ConversionMode)
        return(BDBB_OPSRET_SKIP);
    
    // Capture most recent parameters
    int32_t _vlv = __builtin_mulss(pcInstance->Data.VLowVoltage, pcInstance->Feedback.AdVLowVoltage.Scaling.Factor);
    int32_t _vhv = __builtin_mulss(pcInstance->Data.VHighVoltage, pcInstance->Feedback.AdVHighVoltage.Scaling.Factor);
    int32_t _vth = __builtin_mulss(pcInstance->Startup.VCapMonitor.Threshold, pcInstance->Feedback.AdVHighVoltage.Scaling.Factor);
        
    // Capture compare values and check if output voltage has charged up into VDrop window
    bool _chargeComplete = (bool)(_vhv > (_vlv - _vth));

    Nop();
    Nop();
    Nop();
    
    // Enforce minimum monitoring delay
    if (true == _chargeComplete) // If output voltage has settled...
    {
        // Monitor settled voltage for the given period of time
        if (++pcInstance->Startup.VCapMonitor.DelayCounter > 
            pcInstance->Startup.VCapMonitor.DelayPeriod)
        { 
            Nop();  // For Debugging Breakpoint
            Nop();
            Nop();

            return(BDBB_OPSRET_COMPLETE);
        }
    }
    else // If output voltage is still out of range...
    {
        // Clear delay counter if value is out of range
        pcInstance->Startup.VCapMonitor.DelayCounter = 0; 
    }

    // Timeout fuse
    if (pcInstance->Startup.VCapMonitor.TimeoutCounter++ > 
        pcInstance->Startup.VCapMonitor.TimeoutPeriod)
    { 

        Nop();  // For Debugging Breakpoint
        Nop();
        Nop();

        return (BDBB_OPSRET_ERROR);
    }
    
    // Recall this sub-state
    return(BDBB_OPSRET_REPEAT);
         
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   In this function, a counter is incremented until the relay delay has expired.
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  In this phase of the soft-start procedure the relay is immediately turned ON
 *  and a delay was introduce to allow the capacitor to charge up until the period
 *  has expired before the power supply is started.
 *  If option for driving a user-defined general purpose output (PG output) is enabled
 *  in proprietary user code, this pin will be set automatically.
 *
 **********************************************************************************/
int State_Relay(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Skip this state in step-down/buck mode, 
    if (false == pcInstance->Gpio.EnableRelay.Enabled)
        return(BDBB_OPSRET_SKIP);
    
    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit
    
    // Set Relay output
    retval &= BdbbGPIO_SetRelay(pcInstance);

    // increment delay counter until the Relay delay has expired
    if(pcInstance->Startup.RelayDelay.Counter++ > pcInstance->Startup.RelayDelay.Period)
    {
        pcInstance->Startup.RelayDelay.Counter = 
            (pcInstance->Startup.RelayDelay.Period + 1); // Clamp to PERIOD_EXPIRED for future startups

        if(!retval)
            retval = BDBB_OPSRET_ERROR;
        else
            retval = BDBB_OPSRET_COMPLETE;
    }
    else
    // if period has not expired yet, remain in this state
    { retval = BDBB_OPSRET_REPEAT; }

    return(retval);
    
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-substates-functions
 * @brief   This function calculate and pre-charge PWM outputs with ideal duty cycle 
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  After the POWER ON DELAY has expired, the ramp up starting point is determined  
 *  by measuring the input and output voltage and calculates the ideal duty ratio 
 *  of the PWM. This value is then programmed into the PWM module duty cycle register 
 *  and is also used to pre-charge the control loop output history. In addition the 
 *  measured output voltage also set as reference to ensure the loop starts without 
 *  jerks and jumps.
 *  
 *  When voltage mode control is enabled, the voltage loop control history is 
 *  charged, when average current mode control is enabled, the current loop control 
 *  history is charged.
 *
 **********************************************************************************/
int State_PrepareVRampUp(void* handle)
{
    int retval = 1;     // Function return value
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Pre-biased output startup point estimation
    uint16_t _start_dc=0;  // Startup-Duty Cycle

    // Open Loop Startup Variables
    uint16_t _minLimit=0;  // buffer variable capturing most recent maximum control loop output limit
    uint16_t _maxLimit=0;  // buffer variable capturing most recent maximum control loop output limit
    uint32_t _ol_step=0;   // Control output step size in open loop

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit
    
    // Recheck the power transfer direction and copy user setting of voltage reference into loop reference
    retval &= BdbbConverter_SetDirectionSwap(pcInstance);
    retval &= BdbbControl_UpdateUserReferences(pcInstance);
    
    // Capture conversion mode control flag bit for soft- and hot-swap 
    // comparisons performed later in state ONLINE
    pcInstance->Status.bits.SwapDetect = pcInstance->Properties.ConversionMode;

    // Hijack voltage and current loop controller references
    retval &= BdbbControl_SetVRampReferencePointers(pcInstance);
    
    // Sanity-Check voltage and current ramp increment values specified by user
    if (pcInstance->Startup.LVRamp.StepSize == 0) pcInstance->Startup.LVRamp.StepSize = 1;
    if (pcInstance->Startup.HVRamp.StepSize == 0) pcInstance->Startup.HVRamp.StepSize = 1;
    if (pcInstance->Startup.IRamp.StepSize == 0) pcInstance->Startup.IRamp.StepSize = 1;
    
    // Estimate required startup control output for seamless startup of pre-biased outputs
    _start_dc = BdbbControl_GetDutyRatio(pcInstance, pcInstance->Data.VHighVoltage, pcInstance->Data.VLowVoltage);
    
    // Clear loops and pre-charge PWM and control loop history
    switch (pcInstance->Control.ControlMode)
    {
        case BDBB_CONTROL_MODE_VMC:

            // Program PWM module and pre-charge control loop
            _minLimit = BdbbLoopControl_GetLoopMinLimit(pcInstance->Control.VLoop.Controller);
            _maxLimit = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.VLoop.Controller);
            
            if(_start_dc < _minLimit) _start_dc = _minLimit; 
            else if(_start_dc > _maxLimit) _start_dc = _maxLimit; 
            else { /* continue */ }
                
            // Pre-Charge voltage loop controller with calculated starting point
            pcInstance->Control.VLoop.Methods.Precharge(pcInstance->Control.VLoop.Controller, 0, _start_dc);
            
            // pre-charge PWM outputs with ideal duty cycle
            BdbbPWM_SetDutyCycle(pcInstance, _start_dc);

            retval = BDBB_OPSRET_COMPLETE;
            break;
        
        // In peak current mode, set duty cycle to max to allow the comparator take control
        case BDBB_CONTROL_MODE_PCMC:

            // Set duty cycle limit to allow comparator control
            for (int i=0; i<pcInstance->Properties.PhaseCount; i++) {
                
                BdbbPWM_SetDutyCycle(pcInstance, pcInstance->SwitchNode[i].DutyRatioMaximum);
                
            }
            
            // Reset control loop to ensure voltage feedback loop always starts with minimum output
            pcInstance->Control.VLoop.Methods.Reset(pcInstance->Control.VLoop.Controller);
            
            retval = BDBB_OPSRET_COMPLETE;
            break;
        
        // In average current mode, set current reference limit to max startup current level
        case BDBB_CONTROL_MODE_ACMC:
            
            // Set startup current reference limit
            if (!pcInstance->Control.VLoop.Controller->Status.bits.SwapTarget)
                pcInstance->Control.VLoop.Properties.Maximum = pcInstance->Startup.IRamp.Start;
            else
                pcInstance->Control.VLoop.Properties.AltMaximum = pcInstance->Startup.IRamp.Start;
            
            BdbbLoopControl_SetLoopMaxLimit(
                pcInstance->Control.VLoop.Controller, 
                pcInstance->Startup.IRamp.Start);
        
            // Limit startup duty cycle to largest minimum/smallest maximum
            for (int i=0; i<pcInstance->Properties.PhaseCount; i++) {

                _minLimit = BdbbLoopControl_GetLoopMinLimit(pcInstance->Control.ILoop[i].Controller);
                _maxLimit = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.ILoop[i].Controller);

                if(_start_dc < _minLimit) _start_dc = _minLimit; 
                else if(_start_dc > _maxLimit) _start_dc = _maxLimit; 
                else { /* continue */ }

            }

            // Set initial duty cycle
            for (int i=0; i<pcInstance->Properties.PhaseCount; i++) 
            {
                // pre-charge current loop control histories with ideal duty cycle
                pcInstance->Control.ILoop[i].Methods.Precharge(
                            pcInstance->Control.ILoop[i].Controller, 0, _start_dc
                        );

                // Set initial PWM duty ratio
                BdbbLoopControl_SetLoopOutput(pcInstance->Control.ILoop[i].Controller, _start_dc);
            }
            
            retval = BDBB_OPSRET_COMPLETE;
            break;
            
        default:
            retval = BDBB_OPSRET_ERROR;
            break;
    }
    
    // If converter needs to be started in open loop, calculate the open loop control
    // output increment steps size, skip in reverse boost mode
    if ((true == pcInstance->Startup.OpenLoopStart.Enabled) && (false == pcInstance->Properties.ConversionMode))
    {
        _ol_step = BdbbControl_GetOpenLoopDutyRatioStep(pcInstance);
        if (_ol_step == 0) _ol_step = 1;
        pcInstance->Startup.OpenLoopStart.StepSize = _ol_step;
    }

    return(retval);
    
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-substates-functions
 * @brief   This function injects a time delay compensating for pre-biased startup voltages
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  The startup ramp timing in sequencing schemes is references to an output 
 *  voltage of zero. If the output is pre-biased, the ramp would start early,
 *  eventually shortening, and thus violating, the desired startup ramp period.
 *  This state injects an additional startup delay postponing the start to the
 *  voltage ramp to keep the overall startup timing in the limits specified
 *  by the user.
 *
 **********************************************************************************/
int State_VRampBiasCompensation(void* handle)
{
    int retval = 1;     // Function return value
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit

    volatile BDBB_REFERENCE_RAMP_HANDLER_t* _refHandler; 
    
    // Triage voltage ramp to high- or low- voltage port
    if (false == pcInstance->Properties.ConversionMode)
    { _refHandler = &(pcInstance->Startup.LVRamp); }
    else
    { _refHandler = &(pcInstance->Startup.HVRamp); }
    
    // Increment voltage ramp-up counter
    _refHandler->Counter++;

    if ((_refHandler->preDelayBuffer + _refHandler->StepSize) > _refHandler->Start)
    {
        _refHandler->Counter = 0;           // Reset ramp counter
        _refHandler->preDelayBuffer = 0;    // REset bias-compensation auxiliary data buffer
        retval = BDBB_OPSRET_COMPLETE;      // Set State = COMPLETE
    }
    else
    {   // Increment voltage reference
        _refHandler->preDelayBuffer += _refHandler->StepSize;
        retval = BDBB_OPSRET_REPEAT;
    }
    
    return(retval);
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-substates-functions
 * @brief   This function ramps up the output voltage in open loop mode up to the declared cut off level
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details // To-Do: add details
 * 
 **********************************************************************************/
int State_OpenLoopVRampUp(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit

    // Check if open loop startup step must be executed
    bool _runOpenLoop=false;
    
    if (false == pcInstance->Properties.ConversionMode) // Buck mode
    {
        _runOpenLoop = (bool)(
            (pcInstance->Startup.OpenLoopStart.Enabled == true) && 
            (pcInstance->Control.ControlMode != BDBB_CONTROL_MODE_PCMC) && 
            (pcInstance->Data.VLowVoltage < (pcInstance->Startup.OpenLoopStart.CutOffVoltage +
                                             pcInstance->Startup.OpenLoopStart.CutOffHysteresis))
            );
    }
    else
    {
        _runOpenLoop = (bool)(
            (pcInstance->Startup.OpenLoopStart.Enabled == true) && 
            (pcInstance->Control.ControlMode != BDBB_CONTROL_MODE_PCMC) && 
            (pcInstance->Data.VHighVoltage < (pcInstance->Startup.OpenLoopStart.CutOffVoltage +
                                             pcInstance->Startup.OpenLoopStart.CutOffHysteresis))
            );
    }
    
    // ============================================================================
    // Skip state if:
    //    - open loop startup is disabled
    //    - PCMC is selected  (ToDo: Evaluate implementation of Open Loop Startup in PCMC)
    //    - output voltage is above open loop cut-off level
    // 
    //ToDo: add protection against corrupted signals
    if (false == _runOpenLoop)
    {
        // Set and enable switch node
        retval &= BdbbPWM_Update(pcInstance); // Enforce PWM timing registers to update
        retval &= BdbbPWM_Resume(pcInstance); // Enable PWM outputs
        retval &= BdbbControl_SetVRampReferencePointers(pcInstance); // Update voltage ramp reference

        // Enable control loop
        if(retval) 
        {
            // Bypass loop enable procedure if converter is in enforced open loop mode
            if (false == pcInstance->Properties.EnforcedOpenLoop)
            {
                // Control loops are enabled
                retval &= BdbbLoopControl_On(pcInstance); // control loop(s)
            }
            
            if(retval) { retval = BDBB_OPSRET_COMPLETE; } // Move on to next state machine step
            else { retval = BDBB_OPSRET_ERROR; } // Trigger state error
            
        }
        else
        { retval = BDBB_OPSRET_ERROR; } // Trigger state error

    }

    // ============================================================================
    // Execute open loop startup step if
    // - open loop startup is enabled
    // - output voltage is below cut-off voltage + hysteresis
    //
    else
    {
        // ============================================================================
        // In first iteration, turn on PWM outputs
        if (pcInstance->Startup.OpenLoopStart.Counter == 0)
        {
            // Set and enable switch node
            retval &= BdbbPWM_Update(pcInstance); // Enforce PWM timing registers to update
            retval &= BdbbPWM_Resume(pcInstance); // Enable PWM outputs
        }

        // Increment output of control loop(s) bypassing control computation
        retval &= BdbbControl_IncrementOpenLoopOutput(pcInstance);

        // Repeat open loop steps until converter output is higher than specified ramp-up minimum
        if(retval) { retval = BDBB_OPSRET_REPEAT; }
        else { retval = BDBB_OPSRET_ERROR; }

        // Increment open loop startup counter
        if (++pcInstance->Startup.OpenLoopStart.Counter >= 0xFFFE)
        { retval = BDBB_OPSRET_ERROR; }
        
    }
    
    return(retval);
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-substates-functions
 * @brief   This function ramps up the output voltage to its nominal regulation point in closed loop operation
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  This is the essential step in which the output voltage is ramped up by 
 *  incrementing the voltage control loop reference. In voltage mode the output  
 *  voltage will ramp up to the nominal regulation point. 
 *  In average current mode the inner loop will limit the voltage as soon as the 
 *  current reference limit is hit and the output is switched to constant current 
 *  mode.
 *
 **********************************************************************************/
int State_ClosedLoopVRampUp(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    uint16_t _valBuf=0;
    uint16_t _valLim=0;
    bool _rampComplete=false;

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit
    
    // Increment voltage reference
    _valBuf = BdbbControl_GetActiveOutputVoltage(pcInstance);
    
    volatile BDBB_REFERENCE_RAMP_HANDLER_t* _refHandler; 
    
    // Triage voltage ramp to high- or low- voltage port
    if (false == pcInstance->Properties.ConversionMode)
    { _refHandler = &(pcInstance->Startup.LVRamp); }
    else
    { _refHandler = &(pcInstance->Startup.HVRamp); }

    // Increment voltage ramp-up counter
    _refHandler->Counter++;

    if (((_refHandler->refBuffer + _refHandler->StepSize) > _refHandler->Stop) || (_valBuf > _refHandler->Stop))
    {
        _refHandler->refBuffer= _refHandler->Stop;
        _rampComplete = true;
    }
    else
    {
        // Increment voltage reference
        _refHandler->refBuffer += _refHandler->StepSize;
    }
    
    // in average current mode control, ramp up current synchronously
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        // Increment maximum current limit
        _valLim = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.VLoop.Controller);
        _valLim += pcInstance->Startup.IRamp.StepSize;
        if (_valLim > pcInstance->Startup.IRamp.Stop) _valLim = pcInstance->Startup.IRamp.Stop;
        
        BdbbLoopControl_SetLoopMaxLimit(pcInstance->Control.VLoop.Controller, _valLim);
    }
    
    // if ramp is complete, exit here
    if (_rampComplete == true)
    {
        // Hand back hijacked reference
        retval &= BdbbControl_UpdateUserReferences(pcInstance);
        retval &= BdbbControl_SetDefaultReference(pcInstance);
        
        // Exit VRAMP state
        retval = BDBB_OPSRET_COMPLETE;
        
    }
    else
    // remain in this state until ramp is complete
    { retval = BDBB_OPSRET_REPEAT; }

    
    return(retval);
    
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-substates-functions
 * @brief   This function is for the average current mode control where the output current is ramped up to nominal current
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  This phase of the soft-start ramp is only executed in average current mode and 
 *  will only take effect when the declared current limit is hit before the nominal 
 *  voltage regulation point. In this case the outer voltage loop will clamp the 
 *  current reference value and the state will be switched to IRampUp, where the 
 *  clamping value of the voltage loop output is ramped up to the declared maximum
 *  current reference value.
 *
 *  This additional state allows powering up the power converter against highly 
 *  capacitive loads preventing overshoots caused by down-stream capacitor inrush 
 *  currents or when the converter is used as battery charger, where this state 
 *  will tune into the constant current charging mode.
 *
 **********************************************************************************/
int State_ClosedLoopIRampUp(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    uint16_t _valBuf=0;

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit

    // in average current mode if voltage limit is hit, increment 
    // voltage loop limit to ramp up current loop
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC)
    {
        // Increment maximum current limit
        _valBuf = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.VLoop.Controller); 
        
        // check if ramp is complete
        if (_valBuf >= pcInstance->Startup.IRamp.Stop) 
        { 
            BdbbLoopControl_SetLoopMaxLimit( pcInstance->Control.VLoop.Controller, pcInstance->Startup.IRamp.Stop); 
            
            retval = BDBB_OPSRET_COMPLETE;
        } 
        else // if ramp is not complete yet, remain in this state 
        { 
            _valBuf += pcInstance->Startup.IRamp.StepSize; 
            
            if (_valBuf > pcInstance->Startup.IRamp.Stop) _valBuf = pcInstance->Startup.IRamp.Stop; 
            
            BdbbLoopControl_SetLoopMaxLimit(pcInstance->Control.VLoop.Controller, _valBuf); 
            
            pcInstance->Startup.IRamp.Counter++; // Increment current ramp-up counter 
            retval = BDBB_OPSRET_REPEAT;
        }
    }    
    else // Skip this state in Non-Average Current Mode Control
    { 
        retval = BDBB_OPSRET_COMPLETE; 
    }

    return(retval);
    
}

// END OF STARTUP STATES ==================================

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   Executes current feedback input channel calibration if enabled
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  If current sense feedback channel calibration is enabled, this function will
 *  oversample the current feedback inputs while the converter is disabled, capturing
 *  the current feedback zero-line.
 * 
 *  If current sense feedback channel calibration is disabled, this state will be
 *  skipped and the state machine moves on to the next state in the next state machine
 *  execution cycle.
 *
 **********************************************************************************/
int State_CsCalibration(void* handle) 
{
    BDBB_OPSTATE_RETURNS_t retval = BDBB_OPSRET_ERROR;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    if (pcInstance->Properties.CsCalibEnable)
    {   
        // Execute current sense calibration
        retval = BdbbConverter_CurrentSenseOffsetCalibration(pcInstance);

        // Set status bits
        pcInstance->Status.bits.Busy = true; // Set BUSY bit
        pcInstance->Status.bits.Active = false; // Clear ACTIVE bit
    }
    else
        // Bypass calibration sense state
        retval = BDBB_OPSRET_COMPLETE; 
        
    switch (retval)
    {
        case BDBB_OPSRET_COMPLETE:
            pcInstance->Status.bits.CsCalibComplete = true; // Set CS Calibration Flag to COMPLETE
            pcInstance->Status.bits.Busy = false; // Clear BUSY bit indicating on-going activity
            break;
            
        case BDBB_OPSRET_REPEAT:
            pcInstance->Status.bits.CsCalibComplete = false; // Set CS Calibration Flag to COMPLETE
            pcInstance->Status.bits.Busy = true; // Set BUSY bit indicating on-going activity
            break;
            
        default:
            pcInstance->Status.bits.CsCalibComplete = false; // Set CS Calibration Flag to COMPLETE
            pcInstance->Status.bits.Busy = false; // Clear BUSY bit indicating on-going activity
            retval = BDBB_OPSRET_ERROR;
            break;
    }
    
    return (retval);
}

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   In this function, a counter is incremented until the power good delay has expired.
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  In this phase of the soft-start procedure a counter is incremented until the 
 *  power good delay has expired before the soft-start process is marked as COMPLETED.
 *  If option for driving a user-defined general purpose output (PG output) is enabled
 *  in proprietary user code, this pin will be set automatically.
 *
 **********************************************************************************/
int State_PowerGoodDelay(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Set converter status
    pcInstance->Status.bits.Busy = true; // Set BUSY bit until process is complete
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit

    // increment delay counter until the GOWER GOOD delay has expired
    if(pcInstance->Startup.PowerGoodDelay.Counter++ > pcInstance->Startup.PowerGoodDelay.Period)
    {
        pcInstance->Startup.PowerGoodDelay.Counter = 
            (pcInstance->Startup.PowerGoodDelay.Period + 1); // Clamp to PERIOD_EXPIRED for future startups

        // Set POWER_GOOD output
        retval &= BdbbGPIO_SetPowerGood(pcInstance);
        
        if(!retval)
            retval = BDBB_OPSRET_ERROR;
        else
            retval = BDBB_OPSRET_COMPLETE;
    }
    else
    // if period has not expired yet, remain in this state
    { retval = BDBB_OPSRET_REPEAT; }

    return(retval);
    
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   This function tunes the controller reference to the new user control reference level.
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 * 
 * @details
 *  After soft-start and when state POWER_GOOD_DELAY has expired, the converter 
 *  enters normal operation.
 * 
 * a) Control Reference Changes
 * 
 *  During normal operation the state machine scans the user reference setting. 
 *  Once a change between the user reference setting and the most recent controller 
 *  reference has been detected, the state machine will inject a transition ramp from
 *  the most recent controller reference to the new user control reference level in 
 *  incremental steps, applying the same ramp slope as during soft-start.
 * 
 *  While ramping the output voltage up or down, the BUSY bit will be set until the 
 *  ramp up/down transition is complete, where it will be automatically cleared.
 * 
 * b) Swap of Conversion Mode
 * 
 *  The bidirectional converter can change the conversion mode from step-down
 *  to step-up and vice versa. It is important to differentiate between 
 *  bidirectional conduction and bidirectional control. Bidirectional conduction 
 *  allows positive and negative currents in ether conversion mode while bidirectional 
 *  control is implemented by swapping the regulated output port as well as related 
 *  switch commutation and transfer function from step-down mode (high-voltage
 *  to low-voltage) to step-up mode (low-voltage to high-voltage) and back. 
 * 
 *  The swap of conversion mode can be performed instantly (= HotSwap, within one 
 *  or two switching cycles) or by forcing the power converter through a reset 
 *  cycle (= SoftSwap, shut-down-reverse-soft-start)
 *  
 *  A conversion swap must be commanded by an external instance and is initiated
 *  by setting the sequence
 * 
 *  @code {.c}
 * 
 *  // Select the new conversion mode
 *  myConverter.Properties.ConversionMode = [new conversion mode];
 * 
 *  // Select swap method (HotSwap or SoftSwap)
 *  myConverter.Properties.HotSwapEnable = [true or false];
 * 
 *  // Flag bit 'Properties.Swap' triggers the swap during the next 
 *  // state function call of state 'Online'
 *  // Note: The Swap flag is cleared automatically by the state machine
 * 
 *  myConverter.Properties.Swap = true; 
 * 
 *  @endcode
 *
 *********************************************************************************/
int State_Online(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Set converter status
    pcInstance->Status.bits.Active = true; // Set ACTIVE bit
    pcInstance->Status.bits.Online = true; // Set ONLINE bit
    
    // Call current balancing engine
    if ((pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC) && (pcInstance->Properties.PhaseCount > 1))
    { pcInstance->Control.ILoop[0].Methods.CurrentBalancing(pcInstance->Control.ILoop[0].Controller); }
    
    // When regulated, apply output voltage changes when reference has been changed
    if (false == pcInstance->Properties.EnforcedOpenLoop)
    {
        // Check for changes in control reference
        retval &= BdbbConverter_CheckForReferenceChange(pcInstance);
    }

    // Check for changes in power transfer direction
    int resetFlag = BdbbConverter_CheckForConversionSwap(pcInstance);
    if (1 == resetFlag) { retval &= 1; }
    
    // Synchronize the Swap Event Detect bit with the conversion mode setting
    // in case a HotSwap has been performed
    pcInstance->Status.bits.SwapDetect = pcInstance->Properties.ConversionMode;
    
    // If any sub-function call went unsuccessful, trigger error state transition.
    // If a conversion mode SoftSwap is scheduled, enforce state machine reset.
    // else, recall state function Online
    if (BDBB_OPSRET_RESET == resetFlag)
    { return(BDBB_OPSRET_RESET); }
    else if (1 == retval)
    { return(BDBB_OPSRET_REPEAT); } // Otherwise remain in recent state
    else { return(BDBB_OPSRET_ERROR); } // Invoke state machine reset if error occurred 
        
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-opstates-functions
 * @brief   If this function is called, the state machine is reset to INITIALIZE (URL=@ref State_Initialize).
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function is a default anchor in case task list index #0 is ever called.
 *  This is the equivalent of a switch case "default".
 *
 *********************************************************************************/
int State_Error(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // If this function is ever called, bring state machine back on track 
    // by resetting it to INITIALIZE
    
    retval &= BdbbPWM_Suspend(pcInstance);       // Hold PWM output (turning off power)
    retval &= BdbbLoopControl_Off(pcInstance);   // Turn off control loops
    retval &= BdbbLoopControl_Reset(pcInstance); // Reset control loops (clearing state and history)

    pcInstance->Status.bits.Active = false;      // Reset ACTIVE bit
    pcInstance->Status.bits.Busy = false;        // Reset BUSY bit
    pcInstance->Status.bits.IsInitialized = false; // Reset INITIALIZED bit
    pcInstance->Status.bits.Online = false;      // Reset ONLINE bit
    pcInstance->Status.bits.Ready = false;       // Reset READY bit
    pcInstance->Status.bits.IsSuspended = false; // Reset SUSPEND bit

    // If any sub-function call went unsuccessful, reset state machine
    // else, trigger transition to next state
    if (retval) return(BDBB_OPSRET_COMPLETE); // Trigger transition to next state
    else return(BDBB_OPSRET_ERROR);  // Invoke state machine reset if error occurred
    
    return(retval);
}

// ______________________________________
// end of file
