/**
 * @file    svc_bdbb_subtasks.c
 * @brief   Bidirectional Buck-Boost Converter state machine special function sub-states
 * @author  M91406
 */

#include <xc_pral.h> // include device peripheral register abstraction layer

#include "../../IBdbbConverter.h" // include bidirectional buck-boost converter type definitions
#include "../../IBdbbPControl.h"  // include bidirectional buck-boost converter peripheral control functions

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-properties-private
 * @struct  CS_CALIBRATION_s
 * @brief   Current sense calibration data structure
 **********************************************************************************/
struct CS_CALIBRATION_s  ///< Current sense calibration data object
{
    bool Start; ///< Current sense Start trigger starting the oversampling process
    bool Stop; ///< Current sense Stop trigger stopping the oversampling process
    uint16_t Counter; ///< Current sense calibration oversapling step counter
    uint32_t Buffer; ///< Current sense calibration averaging value buffer
};
typedef struct CS_CALIBRATION_s CS_CALIBRATION_t; ///< Current sense calibration data object data type

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-properties-private
 * @brief   Array of current sense calibration data objects of type CS_CALIBRATION_t
 * @details
 *  The current sense feedback offset calibration requires a data space to 
 *  accumulate and average the static feedback offset value. The final result
 *  is written to the feedback channel configuration of the bidirectional
 *  buck-boost converter data object, hence, the CS_CALIBRATION_s data structure 
 *  used for the current sense feedback offset calibration is kept private and 
 *  inaccessible from external firmware modules. This variable is declared as array 
 *  reserving a data set for each current sense feedback channel.
 *
 **********************************************************************************/
static CS_CALIBRATION_t calibCurrentSense[BDBB_MPHASE_COUNT];

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-defines
 * @brief   Number of signal oversampling steps used to determine the calibration value
 **********************************************************************************/
#define CS_CALIB_STEPS  8

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Performs an offset calibration of the current sense feedback signal(s)
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function performs a current sense feedback channel zero-offset 
 *  calibration. The calibration is executed when the reference voltage is 
 *  applied to the current sense shunt amplifiers but the power supply is still
 *  turned off. The offset value is determined by a 4x oversampling of each of
 *  the feedback signals to eliminate high-frequency noise. 
 *  Once the calibration is complete, the 'CsCalibComplete' status bit in 
 *  the bidirectional buck-boost converter power controller object status word  
 *  is set, allowing the state machine to run.
 *
 *********************************************************************************/
int BdbbConverter_CurrentSenseOffsetCalibration(void* handle) 
{
    BDBB_OPSTATE_RETURNS_t retval = BDBB_OPSRET_ERROR;
    bool _complete=true;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // if current sense calibration is disabled, return COMPLETE and leave
    if(!pcInstance->Properties.CsCalibEnable)
        return((uint16_t)BDBB_OPSRET_COMPLETE); // Set return value to COMPLETE and leave function

    // if current sense calibration is enabled but sense circuit requires a minimum 
    // common mode voltage which is not available, return COMPLETE and leave
    if ((pcInstance->Startup.OpenLoopStart.Enabled) && 
        (pcInstance->Data.VLowVoltage < pcInstance->Startup.OpenLoopStart.CutOffVoltage))
    {
        return((uint16_t)BDBB_OPSRET_COMPLETE); // Set return value to COMPLETE and leave function
    }

    // Protect against floored Current Calibration Procedure
    if ((!pcInstance->Status.bits.AdcActive) ||
        (!pcInstance->Status.bits.PwmActive)
       )
    { return((uint16_t)BDBB_OPSRET_REPEAT); } // Return REPEAT
        
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Perform Calibration Step
    
    for (int i = 0; i<pcInstance->Properties.PhaseCount; i++)
    {
        // Reset Calibration Data Buffers if Calibration is rerun
        if ((calibCurrentSense[i].Start && calibCurrentSense[i].Stop))
        {
            // Clear calibration data buffer
            calibCurrentSense[i].Buffer = 0; // Clear buffer
            calibCurrentSense[i].Counter = 0; // Clear counter
            calibCurrentSense[i].Start = false; // Clear START flag bit
            calibCurrentSense[i].Stop = false; // Clear STOP flag bit
        }

        // Collect data samples
        if (calibCurrentSense[i].Counter++ < CS_CALIB_STEPS)
        {
            calibCurrentSense[i].Start = true; // Clear START flag bit
            calibCurrentSense[i].Stop = false; // Clear STOP flag bit

            uint16_t _valBuf = BdbbLoopControl_GetSource(pcInstance->Control.ILoop[i].Controller);
            calibCurrentSense[i].Buffer += _valBuf;
            
        }
        // Collection of data samples is complete
        else
        {
            calibCurrentSense[i].Buffer >>= 3; // Divide accumulated ADC samples (calculate average)
            calibCurrentSense[i].Stop = true;  // Set STOP flag bit

            // Write calibration data to control loop parameter buffer data structure
            pcInstance->Feedback.AdISense[i].Scaling.Offset = calibCurrentSense[i].Buffer;
            
            // Write calibration result to control loop input source offset data field 
            BdbbLoopControl_SetSourceOffset(pcInstance->Control.ILoop[i].Controller, calibCurrentSense[i].Buffer);

        }
        
        _complete &= calibCurrentSense[i].Stop; // track STOP flag bits
    }

    // Return REPEAT until calibration is complete
    if (_complete) 
    {
        Nop();
        Nop();
        Nop();

        retval = (uint16_t)BDBB_OPSRET_COMPLETE; // Return COMPLETE
    }
    else retval = (uint16_t)BDBB_OPSRET_REPEAT; // Return REPEAT
    
    return(retval);
    
}  

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Resets the BDBB state machine runtime data members
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details This method clears all runtime data values, forcing them to =0. 
 **********************************************************************************/
int BdbbConverter_ResetRuntimeData(void* handle)
{
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Clear Runtime Data
    pcInstance->Data.VLowVoltage = 0; // Reset output voltage value
    pcInstance->Data.VHighVoltage = 0;  // Reset input voltage value
    pcInstance->Data.IAverage = 0;  // Reset output current value
    pcInstance->Data.Temperature = 0;  // Reset output temperature value

    for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
    {
        pcInstance->Data.ISense[i] = 0; // Reset phase current value
    }

    pcInstance->Data.ControlError = 0;  // Reset Control Error runtime data
    pcInstance->Data.ControlInput = 0;  // Reset Control Input runtime data
    pcInstance->Data.ControlOutput = 0; // Reset Control Output runtime data
    
    return(1);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Updates the BDBB state machine runtime data members from their respective ADC input channels
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  This function updates the power supply runtime data members from their
 *  source ADC buffer registers, conditioned by applying the user-defined
 *  signal limits and offset values. The data content reflects the 
 *  instantaneous state of the circuit, no averaging is applied.
 * 
 *  Update scheme:
 *  - Runtime data is updated at every state machine call. Thus, the 
 *    update frequency is equal the state machine call frequency.
 *  - If a feedback channel is disabled, the value in the data structure
 *    is forced to =0 and will not get updated. 
 *  - When the power supply is turned off (Enable=false), the runtime data 
 *    structure is cleared and all values are forced to =0.
 *  - If the power supply is suspended, the runtime data structure is 
 *    continuously updated to support startup, fault recovery and system
 *    check procedures.
 *
 **********************************************************************************/
int BdbbConverter_UpdateRuntimeData(void* handle) 
{
    int retval = 1;
    int32_t  iAvgBuffer = 0;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

	volatile P33C_ADCAN_BUFFER_INSTANCE_t* adc;

    // Read and accumulate phase currents 
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
    { 
        adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdISense[i].Input.AdcInput);
        if (pcInstance->Feedback.AdISense[i].Enabled)
        {
            pcInstance->Data.ISense[i]  = adc->ADCBUFx.value;
            pcInstance->Data.ISense[i] -= pcInstance->Feedback.AdISense[i].Scaling.Offset;
        }
        else
        { pcInstance->Data.ISense[i] = 0; } 
    
        if(pcInstance->Feedback.AdISense[i].Scaling.Polarity ^ pcInstance->Properties.ConversionMode)
        { pcInstance->Data.ISense[i] = -pcInstance->Data.ISense[i]; }

        iAvgBuffer += pcInstance->Data.ISense[i]; // Accumulate phase currents
    }
    
    if (iAvgBuffer > (int32_t)INT16_MAX) pcInstance->Data.IAverage = INT16_MAX;
    else if (iAvgBuffer < (int32_t)INT16_MIN) pcInstance->Data.IAverage = INT16_MIN;
    else  pcInstance->Data.IAverage = (uint16_t)iAvgBuffer; // Update API data member
    
    // Capture runtime data of the low-voltage feedback channel
    if (pcInstance->Feedback.AdVLowVoltage.Enabled){
        adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdVLowVoltage.Input.AdcInput);
        pcInstance->Data.VLowVoltage = (adc->ADCBUFx.value - pcInstance->Feedback.AdVLowVoltage.Scaling.Offset);
    }
    else
        pcInstance->Data.VLowVoltage = 0;
    
    // Capture runtime data of the high-voltage feedback channel
    if (pcInstance->Feedback.AdVHighVoltage.Enabled)
    {
        adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdVHighVoltage.Input.AdcInput);
        pcInstance->Data.VHighVoltage = (adc->ADCBUFx.value - pcInstance->Feedback.AdVHighVoltage.Scaling.Offset);
    }
    else
        pcInstance->Data.VHighVoltage = 0;
        
    // Capture runtime data of further, secondary feedback channels
    if (pcInstance->Feedback.AdTemperature.Enabled)
    {
        adc = p33c_AdcAnBuffer_GetHandle(pcInstance->Feedback.AdTemperature.Input.AdcInput);
        pcInstance->Data.Temperature = adc->ADCBUFx.value - pcInstance->Feedback.AdTemperature.Scaling.Offset;
    }
    else { pcInstance->Data.Temperature = 0; }

    return(retval); 
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Resets the BDBB state machine process counters
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
int BdbbConverter_ResetProcessCounters(void* handle)
{
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Reset all startup counters
    pcInstance->Startup.OpenLoopStart.Counter = 0; // Clear the open loop startup counter
    pcInstance->Startup.PowerOnDelay.Counter = 0; // Clear Power On Delay counter
    pcInstance->Startup.PowerGoodDelay.Counter = 0; // Clear Power Good Delay counter
    pcInstance->Startup.IRamp.Counter = 0; // Clear current ramp-up counter
    pcInstance->Startup.LVRamp.Counter = 0; // Clear voltage ramp-up counter
    pcInstance->Startup.HVRamp.Counter = 0; // Clear voltage ramp-up counter
    pcInstance->Startup.VCapMonitor.DelayCounter = 0; // Clear VCap Monitor Delay counter
    pcInstance->Startup.VCapMonitor.TimeoutCounter = 0; // Clear VCap Monitor Timeout counter

    return(1);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Updates status of external, hardware Enable pin and enables/disables the power controller accordingly
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbConverter_GetEnableInput(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    /* CAPTURE ENABLE PIN STATE IF ENABLED BY USER CODE                                   */
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    if(pcInstance->Gpio.EnableInput.Enabled)
    {
        // Capture Enable Input pin status (true=ACTIVE/ENABLED, false=INACTIVE/DISABLED)
        bool preEnable = pcInstance->Properties.Enable;
        bool pinState = BdbbGPIO_GetEnableInputState(pcInstance);
        pcInstance->Properties.Enable = pinState;
        
        if ((preEnable != pinState) && (pinState == false)) // Power Supply has been disabled
        { retval &= BdbbConverter_Suspend(pcInstance); }
        else if ((preEnable != pinState) && (pinState == true)) // Power Supply has been enabled 
        { retval &= BdbbConverter_Resume(pcInstance); }
        
    }
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    return(retval);
}
    
/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   This function tunes the voltage reference to the new user control reference level.
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  After soft-start and when state POWER_GOOD_DELAY has expired, the converter 
 *  enters normal operation.
 *  
 *  During normal operation the state machine scans the user reference setting. 
 *  Once a change between the user reference setting and the most recent controller 
 *  reference has been detected, the state machine will tune the controller 
 *  reference to the new user control reference level in incremental steps,
 *  applying the same ramp slope as during soft-start.
 *  
 *  While ramping the output voltage up or down, the BUSY bit will be set and any 
 *  new changes to the reference will be ignored until the ramp up/down is complete.
 *
 *********************************************************************************/
int BdbbConverter_SetVoltageReference(void* handle) 
{
    int retval = 1;
    uint16_t _vrefBuf = 0;
    uint16_t _vrefProp = 0;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Capture current limit value
    _vrefBuf  = BdbbLoopControl_GetLoopReference(pcInstance->Control.VLoop.Controller);
    _vrefProp = BdbbControl_GetUserVoltageReference(pcInstance);
    
    // New reference value is less than controller reference value => ramp down
    if(_vrefProp < _vrefBuf)
    {
        // decrement reference until new reference level is reached
        if(false == pcInstance->Properties.ConversionMode)
            _vrefBuf -= pcInstance->Startup.LVRamp.StepSize; // decrement reference
        else
            _vrefBuf -= pcInstance->Startup.HVRamp.StepSize; // decrement reference

        if(_vrefBuf <= _vrefProp) // check if ramp is complete
            _vrefBuf = _vrefProp; // clamp reference level to setting

        // Set new current limit
        retval &= BdbbLoopControl_SetLoopReference(pcInstance->Control.VLoop.Controller, _vrefBuf);
        
    }
    // New reference value is greater than controller reference value => ramp up
    else if(_vrefProp > _vrefBuf)
    {
        // increment reference until new reference level is reached
        if(false == pcInstance->Properties.ConversionMode)
            _vrefBuf += pcInstance->Startup.LVRamp.StepSize; // decrement reference
        else
            _vrefBuf += pcInstance->Startup.HVRamp.StepSize; // decrement reference
        
        if(_vrefBuf >= _vrefProp) // check if ramp is complete
            _vrefBuf = _vrefProp; // clamp reference level to setting

        // Set new current limit
        retval &= BdbbLoopControl_SetLoopReference(pcInstance->Control.VLoop.Controller, _vrefBuf);

    }
    
    return(retval);
    
}

/*******************************************************************************
 * @ingroup lsvc-layer-bdbb-sm-get-state-functions-public
 * @brief   This function tunes the current reference to the new user control reference level.
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  After soft-start and when state POWER_GOOD_DELAY has expired, the converter 
 *  enters normal operation.
 *  
 *  During normal operation the state machine scans the user reference setting. 
 *  Once a change between the user reference setting and the most recent controller 
 *  reference has been detected, the state machine will tune the controller 
 *  reference to the new user control reference level in incremental steps,
 *  applying the same ramp slope as during soft-start.
 *  
 *  While ramping the output voltage up or down, the BUSY bit will be set and any 
 *  new changes to the reference will be ignored until the ramp up/down is complete.
 *
 *********************************************************************************/
int BdbbConverter_SetCurrentReference(void* handle) 
{
    int fret = 1;
    BDBB_OPSTATE_RETURNS_t retval = BDBB_OPSRET_ERROR;
    uint16_t _ilimBuf = 0;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Set the BUSY bit indicating a delay/ramp period being executed
    pcInstance->Status.bits.Busy = true;
    
    // Capture current limit value
    _ilimBuf = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.VLoop.Controller);

    // New reference value is less than controller reference value => ramp down
    if(pcInstance->Properties.IReference < _ilimBuf)
    {
        // decrement reference limit until new reference limit level is reached
        _ilimBuf -= pcInstance->Startup.IRamp.StepSize; // decrement reference

        if(_ilimBuf <= pcInstance->Properties.IReference) // check if ramp is complete
        {
            _ilimBuf = pcInstance->Properties.IReference; // clamp reference level to setting
            retval = BDBB_OPSRET_COMPLETE; // Return completion of reference change
        }
        else
        { retval = BDBB_OPSRET_REPEAT; } // Return REPEAT to continue change of reference

        // Set new current limit
        fret &= BdbbLoopControl_SetLoopMaxLimit(pcInstance->Control.VLoop.Controller,_ilimBuf);
        
    }
    // New reference value is greater than controller reference value => ramp up
    else if(pcInstance->Properties.IReference > _ilimBuf)
    {
        // increment reference until new reference level is reached
        _ilimBuf += pcInstance->Startup.IRamp.StepSize; // increment reference

        if(_ilimBuf >= pcInstance->Properties.IReference) // check if ramp is complete
        {
            _ilimBuf = pcInstance->Properties.IReference; // clamp reference level to setting
            retval = BDBB_OPSRET_COMPLETE; // Return completion of reference change
        }
        else
        { retval = BDBB_OPSRET_REPEAT; } // Return REPEAT to continue change of reference

        // Set new current limit
        fret &= BdbbLoopControl_SetLoopMaxLimit(pcInstance->Control.VLoop.Controller,_ilimBuf);
        
    }
    else
    { retval = BDBB_OPSRET_COMPLETE; } // Return completion of reference change
    
    // Capture error
    if (0 == fret) retval = BDBB_OPSRET_ERROR;
    
    return(retval);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Enables or disables synchronous rectification of the power supply
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  If the power converter PWM drive supports asynchronous and synchronous mode,
 *  users may define output current thresholds above/below which the synchronous 
 *  drive is turned on/off automatically. This function must be enabled 
 *  in the power converter data object by setting the related control bit 
 *  'AutoAsyncMode = true' and defining output current on/off thresholds in 
 *  sub data structure 'Control.SyncRectifierControl'.
 *
 **********************************************************************************/
int BdbbConverter_SetSynchronousRectifier(void* handle)
{
    int retval = 1;
    bool _bTest = false; // ON/OFF flag where TRUE means Synchronous Rectifier Output Is ON (Sync Mode)
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // If synchronous rectifier auto-control is enabled, 
    // check if all phase currents are above the thresholds
    if (pcInstance->Properties.AutoAsyncMode == true)
    {
        // If asynchronous mode is active, check if signal is above Turn On Threshold
        // => SYNCHRONOUS RECIFIER PWM OUTPUT IS DISABLED => CHECK TO TURN ON
        // => All synchronous rectifiers of all converter phases are turning on/off 
        //    simultaneously (never independently)
        if (pcInstance->Status.bits.AsyncActive)
        {
            _bTest = true;  // Initiate ON state for AND-ing all conditions

            for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
            {
                _bTest &= (bool)(pcInstance->Data.ISense[i] > 
                              pcInstance->Properties.SyncRectifier.OnThreshold);
            }

            // Set/Clear Synchronous/Asynchronous Mode Control Bit
            if (_bTest) pcInstance->Properties.AsyncMode = false; // Turn on sync switch PWM channel / disable async mode
        }
        // If asynchronous mode is inactive, check if signal is below Turn Off Threshold
        // => SYNCHRONOUS RECIFIER PWM OUTPUT IS ENABLED => CHECK TO TURN OFF
        // => All synchronous rectifiers of all converter phases are turning on/off 
        //    simultaneously (never independently)
        else
        {
            _bTest = true;  // Initiate OFF state for AND-ing all conditions

            for (int i=0; i<pcInstance->Properties.PhaseCount; i++)
            {
                _bTest &= (bool)(pcInstance->Data.ISense[i] < 
                              pcInstance->Properties.SyncRectifier.OffThreshold);
            }

            // Set/Clear Synchronous/Asynchronous Mode Control Bit
            if (_bTest) pcInstance->Properties.AsyncMode = true;
        }

        // Check if converter operates in open loop conditions
        if (pcInstance->Startup.OpenLoopStart.Enabled) 
        {   
            // Enforce asynchronous mode below minimum common mode voltage range
            if (pcInstance->Data.VLowVoltage <= pcInstance->Startup.OpenLoopStart.CutOffVoltage) 
                pcInstance->Properties.AsyncMode = true;
        }
        
    }
    
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Set synchronous rectifier control output
    if (pcInstance->Properties.AsyncMode != pcInstance->Status.bits.AsyncActive) 
    {
        retval &= BdbbPWM_SetSynchronousMode(pcInstance);
    }
    
    // Return COMPLETE if action was successful, ERROR if not
    if (retval) retval = (uint16_t)BDBB_OPSRET_COMPLETE; // Return COMPLETE
    else retval = (uint16_t)BDBB_OPSRET_ERROR; // Return ERROR
    
    return(retval);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Checks the control bit of the synchronous rectifier control and sets the state of the synchronous rectifier PWM outputs
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
int BdbbConverter_SetAsyncDriveMode(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    /* CHECK FOR SWITCH NODE DRIVE CHANGES                                                */
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    if ((pcInstance->Properties.Enable) && (pcInstance->Status.bits.Active))
    {
        // Check if there was a transition between synchronous/asynchronous switch node drive
        // set new state of the synchronous rectifier PWM output
        if ((pcInstance->Properties.AutoAsyncMode == true) ||   // always check if auto sync rectifier control is enabled
            (pcInstance->Properties.AsyncMode != pcInstance->Status.bits.AsyncActive)) // or Async control bit has been set/cleared manually
            retval &= BdbbConverter_SetSynchronousRectifier(pcInstance);
    }
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Monitors differences between voltage and current user reference values and the one recently used by the respective control loop, performs transition when different
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=error, 1=repeat, 2=complete)
 *********************************************************************************/
int BdbbConverter_CheckForReferenceChange(void* handle)
{
    int retval = 1;
    uint16_t _vrefBuf = 0;
    uint16_t _vrefProp = 0;
    uint16_t _ilimBuf = 0;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Capture value of recently active voltage reference
    _vrefBuf  = BdbbLoopControl_GetLoopReference(pcInstance->Control.VLoop.Controller);
    _ilimBuf  = BdbbLoopControl_GetLoopMaxLimit(pcInstance->Control.VLoop.Controller);
    _vrefProp = BdbbControl_GetUserVoltageReference(pcInstance);

    // Check for changes in output voltage reference invoking a controlled transition
    if(_vrefProp != _vrefBuf) 
    {
        // Set the BUSY bit indicating a delay/ramp period being executed
        pcInstance->Status.bits.Busy = true;
        retval = BdbbConverter_SetVoltageReference(pcInstance);
    }

    // Check for changes in output current reference invoking a controlled transition
    else if ((pcInstance->Properties.IReference != _ilimBuf) && 
             (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_ACMC))
    {
        // Set the BUSY bit indicating a delay/ramp period being executed
        pcInstance->Status.bits.Busy = true;
        retval = BdbbConverter_SetCurrentReference(pcInstance);
    }
    
    else{
        // Clear the BUSY bit indicating "no state machine activity"
        pcInstance->Status.bits.Busy = false;
    }
    
    return(retval);  // return success/failure
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Sets source, reference and feedback inversion status bits for down-stream (buck) or upstream (boost) mode
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbConverter_SetDirectionSwap(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Set control loop conversion direction
    retval &= BdbbLoopControl_SetControlSwap(pcInstance);
    
    // Set PWM output channel
    retval &= BdbbPWM_SetOutputSwap(pcInstance);
    
    return(retval);  // return success/failure
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   Flips power transfer direction on the fly during operation
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
int BdbbConverter_HotSwap(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Guarding condition, preventing hot-swap when disabled
    if (false == pcInstance->Properties.HotSwapEnable) return(retval);
    
    // Capture Enable state
    bool _enable = pcInstance->Control.VLoop.Controller->Status.bits.Enable;

    // Turn off control loop
    retval &= BdbbLoopControl_Off(pcInstance);
    
    // PWM outputs
    retval &= BdbbPWM_Suspend(pcInstance);
    
    // Capture inverted duty cycle
    uint16_t _invDC = BdbbPWM_GetDutyCycleComplement(pcInstance);
    retval &= BdbbLoopControl_SetOutputLoopHistory(pcInstance, _invDC);
    
    // Set control loop conversion direction
    retval &= BdbbLoopControl_SetControlSwap(pcInstance);

    // Set PWM output channel
    retval &= BdbbPWM_SetOutputSwap(pcInstance);
    
    // Update PWM timing registers
    retval &= BdbbPWM_Update(pcInstance);
    
    // If loop was enabled and active, re-enable loop and PWM outputs
    if(_enable)
    {
        retval &= BdbbPWM_Resume(pcInstance);
        retval &= BdbbLoopControl_On(pcInstance);
    }
    
    return(retval);  // return success/failure
}

/*******************************************************************************
 * @ingroup lib-layer-bdbb-converter-sm-methods
 * @brief   Monitors for commanded power transfer switch-over
 * @param   pcInstance  Pointer to a power converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success, 3=reset)
 *********************************************************************************/
int BdbbConverter_CheckForConversionSwap(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // If a switch-over command has been detected, swap power transfer direction
    if (!pcInstance->Properties.Swap)return(retval);

    // Only swap power transfer direction when it is different from recent direction
    if (pcInstance->Properties.ConversionMode == pcInstance->Status.bits.SwapDetect) return(retval);
    
    // Guarding condition, preventing hot-swap when disabled
    if (true == pcInstance->Properties.HotSwapEnable)
    {
        retval &= BdbbConverter_HotSwap(pcInstance); // Call hot-swap function
        pcInstance->Properties.Swap = false;         // Clear SWAP control bit
    }
    else{
        retval = BDBB_OPSRET_RESET;
        pcInstance->Properties.Swap = false;         // Clear SWAP control bit
        pcInstance->Properties.PowerOnDelayByPass = true; // Set the Power on delay bypass control bit
    }
    
    return(retval);  // return success/failure
}

// ______________________________________
// end of file
