/**
 * @file    svc_bdbb_sm_exec.c
 * @brief   Bidirectional Buck-Boost Converter device driver top-level functions
 * @author  M91406
 */

#include "states/svc_bdbb_subtasks.h" // include buck converter special function declarations
#include "../IBdbbPControl.h" // include buck converter peripheral configuration declarations
#include "../IBdbbConverter.h" // include data object interface header file

#if (__XC16_VERSION < 2000)
// Compiler version lower than v2.0 are not supported

  #define STR(x)   #x
  #define XSTR(x)  STR(x)
  #define MSG(x)   STR(x)
  #pragma message MSG(Selected compiler version XSTR(__XC16_VERSION) not supported)
  #error "compiler version conflict"

#elif ((__XC16_VERSION < 2000) || (__XC16_VERSION > 3100))
// Throw warning if this code has not been tested with the recently selected compiler version

  #warning "this code has not been tested with the recently selected compiler version"

#endif
// ------------------------------------------------------------------------------------------------
// Declaration of Bidirectional Buck-Boost Converter Device Driver Constants
const BDBB_CONVERTER_CONSTANTS_t BdbbConverterConstants;

// ------------------------------------------------------------------------------------------------
// Bidirectional Buck-Boost Converter State Machine Function Prototypes
extern int BdbbConverter_GetState(IBdbbConverter_t *pcInstance);
extern int BdbbConverter_SetState(IBdbbConverter_t *pcInstance);
extern int BdbbConverter_GetStateListSize();
// ------------------------------------------------------------------------------------------------

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions
 * @brief   This function initializes all peripheral modules and their instances used by the power controller
 * @param   pcInstance  Pointer to a Buck Converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function initializes the PWM module, PWM channels, ADC channels for 
 *  temperature, input voltage, output voltage and phase current. The buck IO 
 *  pins are also initialize while keeping the buck converter operation disabled. 
 *  The state machine is set to Initialize. 
 *
 *********************************************************************************/
int BdbbConverter_Initialize(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Initialize PWM peripheral module and channels
    retval &= BdbbPWM_ModuleInitialize(pcInstance); // Initialize PWM Module
    retval &= BdbbPWM_ChannelInitialize(pcInstance); // Initialize PWM Channel of Buck Converter
    
    // Initialize ADC module and input channels
    retval &= BdbbADC_ModuleInitialize();     // Initialize ADC Module base registers
    retval &= BdbbADC_ChannelInitialize(&pcInstance->Feedback.AdTemperature); // Initialize Temperature Channel
    retval &= BdbbADC_ChannelInitialize(&pcInstance->Feedback.AdVHighVoltage);  // Initialize Input Voltage Channel
    retval &= BdbbADC_ChannelInitialize(&pcInstance->Feedback.AdVLowVoltage); // Initialize Output Voltage Channel
    
    for (int i=0; i<pcInstance->Properties.PhaseCount; i++) // Reset phase current values
        retval &= BdbbADC_ChannelInitialize(&pcInstance->Feedback.AdISense[i]); // Initialize Phase Current Channel
    
    // Initialize GPIO registers of control pins supporting library features
    retval &= BdbbGPIO_Initialize(pcInstance); // Initialize additional control IOs

    // ~~~~~~~~~~~~~~~~~~~~~
    
    // Disable control loops
    retval &= BdbbLoopControl_Reset(pcInstance);
    
    // Reset Power Good output
    retval &= BdbbGPIO_ClearPowerGood(pcInstance);    
    
    // Set initial off state
    pcInstance->Properties.Enable = false;  // Disable Buck Converter
    pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_INITIALIZE; // Reset state machine
    pcInstance->StateId.OpStateListSize = BdbbConverter_GetStateListSize(); // Capture size of state machine function pointer list
    
    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions
 * @brief   This function is the main bidirectional buck-boost converter state machine executing the most recent state 
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_REPEAT
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 *
 * @details
 *  This function performs tasks in the state machine.  
 *   - If state machine state returns ERROR, switch to ERROR state in next execution cycle
 *   - If state machine state signals state completion, move on to next state in line
 *   - When state machine state returns REPEAT, the recent state function will be called again
 *   - When state machine state returns an unknown result, the state machine will be reset to INITIALIZE again 
 *
 *********************************************************************************/
int BdbbConverter_Execute(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Call Bidirectional Buck-Boost converter State Machine
    retval &= BdbbConverter_GetState(pcInstance);
    retval &= BdbbConverter_SetState(pcInstance);
    
    // Clear Feedback Loop Control ISR Active Flag bit
    pcInstance->Status.bits.LoopCtrlActive = false;
    
    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions
 * @brief   This function starts the bidirectional buck-boost converter
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *  
 * @details
 *  This function starts the bidirectional buck-boost converter operation by enabling the the buck PWM 
 *  and ADC peripherals, enabling the bidirectional buck-boost converter and reseting the state machine 
 *  to Initialize. 
 *
 *********************************************************************************/
int BdbbConverter_Start(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Disable and reset control loops
    retval &= BdbbLoopControl_Reset(pcInstance);

    // Sequence PWM, DAC and ADC peripheral startup
    retval &= BdbbPWM_Start(pcInstance);   // Start PWM (All Outputs Disabled)
    if (retval == 1) pcInstance->Status.bits.PwmActive = true; // If PWM startup was successful, set PWM_ACTIVE flag
    if (pcInstance->Control.ControlMode == BDBB_CONTROL_MODE_PCMC)
    { retval &= BdbbDAC_Start(pcInstance); } // Enable analog comparator + DAC instances
    retval &= BdbbADC_Start();              // Start ADC
    if (retval == 1) 
        pcInstance->Status.bits.AdcActive = true; // If ADC startup was successful, set ADC_ACTIVE flag
    
    // Enable bidirectional buck-boost converter and reset state machine to INITIALIZE
    pcInstance->Properties.Enable = true;   // Enable bidirectional buck-boost converter
    pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_INITIALIZE; // Reset state machine

    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions-public
 * @brief   This function stop the bidirectional buck-boost converter opration
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function stops the bidirectional buck-boost converter operation 
 *  by shutting down the PWM generator, disabling the voltage/current loop 
 *  and reset the state machine to initialize.
 *
 *********************************************************************************/
int BdbbConverter_Stop(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    // Stop PWM completely (shuts down PWM generator)
    retval &= BdbbPWM_Stop(pcInstance); // Stop PWM
    retval &= BdbbLoopControl_Off(pcInstance); // Turn off control loops immediately
    retval &= BdbbGPIO_ClearPowerGood(pcInstance); // Reset POWER_GOOD output (function is internally guarded)
    
    // Disable and reset control loops
    retval &= BdbbLoopControl_Reset(pcInstance);
    
    // Disable bidirectional buck-boost converter and reset state machine to INITIALIZE
    pcInstance->Properties.Enable = false;  // Disable bidirectional buck-boost converter
    pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_INITIALIZE; // Reset state machine

    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions
 * @brief   This function suspends the operation of the bidirectional buck-boost converter
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *
 * @details
 *  This function sets the suspend bit terminating operation. This bit will 
 *  be evaluated in the State machine tasks which eventually shuts down the 
 *  operation of the bidirectional buck-boost converter.
 *********************************************************************************/
int BdbbConverter_Suspend(void* handle) 
{    
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    static bool suspend_active = false;
    
    //Recursion protection for the resume
    if(suspend_active) return(retval);
    suspend_active = true;    

    pcInstance->Properties.Suspend = true;  // Set SUSPEND bit terminating operation
    pcInstance->Status.bits.IsSuspended = true; // Set SUSPENDED status bit
    pcInstance->Status.bits.Ready = false; // Clear READY bit to force state machine back into RESET state
    retval &= BdbbPWM_Suspend(pcInstance); // Shut down PWM immediately 
    retval &= BdbbLoopControl_Off(pcInstance); // Turn off control loops immediately
    retval &= BdbbGPIO_ClearPowerGood(pcInstance); // Reset POWER_GOOD output (function is internally guarded)
    retval &= BdbbConverter_Execute(pcInstance);  // Call state machine to clean up the mess afterwards
    
    suspend_active = false;
    
    return(retval);
}

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-exec-functions
 * @brief   This function resume the operation of the bidirectional buck-boost converter
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function executes the latest task in the state machine enforcing state 
 *  switch immediately.
 *
 *********************************************************************************/
int BdbbConverter_Resume(void* handle) 
{    
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    static bool resume_active = false;
    if(resume_active) return(retval);
    resume_active = true;
    
    pcInstance->Properties.Suspend = false;  // Reset SUSPEND bit allowing state machine to start up
    pcInstance->Status.bits.IsSuspended = false; // Reset SUSPENDED status bit
    retval &= BdbbConverter_Execute(pcInstance);  // Enforce state switch immediately
    
    resume_active = false;
    
    return(retval);
}


// ______________________________________
// end of file
