/**
 * @file    svc_bdbb_sm_get_state.c
 * @brief   Set of functions analyzing the most recent operating state updating status information
 * @author  M91406
 * @date    05/22/2022
 */

#include "../IBdbbPControl.h"
#include "../IBdbbConverter.h"
#include "states/svc_bdbb_subtasks.h"

static int BdbbConverter_UpdateConverterData(void* handle);
static int BdbbConverter_GetControlFlags(void* handle);
static int BdbbConverter_ValidateStatePointer(void* handle);
static int BdbbConverter_CheckEnforcedOpenLoop(void* handle); 

// Declaration of function pointer array listing op-state functions in order of execution
int (*BdbbConverterStateCheckList[])(void* handle) = 
{
    BdbbConverter_UpdateConverterData,  ///< State Check #0:  Read ADC data of all feedback channels
    BdbbConverter_GetEnableInput,       ///< State Check #1:  Updates status of external, hardware Enable pin
    BdbbConverter_GetControlFlags,      ///< State Check #2:  Verifies state-machine startup-flags and sets the desired state
    BdbbConverter_SetAsyncDriveMode,    ///< State Check #3:  Checks the control bit of the synchronous rectifyer control and sets the state of the synchronous rectifier PWM outputs
    BdbbConverter_ValidateStatePointer, ///< State Check #4:  Verifies if the state pointer is pointing to a valid state
    BdbbConverter_CheckEnforcedOpenLoop ///< State Check #5:  Checks if power converter is set in enforced open loop mode
};

// Declaration variable capturing the size of the sub-state function pointer array 
static int BdbbConverterStateCheckList_size = (sizeof(BdbbConverterStateCheckList)/sizeof(BdbbConverterStateCheckList[0]));


/*******************************************************************************
 *******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-public
 * @brief   This function is the main bidirectional buck-boost converter state machine executing the most recent state 
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  0 = BDBB_OPSRET_ERROR
 * @return  1 = BDBB_OPSRET_COMPLETE
 * @return  2 = BDBB_OPSRET_REPEAT
 *
 * @details
 *  This function performs tasks in the state machine.  
 *   - If state machine state returns ERROR, switch to ERROR state in next execution cycle
 *   - If state machine state signals state completion, move on to next state in line
 *   - When state machine state returns REPEAT, the recent state function will be called again
 *   - When state machine state returns an unknown result, the state machine will be reset to INITIALIZE again 
 *
 *********************************************************************************/
int BdbbConverter_GetState(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Execute list of state checks
    for (int i=0; i<BdbbConverterStateCheckList_size; i++)
    { retval &= BdbbConverterStateCheckList[i](pcInstance); }
    
    return(retval);
}

// =========================================================================================
// PRIVATE GET STATE FUNCTIONS
// =========================================================================================

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-private
 * @brief   Reads data of all feedback channels of the converter updating the converter Data object members
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 *********************************************************************************/
static int BdbbConverter_UpdateConverterData(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    // Skip data capture if power supply is disabled
    if ((true == pcInstance->Status.bits.AdcActive) && 
        (true == pcInstance->Status.bits.PwmActive))
    { retval &= BdbbConverter_UpdateRuntimeData(pcInstance); }
    else 
    { retval &= BdbbConverter_ResetRuntimeData(pcInstance); }

    return(retval); 
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-private
 * @brief   Verifies state-machine startup-flags and sets the desired state
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
static int BdbbConverter_GetControlFlags(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    /* DISABLE/SUSPEND/FAULT-RESET                                                        */
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    // When enable status has changed from ENABLED to DISABLED or a fault condition 
    // is active, reset the state machine and hold it in RESET state
    if ((false == pcInstance->Properties.Enable)            || 
        (true  == pcInstance->Properties.Suspend)           ||
        (true  == pcInstance->Status.bits.FaultStatus)      || 
        (false == pcInstance->Status.bits.LoopCtrlActive)   ||
        (false == pcInstance->Status.bits.AdcActive)        ||
        (false == pcInstance->Status.bits.PwmActive)        || 
        (true  == pcInstance->Status.bits.IsSuspended))
    {
        if (false == pcInstance->Status.bits.IsInitialized)
            pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_INITIALIZE;
        else if ((false == pcInstance->Status.bits.Ready) || (false == pcInstance->Status.bits.LoopCtrlActive))
            pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_RESET;
        else if (false == pcInstance->Status.bits.PodComplete)
            pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_POWER_ON_DELAY;
        else
            pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_STANDBY;
    }
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    return(retval);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-private
 * @brief   Verifies if the state pointer is pointing to a valid state
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
static int BdbbConverter_ValidateStatePointer(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    /* FUNCTION CALL PROTECTION                                                           */
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    // If the state array pointer is out of range, roll over and start from first 
    // valid state
    if(pcInstance->StateId.OpStateId >= pcInstance->StateId.OpStateListSize)
        pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_INITIALIZE;
    
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    
    return(retval);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-get-state-functions-private
 * @brief   In Enforced Open Loop mode, the state machine is forced to state ONLINE after having started the peripherals without closing the loop
 * @param   pcInstance  Pointer to a bidirectional buck-boost converter data object of type IBdbbConverter_t
 * @return  unsigned integer (0=failure, 1=success)
 **********************************************************************************/
static int BdbbConverter_CheckEnforcedOpenLoop(void* handle) 
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;
    
    if ((true == pcInstance->Properties.EnforcedOpenLoop) && 
        (pcInstance->StateId.OpStateId > BDBB_OPSTATE_V_OL_RAMP_UP))
        { pcInstance->StateId.OpStateId = BDBB_OPSTATE_ONLINE; }
   
    return(retval);
}

// ____________________________
// end of file
