/**
 * @file    svc_bdbb_sm_set_state.c
 * @brief   Bidirectional Buck-Boost Converter device driver operating state execution
 * @author  M91406
 * @date    05/22/2022
 */

#include "../IBdbbConverter.h" // include data object interface header file
#include "./states/svc_bdbb_statelist.h" // list ofavailable state functions of this power supply

/*******************************************************************************
 * @ingroup svc-layer-bdbb-sm-set-state-functions
 * @return  unsigned integer (0=failure, 1= success)
 * @brief   This function is the main bidirectional buck-boost converter state machine executing the most recent state 
 * @return  unsigned integer: 0=failure, 1= success
 *
 * @details
 *  This function executes the most recent operating state by calling the recently
 *  selected sub-state function and responds to its return value.
 *  
 *  Sub-state functions can return three different values:
 *  
 *  - BDBB_OPSRET_ERROR:
 *    Something went wrong during the execution of the sub-state function and
 *    the state machine must be reset in the next execution cycle.
 *  
 *  - BDBB_OPSRET_COMPLETE:
 *    The task of the most recent sub-state function has completed and the state 
 *    machine can move on to the next sub-state function of the queue
 * 
 *  - BDBB_OPSRET_REPEAT:
 *    The task of the most recent sub-state function has not completed and must be
 *    called again in the next state machine execution cycle
 * 
 **********************************************************************************/

int BdbbConverter_SetState(void* handle)
{
    int retval = 1;
    IBdbbConverter_t* pcInstance = (IBdbbConverter_t*)handle;

    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    /* NULL POINTER PROTECTION                                                            */
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    // If no power converter instance has been declared, leave here
    if(pcInstance == NULL)
        return(0);

    // If state machine function pointer points nowhere, leave here
    if (BdbbConverterStateMachine[pcInstance->StateId.OpStateId] == NULL)
        return(0);
    
    // Execute state machine step
    do
    {
        /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
        /* EXECUTE MOST RECENT STATE                                                          */
        /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
        // Call most recent state function
        retval = BdbbConverterStateMachine[pcInstance->StateId.OpStateId](pcInstance);
 
        /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
        /* PROCESS STATE RESPONSE                                                             */
        /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
        switch (retval) 
        {
            /* If state machine state returns ERROR, switch to ERROR state in next execution cycle */
            case BDBB_OPSRET_ERROR:

                pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_ERROR;
                retval = 0;
                break;

            /* IF state machine state signals state completion, move on to next state in line */
            case BDBB_OPSRET_COMPLETE:

                // Increment main operating state pointer by one tick
                (pcInstance->StateId.OpStateId++);

                // Check if new index is out of range, reset to RESET if so
                if (pcInstance->StateId.OpStateId >= BdbbConverterStateList_size)
                    pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_RESET;

                retval = 1;
                break;

            case BDBB_OPSRET_SKIP:
                // Increment main operating state pointer by one tick
                (pcInstance->StateId.OpStateId++);
                // 'retval' is kept to enforce immediate execution of the following state function
                retval = BDBB_OPSRET_SKIP; 
                break;
                
            /* When state machine state returns REPEAT, the recent state function will be called again */
            case BDBB_OPSRET_REPEAT:
                // Do nothing, same state will be called next time
                retval = 1;
                break;

            /* When state machine state returns RESET, the state machine will be thrown back to state RESET in the next execution step */
            case BDBB_OPSRET_RESET:
                pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_RESET;
                retval = 1;
                break;

            /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
             * When state machine state returns an unknown result, the 
             * state machine will be reset to INITIALIZE again */
            default:
                // In case an undefined return value has been received,
                // Reset state machine and start from scratch
                pcInstance->StateId.OpStateId = (uint16_t)BDBB_OPSTATE_ERROR;
                retval = 0;
                break;
        }
        
    } while (retval == BDBB_OPSRET_SKIP);
    
    return(retval);
}

/*********************************************************************************
 * @ingroup svc-layer-bdbb-sm-set-state-functions
 * @brief   Callback function returning number of state machine states
 * @return  unsigned integer: number of state machine states
 *
 * @details
 *  This is an internal auxiliary function exposing the size of the state machine
 *  function pointer list to other instances of this device driver.
 *
 **********************************************************************************/
int BdbbConverter_GetStateListSize() { return(BdbbConverterStateList_size); }

// ______________________________________
// end of file
