/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    svc_bdbb_typedef.h
 * @brief   Type definitions of single- and multiphase bidirectional buck-boost converter data object
 * @author  M91406
 */
/**
 * Revision history: 
 * 1.0  initial release
 * 1.1  restructured phase arrays in data object to optimize code execution and stay with 
 *      unified API with regards to other power converter object libraries
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef BDBB_CONVERTER_TYPE_DEF_H
#define	BDBB_CONVERTER_TYPE_DEF_H

#include <stdint.h> // include standard integer number data types
#include <stdbool.h> // include standard boolean data types (true/false)
#include <stddef.h> // include standard definition data types
#include <dsp.h> // include DSP related standard data types

#include "config/paths.h" // include project common header file locations header file
#include NPNZ16B_HEADER // include NPNZ16 controller header file
#include HAL_HEADER // include hardware abstraction layer header file

#ifndef NO_OF_PHASES
#define NO_OF_PHASES 1U
#pragma message "warning: macro 'NO_OF_PHASES' has not been found. Number of phases has been set to default=1"
#endif

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-defines
 * @brief   Declaration of the number of power train phases of the BDBB Converter
 **************************************************************************************************** */
#define BDBB_MPHASE_COUNT           NO_OF_PHASES

// Converter Status Bits
/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Enumeration of status flags
 * @extends BDBB_CONVERTER_CONSTANTS_s
 **************************************************************************************************** */
enum BDBB_STATUS_FLAGS_e 
{  
    // Status Flags
    BDBB_STAT_NONE                 = 0b0000000000000000, ///< No power supply status has been set
    BDBB_STAT_INITIALIZED          = 0b0000000000000001, ///< Bit #0:  Power supply state machine has been successfully initialized and is ready to run
    BDBB_STAT_READY                = 0b0000000000000010, ///< Bit #1:  Power supply state machine has been successfully initialized and is ready to run
    BDBB_STAT_POD_COMPLETE         = 0b0000000000000100, ///< Bit #2:  ADC has been started and samples are taken
    BDBB_STAT_ADC_ACTIVE           = 0b0000000000001000, ///< Bit #3:  ADC has been started and samples are taken
    BDBB_STAT_PWM_ACTIVE           = 0b0000000000010000, ///< Bit #4:  PWM has been started and ADC triggers are generated
    BDBB_STAT_LOOP_CTRL_ACTIVE     = 0b0000000000100000, ///< Bit #5:  Feedback loop control interrupt is executing
    BDBB_STAT_FAULT_STATUS         = 0b0000000001000000, ///< Bit #6:  Power supply is in enforced shut down mode (usually due to a fault condition)
    BDBB_STAT_FAULT_LATCH          = 0b0000000010000000, ///< Bit #7:  Power supply is in latched enforced shut down mode (requires manual clearance)

    BDBB_STAT_CS_CALIB_COMPLETE    = 0b0000000100000000, ///< Bit #8:  Current sense calibration has completed successfully
    BDBB_STAT_IS_SUSPENDED         = 0b0000001000000000, ///< Bit #9:  Converter operation has been suspended and state machine has been reset
    BDBB_STAT_PWM_OUTPUT_ACTIVE    = 0b0000010000000000, ///< Bit #10: PWM output pins are enabled and switching signals are generated
    BDBB_STAT_ASYNC_ACTIVE         = 0b0000100000000000, ///< Bit #11: Synchronous rectifier PWM output is disabled
    BDBB_STAT_HOT_SWAP_DETECT      = 0b0001000000000000, ///< Bit #12: Hot swap event detection status flag (mirrors SwapEnable control bit when a switch over has been performed)
    BDBB_STAT_BUSY                 = 0b0010000000000000, ///< Bit #13: State machine is executing an ongoing process (e.g. startup-ramp)
    BDBB_STAT_ACTIVE               = 0b0100000000000000, ///< Bit #14: Converter is active (PWM outputs and feedback loops are active)
    BDBB_STAT_ONLINE               = 0b1000000000000000  ///< Bit #15: Converter has entered normal operation
        
}; ///< Enumeration of status flags of the BDBB Converter status word
typedef enum BDBB_STATUS_FLAGS_e BDBB_STATUS_FLAGS_t;

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Enumeration of state machine operating states 
 * @extends BDBB_STATE_ID_s
 * @extends BDBB_CONVERTER_CONSTANTS_s
 **************************************************************************************************** */
enum BDBB_OPSTATES_e 
{  
    BDBB_OPSTATE_ERROR,            ///< power converter control state #0:  in case of an error, state machine will reset to RESET
    BDBB_OPSTATE_INITIALIZE,       ///< power converter control state #1:  initializes object properties and clears status bits
    BDBB_OPSTATE_RESET,            ///< power converter control state #2:  resets object properties and clears status bits
    BDBB_OPSTATE_POWER_ON_DELAY,   ///< power converter control state #3:  power on delay for startup debouncing (no action until delay period has passed)
    BDBB_OPSTATE_STANDBY,          ///< power converter control state #4:  waiting for status bits allowing power supply to run
    BDBB_OPSTATE_VCAP_MONITOR,     ///< power converter control state #5:  waits until the output capacitor has charged up to forward voltage drop level in boost converter mode
    BDBB_OPSTATE_RELAY,            ///< power converter control state #6:  turns on the relay and added a time delay to allow the capacitor to recover
    BDBB_OPSTATE_CS_CALIBRATION,   ///< power converter control state #7:  performs offset calibration of current sense ADC signals 
    BDBB_OPSTATE_PREPARE_V_RAMP,   ///< power converter control state #8:  estimates the required startup condition and programs PWM and control loops
    BDBB_OPSTATE_V_RAMP_BIAS_COMP, ///< power converter control state #9:  injects additional ramp-up delay to keep sequencing timing constant with pre-biased output capacitors
    BDBB_OPSTATE_V_OL_RAMP_UP,     ///< power converter control state #10:  performs monitored open loop start-up of output voltage with control loops suspended (optional)
    BDBB_OPSTATE_V_CL_RAMP_UP,     ///< power converter control state #11: performs closed loop output voltage ramp-up based on user-constants
    BDBB_OPSTATE_I_CL_RAMP_UP,     ///< power converter control state #12: performs closed loop output current ramp-up based on user-constants (average current mode only)
    BDBB_OPSTATE_PWRGOOD_DELAY,    ///< power converter control state #13: monitoring period after voltage ramp is complete after which output power is considered to be stable and reliable
    BDBB_OPSTATE_ONLINE            ///< power converter control state #14: output is in regulation and power is OK (normal continuous operation)
        
}; ///< Enumeration of state machine operating states 
typedef enum BDBB_OPSTATES_e BDBB_OPSTATES_t; ///< Enumeration of state machine operating states

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Enumeration of state machine operating state return values
 * @extends BDBB_CONVERTER_CONSTANTS_s
 **************************************************************************************************** */
enum BDBB_OPSTATE_RETURNS_e 
{   
    BDBB_OPSRET_ERROR = 0,      ///< power converter state return #0: internal error occurred
    BDBB_OPSRET_COMPLETE = 1,   ///< power converter state return #1: operation state has completed
    BDBB_OPSRET_REPEAT = 2,     ///< power converter state return #2: operation state is in progress and needs to be recalled
    BDBB_OPSRET_RESET = 3,      ///< power converter state return #3: operation state return to reset
    BDBB_OPSRET_SKIP = 4        ///< power converter state return #4: operation state got skipped immideately
        
}; // Enumeration of state machine operating state return values
typedef enum BDBB_OPSTATE_RETURNS_e BDBB_OPSTATE_RETURNS_t; // State Machine Operating State Return Values

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Enumeration of the power supply mode control
 * @extends BDBB_CONVERSION_CONTROL_t
 **************************************************************************************************** */
enum BDBB_CONTROL_MODE_e
{
    BDBB_CONTROL_MODE_VMC,  ///< Voltage Mode Control
    BDBB_CONTROL_MODE_PCMC, ///< Peak Current Mode Control 
    BDBB_CONTROL_MODE_ACMC  ///< Average Current Mode Control (Constant Voltage Source)
    //BDBB_CONTROL_MODE_CCMC  ///< Constant Current Mode Control (Constant Average Current Source) // ToDo: implement strict constant current mode control
        
}; ///< Enumeration of BDBB Converter Control Modes
typedef enum BDBB_CONTROL_MODE_e BDBB_CONTROL_MODE_t; ///< BDBB Converter Control Modes data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Structure providing all public enumerated lists of constants 
 **************************************************************************************************** */
struct BDBB_CONVERTER_CONSTANTS_s
{
    BDBB_STATUS_FLAGS_t StatusFlags; ///< List of status flags of the BDBB Converter status word
    BDBB_OPSTATES_t OpStateOpCodes;  ///< List of State Machine Operating State IDs
    BDBB_OPSTATE_RETURNS_t OpStateReturnValues;  ///< List of State Machine Operating State Return Values
    BDBB_CONTROL_MODE_t ControlModes;  ///< List of Supported Control Modes
};
typedef struct BDBB_CONVERTER_CONSTANTS_s BDBB_CONVERTER_CONSTANTS_t;

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-variables
 * @var     BDBB_CONVERTER_CONSTANTS_t BdbbConverterConstants
 * @brief   Structure providing all public enumerated lists of constants 
 **************************************************************************************************** */
extern const BDBB_CONVERTER_CONSTANTS_t BdbbConverterConstants;

/*****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @struct  BDBB_CONVERTER_STATUS_s
 * @brief   Generic power controller status word
 * @extends IBdbbConverter_s
 * 
 * @details
 *  The power controller status/control word contains status (low-byte) and control bits (high-byte). 
 *  -# Status Bits:
 *      - ADC_ACTIVE: ADC is active and running (read only)
 *      - PWM_STARTED: PWM is active and running generating ADC triggers (read only)
 *      - POWERSOURCE_DETECTED: A valid power source has been detected allowing the converter to run (read only)
 *      - CS_READ: Current sense feedback ready (read only)
 *      - FORCED_SHUT_DOWN: Control(Status bit for external software components forcing the converter to stay off
 *      - BUSY: Converter is currently going through an internal process (e.g. ramp up/down) (read only)
 *      
 **************************************************************************************************** */
struct BDBB_CONVERTER_STATUS_s
{
    bool IsInitialized:1;       ///< Bit #0:  Status bit indicating if power converter has been initialized
    bool Ready:1;               ///< Bit #1:  Status bit indicating if power converter settings have been reset and is ready to run
    bool PodComplete:1;         ///< Bit #2:  Status bit indicating if the Power On Delay has expired
    bool AdcActive:1;           ///< Bit #3:  Status bit indicating if ADC has been started and samples are taken
    bool PwmActive:1;           ///< Bit #4:  Status bit indicating if PWM has been started and ADC triggers are generated
    bool LoopCtrlActive:1;      ///< Bit #5:  Status bit indicating if feedback loop control interrupt is executed
    bool FaultStatus :1;        ///< Bit #6:  Status bit indicating if system is in enforced shut down mode (usually due to a fault condition)
    bool FaultLatch :1;         ///< Bit #7:  Status bit indicating if system is in latched enforced shut down mode (requires manual clearance)

    bool CsCalibComplete :1;    ///< Bit #8:  Status bit indicating if current sense calibration process is complete
    bool IsSuspended :1;        ///< Bit #9:  Status bit indicating if converter is in suspend mode and state machine is hold in reset
    bool PwmOutputActive:1;     ///< Bit #10: Status bit indicating if the PWM outputs are enabled or disabled
    bool AsyncActive :1;        ///< Bit #11: Status bit indicating if the switch node is driven in asynchronous mode (=true) or synchronous mode (=false)
    bool SwapDetect :1;         ///< Bit #12: Status bit indicating if hot swap event has been detected (matches SwapEnable control bit after a switch over has been performed)
    bool Busy :1;               ///< Bit #13: Status bit indicating that the state machine is executing a process (e.g. startup-ramp)
    bool Active :1;             ///< Bit #14: Status bit indicating if switch-node and control loops are active
    bool Online :1;             ///< Bit #15: Status bit indicating if the state machine has reached continuous operation in state ONLINE

}__attribute__((packed)); ///< BDBB Converter Status & Control word

union BDBB_CONVERTER_STATUS_u 
{
    struct BDBB_CONVERTER_STATUS_s bits; ///< BDBB Converter Status & Control bit-field
	uint16_t value; ///< BDBB Converter Status & Control word for 16-bit word read/write operations
};

typedef union BDBB_CONVERTER_STATUS_u BDBB_CONVERTER_STATUS_t; // BDBB Converter Status & Control word data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Data structure for capturing the state ID of the most recent operating state 
 * @extends IBdbbConverter_s
 **************************************************************************************************** */
struct BDBB_STATE_ID_s 
{
    BDBB_OPSTATES_t  OpStateId;    ///< Gets/sets most recent operating state of main state machine
    uint16_t   OpStateListSize;    ///< Gets number of sub-states of main state machine
}; // BDBB Converter State Machine operating state ID

typedef struct BDBB_STATE_ID_s BDBB_STATE_ID_t;  // BDBB Converter State Machine operating state ID data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power controller delay period settings
 * @extends BDBB_CONVERTER_STARTUP_s
 * 
 * @details
 *  This data structure is used to store and buffer settings such as power on delay, power good delay
 *  and ramp up time. It further covers private values like startup counters and reference values
 *  for voltage and current, which are used internally by the controller (read only) but are still
 *  accessible for external code modules for monitoring, diagnostics and fault handling purposes.
 *
 **************************************************************************************************** */
struct BDBB_PERIOD_HANDLER_s 
{
    uint16_t Counter;       ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t Period;        ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    
}; // Power converter soft-start auxiliary variables
typedef struct BDBB_PERIOD_HANDLER_s BDBB_PERIOD_HANDLER_t; // Power converter soft-start auxiliary variables data types

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic parameter monitor settings
 * @extends BDBB_CONVERTER_STARTUP_s
 * 
 * @details
 *  This startup condition data set is used to verify that a specific parameter is within
 *  a valid operating range before continuing the startup procedure. The parameter 'Threshold'
 *  specifies a signal level or reference value, which is used to determine if the software
 *  is allowed to move on. The 'Period' setting is used to determine when the wait period
 *  should be terminated and a timeout error should be returned
 * 
 **************************************************************************************************** */
struct BDBB_PARAM_MONITOR_HANDLER_s 
{
    uint16_t Threshold;        ///< Soft-Start parameter Ok/Fail threshold
    uint16_t DelayCounter;     ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t DelayPeriod;      ///< Soft-Start Minimum monitoring. Enforces a minimum monitoring period while the parameter is in a valid range.
    uint16_t TimeoutCounter;   ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t TimeoutPeriod;    ///< Soft-Start Timeout Period. Maximum monitoring period after which a timeout error is returned.
    
}; // Power converter soft-start auxiliary variables
typedef struct BDBB_PARAM_MONITOR_HANDLER_s BDBB_PARAM_MONITOR_HANDLER_t; // Power converter soft-start auxiliary variables data types

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic voltage or current reference transition ramp settings
 * @extends BDBB_CONVERTER_STARTUP_s
 * 
 * @details
 *  In some cases power supplies must be started in open loop due to limited available of feedback 
 *  signals (e.g. due to common mode voltage range limitations, for example, when feedback components 
 *  are biased from the converted voltage). To overcome this obstacle, the startup engine of the 
 *  state machine provides the declaration of an open loop startup period, which is defined by the 
 *  control output value step size and an target cut-off voltage level at which the state machine 
 *  will close the control loop and switch to a controlled ramp up period.
 *
 **************************************************************************************************** */
struct BDBB_REFERENCE_RAMP_HANDLER_s 
{
    uint16_t Counter;          ///< Soft-Start Execution Counter. This setting is set/cleared by the device driver and is 'read only'.
    uint16_t Period;           ///< Soft-Start Period (POD, RAMP PERIOD, PGD, etc.)
    uint16_t StepSize;         ///< Size/value of one reference increment/decrement or this period
    uint16_t Start;            ///< Reference value transition start point 
    uint16_t Stop;             ///< Reference value transition end point 
    uint16_t refBuffer;        ///< READ ONLY: \n Internal reference value buffer manipulated by state machine during reference level transition
    uint16_t preDelayBuffer;   ///< READ ONLY: \n Internal value buffer used to inject a startup delay compensating for variations of the voltage ramp start level caused by pre-biased output voltages \n This buffer is only used during the startup process and does not affect runtime changes of output references.
    
}; // Power converter soft-start auxiliary variables
typedef struct BDBB_REFERENCE_RAMP_HANDLER_s BDBB_REFERENCE_RAMP_HANDLER_t; ///< Power converter soft-start auxiliary variables data types

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Power Converter Monitored Open Loop Startup Settings
 * @extends BDBB_CONVERTER_STARTUP_s
 * 
 * @details
 *  This data structure is used to set the monitored open loop startup settings such as the open 
 *  loop soft-start increment per execution tick.
 * 
 **************************************************************************************************** */
struct BDBB_OPEN_LOOP_STARTUP_s 
{
    bool     Enabled; ///< User option enabling the open loop startup feature, which will increment the control output until the output voltage is above the specified minimum cut-off voltage
    uint16_t CutOffVoltage; ///< Monitored open loop startup cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t CutOffHysteresis; ///< Hysteresis above cut-off voltage, at which the state machine will switch over into closed loop output ramp-up mode
    uint16_t StepSize; ///< Value increment per execution step during monitored open loop startup. This value will be determined by the state machine during startup and is 'read only'.
    uint16_t Counter; ///< internal open loop startup process counter
    
}; // Power converter start-up settings and variables
typedef struct BDBB_OPEN_LOOP_STARTUP_s  BDBB_OPEN_LOOP_STARTUP_t; // Power converter open-loop start-up settings and variables data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Power Converter Startup Timing Settings
 * @extends IBdbbConverter_s
 *************************************************************************************************** */
 struct BDBB_CONVERTER_STARTUP_s 
 {
    BDBB_PERIOD_HANDLER_t PowerOnDelay;        ///< Declaration of Power On Delay period and timeout
	BDBB_PERIOD_HANDLER_t RelayDelay;          ///< Declaration of Relay period and timeout
    BDBB_PERIOD_HANDLER_t PowerGoodDelay;      ///< Declaration of Power Good period and timeout
    BDBB_REFERENCE_RAMP_HANDLER_t IRamp;       ///< Declaration of output current ramp reference increment step size and period
    BDBB_REFERENCE_RAMP_HANDLER_t LVRamp;      ///< Declaration of output voltage ramp reference increment step size and period in buck mode
    BDBB_REFERENCE_RAMP_HANDLER_t HVRamp;      ///< Declaration of output voltage ramp reference increment step size and period in boost mode
    BDBB_OPEN_LOOP_STARTUP_t OpenLoopStart;    ///< Monitored Open Loop Startup Settings
    BDBB_PARAM_MONITOR_HANDLER_t VCapMonitor;  ///< Output capacitor charge monitor (in Boost mode only)
    
}; // Power converter start-up settings and variables
typedef struct BDBB_CONVERTER_STARTUP_s  BDBB_CONVERTER_STARTUP_t; // Power converter start-up settings and variables data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Publicly accessible data buffer of most recent runtime data values
 * @extends IBdbbConverter_s
 * 
 **************************************************************************************************** */
struct BDBB_CONVERTER_DATA_s 
{
    int16_t ISense[BDBB_MPHASE_COUNT];  ///< BDBB phase current sense value (rectified)
    int16_t IAverage;                   ///< BDBB accumulated, averaged, rectified sum of phase currents
    int16_t VHighVoltage;               ///< BDBB high voltage port
    int16_t VLowVoltage;                ///< BDBB low voltage port
    int16_t Temperature;                ///< BDBB board temperature
    
    int16_t ControlInput;               ///< BDBB most recent control input value
    int16_t ControlError;               ///< BDBB most recent control error value
    int16_t ControlOutput;              ///< BDBB most recent control output value
    
}; // BDBB runtime data buffer
typedef struct BDBB_CONVERTER_DATA_s BDBB_CONVERTER_DATA_t; // BDBB runtime data buffer data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Power Converter Synchronous Rectifier Control Settings
 * @extends BDBB_CONVERTER_SETTINGS_s
 * 
 * @details
 *  This data structure is used to set the required parameters for automated synchronous rectifier 
 *  control. This control scheme enables and disables the synchronous rectifier drive output in
 *  depending on load. This data structure holds threshold parameters for when the synchronous 
 *  rectifier should turn on and off.
 * 
 **************************************************************************************************** */
struct BDBB_SYNC_RECTIFIER_CONTROL_s 
{
    int16_t OnThreshold;           ///< Signal level above which the synchronous rectifier PWM output turns on
    int16_t OffThreshold;          ///< Signal level below which the synchronous rectifier PWM output turns off
    
}; // Power converter automatic synchronous rectifier control settings
typedef struct BDBB_SYNC_RECTIFIER_CONTROL_s  BDBB_SYNC_RECTIFIER_CONTROL_t; // Power converter automatic synchronous rectifier control settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Fundamental power controller control settings
 * @extends IBdbbConverter_s
 * 
 * @details
 *  This data structure allows external software instances to control the power converter 
 *  at runtime, such as references and features. This control interface is decoupled from
 *  the cycle-by-cycle control. Most recent parameter values changed by external firmware 
 *  modules will be picked up and applied by the power converter state machine at the next
 *  execution cycle.
 * 
 *  - Control Flags
 *      - Enable        Enables/Disables the power converter
 *      - Autorun       When set, the power converter will automatically start up once all start up conditions are met
 *      - GO            Control flag to manually start the power converter if (AUTOSTART=false)
 *      - SwapEnable    When set, swaps the feedback input/output and reference of the real-time control stage, switching the power converter operation from buck- to boost mode and vice versa
 *      - AsyncMode     Enables/disables the synchronous rectifier control output(s), effectively turning on/off the synchronous rectification of the power stage
 *      - AutoAsyncMode Enables/disables the synchronous rectifier control output(s) based on the on/off thresholds declared in option 'SyncRectifier'
 *      - CsCalibEnable Enables/disables the automatic current sense feedback signal calibration, which will be executed/skipped after the converter comes out of state 'Standby'
 *  
 **************************************************************************************************** */
struct BDBB_CONVERTER_PROPERTIES_s 
{
    BDBB_SYNC_RECTIFIER_CONTROL_t SyncRectifier; ///< Power converter automatic synchronous rectifier control settings

    uint16_t PhaseCount;       ///< number of converter phases
    uint16_t LVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in buck mode
    uint16_t HVReference;      ///< User low-voltage port reference setting used to control the power converter output voltage in boost mode
    int16_t  IReference;       ///< User current reference setting used to control the power converter controller

    bool Go;                   ///< Control Flag: While GO = false, the power supply is kept in standby, the power supply will start, when GO is set to 'true'. GO is set automatically, when option 'Autostart' is enabled.
    bool Swap;                 ///< Control Flag: When SWAP = true, triggers the switch over of power transfer direction at runtime without state change. This flag is reset when the switch-over has been performed.
    bool Suspend;              ///< Control Flag: When set, state machine is pushed to and hold in RESET

    bool EnforcedOpenLoop;     ///< Control Flag: When set, enables PWM output while keeping control loop disabled; Skips closed loop ramp up states and directly jumps to state Online 
    bool PowerOnDelayByPass;   ///< Control Flag: When set, it bypasses the power on delay state in state machine once and will be auto cleared 
    bool HotSwapEnable;        ///< Control Flag: When set, allows switching of power transfer direction immediately at runtime without state change. When cleared, a change of power transfer direction forces the power converter through a shut-down and restart cycle
    bool AsyncMode;            ///< Control Flag: When set, suspending the synchronous rectifier switch drive PWM channel(s)
    bool AutoAsyncMode;        ///< Control Flag: When set, enables the automatic synchronous rectifier control using thresholds declared in 'Settings.SyncRectifierControl'
    bool CsCalibEnable;        ///< Control Flag: When set, enforcing current sense calibration
    bool ConversionMode;       ///< Control Flag: When set, power transfer direction is set to up-stream (boost) mode, when cleared, power transfer direction is set to down-stream (buck) mode)
    bool Autorun;              ///< Control Flag: When set, converter starts up automatically as soon as all start conditions are met. \nWhen cleared, converter will be held in standby until the GO bit is set
    bool Enable;               ///< Control Flag: When set, enables the converter triggering a startup sequence; When cleared, forces teh power supply to reset and turn off

};  ///> BDBB converter main settings
typedef struct BDBB_CONVERTER_PROPERTIES_s BDBB_CONVERTER_PROPERTIES_t;  ///> BDBB converter main settings

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @extends BDBB_LOOP_SETTINGS_s
 * @brief   Set of control loop default functions of a control loop data object of type NPNZ16b_t
 **********************************************************************************/
struct BDBB_LOOP_PARAMETER_BUFFER_s
{   // Properties (user settings)
    
    int16_t Reference;             ///< Nominal control loop voltage reference variable
    int16_t Minimum;               ///< output clamping value (minimum)
    int16_t Maximum;               ///< output clamping value (maximum)

    int16_t AltReference;          ///< Alternate nominal control loop voltage reference variable in reverse mode
    int16_t AltMinimum;            ///< Alternate output clamping value (minimum)
    int16_t AltMaximum;            ///< Alternate output clamping value (maximum)

};
typedef struct BDBB_LOOP_PARAMETER_BUFFER_s BDBB_LOOP_PARAMETER_BUFFER_t; ///< User defined settings for control loops data type

/***********************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @extends BDBB_LOOP_SETTINGS_s
 * @brief   Set of control loop default functions of a control loop data object of type NPNZ16b_t
 **********************************************************************************/
struct BDBB_LOOP_METHODS_s
{
    // Methods (Function pointers to control loop object functions)
    volatile uint16_t (*Initialize)(volatile NPNZ16b_t*); ///< Function pointer to INIT routine
    void (*Precharge)(volatile NPNZ16b_t*, volatile fractional, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control output and error histories
    void (*PrechargeControlHistory)(volatile NPNZ16b_t*, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control output history
    void (*PrechargeErrorHistory)(volatile NPNZ16b_t*, volatile fractional); ///< Function pointer to PRECHARGE routine, loading user values into control error history
    void (*Reset)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control output and error histories
    void (*ResetControlHistory)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control output history
    void (*ResetErrorHistory)(volatile NPNZ16b_t*); ///< Function pointer to RESET routine, clearing the control error history
    void (*Update)(volatile NPNZ16b_t*); ///< Function pointer to UPDATE routine, calling a regular loop control step
    void (*PTermUpdate)(volatile NPNZ16b_t*); ///< Function pointer to PTerm UPDATE routine, calling a unity gain loop control step
    void (*OutputMultiplexing)(volatile NPNZ16b_t*); ///< Function pointer to control output multiplexing function (e.g. for PWM and ADC trigger distribution)
    void (*CurrentBalancing)(volatile NPNZ16b_t*); ///< Function pointer to current balancing function (e.g. for control output offset adjustment based on ADC feedback deviation)

};
typedef struct BDBB_LOOP_METHODS_s BDBB_LOOP_METHODS_t; ///< User defined settings for control loops data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power control loop settings
 * @extends BDBB_CONVERSION_CONTROL_t
 * 
 * @details
 *  This data structure is used to set the control loop settings such as pointers to controller 
 *  objects and its function calls as well as basic user settings such as reference, feedback
 *  signal offsets, trigger delays and minimum/maximum output clamping values.
 * 
 **************************************************************************************************** */
struct BDBB_LOOP_SETTINGS_s 
{
    // Control Loop Object
    volatile NPNZ16b_t* Controller; ///< pointer to control loop object data structure

    // Control Loop Default Settings & Methods
    BDBB_LOOP_PARAMETER_BUFFER_t Properties; // Control loop properties store default configuration information
    BDBB_LOOP_METHODS_t Methods; // Control loop methods hold function pointers to control loop default functions

}; ///< User defined settings for control loops; 
typedef struct BDBB_LOOP_SETTINGS_s BDBB_LOOP_SETTINGS_t; ///< User defined settings for control loops data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter switch-node specifications
 * @extends BDBB_SWITCH_NODE_SETTINGS_s
 * 
 * @details
 *  Settings covered in this sub data structure are only used in peak current mode control (PCMC) and are
 *  ignored in all other control modes such as voltage mode control (VMC) or average current mode control 
 *  (ACMC). Peak current mode control is enabled by setting 
 * 
 * @code{.c}
 *  bdbb.set_values.control_mode = BDBB_CONTROL_MODE_PCMC. 
 * @endcode
 * 
 *  By enabling peak current mode control, an on-chip analog comparator with integrated ramp generator 
 *  DAC will be initialized and configured to truncate the active duty cycle of the main PWM generator 
 *  instance, declared in the switch node settings of the BDBB converter object. The compensation ramp 
 *  is controlled by the PWM generator triggers 1 and 2, where the setting of Trigger 1 determines the 
 *  starting point of the ramp and Trigger 2 determines the stop-point of the ramp. The ramp will 
 *  automatically reset when the analog comparator has tripped or when the stop-point has been reached 
 *  without, whatever event occurs first.
 * 
 **************************************************************************************************** */
struct BDBB_SLOPE_COMP_SETTINGS_s
{
    uint16_t DacInstance; ///< Digital-to-analog converter / Analog comparator instance used to triggering the the slope compensation ramp generator
    uint16_t CompInput; ///< Analog comparator input option A to D (0=A, 1=B, 2=C, 3=D)
    uint16_t LebPeriod; ///< Leading-Edge Blanking period
    uint16_t PwmTriggerInput; ///< PWM current limit PCI input source (must match comparator declared by cmp_instance)
    uint16_t SlewRate; ///< Integer equivalent of compensation ramp slew rate in [V/�s]
    uint16_t SlopeStartTrigger; ///< Period counter value at with the slope start is triggered
    uint16_t SlopeStopTrigger; ///< Period counter value at with the slope stop is triggered
    uint16_t DacValueInit;     ///< Initial Digital-to-Analog converter reference value (used when peripheral is enabled or being reset)
    uint16_t DutCycleLimit;    ///< Switch node duty cycle limit value
    
}; ///< Peak Current Mode Control Slope Compensation Settings
typedef struct BDBB_SLOPE_COMP_SETTINGS_s BDBB_SLOPE_COMP_SETTINGS_t; ///< Peak Current Mode Control Slope Compensation Settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter switch-node specifications
 * @extends BDBB_CONVERTER_s
 * 
 * @details
 *  This data structure is used to set the converter switch-node specifications declaring which
 *  PWM channel is used as well as its switching frequency, phase-shift, dead times and duty ratio
 *  limits.
 * 
 **************************************************************************************************** */
struct BDBB_SWITCH_NODE_SETTINGS_s 
{
    P33C_PWM_INSTANCE_t* PwmInstance;   ///< Pointer to PWM generator instance used
    P33C_GPIO_t GpioHigh;               ///< GPIO port pin-number of PWMxH of the selected PWM generator
    P33C_GPIO_t GpioLow;                ///< GPIO port pin-number of PWMxL of the selected PWM generator
    bool        SwapOutputs;            ///< Selecting if PWMxH (default) or PWMxL should be the leading default PWM output
    bool        MasterPeriodEnable;     ///< Selecting MASTER or Individual period register
    bool        HighResolutionEnable;   ///< Selecting if PWM module should use high-resolution mode 
    bool        SyncDrive;              ///< Selects if switch node supports synchronous or asynchronous commutation
    uint16_t    Period;                 ///< Switching period
    uint16_t    PhaseShift;             ///< Switching signal phase-shiftbetween multiple phases
    uint16_t    DutyRatioMinimum;       ///< Absolute duty cycle minimum during normal operation
    uint16_t    DutyRatioMaximum;       ///< Absolute duty cycle maximum during normal operation
    uint16_t    DeadTimeRising;         ///< Dead time setting at rising edge of a half-bridge drive
    uint16_t    DeadTimeFalling;        ///< Dead time setting at falling edge of a half-bridge drive
    uint16_t    ControlTriggerDivider;  ///< PWM triggers for ADC will be generated every n-th cycle  (only affects ADC Trigger #1, see register PGxEVTL for details)
    uint16_t    ControlTriggerOffset;   ///< PWM triggers for ADC will be offset by n cycles (only affects ADC Trigger #1, see register PGxEVTL for details)
    
    BDBB_SLOPE_COMP_SETTINGS_t SlopeCompensation; ///< Peak current mode slope compensation settings (peak current mode control only)
    
}; ///< Switching signal timing settings
typedef struct BDBB_SWITCH_NODE_SETTINGS_s  BDBB_SWITCH_NODE_SETTINGS_t; ///< Switching signal timing settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter feedback specifications
 * @extends BDBB_FEEDBACK_CHANNEL_t
 * @details
 *  This data structure is used to set the converter feedback specifications declaring which
 *  ADC channels are used including the individual AD input configuration such as trigger mode,
 *  input mode, result format and value normalization.
 * 
 **************************************************************************************************** */
struct BDBB_ADC_SIGNAL_SCALING_s 
{
    int16_t Factor; ///< Fractional scaling factor (range -1 ... 0.99969)
    int16_t Scaler; ///< Feedback bit-shift scaler used for number normalization
    int16_t Offset; ///< Signal offset as signed integer to be subtracted from ADC input
    bool  Polarity; ///< Signal polarity declarations, where (0 = positive, 1 = negative)

}; ///< ADC input signal scaling = (ADCBUF - [offset]) * [factor] >> 2^[scaler]
typedef struct BDBB_ADC_SIGNAL_SCALING_s BDBB_ADC_SIGNAL_SCALING_t; ///< ADC input signal scaling data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter feedback specifications
 * @extends BDBB_FEEDBACK_CHANNEL_t
 *
 * @details
 *   This data structure holds information about the valid signal level range at this 
 *   feedback channel. Declared min/max values may be used for fault detection, nominal
 *   signal level may serve as reference.
 * 
 **************************************************************************************************** */
struct BDBB_ADC_SIGNAL_RANGE_s 
{
    int16_t Minimum;    ///< Minimum valid signal level value
    int16_t Nominal;    ///< Nominal expected signal level value
    int16_t Maximum;    ///< Maximum valid signal level value

}; ///< ADC input signal range
typedef struct BDBB_ADC_SIGNAL_RANGE_s BDBB_ADC_SIGNAL_RANGE_t; ///< ADC input signal range data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types 
 * @brief   Generic power converter feedback channel interrupt configuration
 * @extends BDBB_FEEDBACK_CHANNEL_t
 *
 * @details
 *  This data structure is used to configure the software interrupt service routine settings
 *  of a converter feedback channel. 
 * 
 * @note
 *  These settings only refer to the software interrupt service routine of the declared
 *  analog input channel and does not cover the configuration of its sample trigger as 
 *  ADC samples may be triggered without the intend to raise the related software interrupt.
 * 
 **************************************************************************************************** */
struct BDBB_ADC_INTERRUPT_s 
{
    bool InterruptEnable;      ///< input channel interrupt enable bit (allows tripping an Interrupt Service Routine)
    bool EarlyInterruptEnable; ///< input channel early interrupt enable bit (triggers the interrupt early before conversion is complete)

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_INTERRUPT_s BDBB_ADC_INTERRUPT_t; ///< ADC input interrupt settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter feedback channel interrupt configuration
 * @extends BDBB_FEEDBACK_CHANNEL_s
 *
 * @details
 *  This data structure is used to configure the software interrupt service routine settings
 *  of a converter feedback channel. 
 * 
 * @note
 *  These settings only refer to the software interrupt service routine of the declared
 *  analog input channel and does not cover the configuration of its sample trigger as 
 *  ADC samples may be triggered without the intend to raise the related software interrupt.
 * 
 ***************************************************************************************************** */
struct BDBB_ADC_INPUT_s 
{
    uint8_t   AdcInput;            ///< number of the ADC input channel used
    uint8_t   AdcAltInput;         ///< selection of the alternate ADC input channel used (dedicated ADC cores only, set =0 for shared ADC inputs)
    uint8_t   AdcCore;             ///< index of the ADC core the selected analog input channel is assigned to
    bool      DifferentialInput;   ///< input channel differential mode enable bit
    bool      SignedResult;        ///< input channel singed result mode enable bit
    uint16_t  GpioInstance;        ///< GPIO instance of the selected PWM generator

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_INPUT_s BDBB_ADC_INPUT_t; ///< ADC input settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter feedback channel sample trigger configuration
 * @extends BDBB_FEEDBACK_CHANNEL_s
 *
 * @details
 *  This data structure is used to configure the ADC sample trigger configuration 
 *  of a converter feedback channel. 
 * 
 * @note
 *  ADC samples are triggered by other peripherals such as PWM modules or timers 
 *  or manually from software. This data structure is only used to configure the
 *  trigger input to the ADC but does not configure the selected trigger source.
 * 
 *****************************************************************************************************/
struct BDBB_ADC_TRIGGER_s 
{
    volatile uint16_t* ptrTriggerRegister;  ///< pointer to trigger register of this 
    uint8_t   TriggerSource;       ///< input channel trigger source
    uint16_t  TriggerDelay;        ///< input channel trigger delay (fixed, will be added to trigger delay set in PWM generator register PGxTRIGy)
    bool      LevelTrigger;        ///< input channel level trigger mode enable bit

}; ///< ADC input interrupt settings
typedef struct BDBB_ADC_TRIGGER_s BDBB_ADC_TRIGGER_t; ///< ADC input interrupt settings data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter ADC input channel configuration
 * @extends BDBB_FEEDBACK_SETTINGS_s
 *****************************************************************************************************/
struct BDBB_FEEDBACK_CHANNEL_s 
{
    bool Enabled;                      ///< input channel enable bit
    BDBB_ADC_INPUT_t Input;            ///< ADC input channel specifications
    BDBB_ADC_TRIGGER_t Trigger;        ///< ADC input sample trigger configuration
    BDBB_ADC_INTERRUPT_t Interrupt;    ///< ADC sample interrupt service routine call configuration
    BDBB_ADC_SIGNAL_SCALING_t Scaling; ///< ADC result normalization settings
    BDBB_ADC_SIGNAL_RANGE_t Range;     ///< ADC result valid data range specification

}; ///< ADC input channel configuration
typedef struct BDBB_FEEDBACK_CHANNEL_s BDBB_FEEDBACK_CHANNEL_t; ///< ADC input channel configuration data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Power converter feedback input channel declarations
 * @extends BDBB_CONVERTER_s
 ***************************************************************************************************** */
struct BDBB_FEEDBACK_SETTINGS_s 
{
    BDBB_FEEDBACK_CHANNEL_t AdVHighVoltage;             ///< ADC input sampling input voltage
    BDBB_FEEDBACK_CHANNEL_t AdVLowVoltage;              ///< ADC input sampling output voltage
    BDBB_FEEDBACK_CHANNEL_t AdISense[BDBB_MPHASE_COUNT]; ///< ADC input sampling phase current
    BDBB_FEEDBACK_CHANNEL_t AdTemperature;              ///< ADC input sampling temperature
    
}; ///< BDBB converter feedback declarations
typedef struct BDBB_FEEDBACK_SETTINGS_s BDBB_FEEDBACK_SETTINGS_t; ///< BDBB converter feedback declarations data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter GPIO specifications
 * @extends BDBB_GPIO_SETTINGS_t
 * 
 * @details
 *  This data structure is used to set the converter GPIO specifications declaring which
 *  if and which additional GPIOs are used by the converter controller, such as POWER_GOOD.
 * 
 **************************************************************************************************** */
struct BDBB_GPIO_INSTANCE_s 
{
    bool        Enabled;    ///< Specifies, if this IO is used or not
    P33C_GPIO_t IoPin;      ///< Assigns a device pin to be controlled by the library
    uint16_t    Polarity;   ///< Output polarity, where 0=ACTIVE HIGH, 1=ACTIVE_LOW
    uint16_t    IoType;     ///< Input/Output definition (0=push-pull output, 1=input, 2=open-drain output)

}; ///< GPIO instance of the converter control GPIO
typedef struct BDBB_GPIO_INSTANCE_s BDBB_GPIO_INSTANCE_t; ///< GPIO instance of the converter control GPIO data type

/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter GPIO specifications
 * @extends BDBB_CONVERTER_s
 ****************************************************************************************************** */
struct BDBB_GPIO_SETTINGS_s 
{
    BDBB_GPIO_INSTANCE_t EnableInput; ///< External ENABLE input
    BDBB_GPIO_INSTANCE_t PowerGood; ///< Power Good Output
    BDBB_GPIO_INSTANCE_t EnableRelay; ///< Power Good Output

}; ///< GPIO instance of the converter control GPIO
typedef struct BDBB_GPIO_SETTINGS_s BDBB_GPIO_SETTINGS_t; ///< GPIO instance of the converter control GPIO data type


/****************************************************************************************************
 * @ingroup svc-layer-bdbb-sm-type-definitions-data-types
 * @brief   Generic power converter real-time control data objects
 * @extends BDBB_CONVERTER_s
 **************************************************************************************************** */
struct BDBB_CONVERSION_CONTROL_s {

    BDBB_CONTROL_MODE_t ControlMode;   ///< Fundamental control mode 
    
    // Power Supply Control Configuration 
    BDBB_LOOP_SETTINGS_t VLoop;            ///< BDBB voltage control loop object
    BDBB_LOOP_SETTINGS_t ILoop[BDBB_MPHASE_COUNT]; ///< BDBB Current control loop objects
    
}; ///< BDBB converter real-time control data objects
typedef struct BDBB_CONVERSION_CONTROL_s BDBB_CONVERSION_CONTROL_t; ///< BDBB converter real-time control data objects data type

#endif  /* BDBB_CONVERTER_TYPE_DEF_H */

// ______________________________________
// end of file
