/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    moving_average.h
 * @brief   Assembler routine function calls and data types declarations of generic moving average calculation
 * @author  M91406
 * @date    04/21/22
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef P33C_LIB_MOVING_AVERAGE_H
#define	P33C_LIB_MOVING_AVERAGE_H

/***********************************************************************************
 * @ingroup lib-layer-p33c-functions-public-moving-average
 * @struct  AVG_TEMP_DATA_s
 * @extends AVERAGE_DATA_SOURCE_s
 * @brief   Temporary data buffer for managing moving average value
 * @details
 *  This is the temporary data buffer of the moving average filter function.
 *  It is managed by the moving average filter routine and should not be 
 *  accessed or changed by user code.
 **********************************************************************************/
struct AVG_TEMP_DATA_s {

    uint16_t  pointer;     ///< most recent array index pointer
    uint16_t  samDepth;    ///< Number of data points included in average calculation
    int32_t   avgSum;      ///< Running sum of data values across data array
};
typedef struct AVG_TEMP_DATA_s AVG_TEMP_DATA_t; ///< internal runtime data buffer. Do not use

/***********************************************************************************
 * @ingroup lib-layer-p33c-functions-public-moving-average
 * @struct  AVERAGE_DATA_SOURCE_s
 * @brief   Moving average data object
 * @details
 *  This data object holds all user configurations and results of the moving
 *  average across a user-specified data array. 
 **********************************************************************************/
struct AVERAGE_DATA_SOURCE_s {

    int16_t*  ptrSource;   ///< Pointer to data source register or variable
    int16_t*  ptrArray;    ///< Pointer to user defined data array
    uint16_t  ArraySize;   ///< Size of user defined data array
    int16_t   minInput;    ///< Minimum input value limit 
    int16_t   maxInput;    ///< Maximum input value limit 
    int16_t   Average;     ///< Running average value tracked across user array
    AVG_TEMP_DATA_t tmp;   ///< internal runtime data buffer. Do not use
};
typedef struct AVERAGE_DATA_SOURCE_s AVERAGE_DATA_SOURCE_t;

/*********************************************************************************
 * @ingroup lib-layer-p33c-functions-public-moving-average
 * @brief   Update the moving average value across a given data array
 * @param   Moving average data object of type AVERAGE_DATA_SOURCE_t
 * 
 * @details
 *  By calling this function, a new value will be read from the declared
 *  source address, which will overwrite the oldest value in the array.  
 *  After the data contents have been updated, the average value of all 
 *  data in the array and its extremes (min, max) is updated.
 *
 **********************************************************************************/
extern void UpdateAverage(AVERAGE_DATA_SOURCE_t* dataSource);

/*********************************************************************************
 * @ingroup lib-layer-p33c-functions-public-moving-average
 * @brief   Clears the moving average data object
 * @param   Moving average data object of type AVERAGE_DATA_SOURCE_t
 * 
 * @details
 *  By calling this function, the declared moving average data object will 
 *  be cleared and all data in the data array as well as the extremes values 
 *  (min, max) and the tracked average value will be reset to zero.
 *
 **********************************************************************************/
extern void ClearAverage(AVERAGE_DATA_SOURCE_t* dataSource);

#endif	/* P33C_LIB_MOVING_AVERAGE_H */

// ______________________________________
// end of file

