/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    II2cHandler.h
 * @author  A69612
 * @brief   Global i2c handler interface header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef I2C_HANDLER_INTERFACE_H
#define	I2C_HANDLER_INTERFACE_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer types header file
#include <stdbool.h> // include standard boolean types header file

#include "xc_pral.h"
#include "core/p33c_plib_i2c_handler.h"
#include "core/p33c_plib_i2c_datamngmnt.h"

/*****************************************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @struct  PLIB33C_I2C_CONTROL_CONFIG_t
 * @brief   I2C Control Configuration
 *****************************************************************************************************/
struct PLIB33C_I2C_CONTROL_CONFIG_s {
    PLIB33C_I2C_HOST_CLIENT_MODE_t  ModeSelect;         ///< User I2C mode selection
    PLIB33C_I2C_BAUDRATE_t          BaudRate;           ///< User I2C baud rate for Host mode
    PLIB33C_I2C_SMBUS_IO_LEVEL_t    SMBusThreshold;     ///< User I2C SMBus input threshold level
    uint16_t                        AddressHold;        ///< User I2C address hold enable bit for Client Mode
    uint16_t                        DataHold;           ///< User I2C data hold enable bit for Client Mode
    uint16_t                        ClockStretch;       ///< User I2C clock stetch enable bit for Client Mode
    uint16_t                        CollisionDetect;    ///< User I2C collision detect enable bit for Client Mode
};
typedef struct PLIB33C_I2C_CONTROL_CONFIG_s PLIB33C_I2C_CONTROL_CONFIG_t;

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @struct  PLIB33C_I2C_OBJECT_t
 * @brief   This data structure is a collection of parameters that the user can configure.
 *          This data object can be initialize as follows.
 * @code {.c}
	PLIB33C_I2C_OBJECT_t i2cConfig = {
                .Control.ModeSelect = I2C_CLIENT_MODE,
                .Control.SMBusThreshold = I2C_DISABLE_SMBUS_IO_LEVEL_THRESHOLD,
                .Control.AddressHold = true,
                .Control.DataHold = true,
                .Control.ClockStretch = true,
                .Control.CollisionDetect = false,
                .Address = 0x3C,
                .CommPort.i2cInstance = I2C_NUM_2,
                .CommPort.IsrPriorityLvl = 4,
                .CommPort.DmaInstance = 0
            };
			
	@endcode
 *********************************************************************************/
struct PLIB33C_I2C_OBJECT_s {
    PLIB33C_I2C_CONTROL_CONFIG_t    Control;    ///< User I2C control configuration
    PLIB33C_I2C_INTERFACE_t         CommPort;   ///< User I2C communication port
    uint16_t                        Address;    ///< User I2C address
}; ///< Generic i2c object 
typedef struct PLIB33C_I2C_OBJECT_s PLIB33C_I2C_OBJECT_t; ///< Generic i2c object data type

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @struct  PLIB33C_I2C_SEND_BUFFER_t
 * @brief   This data structure will hold the I2C Send/Transmit Buffer. The user
 *          can initialize this data object as follows: 
 * @code {.c} 
    PLIB33C_I2C_SEND_BUFFER_t sendDataBuffer = {
                        .TxD = (uint8_t*)&i2cSendBuff,
                        .TxDSize = PLIB33C_I2C_TX_BUFFER_SIZE
                    };
	@endcode
 *********************************************************************************/
struct PLIB33C_I2C_SEND_BUFFER_s
{
    uint8_t*    TxD;            ///< User transmit buffer
    uint16_t    TxD8BitSize;    ///< Size of the transmit buffer
}; ///< Transmit/Send data object  
typedef struct PLIB33C_I2C_SEND_BUFFER_s PLIB33C_I2C_SEND_BUFFER_t; ///< Transmit/Send object data type  

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @struct  PLIB33C_I2C_RECEIVE_BUFFER_t
 * @brief   This data structure will hold the I2C Receive Buffer. The user
 *          can initialize this data object as follows: 
 * @code {.c} 
    PLIB33C_I2C_RECEIVE_BUFFER_t rcvDataBuffer = {
                        .RxD = (uint8_t*)&i2cReceiveBuff,
                        .RxDSize = PLIB33C_I2C_RX_BUFFER_SIZE
                    };
	@endcode
 *********************************************************************************/
struct PLIB33C_I2C_RECEIVE_BUFFER_s
{
    uint8_t*    RxD;            ///< User receive buffer
    uint16_t    RxD8BitSize;    ///< Size of the receive buffer
}; ///< Receive data object  
typedef struct PLIB33C_I2C_RECEIVE_BUFFER_s PLIB33C_I2C_RECEIVE_BUFFER_t; ///< Receive object data type  

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @def     PLIB33C_I2C_RX_BUFFER_SIZE
 * @def     PLIB33C_I2C_TX_BUFFER_SIZE
 * @var     plib33cI2cRcvBufferArr[]
 * @var     plib33cI2cRcvBufferArrSize
 * @var     plib33cI2cSendBufferArr[]
 * @var     plib33cI2cSendBufferArrSize
 * 
 * @brief   Receive/Transmit buffer default declarations
 * 
 * @details
 *     This I2C driver provides built-in receive and transmit data buffers. The 
 *     macros PLIB33C_I2C_RX_BUFFER_SIZE and PLIB33C_I2C_TX_BUFFER_SIZE define 
 *     their default size. Should the declaration be overwritten by user 
 *     declarations, variables plibI2cRcvBufferArrSize and plibI2cSendBufferArrSize 
 *     can be used to derive the real size of these buffer byte arrays.
 * 
 * @note
 *     The receive and transmit buffers are defined in file p33c_plib_i2c_datamngmnt.c 
 *     using the 'weak' attribute to allow users to override the buffer array 
 *     declarations with another declaration of the same name in user code.
 *     By doing so, users can declare a new array size and thus adapt the buffer
 *     size to their individual application requirements.
 **********************************************************************************/
#define PLIB33C_I2C_RX_BUFFER_SIZE     256
#define PLIB33C_I2C_TX_BUFFER_SIZE     256

extern uint8_t  plib33cI2cRcvBufferArr[];
extern uint16_t plib33cI2cRcvBufferArrSize;

extern uint8_t  plib33cI2cSendBufferArr[];
extern uint16_t plib33cI2cSendBufferArrSize;

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @var     plib33cI2cSendBuffer
 * @brief   This I2C driver allows the user to access the buffer of the 
 *          transmitter in case the transmitted data needs to be manage in a 
 *          particular use case.
 *          When using plib33cI2cSendBuffer[], the index of the variable should be 
 *          the instance of I2C minus 1. This is an example of how to use this variable. 
 * @code{.c}    
 *     plib33cI2cSendBuffer[i2cInstance-1]
 * @endcode
 *********************************************************************************/
extern PLIB33C_I2C_SEND_BUFFER_t plib33cI2cSendBuffer[];

/**********************************************************************************
 * @ingroup p33c-plib-i2c-properties
 * @var     plib33cI2cReceiveRingBuffer
 * @brief   This I2C driver allows the user to access the buffer of the 
 *          receiver in case the received data needs to be manage in a 
 *          particular use case.
 *          When using plib33cI2cReceiveRingBuffer[], the index of the variable should be 
 *          the instance of I2C minus 1. This is an example of how to use this variable. 
 * @code{.c}    
 *     plib33cI2cReceiveRingBuffer[i2cInstance-1]
 * @endcode
 *********************************************************************************/
extern PLIB33C_I2C_RECEIVE_RINGBUFFER_t plib33cI2cReceiveRingBuffer[];

/**********************************************************************************
 * Public Functions
 *********************************************************************************/
extern uint16_t plib33c_i2c_DataStatusInitialize(uint16_t i2cInstance);
extern uint16_t plib33c_i2c_DataBufferInitialize(uint16_t i2cInstance,
                        PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer, 
                        PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer);
extern uint16_t plib33c_i2c_InterruptEnable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort,
                        uint16_t modeSelect);
extern uint16_t plib33c_i2c_InterruptDisable(PLIB33C_I2C_INTERFACE_t* interfaceCommPort);
extern bool     plib33c_i2c_RingBufferGet(uint16_t i2cInstance,
                        PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, 
                        uint8_t* bytePointer);
extern uint16_t plib33c_i2c_RingBufferSet(uint16_t i2cInstance,
                        PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, 
                        uint8_t data);

extern uint16_t plib33c_i2c_Initialize(PLIB33C_I2C_OBJECT_t* i2cConfig,
                        PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer,
                        PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer,
                        const P33C_GPIO_t* pinSDA, const P33C_GPIO_t* pinSCL);
extern uint16_t plib33c_i2c_Start(PLIB33C_I2C_OBJECT_t* i2cConfig);
extern uint16_t plib33c_i2c_SendData(PLIB33C_I2C_OBJECT_t* i2cConfig, uint8_t buffer);
extern uint8_t  plib33c_i2c_ReceiveData(PLIB33C_I2C_OBJECT_t* i2cConfig);
extern uint16_t plib33c_i2c_Close(PLIB33C_I2C_OBJECT_t* i2cConfig);

#endif	/* I2C_HANDLER_INTERFACE_H */
