/*
    (c) 2023 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

#include "xc_pral.h" // include peripheral register abstraction layer drivers
#include "../II2cHandler.h"

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cSendBufferArr
 * @brief   I2C send or transmit byte-data buffer
 * @details I2C send or transmit data buffer from which the I2C interface is reading new 
 *          data to be transmitted
 **********************************************************************************/
uint8_t __attribute__((near, weak)) plib33cI2cSendBufferArr[PLIB33C_I2C_TX_BUFFER_SIZE];

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cSendBufferArrSize
 * @brief   I2C send data buffer size
 * @details Size of the I2C send data buffer in bytes
 **********************************************************************************/
uint16_t __attribute__((weak)) plib33cI2cSendBufferArrSize = (sizeof(plib33cI2cSendBufferArr)/sizeof(plib33cI2cSendBufferArr[0])); 

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cRcvBufferArr
 * @brief   I2C receive byte-data buffer
 * @details I2C receive data buffer to which the I2C interface is writing to when
 *          new data is received.
 **********************************************************************************/
uint8_t __attribute__((near, weak)) plib33cI2cRcvBufferArr[PLIB33C_I2C_RX_BUFFER_SIZE];

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cRcvBufferArrSize
 * @brief   I2C receive data buffer size
 * @details Size of the I2C receiver data buffer in bytes
 **********************************************************************************/
uint16_t __attribute__((weak)) plib33cI2cRcvBufferArrSize = (sizeof(plib33cI2cRcvBufferArr)/sizeof(plib33cI2cRcvBufferArr[0])); 

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cSendBuffer
 * @brief   Data object that contains the I2C buffer send properties
 * @details Object 'PLIB33C_I2C_SEND_BUFFER_t' is the object providing access 
 *          to the I2C send data buffers
 **********************************************************************************/
PLIB33C_I2C_SEND_BUFFER_t plib33cI2cSendBuffer[P33C_I2C_COUNT];

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cReceiveRingBuffer
 * @brief   Data object that contains the I2C buffer receive properties
 * @details Object 'PLIB33C_I2C_RECEIVE_BUFFER_t' is the object providing access 
 *          to the I2C receive data buffers
 **********************************************************************************/
PLIB33C_I2C_RECEIVE_RINGBUFFER_t plib33cI2cReceiveRingBuffer[P33C_I2C_COUNT];

/*********************************************************************************
 * @ingroup p33c-plib-i2c-properties-private
 * @var     plib33cI2cDataStatus
 * @brief   Data object that contains the data status of the send and receive
 * @details Object 'PLIB33C_I2C_DATA_STATUS_t' is the object providing access 
 *          to the I2C data status
 **********************************************************************************/
PLIB33C_I2C_DATA_STATUS_t plib33cI2cDataStatus[P33C_I2C_COUNT];

/*********************************************************************************
 * @ingroup p33c-plib-i2c-methods-private
 * @fn      uint16_t plib33c_i2c_DataStatusInitialize(PLIB33C_I2C_INTERFACE_t* interfaceCommPort)
 * @brief   Initializes I2C driver transmit and receive data statuses
 * @param   i2cInstance Instance of the I2C port of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return 0 = failure, Initializing I2C Tx/Rx data status was not successful
 * @return 1 = success, Initializing I2C Tx/Rx data status was successful
 * @details
 *     This function is used to initialize an I2C data statuses. 
 *
 **********************************************************************************/
uint16_t plib33c_i2c_DataStatusInitialize(uint16_t i2cInstance)
{
    int retval = 1;
    uint16_t index = (i2cInstance - 1);

    plib33cI2cDataStatus[index].bits.rxReady = false;
    plib33cI2cDataStatus[index].bits.rxByteReceived = false;
    plib33cI2cDataStatus[index].bits.rxBufferEmpty = true;
    plib33cI2cDataStatus[index].bits.rxBufferOverrun = false;
    plib33cI2cDataStatus[index].bits.txReady = true;
    plib33cI2cDataStatus[index].bits.txDataSent = true;
    
    return(retval);
}

/*********************************************************************************
 * @ingroup p33c-plib-i2c-methods-private
 * @fn      uint16_t plib33c_i2c_RingBufferInitialize(PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, uint8_t buf[], uint16_t size)
 * @brief   Initializes the I2C ring buffer
 * @param   ringbuf Pointer to ring buffer object of type PLIB33C_I2C_RECEIVE_RINGBUFFER_t
 * @param   buf Ring buffer array that will hold the receive bytes of type unsigned char
 * @param   size Size of the ring buffer array of type unsigned integer
 * @return  0 = failure, Initialization of ring buffer is not successful
 * @return  1 = success, Initialization of ring buffer is successful
 *  
 * @details 
 *     This function initializes the ring buffers used by the I2C Receiver.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_RingBufferInitialize(PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf,
                            uint8_t buf[], uint16_t size) 
{
    int retval = 1;

    // Initialize ring buffer
    ringbuf->buf  = &buf[0];
    ringbuf->end  = size;
    ringbuf->head = 0;
    ringbuf->tail = 0;
    
    return(retval);
}

/*********************************************************************************
 * @ingroup p33c-plib-i2c-methods-private
 * @fn      uint16_t plib33c_i2c_DataBufferInitialize(uint16_t i2cInstance, 
 *                                  PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer, 
 *                                  PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer)
 * @brief   Initializes a I2C data buffer with predefined user settings
 * @param   i2cInstance Instance of the I2C port of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param   interfaceSendBuffer Pointer to I2C send buffer object of type PLIB33C_I2C_SEND_BUFFER_t
 * @param   interfaceReceiveBuffer Pointer to I2C receive buffer object of type PLIB33C_I2C_RECEIVE_BUFFER_t
 * @return 0 = failure, Initializing I2C data buffer was not successful
 * @return 1 = success, Initializing I2C data buffer was successful
 * @details
 *     This function can be used to initialize a I2C data buffer by handing over
 *     a predefined user-I2C data buffer object.  
 *
 **********************************************************************************/
uint16_t plib33c_i2c_DataBufferInitialize(uint16_t i2cInstance,
                    PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer, 
                    PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer)
{
    int retval = 1;
    uint8_t index = (i2cInstance - 1);
    
    if(interfaceSendBuffer == NULL)
    {
        plib33cI2cSendBuffer[index].TxD = plib33cI2cSendBufferArr;  ///< Collected data from the frame parser
        plib33cI2cSendBuffer[index].TxD8BitSize = plib33cI2cSendBufferArrSize;  ///< Tx buffer data length
    }
    else
    {
        // Initialize interface data buffer
        plib33cI2cSendBuffer[index].TxD = interfaceSendBuffer->TxD; ///< Collected data from the frame parser
        plib33cI2cSendBuffer[index].TxD8BitSize = interfaceSendBuffer->TxD8BitSize; ///< Tx buffer data length
    }

    if(interfaceReceiveBuffer == NULL)
    {
        retval &= plib33c_i2c_RingBufferInitialize(&plib33cI2cReceiveRingBuffer[index], 
                            plib33cI2cRcvBufferArr, plib33cI2cRcvBufferArrSize);
    }
    else
    {
        retval &= plib33c_i2c_RingBufferInitialize(&plib33cI2cReceiveRingBuffer[index], 
                            interfaceReceiveBuffer->RxD, interfaceReceiveBuffer->RxD8BitSize);
    }

    return (retval);
}

/*********************************************************************************
 * @ingroup p33c-plib-i2c-methods-private
 * @fn      bool plib33c_i2c_RingBufferGet(uint16_t i2cInstance, PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, uint8_t* bytePointer) 
 * @brief   Acquire and manage the receive byte
 * @param   i2cInstance Instance of the I2C port of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param   ringbuf Pointer to ring buffer object of type PLIB33C_I2C_RECEIVE_RINGBUFFER_t
 * @param   bytePointer Pointer to the variable byte of type unsigned char
 * @return  0 = failure, receive ring buffer is not empty
 * @return  1 = success, receive ring buffer is empty
 *  
 * @details 
 *     This function gets the receive byte/s until the ring buffer is empty.
 *     When the ring buffer tail is not equal ring buffer head, the byte is transferred 
 *     to the variable byte pointer. 
 * 
 *********************************************************************************/
bool plib33c_i2c_RingBufferGet(uint16_t i2cInstance,
                    PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, 
                    uint8_t* bytePointer)
{
    uint16_t tail = ringbuf->tail;
    
    if (ringbuf->head != tail) 
    { //ring buffer is not empty
        *bytePointer = ringbuf->buf[tail];
        ++tail;
        
        if (tail == ringbuf->end) { tail = 0; }
        
        ringbuf->tail = tail; // update the tail to a *valid* index 
        
        plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferEmpty =  false;
        plib33cI2cDataStatus[i2cInstance-1].bits.rxByteReceived = true;
        return((bool)!plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferEmpty);
    }
    else 
    {
        plib33cI2cDataStatus[i2cInstance-1].bits.rxByteReceived = false;
        plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferEmpty =  true;
        plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferLocked = false;
        return(0);
    }
}

/*********************************************************************************
 * @ingroup p33c-plib-i2c-methods-private
 * @fn      uint16_t plib33c_i2c_RingBufferSet(uint16_t i2cInstance, PLIB33C_I2C_RECEIVE_RINGBUFFER_t * const ringbuf, uint8_t data) 
 * @brief   transfer and set the receive byte to the user defined buffer 
 * @param   i2cInstance Instance of the I2C port of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.) 
 * @param   ringbuf Pointer to ring buffer object of type PLIB33C_I2C_RECEIVE_RINGBUFFER_t
 * @param   data Received I2C byte of type unsigned char
 * @return  0 = failure, Transfer the data to the ring buffer was not successful
 * @return  1 = success, Transfer the data to the ring buffer was successful
 *  
 * @details 
 *     This function saves the byte receive when the I2C interrupts.
 * 
 * @note    
 *     In the event that the ring buffer overflows, the overflow bit is set,
 *     as well as the rxBufferLocked bit is set. In this manner, the user needs to
 *     process all the data on the ring buffer before the ring buffer can be 
 *     used again (rxBufferLocked = false).
 *********************************************************************************/
uint16_t plib33c_i2c_RingBufferSet(uint16_t i2cInstance,
                    PLIB33C_I2C_RECEIVE_RINGBUFFER_t* const ringbuf, 
                    uint8_t data)
{
    int retval = 1;
    uint16_t head = 0;
 
    if(plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferLocked == false)
    {
        head = ringbuf->head + 1;

        if (head == ringbuf->end) { head = 0; }

        if (head != ringbuf->tail) 
        { // buffer NOT full?
            ringbuf->buf[ringbuf->head] = data; // copy the byte into the buffer 
            ringbuf->head = head; // update the head to a *valid* index 
            plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferOverrun = false;
        }
        else { plib33cI2cDataStatus[i2cInstance-1].bits.rxBufferOverrun = true; }
    }
    
   return(retval);  
}