/*
    (c) 2023 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types
#include <stdlib.h> // include standard libraries data type

#include "xc_pral.h"
#include "p33c_plib_i2c_handler.h"
#include "p33c_plib_i2c_datamngmnt.h"
#include "../II2cHandler.h"

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Enable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enables the specified I2C module
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, enabling I2C module was not successful
 * @return  1 = success, enabling I2C module was successful
 * 
 * @details
 *     This function enables the I2C module.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Enable(uint16_t i2cInstance)
{
    int retval = 1;
    
    // Set I2C module enable bit
    retval &= p33c_i2cInstance_Enable(i2cInstance);

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Disable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Disables the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, disabling I2C module was not successful
 * @return  1 = success, disabling I2C module was successful
 * 
 * @details
 *     This function disables the I2C module. All communication will stop immediately.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Disable(uint16_t i2cInstance)
{
    int retval = 1;
    
    // Set pointer to memory address of desired I2C instance
    P33C_I2C_INSTANCE_t* i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    while(!i2c->I2CxSTAT.bits.P);  // Wait until stop bit has been detected last
    
    // Clear I2C module enable bit
    retval &= (bool)(!p33c_i2cInstance_Disable(i2cInstance));

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_Start(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Initiates start condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, initiating start condition was not successful
 * @return  1 = success, initiating start condition was successful
 * 
 * @details
 *     This function initiates start condition on the I2C module. Can be used by
 *     the I2C host mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_Start(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Initiates the start condition on SDAx and SCLx pins
    i2c->I2CxCONL.bits.SEN = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.SEN == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_Stop(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Initiates stop condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, initiating stop condition was not successful
 * @return  1 = success, initiating stop condition was successful
 * 
 * @details
 *     This function initiates stop condition on the I2C module. Can be used by
 *     the I2C host mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_Stop(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Initiates the stop condition on SDAx and SCLx pins
    i2c->I2CxCONL.bits.PEN = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.PEN == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_Restart(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Initiates restart condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, initiating restart condition was not successful
 * @return  1 = success, initiating restart condition was successful
 * 
 * @details
 *     This function initiates restart condition on the I2C module. Can be used by
 *     the I2C host mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_Restart(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Initiates the restart condition on SDAx and SCLx pins
    i2c->I2CxCONL.bits.RSEN = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.RSEN == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_ReceiveEnable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enables host receive mode on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, enabling host receive mode was not successful
 * @return  1 = success, enabling host receive mode was successful
 * 
 * @details
 *     This function enables receive mode on the I2C module. Can be used by
 *     the I2C host mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_ReceiveEnable(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Enables receive mode for host I2C
    i2c->I2CxCONL.bits.RCEN = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.RCEN == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_SetDataAcknowledgment(uint16_t i2cInstance, uint16_t ackState)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Sets the data acknowledgment on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param   ackState Status of data if ACK or NACK
 * @return  0 = failure, setting of the data acknowledgment was not successful
 * @return  1 = success, setting of the data acknowledgment was successful
 * 
 * @details
 *     This function transmits data acknowledgment either ACK or NACK on the I2C module.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_SetDataAcknowledgment(uint16_t i2cInstance, uint16_t ackState)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Set acknowledge data bit to ACK
    i2c->I2CxCONL.bits.ACKDT = ackState;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.ACKDT == ackState);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_EnableAckSequence(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Initiates the data acknowledge sequence on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, Initiating the data acknowledge sequence was not successful
 * @return  1 = success, Initiating the data acknowledge sequence was successful
 * 
 * @details
 *     This function initiates data acknowledge sequence on SDAx and SCLx pins of
 *     specified I2C module and transmits ACKDT data bit.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_EnableAckSequence(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Initiates the acknowledge sequence and transmit ACKDT data bit
    i2c->I2CxCONL.bits.ACKEN = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.ACKEN == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Host_OpenPort(uint16_t i2cInstance, PLIB33C_I2C_BAUDRATE_t baud, 
                        uint16_t address, PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enable I2C host communication on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param   baud 
 * @param   address 
 * @param   smbusThreshold 
 * @return  0 = failure, Enabling I2C host communication was not successful
 * @return  1 = success, Enabling I2C host communication was successful
 * 
 * @details
 *     This function enables host communication on the specified I2C module.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Host_OpenPort(uint16_t i2cInstance, PLIB33C_I2C_BAUDRATE_t baud, 
                        uint16_t address, PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;

    // Check if I2C instance is within valid range
    if (!(p33c_i2cInstance_Exists(i2cInstance)))
        return(0);
        
    // Update oscillator setting
    retval &= p33c_UpdateSystemFrequencies();

    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);

    // Check system oscillator settings
    if (p33c_SystemFrequencies.Fosc == 0)
        return(0);

    // Selection of address length for I2CxADD
    if (address > 0x7F)
        i2c->I2CxCONL.bits.A10M = true;
    else
        i2c->I2CxCONL.bits.A10M = false;
    
    // Set address
    i2c->I2CxADD.value = address;
    
    // Set SMBus dependency for the input threshold
    i2c->I2CxCONL.bits.SMEN = smbusThreshold;
    
    // Calculate baud rate generator
    retval &= plib33c_i2c_SetBaudrate(i2cInstance, baud);
    
    // Check baud for setting of slew rate control disable bit
    if (baud == I2C_BAUDRATE_400K)
        i2c->I2CxCONL.bits.DISSLW = 0;  // For high-speed mode
    else
        i2c->I2CxCONL.bits.DISSLW = 1;  // For standard speed
    
    // Clear I2C status bits
    i2c->I2CxSTAT.value = 0;
    // Enable I2C module instance
    retval &= p33c_i2cInstance_Enable(i2cInstance);

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_OpenPort(uint16_t i2cInstance, uint16_t address,
                        PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enable I2C client communication on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param   address 
 * @param   smbusThreshold 
 * @return  0 = failure, Enabling I2C client communication was not successful
 * @return  1 = success, Enabling I2C client communication was successful
 * 
 * @details
 *     This function enables client communication on the specified I2C module.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Client_OpenPort(uint16_t i2cInstance, uint16_t address,
                                     PLIB33C_I2C_SMBUS_IO_LEVEL_t smbusThreshold,
                                     uint16_t addrHoldFlg, uint16_t dataHoldFlg, 
                                     uint16_t clkStretchFlg, uint16_t collisionDtctFlg)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;

    // Check if I2C instance is within valid range
    if (!(p33c_i2cInstance_Exists(i2cInstance)))
        return(0);

    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);

    // Selection of address length for I2CxADD
    if (address > 0x7F)
        i2c->I2CxCONL.bits.A10M = true;
    else
        i2c->I2CxCONL.bits.A10M = false;
    
    // Set client address
    i2c->I2CxADD.value = address;
    
    // Set control bit flags
    i2c->I2CxCONH.bits.AHEN = addrHoldFlg;
    i2c->I2CxCONH.bits.DHEN = dataHoldFlg;
    i2c->I2CxCONL.bits.STREN = clkStretchFlg;
    i2c->I2CxCONH.bits.SBCDE = collisionDtctFlg;
    
    // Set SMBus dependency for the input threshold
    i2c->I2CxCONL.bits.SMEN = smbusThreshold;
    
    // Enable interrupt on detection of start/restart condition
    retval &= plib33c_i2c_Client_StartCndInt_Enable(i2cInstance);
    // Enable interrupt on detection of stop condition
    retval &= plib33c_i2c_Client_StopCndInt_Enable(i2cInstance);
    
    // Clear I2C status bits
    i2c->I2CxSTAT.value = 0;
    // Enable I2C module instance
    retval &= p33c_i2cInstance_Enable(i2cInstance);

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_SetBaudrate(uint16_t i2cInstance, uint32_t baud)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Sets the Baud Rate of the specified I2C instance
 * @param   i2cInstance Index of I2C peripheral instance of type unsigned integer
 * @param   baud Baud rate of type unsigned integer of the I2C peripheral instance Baud Rate generator
 * @return  0 = failure, setting I2C Baud Rate was not successful
 * @return  1 = success, setting I2C Baud Rate was successful
 * 
 * @details
 *     This function writes the value specified by 'baud' to the Baud Rate 
 *     register of the specified I2C peripheral instance.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_SetBaudrate(uint16_t i2cInstance, uint32_t baud)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    uint16_t brg = 0;

    // Check if I2C instance is within valid range
    if (!(p33c_i2cInstance_Exists(i2cInstance)))
        return(0);

    // Capture I2C instance SFR block
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    if (i2c != NULL) 
    {
        // Update oscillator setting
        retval &= p33c_UpdateSystemFrequencies();
        
        // Calculate for the baud rate generator value, delay = 130ns
        brg = (uint16_t)(((float)(1/((float)baud * 2.0)) - 0.000000130) * (float)p33c_SystemFrequencies.Fp) - (uint16_t)2;

        // I2CxBRG value cannot be from 0 to 3
        if ((brg >= 4) && (brg <= 65535)) 
        {
            // Disable i2c module instance
            p33c_i2cInstance_Disable(i2cInstance);
            // Set I2C baud rate
            i2c->I2CxBRG.value = brg;
        }
        else { retval = 0; } // return error
    }
    else { retval = 0; } // return error

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_StartCndInt_Enable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enables interrupt on detection of start/restart condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, enabling start/restart condition interrupt was not successful
 * @return  1 = success, enabling start/restart condition interrupt was successful
 * 
 * @details
 *     This function enables interrupt on detection of start/restart condition on the I2C module.
 *     Can be used by the I2C client mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Client_StartCndInt_Enable(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Enables the start/restart condition interrupt
    i2c->I2CxCONH.bits.SCIE = true;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONH.bits.SCIE == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_StartCndInt_Disable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Disables interrupt on detection of start/restart condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, disabling start/restart condition interrupt was not successful
 * @return  1 = success, disabling start/restart condition interrupt was successful
 * 
 * @details
 *     This function disables interrupt on detection of start/restart condition on the I2C module.
 *     Can be used by the I2C client mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Client_StartCndInt_Disable(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Disables the start/restart condition interrupt
    i2c->I2CxCONH.bits.SCIE = false;

    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONH.bits.SCIE == false);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_StopCndInt_Enable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Enables interrupt on detection of stop condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, enabling stop condition interrupt was not successful
 * @return  1 = success, enabling stop condition interrupt was successful
 * 
 * @details
 *     This function enables interrupt on detection of stop condition on the I2C module.
 *     Can be used by the I2C client mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Client_StopCndInt_Enable(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Enables the stop condition interrupt
    i2c->I2CxCONH.bits.PCIE = true;
    
    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONH.bits.PCIE == true);

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_StopCndInt_Disable(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Disables interrupt on detection of stop condition on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, disabling stop condition interrupt was not successful
 * @return  1 = success, disabling stop condition interrupt was successful
 * 
 * @details
 *     This function disables interrupt on detection of stop condition on the I2C module.
 *     Can be used by the I2C client mode only.
 * 
 *********************************************************************************/
uint16_t plib33c_i2c_Client_StopCndInt_Disable(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Disables the stop condition interrupt
    i2c->I2CxCONH.bits.PCIE = false;
    
    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONH.bits.PCIE == false);

    return(retval);
}


/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_ReleaseClock(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Releases the SCLx clock on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, releasing the SCLx clock was not successful
 * @return  1 = success, releasing the SCLx clock was successful
 * 
 * @details
 *     This function releases the SCLx clock on the I2C module. 
 *     Can be used by the I2C client mode only.
 *     
 *********************************************************************************/
uint16_t plib33c_i2c_Client_ReleaseClock(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Releases the SCLx clock
    i2c->I2CxCONL.bits.SCLREL = true;
    
    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.SCLREL == true);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_ClockStretchCtrl(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Holds the SCLx clock low (clock stretch) on the specified I2C module 
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return  0 = failure, holding the SCLx clock low was not successful
 * @return  1 = success, holding the SCLx clock low was successful
 * 
 * @details
 *     This function holds the SCLx clock low or clock stretching on the I2C module. 
 *     Can be used by the I2C client mode only.
 *     
 *********************************************************************************/
uint16_t plib33c_i2c_Client_ClockStretchCtrl(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    
    // Set pointer to memory address of desired I2C instance
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Holds the SCLx clock low (clock stretch)
    i2c->I2CxCONL.bits.SCLREL = false;
    
    // Verifying configuration process
    retval &= (bool)(i2c->I2CxCONL.bits.SCLREL == false);
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Client_GetAddress(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Gets the I2C client address of selected I2C instance
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return	0 = failure, Getting the client address was not successful
 * @return  1 = success, Getting the client address was successful
 * 
 * @details
 *     This function gets the client address of the selected I2C instance.
 *     
 *********************************************************************************/
uint16_t plib33c_i2c_Client_GetAddress(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Null-pointer protection
    if (i2c != NULL) {
        retval = i2c->I2CxADD.value;
    }
    else { retval = 0; }

    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_WriteByte(uint16_t i2cInstance, uint8_t buffer)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Sends a single byte via selected I2C instance
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @param 	buffer Data transmit buffer of type unsigned char
 * @return	0 = failure, Sending the data was not successful
 * @return  1 = success, Sending the data was successful
 * 
 * @details
 *     This function sends a single byte via the selected I2C instance.
 *     
 *********************************************************************************/
uint16_t plib33c_i2c_WriteByte(uint16_t i2cInstance, uint8_t buffer)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Null-pointer protection
    if (i2c != NULL) { 
        if (i2c->I2CxSTAT.bits.IWCOL) {    
            retval = 0;
        }
        else {
            while(i2c->I2CxSTAT.bits.TBF);
            i2c->I2CxTRN.value = (uint16_t)buffer;
        }
    }
    else { retval = 0; }

    return(retval);
}

/*********************************************************************************
 * @fn      uint8_t plib33c_i2c_ReadByte(uint16_t i2cInstance)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Reads a single byte via selected I2C instance
 * @param   i2cInstance Instance of the I2C module of type unsigned integer (e.g. 1=I2C1, 2=I2C2, etc.)
 * @return	Single data byte read from the specified I2C interface
 * 
 * @details
 *     This function reads a single byte from the selected I2C instance.
 *     
 *********************************************************************************/
uint8_t plib33c_i2c_ReadByte(uint16_t i2cInstance)
{
    int retval = 1;
    P33C_I2C_INSTANCE_t* i2c;
    i2c = p33c_i2cInstance_GetHandle(i2cInstance);
    
    // Null-pointer protection
    if (i2c != NULL) {
        // Receive data buffer via I2CxRCV register of selected I2C instance
        retval = (uint8_t)(i2c->I2CxRCV.bits.I2CRXDATA);
    }
    else { retval = 0; }
    
    return(retval);
}

// ___________________
// end of file