/*
    (c) 2023 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
*/

#include "II2cHandler.h"

extern PLIB33C_I2C_DATA_STATUS_t plib33cI2cDataStatus[];

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Initialize(PLIB33C_I2C_OBJECT_t* i2cConfig, PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer, PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer)
 * @ingroup p33c-plib-i2c-methods
 * @brief   Initializes a given I2C port by resetting all its registers to default 
 *          as well as the data buffers that user configure
 * @param   i2cConfig I2C property configuration of type PLIB33C_I2C_OBJECT_t
 * @param   interfaceSendBuffer Pointer to I2C send buffer object of type PLIB33C_I2C_SEND_BUFFER_t
 * @param   interfaceReceiveBuffer Pointer to I2C receive buffer object of type PLIB33C_I2C_RECEIVE_BUFFER_t
 * @param   pinSDA pointer to GPIO I2C SDA pin of type P33C_GPIO_t
 * @param   pinSCL pointer to GPIO I2C SCL pin of type P33C_GPIO_t
 * @return  0 = failure, Initializing I2C module was not successful
 * @return  1 = success, Initializing I2C module was successful
 * 
 * @details
 *     This function initializes the specified I2C module. The data buffers and 
 *     handlers were also initialized, as well as the pins for the SCLx and SDAx.
 * 
 **********************************************************************************/
uint16_t plib33c_i2c_Initialize(PLIB33C_I2C_OBJECT_t* i2cConfig,
                                PLIB33C_I2C_SEND_BUFFER_t* interfaceSendBuffer,
                                PLIB33C_I2C_RECEIVE_BUFFER_t* interfaceReceiveBuffer,
                                const P33C_GPIO_t* pinSDA, const P33C_GPIO_t* pinSCL)
{
    int retval = 1;
    
    i2cConfig->CommPort.SDAx.Io = pinSDA;
    i2cConfig->CommPort.SCLx.Io = pinSCL;
    
    // Power on the module
    retval &= p33c_i2cInstance_PowerOn(i2cConfig->CommPort.i2cInstance);

    // Reset all SFRs to default
    retval &= p33c_i2cInstance_SetConfig(i2cConfig->CommPort.i2cInstance, i2cConfigClear);
    
    // Initialization of data status and buffer
    retval &=  plib33c_i2c_DataStatusInitialize(i2cConfig->CommPort.i2cInstance);
    retval &=  plib33c_i2c_DataBufferInitialize(i2cConfig->CommPort.i2cInstance,
                                interfaceSendBuffer, interfaceReceiveBuffer);
    
    plib33cI2cDataStatus[i2cConfig->CommPort.i2cInstance - 1].bits.ready = true;
    
    return (retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Start(PLIB33C_I2C_OBJECT_t* i2cConfig)
 * @ingroup p33c-plib-i2c-methods
 * @brief   Opens the I2C communication
 * @param   i2cConfig I2C property configuration of type PLIB33C_I2C_OBJECT_t
 * @return  0 = failure, Opening communication was not successful
 * @return  1 = success, Opening communication was successful
 
 * @details 
 *     Opens the I2C communications with the respective settings 
 *     specified by the user.
 * 
 **********************************************************************************/
uint16_t plib33c_i2c_Start(PLIB33C_I2C_OBJECT_t* i2cConfig)
{
    int retval = 1;

    if (!(plib33cI2cDataStatus[i2cConfig->CommPort.i2cInstance - 1].bits.enable))
    {
        if (i2cConfig->Control.ModeSelect == I2C_HOST_MODE) 
        {
            retval &= plib33c_i2c_Host_OpenPort(i2cConfig->CommPort.i2cInstance, 
                                                i2cConfig->Control.BaudRate, 
                                                i2cConfig->Address,
                                                i2cConfig->Control.SMBusThreshold);
        }
        else 
        {
            retval &= plib33c_i2c_Client_OpenPort(i2cConfig->CommPort.i2cInstance,
                                                  i2cConfig->Address, 
                                                  i2cConfig->Control.SMBusThreshold,
                                                  i2cConfig->Control.AddressHold,
                                                  i2cConfig->Control.DataHold,
                                                  i2cConfig->Control.ClockStretch,
                                                  i2cConfig->Control.CollisionDetect);
        }
        
        retval &= plib33c_i2c_InterruptEnable(&i2cConfig->CommPort, 
                                              i2cConfig->Control.ModeSelect);
    
        plib33cI2cDataStatus[i2cConfig->CommPort.i2cInstance - 1].bits.enable = true;
        
        i2cConfig->CommPort.status.ready = 1;
        i2cConfig->CommPort.status.busy = 0;
    }
    else { retval = 0; }
    
    return(retval);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_SendData(PLIB33C_I2C_OBJECT_t* i2cConfig, uint8_t buffer)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Sends the data via I2C
 * @param   i2cConfig I2C property configuration of type PLIB33C_I2C_OBJECT_t
 * @param 	buffer Data transmit buffer of type unsigned char
 * @return	0 = failure, Sending the data was not successful
 * @return  1 = success, Sending the data was successful
 * 
 * @details
 *     This function send the data to the selected I2C instance.
 *     
 *********************************************************************************/
uint16_t plib33c_i2c_SendData(PLIB33C_I2C_OBJECT_t* i2cConfig, uint8_t buffer)
{
    int retval = 1;
    uint16_t index = i2cConfig->CommPort.i2cInstance - 1;

    plib33cI2cDataStatus[index].bits.txDataSent = false;
    
    retval &= plib33c_i2c_WriteByte(i2cConfig->CommPort.i2cInstance, buffer);
    
    plib33cI2cDataStatus[index].bits.txDataSent = true;

    return(retval);
}

/*********************************************************************************
 * @fn      uint8_t plib33c_i2c_ReceiveData(PLIB33C_I2C_OBJECT_t* i2cConfig)
 * @ingroup p33c-plib-i2c-methods-private
 * @brief   Processes the received data from I2C
 * @param   i2cConfig I2C property configuration of type PLIB33C_I2C_OBJECT_t
 * @return	rxChar Returns the processed receive byte
 * 
 * @details
 *     This function returns the received character or byte. It calls the 
 *     plib33c_i2c_RingBufferGet function to keep the ring buffer updated of the 
 *     position of the byte being processed.
 *     
 *********************************************************************************/
uint8_t plib33c_i2c_ReceiveData(PLIB33C_I2C_OBJECT_t* i2cConfig)
{
    uint8_t rxChar = 0;
    uint16_t index = i2cConfig->CommPort.i2cInstance - 1;
    
    if (plib33c_i2c_RingBufferGet(index, &plib33cI2cReceiveRingBuffer[index], &rxChar))
    {
        return(rxChar);
    }
    
    return(rxChar);
}

/*********************************************************************************
 * @fn      uint16_t plib33c_i2c_Close(PLIB33C_I2C_OBJECT_t* i2cConfig)
 * @ingroup p33c-plib-i2c-methods
 * @brief   Disables the i2c communication
 * @param   i2cConfig I2C property configuration of type PLIB33C_I2C_OBJECT_t
 * @return  0 = failure, Disabling I2C communication was not successful
 * @return  1 = success, Disabling I2C communication was successful
 * 
 * @details 
 *     Closes the I2C communications with the respective settings 
 *     specified by the user.
 * 
 **********************************************************************************/
uint16_t plib33c_i2c_Close(PLIB33C_I2C_OBJECT_t* i2cConfig)
{
    int retval = 1;
    uint16_t i2cInstance = i2cConfig->CommPort.i2cInstance;
    
    if ((!(p33c_i2cInstance_Exists(i2cInstance))) || 
        (i2cConfig->CommPort.status.busy)) {
        return(0);
    }
  
    retval &= plib33c_i2c_Disable(i2cInstance);
    
    i2cConfig->CommPort.status.ready = 0;
    
    plib33cI2cDataStatus[i2cInstance - 1].bits.ready = false;
    plib33cI2cDataStatus[i2cInstance - 1].bits.enable = false;

    return (retval);
}

// ___________________
// end of file