/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    epc9163_r10_board_ext.h
 * @author  M91406
 * @brief   EPC9163 48V-to-12V Rail Converter Extension Card Hardware Descriptor header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef EPC9163_R10_EXTENSION_PINMAP_H
#define	EPC9163_R10_EXTENSION_PINMAP_H
#ifdef __EPC9163_R10__

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types
#include <math.h> // include standard math functions library

#include "config/hal.h" // include hardware abstraction layer declarations

/** 
 * @ingroup pwm-mcal-phase3
 * @{ 
 * @brief   PWM peripheral output pins, control signals and register assignments of converter phase #3
 * @note    Please review the device data sheet for details about register names and settings.
 */

// PWM Phase #3 Configuration (extension card phase#1)
#if (NO_OF_PHASES >= 3)

#define PWM3H               CEC02                       ///< Card Edge Connector pin of Device Port Pin of PWM C High-Output
#define PWM3L               CEC04                       ///< Card Edge Connector pin of Device Port Pin of PWM C Low-Output
#define PWM3_OUTPUT_SWAP    false                       ///< PWM H/L outputs of channel are used "as is" (no swap)

#define PWM3_Interrupt      _PWM3Interrupt              ///< Interrupt Service Routine Label of PG3
#define PWM3_IP             _PWM3IP                     ///< Interrupt vector priority register
#define PWM3_IF             _PWM3IF                     ///< Interrupt vector flag bit register bit
#define PWM3_IE             _PWM3IE                     ///< Interrupt vector enable bit register bit

#endif


/** @} */ // end of group pwm-mcal-phase3 ~~~~~~~~~~~~~~~~~

/** 
 * @ingroup pwm-mcal-phase4
 * @{ 
 * @brief   PWM peripheral output pins, control signals and register assignments of converter phase #4
 * @note    Please review the device data sheet for details about register names and settings.
 */

// PWM Phase #4 Configuration (extension card phase#2)
#if (NO_OF_PHASES >= 4)

#define PWM4H                   CEC06                   ///< Card Edge Connector pin of Device Port Pin of PWM D High-Output
#define PWM4L                   CEC08                   ///< Card Edge Connector pin of Device Port Pin of PWM D Low-Output
#define PWM4_OUTPUT_SWAP        false                   ///< PWM H/L outputs of channel are used "as is" (no swap)

#define PWM4_Interrupt          _PWM2Interrupt          ///< Interrupt Service Routine Label of PG2
#define PWM4_IP                 _PWM2IP                 ///< Interrupt vector priority register
#define PWM4_IF                 _PWM2IF                 ///< Interrupt vector flag bit register bit
#define PWM4_IE                 _PWM2IE                 ///< Interrupt vector enable bit register bit

#endif

/** @} */ // end of group pwm-mcal-phase4 ~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup phase-current-feedback-mcal
 * @{ 
 * @brief ADC input assignments of phase current feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignments of feedback signals in case of hardware changes.
 **************************************************************************************************/

#if (NO_OF_PHASES >= 3)

#define ISNS3                   CEC28                   ///< Card Edge Connector pin this signal is connected to

#define ISNS3_ADCANxInterrupt   CEC28_ADCANxInterrupt   ///< ADC interrupt service routine function call of the low voltage feedback channel
#define ISNS3_ADCANxIF          CEC28_ADCANxIF          ///< ADC interrupt flag bit of the low voltage feedback channel

#define ISNS3_ADCTRIG           PWM3H.Objects.PG->PGxTRIGA ///< Register used for trigger placement
#define ISNS3_TRGSRC            ADCAN_TRGSRC_PWM3_TRG1  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define ISNS3_ADCCORE           CEC28.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define ISNS3_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define ISNS3_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define ISNS3_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define ISNS3_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

#endif

#if (NO_OF_PHASES >= 4)

#define ISNS4                   CEC30                   ///< Card Edge Connector pin this signal is connected to

#define ISNS4_ADCANxInterrupt   CEC30_ADCANxInterrupt   ///< ADC interrupt service routine function call of the low voltage feedback channel
#define ISNS4_ADCANxIF          CEC30_ADCANxIF          ///< ADC interrupt flag bit of the low voltage feedback channel

#define ISNS4_ADCTRIG           PWM4H.Objects.PG->PGxTRIGA ///< Register used for trigger placement
#define ISNS4_TRGSRC            ADCAN_TRGSRC_PWM4_TRG1  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define ISNS4_ADCCORE           CEC30.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define ISNS4_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define ISNS4_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define ISNS4_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define ISNS4_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

#endif

/** @} */ // end of group phase-current-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~


/**************************************************************************************************
 * @ingroup control-interrupt-vector-declarations
 * @{
 * @brief Control loop Interrupt Vector Settings
 * 
 * @details
 *   Control loops are called in dedicated interrupt service routines of PWM events, ADC events
 *   or triggered by timers. This section allows users to set and modify the interrupt service 
 *   routine triggers and their priority to set up and optimize the control system interrupt 
 *   structure.
 * 
 * @note
 *   All available control loop interrupt trigger options are listed in the respective
 *   hardware abstraction layer flags header file
 * 
 * @see
 *   epc9163_r430_flags.h
 * 
 * *************************************************************************************************/

#if (NO_OF_PHASES >= 3)

  #define ISNS3_TRIGGER_MODE    TRIGGER_MODE_NONE       ///< Currently selected current loop #3 interrupt vector
  #define ISNS3_ISR_PRIORITY    6U                      ///< Current loop #3 interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

  #if (ISNS3_TRIGGER_MODE == TRIGGER_MODE_ADC)    
    #define ILOOP3_Interrupt    ISNS3_ADCANxInterrupt   ///< Interrupt vector function call label
    #define ILOOP3_ISR_IP       ISNS3_ADCANxIP          ///< Interrupt vector priority register bits
    #define ILOOP3_ISR_IF       ISNS3_ADCANxIF          ///< Interrupt vector flag bit register bit
    #define ILOOP3_ISR_IE       ISNS3_ADCANxIE          ///< Interrupt vector enable bit register bit
  #elif (ISNS3_TRIGGER_MODE == TRIGGER_MODE_PWM)
    #define ILOOP3_Interrupt    PWM3_Interrupt          ///< Interrupt vector function call label
    #define ILOOP3_ISR_IP       PWM3_IP                 ///< Interrupt vector priority register
    #define ILOOP3_ISR_IF       PWM3_IF                 ///< Interrupt vector flag bit register bit
    #define ILOOP3_ISR_IE       PWM3_IE                 ///< Interrupt vector enable bit register bit
  #endif

#endif

#if (NO_OF_PHASES >= 4)

  #define ISNS4_TRIGGER_MODE    TRIGGER_MODE_NONE       ///< Currently selected current loop #4 interrupt vector
  #define ISNS4_ISR_PRIORITY    6U                      ///< Current loop #4 interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

  #if (ISNS4_TRIGGER_MODE == TRIGGER_MODE_ADC)    
    #define ILOOP4_Interrupt    ISNS4_ADCANxInterrupt   ///< Interrupt vector function call label
    #define ILOOP4_ISR_IP       ISNS4_ADCANxIP          ///< Interrupt vector priority register bits
    #define ILOOP4_ISR_IF       ISNS4_ADCANxIF          ///< Interrupt vector flag bit register bit
    #define ILOOP4_ISR_IE       ISNS4_ADCANxIE          ///< Interrupt vector enable bit register bit
  #elif (ISNS4_TRIGGER_MODE == TRIGGER_MODE_PWM)
    #define ILOOP4_Interrupt    PWM4_Interrupt          ///< Interrupt vector function call label
    #define ILOOP4_ISR_IP       PWM4_IP                 ///< Interrupt vector priority register
    #define ILOOP4_ISR_IF       PWM4_IF                 ///< Interrupt vector flag bit register bit
    #define ILOOP4_ISR_IE       PWM4_IE                 ///< Interrupt vector enable bit register bit
  #endif

#endif

/** @} */ // end of group control-interrupt-vector-declarations

/** 
 * @ingroup temperature-feedback-mcal
 * @{ 
 * @brief ADC input assignments of temperature feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignments of feedback signals in case of hardware changes.
 */

#if (NO_OF_PHASES >= 3)

// Peripheral Assignments
#define TEMP2                   CEC37                   ///< card edge connector pint this signal is connected to
#define TEMP2_ADCTRIG           PWM3H.Objects.PG->PGxTRIGB ///< Register used for trigger placement
#define TEMP2_TRGSRC            ADCAN_TRGSRC_PWM3_TRG2  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define TEMP2_ADCCORE           CEC37.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define TEMP2_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define TEMP2_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define TEMP2_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define TEMP2_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

#endif

/** @} */ // end of group temperature-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~

#endif  /* __EPC9163_R10__ */
#endif	/* EPC9163_R10_EXTENSION_PINMAP_H */

// ________________________
// end if file
