/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    epc9165_r10_config.h
 * @author  M91406
 * @brief   Physical quantity parameter declaration header file 
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef EPC9165_R10_HARDWARE_CONFIGURATION_H
#define	EPC9165_R10_HARDWARE_CONFIGURATION_H
#ifdef __EPC9165_R10__

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

#include "epc9165_r10_flags.h" // include list of generic configuration flags

///**************************************************************************************************
// * @ingroup hardware-id-macro
// * @{
// * @brief   Global macro identifying the hardware version supported by 
// * 
// * @details
// *  This global macro is used to identify the hardware version supported by this hardware 
// *  abstraction layer header file. 
// * 
// **************************************************************************************************/
//#ifndef __EPC9165_R10__
//  #define __EPC9165_R10__
//#endif

/** @} */ // end of group hardware-id-macro

/**************************************************************************************************
 * @ingroup special-options
 * @{
 * @brief   Global defines used to enable/disable special firmware options
 * 
 * @details
 *  This section is used to enable/disable special options of the firmware. 
 * 
 **************************************************************************************************/

/* CUSTOM RUNTIME OPTIONS */
#define PLANT_MEASUREMENT       false ///< If enabled, replaces the common voltage control loop by a simple P-control loop to perform measurements of the plant transfer function.
#define CHARGER_ENABLE          false ///< Enable charger profile engine to charge large capacitive loads on 12V port
#define DBGPIN1_ENABLE          false ///< Enables debug pin indicating control loop execution timing
#define DBGPIN2_ENABLE          false ///< Enables debug pin indicating task scheduler execution timing
#define DBGDAC_ENABLE           false ///< Enables debug DAC output (ot applicable with EPC9528)

#define PWREN_ENABLE            false ///< Enables a POWER ENABLE input pin
#define PWRGOOD_ENABLE          false ///< Enables a POWER GOOD output pin

/** @} */ // end of group special-options

/*********************************************************************************
 * @ingroup special-options
 * @def     DEFAULT_OPERATING_MODE
 * @def     DEFAULT_DIRCTRL_MODE
 * @brief   Determines if the converter is operating in down-stream (buck) mode or up-stream (boost) mode by default
 * 
 * @details
 *  At startup, the converter state machine is monitoring the system conditions 
 *  before allowing the power supply to run. This analysis requires a definition
 *  of the default operating mode of the converter.
 * 
 *  - DEFAULT_OPMODE_BUCK = 0
 *    
 *    In this setting the converter is starting up in step-down mode, 
 *    considering the high-voltage port as input source and the low-voltage
 *    port as output. The current feedback is considered to be direct 
 *    proportional (positive)
 *   
 *  - DEFAULT_OPMODE_BOOST = 1
 *    
 *    In this setting the converter is starting up in step-up mode, 
 *    considering the low-voltage port as input source and the high-voltage
 *    port as output. The current feedback is considered to be indirect
 *    proportional (negative)
 *   
 *  - DEFAULT_OPMODE_AUTO = 2
 *    
 *    In this setting the converter is monitoring the high- and low-voltage 
 *    port simultaneously. Which ever exceeds the under voltage lock-out limit
 *    will be considered as input and the power transfer direction will be set
 *    accordingly.
 * 
 * @note
 *   All available control loop interrupt trigger options are listed in the respective
 *   hardware abstraction layer flags header file
 * 
 * @see
 *   epc9165_r10_flags.h
 * 
 **********************************************************************************/
#define DEFAULT_OPERATING_MODE  DEFAULT_OPMODE_BOOST ///< 0 = Buck Converter Mode (default), 1 = Boost Converter Mode, 2 = Auto-Detect Mode (not supported yet)
#define DEFAULT_SWAP_MODE       DIRCTRL_HOT_SWAP    ///< 0 = Soft Swap of conversion mode, 1 = Hot Swap of conversion mode

/*********************************************************************************
 * @ingroup special-options
 * @def     AUTO_START
 * @brief   Determines if the converter is allowed to run as soon as input power has been detected
 * 
 * @details
 *  - When AUTO_START flag is set = true
 *    Forces the power supply state machine to start up the power converter
 *    as soon as input power has been detected and all other self-test have 
 *    been cleared.
 *
 *  - When AUTO_START flag is set = false
 *    Holds the power supply state machine in STANDBY after input power has 
 *    been detected and all other self-test have been cleared. The GO bit 
 *    in the Status word of the power converter data object must be set to 
 *    start the power converter. The GO bit is cleared automatically and must 
 *    be set at every converter start, including when the converter has been 
 *    shut of and reset due to a fault condition.
 * 
 ***********************************************************************************/
#define AUTO_START              true  ///< Allow converter to start up automatically

/*********************************************************************************
 * @ingroup special-options
 * @def     ISNS_OFFSET_CALIBRATION_ENABLE
 * @brief   Enables/Disables Current Sense Offset Calibration
 * 
 * @details
 *  Flag indicating if the signal offset of the current sense feedback needs to be calibrated.
 *  The calibration procedure is available as Special Function of the power controller and will be 
 *  called automatically when the option ISNS_OFFSET_CALIBRATION_ENABLE is set to 'true' 
 * 
 * @note
 *  Standby current sense feedback offset calibration is not supported by the MCP6C02 shunt amplifier
 *  device used in EPC9165 due to its limited common mode range, which starts above 2.5 V DC. However,
 *  the common reference voltage of both devices, auto-zero self tuning and high sensing precision
 *  makes calibration negligible.
 **********************************************************************************/
#define ISNS_OFFSET_CALIBRATION_ENABLE  true ///< Current Sense Offset Calibration is disabled (see notes)

/*********************************************************************************
 * @ingroup special-options
 * @def     SYNCHRONOUS_RECTIFIER_CONTROL
 * @def     AUTO_SYNCH_RECTIFIER_CONTROL
 * @brief   Determines if the converter is allowed to run as soon as input power has been detected
 * 
 * @details
 *  Feature is still experimental. DO NOT USE !
 ***********************************************************************************/
#define SYNCHRONOUS_RECTIFIER_CONTROL   false ///< Disable support for runtime rectifier control
#define AUTO_SYNCH_RECTIFIER_CONTROL    false ///< Disable support for automatic runtime rectifier control

/*********************************************************************************
 * @ingroup special-options
 * @def     CURRENT_LIMIT_MODE
 * @brief   Selects the over current response of the converter
 * 
 * @details
 *  This feature allows the adjustment of the over current protection response 
 *  of the power converter. By default, the output current will be limited by 
 *  the real-time control loop at the threshold declared by setting ISNS_MAXIMUM.
 *  If the load current further increases, the output voltage will drop accordingly.
 * 
 *  If CURRENT_LIMIT_MODE is set to OUTPUT_OCP_MODE, 
 *    the converter will shut down when the output voltage drop exceeds the regulation 
 *    deviation limit declared by setting V12_TOLERANCE_MAX resp. V48_TOLERANCE_MAX.
 *    This mode is recommended when the converter is operated as conventional constant 
 *    voltage source, where an over-current condition is indicating a system malfunction.
 * 
 *  If CURRENT_LIMIT_MODE is set to OUTPUT_SCL_MODE, 
 *    the converter will not shut down when the output voltage drop exceeds the regulation 
 *    deviation limit but remain in continuous current limit.
 *    This mode is recommended when the converter is operated with high capacitive loads
 *    such as batteries of super capacitors. In this case the voltage regulation error
 *    detection is turned of to allow using the converter as charger.
 * 
 ***********************************************************************************/
#define CURRENT_LIMIT_MODE      OUTPUT_SCL_MODE ///< Default output over current response (OCP vs. CL)

/**************************************************************************************************
 * @ingroup pwm-settings
 * @{
 * @brief User-declaration of global defines for PWM signal generator settings
 * 
 * @details
 *  This section defines fundamental PWM settings required for the interleaved buck converter
 *  of the EPC9165 48V-to-12V rail power module development board. These settings are determined 
 *  by hardware and defined using physical quantities. Pre-compiler macros are used to convert 
 *  physical values into binary (integer) numbers to be written to Special Function Registers (SFR).
 * 
 **************************************************************************************************/

#define NO_OF_PHASES            2U ///< Number of power converter phases of this design

#if (NO_OF_PHASES <= 2)
  #define PHASE_1               0U // Assigns switch node x and current sense feedback channel x to PHASE #1
  #define PHASE_2               1U // Assigns switch node x and current sense feedback channel x to PHASE #2
#else
  #define PHASE_1               0U // Assigns switch node x and current sense feedback channel x to PHASE #1
  #define PHASE_2               2U // Assigns switch node x and current sense feedback channel x to PHASE #2
  #define PHASE_3               1U // Assigns switch node x and current sense feedback channel x to PHASE #3
  #define PHASE_4               3U // Assigns switch node x and current sense feedback channel x to PHASE #4
#endif

#define SWITCHING_FREQUENCY     (float)440.0e+3 ///< Fixed Switching frequency in [Hz]
#define PWM_DUTY_CYCLE_MIN      (float)  0.020  ///< Minimum on/off-time ratio (duty ratio) in [%]
#define PWM_DUTY_CYCLE_MAX      (float)  0.900  ///< Maximum on/off-time ratio (duty ratio) in [%]
#define LEADING_EDGE_BLANKING   (float)120.0e-9 ///< Leading Edge Blanking period in [sec]
#define DEAD_TIME_LEADING_EDGE  (float) 10.0e-9 ///< Leading Edge Dead Time delay in [sec]
#define DEAD_TIME_FALLING_EDGE  (float) 10.0e-9 ///< Falling Edge Dead Time delay in [sec]
#define SAMPLING_DIVIDER        1U              ///< Sampling frequency divider (divides the switching frequency by [n+1])
#define SAMPLING_OFFSET         0U              ///< Sampling event offset (delays the first generation of trigger by n switching cycles)

/** @} */ // end of group pwm-settings ~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup input-voltage-feedback-settings
 * @{
 * @brief   Declaration of input voltage feedback properties
 * 
 * @details
 *  In this section the output voltage feedback signal scaling, gain, valid signal limits and nominal
 *  operating point is specified. Physical quantities are used to define values. Macros are used to 
 *  convert given physical values into binary (integer) number to be written into SFRs and variables.
 **************************************************************************************************/

// Feedback Declarations
#define V48_MINIMUM             (float)20.000   ///< Minimum input voltage in [V]
#define V48_NOMINAL             (float)48.000   ///< Nominal input voltage in [V]
#define V48_MAXIMUM             (float)60.000   ///< Maximum input voltage in [V]
#define V48_VFWD_DROP_MAX       (float) 2.500   ///< Forward voltage drop across high side switch in boost mode

#define V48_UVLO_VOLTAGE        (float)17.500   ///< Under Voltage Lock Out Cut Off Threshold in [V] (when V48 is voltage input)
#define V48_OVLO_VOLTAGE        (float)60.500   ///< Over Voltage Lock Out Cut Off Threshold in [V] (when V48 is voltage input)
#define V48_HYSTERESIS          (float) 2.500   ///< Voltage Extremes Thresholds Hysteresis in [V]

#define V48_OVP_VOLTAGE         (float)62.000   ///< Over Voltage Protection Cut Off Threshold in [V] (always active)

#define V48_TOLERANCE_MAX       (float) 3.000   ///< Output voltage regulation tolerance [V][+/-]
#define V48_TOLERANCE_MIN       (float) 1.500   ///< Output voltage regulation tolerance [V][+/-]

/** @} */ // end of group input-voltage-feedback-settings ~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup output-voltage-feedback-settings
 * @{
 * @brief Declaration of output voltage feedback properties
 * 
 * @details
 * In this section the output voltage feedback signal scaling, gain, valid signal limits and nominal
 * operating point is specified. Physical quantities are used to define values. Macros are used to 
 * convert given physical values into binary (integer) number to be written into SFRs and variables.
 * *************************************************************************************************/

// Feedback Declarations
#define V12_MINIMUM             (float) 7.000   ///< Absolute minimum output voltage
#define V12_NOMINAL             (float)14.300   ///< Nominal output voltage
#define V12_MAXIMUM             (float)16.000   ///< Absolute maximum output voltage (Over Voltage Protection)

#define V12_UNDER_VOLTAGE       (float)11.300   ///< Under Voltage Lock Out Cut Off in [V] (when V12 is voltage input in boost mode)
#define V12_OVER_VOLTAGE        (float)18.500   ///< Over Voltage Lock Out Cut Off in [V]  (when V12 is voltage input in boost mode)
#define V12_HYSTERESIS          (float) 0.500   ///< Over Voltage Protection Hysteresis

#define V12_OVP_VOLTAGE         (float)27.500   ///< Over Voltage Protection Cut Off Threshold in [V] (always active)
#define V12_UVP_VOLTAGE         (float) 4.000   ///< Under Voltage Protection Cut Off Threshold in [V] (used in battery charging mode only)

#define V12_TOLERANCE_MAX       (float) 1.000   ///< Output voltage regulation tolerance [V][+/-]
#define V12_TOLERANCE_MIN       (float) 0.600   ///< Output voltage regulation tolerance [V][+/-]

/** @} */ // end of group output-voltage-feedback-settings ~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup phase-current-feedback-settings
 * @{
 * @brief Declaration of phase-current feedback properties
 * 
 * @details
 * In this section the phase-current feedback signal scaling, gain, valid signal limits and nominal
 * operating point is specified. Physical quantities are used to define parameter values to ease
 * the system configuration. 
 * 
 * Macros are used to convert given physical values into binary (integer) number to be written
 * into SFRs and variables and being used in runtime calculations.  
 * (see \ref phase-current-feedback-macros for details)
 **************************************************************************************************/

// General Current Sense Feedback Declarations
#define ISNS_MINIMUM            (float)-70.000      ///< absolute minimum phase current
#define ISNS_NOMINAL            (float)  0.000      ///< nominal phase current
#define ISNS_MAXIMUM            (float) 70.000      ///< absolute maximum phase current
#define ISNS_LIMIT_START        (float) 25.000      ///< startup phase current limit
 
#define ISNS_OCP_TRIP_LEVEL     (float) 72.500      ///< absolute maximum phase current tripping the over current limit (immediate)
#define ISNS_OCP_RELEASE_LEVEL  (float) 50.000      ///< current reset level after over current event (immediate)
 
#define ISNS_COMMON_MODE_LIMIT  (float)  3.000      ///< phase current sense common mode voltage minimum in [V]
#define ISNS_COMMON_MODE_HYST   (float)  0.200      ///< phase current sense common mode voltage minimum in [V]
#define OPEN_LOOP_STARTUP       true                ///< flag indicating if minium common mode voltage range needs to be secured before regulation can be trusted
 
#define SYNCCTL_ON_THRESHOLD    (float) 10.000      ///< phase current above which the synchronous rectifier will be turned on
#define SYNCCTL_OFF_THRESHOLD   (float)  8.000      ///< phase current below which the synchronous rectifier will be turned off
#define AUTO_SYNCCTL            false               ///< flag enabling/disabling automatic synchronous rectifier control
 
#define ISNS_BALANCING_LIMIT    (float) 200.0e-9    ///< duty cycle offset limit for current balancing in [sec]
#define ISNS_BALANCING_HYST     (float)   4.0e-9    ///< current sense balancing hysteresis in [sec]

// Specific Current Sense Feedback Declarations
// Current Sense Phase #1
#if (NO_OF_PHASES >= 1)
#define ISNS1_ADC_TRG_DELAY     (float) 886.0e-9    ///< ADC trigger delay for current sense #1 in [sec]
#define ISNS1_FEEDBACK_OFFSET   (float) 1.559       ///< current sense #1 feedback offset (average), default = 1.650 V, real = 1.559 V
#endif

// Current Sense Phase #2
#if (NO_OF_PHASES >= 2)
#define ISNS2_ADC_TRG_DELAY     (float) 886.0e-9    ///< ADC trigger delay for current sense #2 in [sec]
#define ISNS2_FEEDBACK_OFFSET   (float) 1.559       ///< current sense #2 feedback offset (average), default = 1.650 V, real = 1.559 V
#endif

// Current Sense Phase #3
#if (NO_OF_PHASES >= 3)
#define ISNS3_ADC_TRG_DELAY     (float) 886.0e-9    ///< ADC trigger delay for current sense #3 in [sec]
#define ISNS3_FEEDBACK_OFFSET   (float) 1.559       ///< current sense #3 feedback offset (average), default = 1.650 V, real = 1.559 V
#endif

// Current Sense Phase #4
#if (NO_OF_PHASES >= 4)
#define ISNS4_ADC_TRG_DELAY     (float) 886.0e-9    ///< ADC trigger delay for current sense #4 in [sec]
#define ISNS4_FEEDBACK_OFFSET   (float) 1.559       ///< current sense #4 feedback offset (average), default = 1.650 V, real = 1.559 V
#endif

/** @} */ // end of group phase-current-feedback-settings ~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup temperature-feedback-settings
 * @{ 
 * @brief   Declaration of temperature feedback properties
 * 
 * @details
 * In this section the temperature feedback signal scaling, gain, valid signal limits and nominal
 * operating point is specified. Physical quantities are used to define parameter values to ease
 * the system configuration. 
 * 
 * Macros are used to convert given physical values into binary (integer) number to be written
 * into SFRs and variables and being used in runtime calculations.  
 * (see \ref temperature-feedback-macros for details)
 **************************************************************************************************/
 
// Temperature Feedback Declarations
//#define TEMP_FEEDBACK_GAIN      (float) 4.3194  ///< AD590 with 3.48k load gain in tick/K
#define TEMP_FEEDBACK_GAIN      (float) 3.480e-3  ///< AD590 with 3.48k load gain in V/K
#define TEMP_FEEDBACK_OFFSET    (float) 0.950   ///< Signal level at 0�C in [V] (AD590 with 3.48k load offset)

#define TEMP_MINIMUM            (float) -40.0   ///< minimum heat-sink temperature
#define TEMP_NOMINAL            (float)   0.0   ///< nominal heat-sink temperature
#define TEMP_MAXIMUM            (float) 125.0   ///< maximum heat-sink temperature

#define TEMP_OTP_ULIMIT         (float) 100.0   ///< maximum heat-sink temperature to trip over temperature fault (deg C)
#define TEMP_OTP_LLIMIT         (float)  80.0   ///< temperature reset level after over temperature event
#define TEMP_ADC_TRG_DELAY      (float) 0.000   ///< ADC trigger delay for temperature sensor in [sec]

/** @} */ // end of group temperature-feedback-settings ~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup adaptive-control-settings
 * @{
 * @brief Declaration of additional hardware-specific defines required for adaptive gain control
 * 
 * @details
 * In this section additional macros are defined to calculate constant parameters for the
 * adaptive gain modulation algorithm using user defined settings declared in their respective
 * sections. Any change of these parameters will also result in a change of the values of the 
 * gain modulation parameters of this section.
 **************************************************************************************************/

#define AGC_EXECUTION_DELAY     (float)(390.0e-9) ///< AGC Observer Algorithm Execution Time in [sec]

/** @} */ // end of group adaptive-control-settings ~~~~~~~~~~~~~~~~~~~~~~~


/**************************************************************************************************
 * @ingroup startup-timing-settings
 * @{
 * @brief Global defines for soft-start specific parameters
 * 
 * @details
 * This section is used to define power supply startup timing settings. The soft-start sequence 
 * is part of the power controller. It allows to program specific timings for 
 *   - Power On Delay
 *   - Ramp Period 
 *   - Power Good Delay
 * 
 * After the startup has passed these three timing periods, the power supply is ending up in 
 * "normal" operation, continuously regulating the output until a fault is detected or the 
 * operating state is changed for any other reason. When the output voltage reference is changed, 
 * the power control state machine will use the voltage ramp slope defined here to tune from the 
 * recent voltage reference to the new reference value. During this period the BUSY-bit of the 
 * power controller (status word, bit #7) will be set. This status bit will be cleared automatically
 * by the power controller state machine once the new reference value has been applied and the 
 * converter is back in constant regulation mode.
 * 
 * Pre-compiler macros are used to translate physical values into binary (integer) numbers to 
 * be written to SFRs and variables.  
 * (see \ref startup-timing-macros for details)
 * 
 * @note
 * On EPC9165 Rev 1.0 it takes roughly 50 ms until the auxiliary power has been started
 * and the controller completes self-test and peripheral configuration. After this period
 * the controller starts to execute the power control state machine.
 *  
 * This additional startup delay of ~50 ms is not considered here and needs to be taken into 
 * account when adjusting startup timing.
 **************************************************************************************************/

#define POWER_ON_DELAY          (float) 200e-3 ///< power on delay in [sec]
#define CHARGE_UP_DELAY         (float) 300e-3 ///< output capacitor charge-up delay in [sec]
#define CHARGE_UP_TIMEOUT       (float) 500e-3 ///< output capacitor charge-up delay timeout in [sec]
#define LVRAMP_PERIOD           (float)  50e-3 ///< voltage ramp-up period of low-voltage output in [sec] (step-down mode)
#define HVRAMP_PERIOD           (float) 200e-3 ///< voltage ramp-up period of high-voltage output in [sec] (step-up mode)
#define IRAMP_PERIOD            (float)  50e-3 ///< output current ramp-up period in [sec]
#define POWER_GOOD_DELAY        (float) 200e-3 ///< power good delay in [sec]

/** @} */ // end of group startup-timing-settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup fault-response-settings
 * @{
 * @brief Global defines for fault-monitor related parameters
 * 
 * @details
 * This section is used to define power supply fault object timing settings. The fault monitor 
 * is continuously monitoring declared data objects at the high-priority task frequency defined by 
 * \ref MAIN_EXECUTION_PERIOD. Based on this known interval, filtering delays for fault trip and 
 * fault recovery events to allow users to adjust the fault detection sensitivity.
 * 
 * - Fault Trip Event Delay   
 *   This setting defines for how long a fault condition has to be continuously 
 *   active before the effective fault trip status/event will be triggered.
 * 
 * - Fault Recovery Event Delay   
 *   This setting defines for how long a fault condition has to be continuously
 *   cleared before the effective fault recovery status/event will be triggered.
 * 
 * - Fault Restart Delay (FLTRSTDLY)
 *   In case a detected fault event tripped a power supply shutdown, and all 
 *   active fault conditions have been cleared, a global restart delay counter 
 *   is started, which has to expire before a restart attempt is initiated.
 * 
 * - Fault Restart Cycles
 *   Defines the number of restart attempts during an active fault recovery period.
 *   If no faults are triggered during the recovery period, the respective restart
 *   counter is reset. If new fault events are triggered, the restart counter is 
 *   incremented. If the restart counter reaches the given number of declared restart 
 *   cycles, the system will enter the Fault Latched mode and no further restart 
 *   attempts will be initiated.
 * 
 *************************************************************************************************/

#define UVLO_TRIP_DELAY         (float)    5e-3 ///< under voltage lock out trip delay in [sec]
#define UVLO_RECOVERY_DELAY     (float)  100e-3 ///< under voltage lock out recovery delay in [sec]
#define OVLO_TRIP_DELAY         (float)    1e-3 ///< over voltage lock out trip delay in [sec]
#define OVLO_RECOVERY_DELAY     (float)  100e-3 ///< over voltage lock out recovery delay in [sec]
#define REGERR_TRIP_DELAY       (float)   50e-3 ///< regulation error trip delay in [sec]
#define REGERR_RECOVERY_DELAY   (float)  100e-3 ///< regulation error recovery delay in [sec]
#define OCP_TRIP_DELAY          (float)    1e-3 ///< over current protection trip delay in [sec]
#define OCP_RECOVERY_DELAY      (float)  100e-3 ///< over current protection recovery delay in [sec]
#define OVP_TRIP_DELAY          (float)    1e-3 ///< over voltage protection trip delay in [sec]
#define OVP_RECOVERY_DELAY      (float)  100e-3 ///< over voltage protection recovery delay in [sec]
#define OTP_TRIP_DELAY          (float)  500e-3 ///< over temperature protection trip delay in [sec]
#define OTP_RECOVERY_DELAY      (float)  500e-3 ///< over temperature protection recovery delay in [sec]
#define UVP_TRIP_DELAY          (float)   50e-3 ///< under voltage protection trip delay in [sec]
#define UVP_RECOVERY_DELAY      (float)  100e-3 ///< under voltage protection recovery delay in [sec]

#define FAULT_RESTART_DELAY     (float) 5000e-3 ///< under voltage protection recovery delay in [sec]
#define FAULT_RESTART_CYCLES    4U  ///< Maximum number of restart cycles: Fault state will be latched until CPU reset or power cycle

/** @} */ // end of group fault-response-settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup battery-charger-settings
 * @{
 * @brief Global Battery Charger Parameters
 * 
 * @details
 * 
 * 
 *************************************************************************************************/

#if (CHARGER_ENABLE == true)

#define BATCRG_VLEVEL_TRICKLE       (float)  4.500 ///< voltage level ending trickle current mode in [V]
#define BATCRG_ILEVEL_TRICKLE       (float)  0.500 ///< reference current level during trickle current mode in [A]

#define BATCRG_VLEVEL_PULSE         (float)  5.000///< voltage level ending pulse current mode in [V]
#define BATCRG_ILEVEL_PULSE         (float)  1.000 ///< reference current level during pulse current mode in [A]

#define BATCRG_VLEVEL_CONSTANT_IL   (float)  6.000 ///< voltage level at which constant current mode will stop in [V]
#define BATCRG_ILEVEL_CONSTANT_IL   (float)  2.000 ///< reference current level during low current mode in [A]

#define BATCRG_VLEVEL_CONSTANT_IH   (float)  7.000 ///< voltage level at which constant current mode will stop in [V]
#define BATCRG_ILEVEL_CONSTANT_IH   (float)  6.000 ///< reference current level during high current mode in [A]

#define BATCRG_VLEVEL_CONSTANT_V    (float)  13.200 ///< voltage level at which constant current mode will stop in [V]
#define BATCRG_ILEVEL_CONSTANT_V    (float)  6.000 ///< reference current level at which constant current mode will stop in [A]

//#define BATCRG_VLEVEL_PEAK_V        (float)  13.200 ///< voltage level at which constant current mode will stop in [V]
//#define BATCRG_ILEVEL_PEAK_V        (float)  6.000 ///< reference current level during high current mode in [A]

#define BATCRG_VLEVEL_HYSTERESIS    (float)  0.030 ///< voltage level threshold hysteresis in [V]
#define BATCRG_ILEVEL_CUTOFF_MIN    (float)  2.000 ///< minimum current level below which charging will stop in [A]
#define BATCRG_OVERSAMPLING_DEPTH   256U    ///< Array size used for oversampling voltage and current values used by the charging profile

#ifdef CURRENT_LIMIT_MODE     // In Charger Mode, always enable sustained current limit mode.
  #undef CURRENT_LIMIT_MODE
  #define CURRENT_LIMIT_MODE OUTPUT_SCL_MODE
#endif

#endif

/** @} */ // end of group battery-charger-settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#endif  /* __EPC9165_R10__ */
#endif	/* EPC9165_R10_HARDWARE_CONFIGURATION_H */

// ________________________
// end if file
