/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    epc9170_r10_board.h
 * @author  M91406
 * @brief   EPC9170 48V-to-12V Rail Converter Hardware Descriptor header file
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef EPC9170_R10_HARDWARE_PINMAP_H
#define	EPC9170_R10_HARDWARE_PINMAP_H
#ifdef __EPC9170_R10__

#include <xc_pral.h> // include device peripheral register abstraction layer
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types
#include <math.h> // include standard math functions library

#include "config/hal/epc9528/epc9528_r30.h" // include dsPIC prozessor module header file

/***************************************************************************************************
 * @ingroup circuit-gpio-mcal
 * @{
 * @brief Global abstraction labels of special circuit signal device pin assignments
 * 
 * @details
 * This section is used to define labels of hardware specific signals, which are directly 
 * assigned to specific device pins. These labels will be used throughout the code and remain
 * unchanged even if this firmware is migrated to another device or pin-out changes between
 * hardware revisions.
 * 
 **************************************************************************************************/

// Device Pin #14 on EPC9170
#if (PWRGOOD_ENABLE == true)
#define PWRGOOD_PORT        DBGPIN2 ///< GPIO port declaration where 0=Port RA, 0=Port RB, 0=Port RC, etc.
#endif

// Device Pin #25 on EPC9170
#if (PWREN_ENABLE == true)
#define PWREN_PORT          DBGPIN1 ///< GPIO port declaration where 0=Port RA, 0=Port RB, 0=Port RC, etc.
#endif

/** @} */ // end of group circuit-gpio-mcal

/**************************************************************************************************
 * @ingroup circuit-peripheral-mcal
 * @{
 * @brief User-declaration of global defines for PWM signal generator settings
 * 
 * @details
 * This section defines fundamental PWM settings required for the interleaved buck converter
 * of the EPC9170 48V-to-12V rail power module development board. These settings are determined 
 * by hardware and defined using physical quantities. Pre-compiler macros are used to convert 
 * physical values into binary (integer) numbers to be written to Special Function Registers (SFR).
 * 
 **************************************************************************************************/

#define ISENSE_REF_DAC_INSTANCE         REF_DAC_INSTANCE ///< Digital-To-Analog Converter instance used to generate current sense amplifier reference voltage
#define ISENSE_REF_BUFFER_OPA_INSTANCE  REF_BUFFER_OPA_INSTANCE ///< Operational amplifier instance used to generate current sense amplifier reference voltage

/** @} */ // end of group circuit-peripheral-mcal

/**************************************************************************************************
 * @ingroup pwm-mcal-phase
 * @brief PWM peripheral output pins, control signals and register assignments of converter phase #1
 * 
 * @details
 * Each converter phase uses a simple half-bridge to commutate the switch node of the non-isolated,
 * bidirectional step down converter. The signal source therefore only requires a single PWM 
 * generator instance to be configured in fixed frequency, complementary mode with dead times. 
 * Additional PWM peripheral features are used by the firmware to respond to interrupts, trigger 
 * ADC conversions, control device output pins during startup and fault responses and to change 
 * timing settings on the fly. 
 * 
 * Please review the device data sheet for details about register names and settings.
 **************************************************************************************************/

/** 
 * @ingroup pwm-mcal-phase1
 * @{ 
 * @brief   PWM peripheral output pins, control signals and register assignments of converter phase #1
 * @note    Please review the device data sheet for details about register names and settings.
 */

// PWM Phase #1 Configuration
#if (NO_OF_PHASES >= 1)

#define PWM1H                   CEC10                   ///< Card Edge Connector pin of Device Port Pin of PWM A High-Output
#define PWM1L                   CEC12                   ///< Card Edge Connector pin of Device Port Pin of PWM A Low-Output
#define PWM1_OUTPUT_SWAP        false                   ///< PWM H/L outputs of channel are used "as is" (no swap)

#define PWM1_Interrupt          _PWM1Interrupt          ///< Interrupt Service Routine Label of PG1
#define PWM1_IP                 _PWM1IP                 ///< Interrupt vector priority register
#define PWM1_IF                 _PWM1IF                 ///< Interrupt vector flag bit register bit
#define PWM1_IE                 _PWM1IE                 ///< Interrupt vector enable bit register bit

#endif
/** @} */ // end of group pwm-mcal-phase1 ~~~~~~~~~~~~~~~~~

/** 
 * @ingroup pwm-mcal-phase2
 * @{ 
 * @brief   PWM peripheral output pins, control signals and register assignments of converter phase #2
 * @note    Please review the device data sheet for details about register names and settings.
 */

// PWM Phase #2 Configuration
#if (NO_OF_PHASES >= 2)

// Manual configuration of PWM generator #4 (PG4) mapped to GPIOs though PPS
#define PWM2PPS                 (P33C_PWM_INSTANCE_t*)&PG4CONL ///< PWM instance of PWM2 (board) routed to outputs through PPS
#define PWM2H_PPSO_ID           PPSO_PWM4H              ///< Identifier of PWM4H signal to be routed to PWM2H (board) output
#define PWM2L_PPSO_ID           PPSO_PWM4L              ///< Identifier of PWM4L signal to be routed to PWM2L (board) output

#define PWM2H                   CEC14                   ///< Card Edge Connector pin of Device Port Pin of PWM B High-Output
#define PWM2L                   CEC18                   ///< Card Edge Connector pin of Device Port Pin of PWM B Low-Output
#define PWM2_OUTPUT_SWAP        false                   ///< PWM H/L outputs of channel should be swapped

#define PWM2_Interrupt          _PWM4Interrupt          ///< Interrupt Service Routine Label of PG4
#define PWM2_IP                 _PWM4IP                 ///< Interrupt vector priority register
#define PWM2_IF                 _PWM4IF                 ///< Interrupt vector flag bit register bit
#define PWM2_IE                 _PWM4IE                 ///< Interrupt vector enable bit register bit

#endif
/** @} */ // end of group pwm-mcal-phase2 ~~~~~~~~~~~~~~~~~


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/** 
 * @ingroup input-voltage-feedback-mcal
 * @{ 
 * @brief ADC input assignments of input voltage feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignments of feedback signals in case of hardware changes.
 */

// Peripheral Assignments
#define V48                     CEC22                   ///< Card Edge Connector pin this signal is connected to

#define V48_R1                  (float) 100.0           ///< Upper voltage divider resistor in [kOhm]
#define V48_R2                  (float) 5.360           ///< Lower voltage divider resistor in [kOhm]
#define V48_FEEDBACK_OFFSET     (float) 0.000           ///< Physical, static signal offset in [V]
#define V48_ADC_TRG_DELAY       (float)120.0e-9         ///< ADC trigger delay in [sec]

// Data Acquisition Settings
#define V48_ADCANxInterrupt     CEC22_ADCANxInterrupt   ///< ADC interrupt service routine function call of the high voltage feedback channel
#define V48_ADCANxIF            CEC22_ADCANxIF          ///< ADC interrupt flag bit of the high voltage feedback channel

#define V48_ADCTRIG             PWM1H.Objects.PG->PGxTRIGB ///< Register used for trigger placement
#define V48_TRGSRC              ADCAN_TRGSRC_PWM1_TRG2  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define V48_ADCCORE             CEC22.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define V48_ADCDIFF             false                   ///< true = differential input, false = single-ended input
#define V48_ADCSIGN             false                   ///< true = sample result is signed, false = sample result is unsigned
#define V48_LVLTRG              false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define V48_POLARITY            0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

/** @} */ // end of group input-voltage-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/** 
 * @ingroup output-voltage-feedback-mcal
 * @{ 
 * @brief ADC input assignments of output voltage feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignment of feedback signals in case of hardware changes.
 */

// Peripheral Assignments
#define V12                     CEC26                   ///< Card Edge Connector pin this signal is connected to

#define V12_DIV_R1              (float) 100.0           ///< Upper voltage divider resistor in kOhm
#define V12_DIV_R2              (float) 5.360           ///< Lower voltage divider resistor in kOhm
#define V12_FEEDBACK_OFFSET     (float) 0.000           ///< Physical, static signal offset in [V]
#define V12_ADC_TRG_DELAY       (float)120.0e-9         ///< Trigger delay in [sec]

// Data Acquisition Settings
#define V12_ADCANxInterrupt     CEC26_ADCANxInterrupt   ///< ADC interrupt service routine function call of the low voltage feedback channel
#define V12_ADCANxIF            CEC26_ADCANxIF          ///< ADC interrupt flag bit of the low voltage feedback channel

#define V12_ADCTRIG             PWM1H.Objects.PG->PGxTRIGA ///< Register used for trigger placement
#define V12_TRGSRC              ADCAN_TRGSRC_PWM1_TRG1  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define V12_ADCCORE             CEC26.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define V12_ADCDIFF             false                   ///< true = differential input, false = single-ended input
#define V12_ADCSIGN             false                   ///< true = sample result is signed, false = sample result is unsigned
#define V12_LVLTRG              false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define V12_POLARITY            0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

/** @} */ // end of group output-voltage-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/** 
 * @ingroup phase-current-feedback-mcal
 * @{ 
 * @brief ADC input assignments of phase current feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignments of feedback signals in case of hardware changes.
 */

#define ISNS_SHUNT              (float) 0.200e-3        ///< Shunt resistance in [Ohm]
#define ISNS_AMP_GAIN           (float) 50.0            ///< Shunt amplifier gain in [V/V]

// Peripheral Assignments
#if (NO_OF_PHASES >= 1)

#define ISNS1                   CEC32                   ///< Card Edge Connector pin this signal is connected to

#define ISNS1_ADCANxInterrupt   CEC32_ADCANxInterrupt   ///< ADC interrupt service routine function call of the low voltage feedback channel
#define ISNS1_ADCANxIF          CEC32_ADCANxIF          ///< ADC interrupt flag bit of the low voltage feedback channel

#define ISNS1_ADCTRIG           PWM1H.Objects.PG->PGxTRIGA ///< Register used for trigger placement
#define ISNS1_TRGSRC            ADCAN_TRGSRC_PWM1_TRG1  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define ISNS1_ADCCORE           CEC32.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define ISNS1_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define ISNS1_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define ISNS1_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define ISNS1_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

#endif

#if (NO_OF_PHASES >= 2)

#define ISNS2                   CEC34                   ///< Card Edge Connector pin this signal is connected to

#define ISNS2_ADCANxInterrupt   CEC34_ADCANxInterrupt   ///< ADC interrupt service routine function call of the low voltage feedback channel
#define ISNS2_ADCANxIF          CEC34_ADCANxIF          ///< ADC interrupt flag bit of the low voltage feedback channel

#define ISNS2_ADCTRIG           PWM2H.Objects.PG->PGxTRIGA ///< Register used for trigger placement
#define ISNS2_TRGSRC            ADCAN_TRGSRC_PWM4_TRG1  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define ISNS2_ADCCORE           CEC34.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define ISNS2_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define ISNS2_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define ISNS2_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define ISNS2_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

#endif

/** @} */ // end of group phase-current-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/** 
 * @ingroup temperature-feedback-mcal
 * @{ 
 * @brief ADC input assignments of temperature feedback signals
 * 
 * @details
 * In this section the ADC input channels, related ADC result buffers, trigger
 * sources and interrupt vectors are defined. These settings allow the fast 
 * re-assignments of feedback signals in case of hardware changes.
 */

// Peripheral Assignments
#define TEMP1                   CEC39                   ///< Card Edge Connector pin this signal is connected to
#define TEMP1_ADCTRIG           PWM1H.Objects.PG->PGxTRIGB ///< Register used for trigger placement
#define TEMP1_TRGSRC            ADCAN_TRGSRC_PWM1_TRG2  ///< PWM generator ADC Trigger 1 or 2 via PGxTRIGx
#define TEMP1_ADCCORE           CEC39.Properties.ADCORE ///< ADC Core this ADC input is tied to
#define TEMP1_ADCDIFF           false                   ///< true = differential input, false = single-ended input
#define TEMP1_ADCSIGN           false                   ///< true = sample result is signed, false = sample result is unsigned
#define TEMP1_LVLTRG            false                   ///< true= sampling is level-triggered, false = sampling is edge-triggered
#define TEMP1_POLARITY          0                       ///< 0 = signal polarity is positive, 1 = signal polarity is negative 

/** @} */ // end of group temperature-feedback-mcal ~~~~~~~~~~~~~~~~~~~~~~~~~


/**************************************************************************************************
 * @ingroup control-interrupt-vector-declarations
 * @{
 * @brief Control loop interrupt vector and trigger settings
 * 
 * @details
 *   Control loops are called in dedicated interrupt service routines of PWM events, ADC events
 *   or triggered by timers. This section allows users to set and modify the interrupt service 
 *   routine triggers and their priority to set up and optimize the control system interrupt 
 *   structure.
 * 
 * @note
 *   All available control loop interrupt trigger options are listed in the respective
 *   hardware abstraction layer flags header file
 * 
 * @see
 *   epc9170_r10_flags.h
 * 
 ***************************************************************************************************/
    
// Hardware-dependent defines

#define VLOOP_TRIGGER_MODE      TRIGGER_MODE_PWM        ///< Currently selected voltage loop interrupt vector
#define VLOOP_ISR_PRIORITY      5U                      ///< Voltage loop interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

#define ISNS1_TRIGGER_MODE      TRIGGER_MODE_NONE       ///< Currently selected current loop #1 interrupt vector
#define ISNS1_ISR_PRIORITY      6U                      ///< Current loop #1 interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

#define ISNS2_TRIGGER_MODE      TRIGGER_MODE_NONE       ///< Currently selected current loop #2 interrupt vector
#define ISNS2_ISR_PRIORITY      6U                      ///< Current loop #1 interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

#define ILOOP_TRIGGER_MODE      ISNS1_TRIGGER_MODE      ///< Automatically apply ISNS1 trigger mode as default trigger mode

#if (VLOOP_TRIGGER_MODE == TRIGGER_MODE_ADC)
  #define VLOOP_Interrupt       V12_ADCANxInterrupt     ///< Interrupt vector function call label
  #define VLOOP_ISR_IP          V12_ADCANxIP            ///< Interrupt vector priority register bits
  #define VLOOP_ISR_IF          V12_ADCANxIF            ///< Interrupt vector flag bit register bit
  #define VLOOP_ISR_IE          V12_ADCANxIE            ///< Interrupt vector enable bit register bit
#elif (VLOOP_TRIGGER_MODE == TRIGGER_MODE_PWM)
  #define VLOOP_Interrupt       PWM1_Interrupt          ///< Interrupt vector function call label
  #define VLOOP_ISR_IP          PWM1_IP                 ///< Interrupt vector priority register
  #define VLOOP_ISR_IF          PWM1_IF                 ///< Interrupt vector flag bit register bit
  #define VLOOP_ISR_IE          PWM1_IE                 ///< Interrupt vector enable bit register bit
#elif (VLOOP_TRIGGER_MODE == TRIGGER_MODE_TIMER)
  #define VLOOP_Interrupt       _CCT1Interrupt          ///< Interrupt vector function call label
  #define VLOOP_ISR_IP          _CCT1IP                 ///< Interrupt vector priority register
  #define VLOOP_ISR_IF          _CCT1IF                 ///< Interrupt vector flag bit register bit
  #define VLOOP_ISR_IE          _CCT1IE                 ///< Interrupt vector enable bit register bit
#endif

#if (NO_OF_PHASES >= 1)

  #if (ISNS1_TRIGGER_MODE == TRIGGER_MODE_ADC)
    #define ILOOP1_Interrupt    ISNS1_ADCANxInterrupt   ///< Interrupt vector function call label
    #define ILOOP1_ISR_IP       ISNS1_ADCANxIP          ///< Interrupt vector priority register bits
    #define ILOOP1_ISR_IF       ISNS1_ADCANxIF          ///< Interrupt vector flag bit register bit
    #define ILOOP1_ISR_IE       ISNS1_ADCANxIE          ///< Interrupt vector enable bit register bit
  #elif (ISNS1_TRIGGER_MODE == TRIGGER_MODE_PWM)
    #define ILOOP1_Interrupt    PWM1_Interrupt          ///< Interrupt vector function call label
    #define ILOOP1_ISR_IP       PWM1_IP                 ///< Interrupt vector priority register
    #define ILOOP1_ISR_IF       PWM1_IF                 ///< Interrupt vector flag bit register bit
    #define ILOOP1_ISR_IE       PWM1_IE                 ///< Interrupt vector enable bit register bit
  #endif

#endif

#if (NO_OF_PHASES >= 2)

  #define ISNS2_TRIGGER_MODE    TRIGGER_MODE_NONE       ///< Currently selected current loop #2 interrupt vector
  #define ISNS2_ISR_PRIORITY    6U                      ///< Current loop #2 interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

  #if (ISNS2_TRIGGER_MODE == TRIGGER_MODE_ADC)
    #define ILOOP2_Interrupt    ISNS2_ADCANxInterrupt   ///< Interrupt vector function call label
    #define ILOOP2_ISR_IP       ISNS2_ADCANxIP          ///< Interrupt vector priority register bits
    #define ILOOP2_ISR_IF       ISNS2_ADCANxIF          ///< Interrupt vector flag bit register bit
    #define ILOOP2_ISR_IE       ISNS2_ADCANxIE          ///< Interrupt vector enable bit register bit
  #elif (ISNS2_TRIGGER_MODE == TRIGGER_MODE_PWM)
    #define ILOOP2_Interrupt    PWM2_Interrupt          ///< Interrupt vector function call label
    #define ILOOP2_ISR_IP       PWM2_IP                 ///< Interrupt vector priority register
    #define ILOOP2_ISR_IF       PWM2_IF                 ///< Interrupt vector flag bit register bit
    #define ILOOP2_ISR_IE       PWM2_IE                 ///< Interrupt vector enable bit register bit
  #endif

#endif

/** @} */ // end of group control-interrupt-vector-declarations

#endif  /* __EPC9170_R10__ */
#endif	/* EPC9170_R10_HARDWARE_PINMAP_H */

// ________________________
// end if file
