/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/* 
 * @file    epc9170_r10_macros.h
 * @author  M91406
 * @brief   Physical quantity parameter declaration header file 
 * @version 1.0
 */

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef EPC9170_R10_HARDWARE_MACROS_H
#define	EPC9170_R10_HARDWARE_MACROS_H
#ifdef __EPC9170_R10__

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types
#include <math.h> // include standard math library

#include "config/os.h" // include operating system configuration header file
#include "config/mcal.h" // include target device abstraction layer header file
#include "epc9170_r10_config.h" // include related converter configuration header file
#include "epc9170_r10_board.h"  // include related board configuration header file
#include "epc9170_r10_board_ext.h"  // include related board configuration header file

/***********************************************************************************
 * @ingroup pwm-macros
 * @{ 
 * @brief Conversion macros for user-declarations of PWM parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 ***********************************************************************************/

// Conversion Macros
#define SWITCHING_PERIOD    (float)(1.0/SWITCHING_FREQUENCY)   ///< Switching period in [sec]
#define PWM_PERIOD          (uint16_t)(float)(SWITCHING_PERIOD / PWM_CLOCK_PERIOD) ///< This sets the switching period of the converter
#define PWM_PHASE_SHIFT     (uint16_t)((float)PWM_PERIOD / (float)NO_OF_PHASES) ///< This sets the phase shift between phase #1 and #2
#define PWM_DC_MIN          (uint16_t)(PWM_DUTY_CYCLE_MIN * (float)PWM_PERIOD) ///< This sets the minimum duty cycle
#define PWM_DC_MAX          (uint16_t)(PWM_DUTY_CYCLE_MAX * (float)PWM_PERIOD) ///< This sets the maximum duty cycle
#define LEB_PERIOD          (uint16_t)(LEADING_EDGE_BLANKING / (float)PWM_CLOCK_PERIOD) ///< Leading Edge Blanking = n x PWM resolution (here: 50 x 2ns = 100ns)
#define PWM_DEAD_TIME_LE    (uint16_t)(DEAD_TIME_LEADING_EDGE / (float)PWM_CLOCK_PERIOD) ///< Rising edge dead time [tick = 250ps]
#define PWM_DEAD_TIME_FE    (uint16_t)(DEAD_TIME_FALLING_EDGE / (float)PWM_CLOCK_PERIOD) ///< Falling edge dead time [tick = 250ps]

/** @} */ // end of group pwm-macros ~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup input-voltage-feedback-macros
 * @{ 
 * @brief Conversion macros of input voltage feedback parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define V48_FEEDBACK_GAIN   (float)((V48_R2) / (V48_R1 + V48_R2)) // DO NOT CHANGE
#define V48_MIN             (uint16_t)(V48_MINIMUM * V48_FEEDBACK_GAIN / ADC_GRANULARITY)   ///< Minimum input voltage
#define V48_NOM             (uint16_t)(V48_NOMINAL * V48_FEEDBACK_GAIN / ADC_GRANULARITY)   ///< Nominal input voltage
#define V48_MAX             (uint16_t)(V48_MAXIMUM * V48_FEEDBACK_GAIN / ADC_GRANULARITY)   ///< Maximum input voltage
#define V48_HYST            (uint16_t)(V48_HYSTERESIS * V48_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Protection thresholds hysteresis 
#define V48_OFFSET          (uint16_t)(V48_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Input voltage feedback offset
#define V48_VFWD_DROP       (uint16_t)(V48_VFWD_DROP_MAX * V48_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the maximum forward voltage drop across teh rectifier diode

#define V48_DEV_TRIP        (uint16_t)(V48_TOLERANCE_MAX * V48_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the maximum allowed output voltage deviation given above in [V], which will lead to a converter shut down when exceeded.
#define V48_DEV_RELEASE     (uint16_t)(V48_TOLERANCE_MIN * V48_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the maximum allowed output voltage deviation given above in [V], which needs to be underrun before a shut-down converter can recover
#define V48_UVLO_TRIP       (uint16_t)(V48_UVLO_VOLTAGE * V48_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Under Voltage Lock Out voltage
#define V48_UVLO_RELEASE    (uint16_t)((V48_UVLO_VOLTAGE + V48_HYSTERESIS) * V48_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Under Voltage Lock Out voltage
#define V48_OVLO_TRIP       (uint16_t)(V48_OVLO_VOLTAGE * V48_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Lock Out voltage
#define V48_OVLO_RELEASE    (uint16_t)((V48_OVLO_VOLTAGE - V48_HYSTERESIS) * V48_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Lock Out voltage
#define V48_OVP_TRIP        (uint16_t)(V48_OVP_VOLTAGE * V48_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Protection voltage trip threshold
#define V48_OVP_RELEASE     (uint16_t)((V48_OVP_VOLTAGE - V48_HYSTERESIS) * V48_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Protection voltage release threshold

#define V48_NORM_INV_G      (float)(1.0/V48_FEEDBACK_GAIN) ///< Inverted feedback gain required for value normalization
#define V48_NORM_SCALER     (int16_t)(ceil(log(V48_NORM_INV_G)/log(2))) ///< VIN normalization  
#define V48_NORM_FACTOR     (int16_t)((V48_NORM_INV_G / pow(2.0, V48_NORM_SCALER)) * (pow(2.0, 15)-1)) ///< VIN normalization factor scaled in Q15

#define V48_ADC_TRGDLY      (uint16_t)(V48_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< Input voltage ADC trigger delay

#define V48_COMM_DIGITS     (int16_t)2 // Power of 10 determining the number of digits of the normalized communication data format
#define V48_COMM_MULX       (float)(pow(10.0, (float)V48_COMM_DIGITS))
#define V48_COMM_GAIN       (float)(V48_COMM_MULX * (ADC_GRANULARITY / V48_FEEDBACK_GAIN))
#define V48_COMM_SCALER     (int16_t)(ceil(log(V48_COMM_GAIN)/log(2.0)))
#define V48_COMM_FACTOR     (int16_t)((V48_COMM_GAIN / pow(2.0, V48_COMM_SCALER)) * (pow(2.0, 15.0) - 1.0)) 

/** @} */ // end of group input-voltage-feedback-macros ~~~~~~~~~~~~~~~~~~~~~~
    
/**************************************************************************************************
 * @ingroup output-voltage-feedback-macros
 * @{ 
 * @brief Conversion macros of output voltage feedback parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define V12_FEEDBACK_GAIN   (float)((V12_DIV_R2) / (V12_DIV_R1 + V12_DIV_R2)) ///< Macro calculating the integer number equivalent of the output voltage feedback gain
 
#define V12_MIN             (uint16_t)(V12_MINIMUM * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the output voltage minimum given above in [V]. This value specifies the minimum startup voltage of teh converter.
#define V12_NOM             (uint16_t)(V12_NOMINAL * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the output voltage reference given above in [V]
#define V12_MAX             (uint16_t)(V12_MAXIMUM * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the output voltage minimum given above in [V]. This value specifies the minimum startup voltage of teh converter.
#define V12_OFFSET          (uint16_t)(V12_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the physical, static signal offset of this feedback channel
#define V12_HYST            (uint16_t)(V12_HYSTERESIS * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Protection thresholds hysteresis

#define V12_DEV_TRIP        (uint16_t)(V12_TOLERANCE_MAX * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the maximum allowed output voltage deviation given above in [V], which will lead to a converter shut down when exceeded.
#define V12_DEV_RELEASE     (uint16_t)(V12_TOLERANCE_MIN * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Macro calculating the integer number equivalent of the maximum allowed output voltage deviation given above in [V], which needs to be underrun before a shut-down converter can recover
 
#define V12_UVLO_TRIP       (uint16_t)(V12_UNDER_VOLTAGE * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Under Voltage Lock Out voltage
#define V12_UVLO_RELEASE    (uint16_t)((V12_UNDER_VOLTAGE + V12_HYSTERESIS) * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Under Voltage LOck Out voltage
#define V12_OVLO_TRIP       (uint16_t)(V12_OVER_VOLTAGE * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Lock Out voltage
#define V12_OVLO_RELEASE    (uint16_t)((V12_OVER_VOLTAGE - V12_HYSTERESIS) * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Lock Out voltage
#define V12_OVP_TRIP        (uint16_t)(V12_OVP_VOLTAGE * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Protection voltage trip threshold
#define V12_OVP_RELEASE     (uint16_t)((V12_OVP_VOLTAGE - V12_HYSTERESIS) * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Over Voltage Protection voltage release threshold
#define V12_UVP_TRIP        (uint16_t)((V12_UVP_VOLTAGE * V12_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Output Under Voltage Protection trip voltage threshold (used in battery charging mode only)
#define V12_UVP_RELEASE     (uint16_t)((V12_UVP_VOLTAGE + V12_HYSTERESIS) * V12_FEEDBACK_GAIN / ADC_GRANULARITY)  ///< Output Under Voltage Protection release voltage threshold (used in battery charging mode only)

#define V12_NORM_INV_G      (float)(1.0/V12_FEEDBACK_GAIN) ///< Inverted feedback gain required for value normalization
#define V12_NORM_SCALER     (int16_t)(ceil(log(V12_NORM_INV_G)/log(2))) ///< VOUT normalization  
#define V12_NORM_FACTOR     (int16_t)((V12_NORM_INV_G / pow(2.0, V12_NORM_SCALER)) * (pow(2.0, 15)-1)) ///< VOUT normalization factor scaled in Q15

#define V12_ADC_TRGDLY      (uint16_t)(V12_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< Macro calculating the integer number equivalent of the signal chain time delay between internal PWM timebase and effective switching edge of the leading FET

#define V12_COMM_DIGITS     (int16_t)2 // Power of 10 determining the number of digits of the normalized communication data format
#define V12_COMM_MULX       (float)(pow(10.0, (float)V12_COMM_DIGITS))
#define V12_COMM_GAIN       (float)(V12_COMM_MULX * (ADC_GRANULARITY / V12_FEEDBACK_GAIN))
#define V12_COMM_SCALER     (int16_t)(ceil(log(V12_COMM_GAIN)/log(2.0)))
#define V12_COMM_FACTOR     (int16_t)((V12_COMM_GAIN / pow(2.0, V12_COMM_SCALER)) * (pow(2.0, 15.0) - 1.0)) 

/** @} */ // end of group output-voltage-feedback-macros ~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup phase-current-feedback-macros
 * @{ 
 * @brief Conversion macros of phase current feedback parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

// Current Sense Amplifier Gain
#define ISNS_FEEDBACK_GAIN  (float)(ISNS_SHUNT * ISNS_AMP_GAIN)     ///< Phase current gain in V/A

// Phase Current Feedback Settings Conversion Macros
#define ISNS_OCL_START      (uint16_t)((ISNS_LIMIT_START * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Output Current Reference
#define ISNS_CMOD_LIMIT     (uint16_t)(ISNS_COMMON_MODE_LIMIT * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Common Mode Voltage Limit specifies the minimum output voltage required before the current sense feedback becomes reliable
#define ISNS_CMOD_HYST      (uint16_t)(ISNS_COMMON_MODE_HYST * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< Common Mode Voltage Hysteresis specifies the minimum output voltage required before the current sense feedback becomes reliable
#define ISNS_BLNC_LIMIT     (int16_t)(ISNS_BALANCING_LIMIT / PWM_CLOCK_PERIOD)     ///< Duty cycle offset limit for current balancing in [PWM ticks]
#define ISNS_BLNC_HYST      (int16_t)(ISNS_BALANCING_HYST / PWM_CLOCK_PERIOD)     ///< Duty cycle offset hysteresis for current balancing in [PWM ticks]
 
#define ISNS_MIN            (int16_t)((ISNS_MINIMUM * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Lower Runtime Current Limit
#define ISNS_NOM            (int16_t)((ISNS_NOMINAL * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Current Feedback Baseline
#define ISNS_MAX            (int16_t)((ISNS_MAXIMUM * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Upper Runtime Current Limit
#define ISNS_START          (int16_t)((ISNS_LIMIT_START * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Upper Runtime Current Limit
#define ISNS_OCP_TRIP       (int16_t)((ISNS_OCP_TRIP_LEVEL * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Over Current Limit
#define ISNS_OCP_RELEASE    (int16_t)((ISNS_OCP_RELEASE_LEVEL * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY)  ///< Over Current Release Level
 
#define SYNCTHLD_ON         (uint16_t)((SYNCCTL_ON_THRESHOLD * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY) ///< Phase Current above which the synchronous rectifier will be turned on
#define SYNCTHLD_OFF        (uint16_t)((SYNCCTL_OFF_THRESHOLD * ISNS_FEEDBACK_GAIN) / ADC_GRANULARITY) ///< Phase Current below which the synchronous rectifier will be turned off
 
#define ISNS_NORM_INV_G     (float)(1.0/ISNS_FEEDBACK_GAIN) ///< Inverted feedback gain required for value normalization
#define ISNS_NORM_SCALER    (int16_t)(ceil(log(ISNS_NORM_INV_G)/log(2))) ///< ISNS normalization  
#define ISNS_NORM_FACTOR    (int16_t)((ISNS_NORM_INV_G / pow(2.0, ISNS_NORM_SCALER)) * (pow(2.0, 15)-1)) ///< ISNS normalization factor scaled in Q15

#define ISNS_COMM_DIGITS    (int16_t)2 // Power of 10 determining the number of digits of the normalized communication data format
#define ISNS_COMM_MULX      (float)(pow(10.0, (float)ISNS_COMM_DIGITS))
#define ISNS_COMM_GAIN      (float)(ISNS_COMM_MULX * (ADC_GRANULARITY / ISNS_FEEDBACK_GAIN))
#define ISNS_COMM_SCALER    (int16_t)(ceil(log(ISNS_COMM_GAIN)/log(2.0)))
#define ISNS_COMM_FACTOR    (int16_t)((ISNS_COMM_GAIN / pow(2.0, ISNS_COMM_SCALER)) * (pow(2.0, 15.0) - 1.0)) 

#if (NO_OF_PHASES > 0) 
#define ISNS1_OFFSET        (uint16_t)(ISNS1_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Phase current feedback signal offset in [V] converted to integer
#define ISNS1_ADC_TRGDLY    (uint16_t)(ISNS1_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< ADC trigger delay of phase current #1 sense input in [s] converted to integer
#endif

#if (NO_OF_PHASES > 1)
#define ISNS2_OFFSET        (uint16_t)(ISNS2_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Phase current feedback signal offset in [V] converted to integer
#define ISNS2_ADC_TRGDLY    (uint16_t)(ISNS2_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< ADC trigger delay of phase current #2 sense input in [s] converted to integer
#endif

#if (NO_OF_PHASES > 2)
#define ISNS3_OFFSET        (uint16_t)(ISNS3_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Phase current feedback signal offset in [V] converted to integer
#define ISNS3_ADC_TRGDLY    (uint16_t)(ISNS3_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< ADC trigger delay of phase current #2 sense input in [s] converted to integer
#endif

#if (NO_OF_PHASES > 3)
#define ISNS4_OFFSET        (uint16_t)(ISNS4_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Phase current feedback signal offset in [V] converted to integer
#define ISNS4_ADC_TRGDLY    (uint16_t)(ISNS4_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< ADC trigger delay of phase current #2 sense input in [s] converted to integer
#endif

/** @} */ // end of group phase-current-feedback-macros ~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup temperature-feedback-macros
 * @{ 
 * @brief Conversion macros of temperature feedback parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define TEMP_MIN            (uint16_t)(TEMP_MINIMUM * TEMP_FEEDBACK_GAIN + TEMP_FEEDBACK_OFFSET)  // Temperature Range Minimum
#define TEMP_NOM            (uint16_t)(TEMP_NOMINAL * TEMP_FEEDBACK_GAIN + TEMP_FEEDBACK_OFFSET)  // Temperature Range Zero �C Point
#define TEMP_MAX            (uint16_t)(TEMP_MAXIMUM * TEMP_FEEDBACK_GAIN + TEMP_FEEDBACK_OFFSET)  // Temperature Range Maximum

#define TEMP_OTP_TRIP       (uint16_t)(TEMP_OTP_ULIMIT * TEMP_FEEDBACK_GAIN + TEMP_FEEDBACK_OFFSET)  // Over Temperature Limit
#define TEMP_OTP_RELEASE    (uint16_t)(TEMP_OTP_LLIMIT * TEMP_FEEDBACK_GAIN + TEMP_FEEDBACK_OFFSET)  // Over Temperature Release Level    

#define TEMP_NORM_INV_G     (float)(1.0/TEMP_FEEDBACK_GAIN) ///< Inverted feedback gain required for value normalization
#define TEMP_NORM_SCALER    (int16_t)(ceil(log(TEMP_NORM_INV_G)/log(2))) ///< ISNS normalization  
#define TEMP_NORM_FACTOR    (int16_t)((TEMP_NORM_INV_G / pow(2.0, TEMP_NORM_SCALER)) * (pow(2.0, 15)-1)) ///< ISNS normalization factor scaled in Q15
#define TEMP_OFFSET         (uint16_t)(TEMP_FEEDBACK_OFFSET / ADC_GRANULARITY) ///< Phase current feedback signal offset in [V] converted to integer
#define TEMP_ADC_TRGDLY     (uint16_t)(TEMP_ADC_TRG_DELAY / PWM_CLOCK_PERIOD) ///< ADC trigger delay of phase current #2 sense input in [s] converted to integer

#define TEMP_COMM_DIGITS    (int16_t)1 // Power of 10 determining the number of digits of the normalized communication data format
#define TEMP_COMM_MULX      (float)(pow(10.0, (float)TEMP_COMM_DIGITS))
#define TEMP_COMM_GAIN      (float)(TEMP_COMM_MULX * (ADC_GRANULARITY / TEMP_FEEDBACK_GAIN))
#define TEMP_COMM_SCALER    (int16_t)(ceil(log(TEMP_COMM_GAIN)/log(2.0)))
#define TEMP_COMM_FACTOR    (int16_t)((TEMP_COMM_GAIN / pow(2.0, TEMP_COMM_SCALER)) * (pow(2.0, 15.0) - 1.0)) 

/** @} */ // end of group temperature-feedback-macros ~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup adaptive-control-macros
 * @{ 
 * @brief Conversion macros of phase current feedback parameters
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define V12_RANGE_MAX       (float)(V12_MAXIMUM - V12_MINIMUM)
#define V48_RANGE_MAX       (float)(V48_MAXIMUM - V48_MINIMUM)

#define VL_MINIMUM          (float)(V48_UVLO_VOLTAGE - V12_RANGE_MAX) ///< Minimum input voltage - maximum output voltage
#define VL_NOMINAL          (float)(V48_NOMINAL      - V12_NOMINAL) ///< Nominal input voltage - nominal output voltage
#define VL_MAXIMUM          (float)(V48_RANGE_MAX    - 0) ///< Maximum input voltage - 0

// To calculate the voltage across the inductor, input and output voltage ADC results need to be normalized. The normalization factor is determined here
// Each input voltage sample has to be multiplied with this scaling factor to allow the calculation of the instantaneous voltage across the inductor
#define VIN_NORM_FCT        (float)(V12_FEEDBACK_GAIN / V48_FEEDBACK_GAIN)   ///< VIN-2-VOUT Normalization Factor
#define AGC_IO_NORM_SCALER  (int16_t)(ceil(log(VIN_NORM_FCT)/log(2))) ///< Nominal VL Q15 scaler  
#define AGC_IO_NORM_FACTOR  (int16_t)((VIN_NORM_FCT / pow(2.0, AGC_IO_NORM_SCALER)) * (pow(2.0, 15)-1)) ///< Nominal VL Q15 factor 

// The AGC compare value is defined at nominal input voltage and output voltage 
// The maximum modulation factor is normalized to fractional '1' to prevent number overruns
#define AGC_FACTOR_MAX      (float)(VL_NOMINAL / VL_MINIMUM) ///< Floating point number of the maximum limit of the adaptive gain modulation factor (float)
#define AGC_NOM_SCALER      (uint16_t)(ceil(log(AGC_FACTOR_MAX)/log(2))) ///< Bit-shift scaler of the floating point number of the maximum limit of the adaptive gain modulation factor
#define AGC_NOM_FACTOR      (uint16_t)(0x7FFF >> AGC_NOM_SCALER) ///< Fractional of the floating point number of the maximum limit of the adaptive gain modulation factor
#define AGC_MEDIAN          (int16_t)(((int16_t)(((float)V48_NOM * VIN_NORM_FCT) - V12_NOM))>>AGC_NOM_SCALER) ///< Adaptive gain modulation factor at nominal operating point

// Additional execution time calculation to be considered in trigger delay and overall control timing
#define AGC_EXEC_DLY       (uint16_t)(AGC_EXECUTION_DELAY / PWM_CLOCK_PERIOD) ///< Macro calculating the integer number equivalent of the AGC algorithm computation time

/** @} */ // end of group adaptive-control-macros ~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup startup-timing-macros
 * @{ 
 * @brief Conversion Macros of Startup Timing Settings
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define START_POD           (uint16_t)(((float)    POWER_ON_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_CUD           (uint16_t)(((float)   CHARGE_UP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_CUD_TO        (uint16_t)(((float) CHARGE_UP_TIMEOUT / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_LVRAMP_PER    (uint16_t)(((float)     LVRAMP_PERIOD / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_HVRAMP_PER    (uint16_t)(((float)     HVRAMP_PERIOD / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_LVREF_STEP    (uint16_t)( (float)           V12_NOM / (float)(START_LVRAMP_PER + 1.0))
#define START_HVREF_STEP    (uint16_t)( (float)           V48_NOM / (float)(START_HVRAMP_PER + 1.0))
#define START_IRAMP_PER     (uint16_t)(((float)      IRAMP_PERIOD / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)
#define START_IREF_STEP     (uint16_t)( (float)(ISNS_MAX-ISNS_START) / (float)(START_IRAMP_PER + 1.0))
#define START_PGD           (uint16_t)(((float)  POWER_GOOD_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0)

/** @} */ // end of group startup-timing-macros ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/**************************************************************************************************
 * @ingroup fault-response-macros
 * @{ 
 * @brief Conversion Macros of Fault Response Timing Settings
 * 
 * @details
 * These conversion macros are used to convert user settings defined as physical 
 * quantities into binary (integer) numbers, which will be written to registers and
 * variables and used in calculations throughout the firmware.
 **************************************************************************************************/

#define UVLO_TDLY           (uint16_t)(((float)      UVLO_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< under voltage lock out trip delay conversion macro
#define UVLO_RDLY           (uint16_t)(((float)  UVLO_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< under voltage lock out recovery delay conversion macro
#define OVLO_TDLY           (uint16_t)(((float)      OVLO_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over voltage lock out trip delay conversion macro
#define OVLO_RDLY           (uint16_t)(((float)  OVLO_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over voltage lock out recovery delay conversion macro
#define REGERR_TDLY         (uint16_t)(((float)    REGERR_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< regulation error trip delay conversion macro
#define REGERR_RDLY         (uint16_t)(((float)REGERR_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< regulation error recovery delay conversion macro
#define OCP_TDLY            (uint16_t)(((float)       OCP_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over current protection trip Delay conversion macro
#define OCP_RDLY            (uint16_t)(((float)   OCP_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over current protection recovery delay conversion macro
#define OVP_TDLY            (uint16_t)(((float)       OVP_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over voltage protection trip Delay conversion macro
#define OVP_RDLY            (uint16_t)(((float)   OVP_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over voltage protection recovery delay conversion macro
#define OTP_TDLY            (uint16_t)(((float)       OTP_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over temperature protection trip Delay conversion macro
#define OTP_RDLY            (uint16_t)(((float)   OTP_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< over temperature protection recovery delay conversion macro
#define UVP_TDLY            (uint16_t)(((float)       UVP_TRIP_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< under voltage protection trip Delay conversion macro
#define UVP_RDLY            (uint16_t)(((float)   UVP_RECOVERY_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< under voltage protection recovery delay conversion macro
    
#define FLTRSTDLY           (uint16_t)(((float)  FAULT_RESTART_DELAY / (float)RTOS_6G2_EXECUTION_PERIOD)-1.0) ///< under voltage protection recovery delay conversion macro

/** @} */ // end of group fault-response-macros ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    

/**************************************************************************************************
 * @ingroup battery-charger-settings
 * @{
 * @brief Global Battery Charger Parameters
 * 
 * @details
 * 
 * 
 *************************************************************************************************/
#if (CHARGER_ENABLE == true)

#define BATCRG_VLEVEL_HYST      (uint16_t)(BATCRG_VLEVEL_HYSTERESIS  * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level threshold hysteresis in [ADC ticks]

#define BATCRG_VLEVEL_TRKL      (uint16_t)(BATCRG_VLEVEL_TRICKLE     * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering trickle current mode in [ADC ticks]
#define BATCRG_VLEVEL_PLS       (uint16_t)(BATCRG_VLEVEL_PULSE       * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering pulse current mode in [ADC ticks]
#define BATCRG_VLEVEL_CONST_IL  (uint16_t)(BATCRG_VLEVEL_CONSTANT_IL * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering low constant current mode in [ADC ticks]
#define BATCRG_VLEVEL_CONST_IH  (uint16_t)(BATCRG_VLEVEL_CONSTANT_IH * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering high constant current mode  in [ADC ticks]
#define BATCRG_VLEVEL_CONST_V   (uint16_t)(BATCRG_VLEVEL_CONSTANT_V  * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering constant voltage mode in [ADC ticks]
//#define BATCRG_VLEVEL_PEAK      (uint16_t)(BATCRG_VLEVEL_PEAK_V      * V12_FEEDBACK_GAIN / ADC_GRANULARITY) ///< voltage level entering peak voltage mode in [ADC ticks]

#define BATCRG_ILEVEL_TRKL      (uint16_t)(BATCRG_ILEVEL_TRICKLE     * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in trickle current mode in [ADC ticks]
#define BATCRG_ILEVEL_PLS       (uint16_t)(BATCRG_ILEVEL_PULSE       * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in pulse current mode in [ADC ticks]
#define BATCRG_ILEVEL_CONST_IL  (uint16_t)(BATCRG_ILEVEL_CONSTANT_IL * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in low constant current mode in [ADC ticks]
#define BATCRG_ILEVEL_CONST_IH  (uint16_t)(BATCRG_ILEVEL_CONSTANT_IH * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in high constant current mode in [ADC ticks]
#define BATCRG_ILEVEL_CONST_V   (uint16_t)(BATCRG_ILEVEL_CONSTANT_V  * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in constant voltage mode in [ADC ticks]
//#define BATCRG_ILEVEL_PEAK      (uint16_t)(BATCRG_ILEVEL_PEAK_V      * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< constant current level in peak voltage mode in [ADC ticks]

#define BATCRG_ILEVEL_CUTOFF    (uint16_t)(BATCRG_ILEVEL_CUTOFF_MIN  * ISNS_FEEDBACK_GAIN / ADC_GRANULARITY) ///< minimum current level below which battery is considered charged in [ADC ticks]

#endif

/** @} */ // end of group battery-charger-settings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


#endif  /* __EPC9170_R10__ */
#endif	/* EPC9170_R10_HARDWARE_MACROS_H */

// ________________________
// end if file
