/* Microchip Technology Inc. and its subsidiaries.  You may use this software 
 * and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION 
 * WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 *
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
 * INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
 * WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS 
 * BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE 
 * FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS 
 * IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF 
 * ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE 
 * TERMS. 
 */

/**
 * @file    epc9528_r30.h
 * @brief   Device abstraction header file for processor plug-in module EPC9528 featuring dsPIC33CK256MP503
 * @author  M91406
 * @date    04/08/2022 2:57 PM
 * @version 1.0
 * @details
 *  This hardware abstraction layer header file provides macros mapping card-edge
 *  connector pins and test points to specific input ports and peripherals of the 
 *  low pin count processor module EPC9528, hardware version 1.0.
 */

/*
 * EPC9528, REV 3.0
 * 

                 ----------------------------------------                  --> Pin #39 (backside): RB0/RP32/OSCI/CLKI/AN5
                |                                        |       J10      /--> Pin #37 (backside): RB1/RP33/OSCO/CLKO/AN6
              |-----------                               ----------------/
              |           |        --------                40  OOOOOOOOO | RC0/RP48/AN12/ANN0
         USB  |           |       |        |               38  OOOOOOOOO | RA1/ANA1/OA1IN-
              |           |       |        |               36  OOOOOOOOO | RA2/AN9/OA1IN+
              |           |       |        |               34  OOOOOOOOO | RB7/RP39/AN2/CMP3A/SDA3/TDO
              |           |        --------                32  OOOOOOOOO | RB2/RP34/AN1/AN7/ANA0/CMP1D/CMP2D/CMP3D/OA2OUT/SCL3
              |           |        MCP2221A                30  OOOOOOOOO | RC1/RP49/AN13/CMP1B/OA3IN-/ISRC0
              |-----------                                 28  OOOOOOOOO | RC2/RP50/AN14/CMP2B/OA3IN+/ISRC1
                |                                          26  OOOOOOOOO | RB9/RP41/AN11/SDA1/PGC1
                |                                          24  OOOOOOOOO | RA3/AN3/CMP1C/DACOUT1
                |                                          22  OOOOOOOOO | RB8/RP40/AN10/SCL1/PGD1
         J20    |                                          20  OOOOOOOOO | RC3/RP51/AN15/CMP2A/IBIAS2
        ICSP    |                                          18  OOOOOOOOO | RB5/RP37/PWM6L/SDA2/PGD3
                --                  ---------              --------------
    ===========|  |=O  MCLR        |         |            |
               |  |                |  dsPIC  |             --------------
    ===========|  |=O  3V3         |  33 CK  |             14  OOOOOOOOO | RB6/RP38/PWM6H/SCL2/PGC3
               |  |                |         |             12  OOOOOOOOO | RB15/RP47/PWM1L
    ===========|  |=O  GND          ---------              10  OOOOOOOOO | RB14/RP46/PWM1H
               |  |                                         8  OOOOOOOOO | RB13/RP45/PWM2L
    ===========|  |=O  PGD2 (RB4/RP36/OA2IN+)               6  OOOOOOOOO | RB12/RP44/PWM2H/TDI
               |  |                                         4  OOOOOOOOO | RB11/RP43/PWM3L/TCK
    ===========|  |=O  PGC2 (RB3/RP35/AN8/OA2IN-)           2  OOOOOOOOO | RB10/RP42/PWM3H/TMS
                --            ---------------            --------------- 
                |             |O  O  O  O  O| J30        |
                 --------------|--|--|--|--|-------------
                               |  |  |  |  |
                       3V3-----   |  |  |  -----SCL (RC5/RP53/ASCL2)
                            GND---   |  ---GND
                                    SDA (RC4/RP52/ASDA2)
*/                                                                              
                                                                                

// This is a guard condition so that contents of this file are not included
// more than once.  
#ifndef HAL_EPC9528_R30_H
#define	HAL_EPC9528_R30_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer data types
#include <stdbool.h> // include standard boolean data types
#include <stddef.h> // include standard definition data types

// Generic declarations
#define PINSTATE_HIGH           1U  // GPIO HIGH level 
#define PINSTATE_LOW            0U  // GPIO LOW level 
#define PINDIR_INPUT            1U  // GPIO tristate-register value defining an input
#define PINDIR_OUTPUT           0U  // GPIO tristate-register value defining an output

#ifndef PPS_Unlock
#define PPS_Unlock()    { asm volatile( \
                            "push.s \n" \
                            "mov #0x55, w0   \n" \
                            "mov #0xAA, w1   \n" \
                            "mov w0, NVMKEY  \n" \
                            "mov w1, NVMKEY  \n" \
                            "bclr RPCON, #11 \n" \
                            "pop.s \n" ); \
                        }
#endif

#ifndef PPS_Lock
#define PPS_Lock()      { asm volatile( \
                            "push.s \n" \
                            "mov #0x55, w0   \n" \
                            "mov #0xAA, w1   \n" \
                            "mov w0, NVMKEY  \n" \
                            "mov w1, NVMKEY  \n" \
                            "bset RPCON, #11 \n" \
                            "pop.s \n" ); \
                        }
#endif

/***************************************************************************************************
 * @ingroup circuit-gpio-mcal-internal-uart
 * @{
 **************************************************************************************************/

//#if __DEBUG
#define VCP_RX                  PIN_RC5 //PIN_RB4 ///< UART Receive Pin
#define VCP_TX                  PIN_RC4 //PIN_RB3 ///< UART Transmit Pin
//#else
//#define VCP_RX                  PIN_RB4 ///< UART Receive Pin
//#define VCP_TX                  PIN_RB3 ///< UART Transmit Pin
//#endif

#define VCP_PORT                1U      ///< Number of UART instance used to communicate with the PC

#define VCP_BAUDRATE            UART_BAUDRATE_115200    ///< UART port configuration: Baud rate 115200
#define VCP_DATABITS            UART_DATABITS_8         ///< UART port configuration: 8 data bits
#define VCP_PARITY              UART_PARITY_NONE        ///< UART port configuration: no parity
#define VCP_STOPBITS            UART_STOPBITS_1         ///< UART port configuration: 1 stop bits
#define VCP_FLOWCONRTOL         UART_FLOWCONTROL_NONE   ///< UART port configuration: no flow control
	
#define VCP_TX_DMA              1U      ///< UART port transmit channel DMA support instance
#define VCP_RX_DMA              0U      ///< UART port receive channel DMA support instance

#define VCP_USE_TX_DMA          true    ///< Use DMA for data transmission 
#define VCP_USE_RX_DMA          false   ///< Use DMA for data reception
#define VCP_REDIRECT_PRINTF     false   ///< Redirect printf to print messages to UART
#define VCP_DATA_BYTE_SWAP      false   ///< When true, swaps the order of bytes of 16-bit and 32-bit values

/** @} */ // end of group circuit-gpio-mcal-internal-uart

/**************************************************************************************************
 * @ingroup comm-interrupt-vector-declarations
 * @{
 * @brief   Communication interface interrupt vector Settings
 * 
 * @details
 * Control loops are called in dedicated interrupt service routines of PWM events, ADC events
 * or triggered by timers. This section allows users to set and modify the interrupt service 
 * routine triggers and their priority to set up and optimize the control system interrupt 
 * structure.
 * 
 * *************************************************************************************************/

#if (VCP_PORT == 1U)

  #define VcpRxInterrupt        _U1RXInterrupt
  #define VCP_Rx_IP             _U1RXIP ///< Receive interrupt priority register
  #define VCP_Rx_IE             _U1RXIE ///< Receive interrupt enable bit
  #define VCP_Rx_IF             _U1RXIF ///< Receive interrupt flag bit

  #define VcpTxInterrupt        _U1TXInterrupt ///< Transmit interrupt serivce routine label 
  #define VCP_Tx_IP             _U1TXIP ///< Transmit interrupt priority register
  #define VCP_Tx_IE             _U1TXIE ///< Transmit interrupt enable bit
  #define VCP_Tx_IF             _U1TXIF ///< Transmit interrupt flag bit

  #define VcpErrInterrupt       _U1EInterrupt ///< Transmit interrupt serivce routine label 
  #define VCP_ErrIP             _U1EIP ///< Transmit interrupt priority register
  #define VCP_ErrIE             _U1EIE ///< Transmit interrupt enable bit
  #define VCP_ErrIF             _U1EIF ///< Transmit interrupt flag bit

#elif (VCP_PORT == 2U)

  #define VCP_RxInterrupt       _U2RXInterrupt ///< Receive interrupt serivce routine label 
  #define VCP_Rx_IP             _U2RXIP ///< Receive interrupt priority register
  #define VCP_Rx_IE             _U2RXIE ///< Receive interrupt enable bit
  #define VCP_Rx_IF              _U2RXIF ///< Receive interrupt flag bit

  #define VCP_TxInterrupt       _U2TXInterrupt ///< Transmit interrupt serivce routine label 
  #define VCP_Tx_IP             _U2TXIP ///< Transmit interrupt priority register
  #define VCP_Tx_IE             _U2TXIE ///< Transmit interrupt enable bit
  #define VCP_Tx_IF             _U2TXIF ///< Transmit interrupt flag bit

#elif (VCP_PORT == 3U)

  #define VCP_RxInterrupt       _U3RXInterrupt ///< Receive interrupt serivce routine label 
  #define VCP_Rx_IP             _U3RXIP ///< Receive interrupt priority register
  #define VCP_Rx_IE             _U3RXIE ///< Receive interrupt enable bit
  #define VCP_Rx_IF              _U3RXIF ///< Receive interrupt flag bit

  #define VCP_TxInterrupt       _U3TXInterrupt ///< Transmit interrupt serivce routine label 
  #define VCP_Tx_IP             _U3TXIP ///< Transmit interrupt priority register
  #define VCP_Tx_IE             _U3TXIE ///< Transmit interrupt enable bit
  #define VCP_Tx_IF             _U3TXIF ///< Transmit interrupt flag bit

#endif

#if  (VCP_TX_DMA == 0)

  #define VcpTxDmaInterrupt     _DMA0Interrupt///< Interrupt serivce routine label 
  #define VcpTxDma_IP           _DMA0IP ///< Transmit DMA interrupt priority register
  #define VcpTxDma_IE           _DMA0IE ///< Transmit DMA interrupt enable bit
  #define VcpTxDma_IF           _DMA0IF ///< Transmit DMA interrupt flag bit

#elif (VCP_TX_DMA == 1)

  #define VcpTxDmaInterrupt     _DMA1Interrupt///< Interrupt serivce routine label 
  #define VcpTxDma_IP           _DMA1IP ///< Transmit DMA interrupt priority register
  #define VcpTxDma_IE           _DMA1IE ///< Transmit DMA interrupt enable bit
  #define VcpTxDma_IF           _DMA1IF ///< Transmit DMA interrupt flag bit

#elif (VCP_TX_DMA == 2)

  #define VcpTxDmaInterrupt     _DMA2Interrupt///< Interrupt serivce routine label 
  #define VcpTxDma_IP           _DMA2IP ///< Transmit DMA interrupt priority register
  #define VcpTxDma_IE           _DMA2IE ///< Transmit DMA interrupt enable bit
  #define VcpTxDma_IF           _DMA2IF ///< Transmit DMA interrupt flag bit

#elif (VCP_TX_DMA == 3)

  #define VcpTxDmaInterrupt     _DMA3Interrupt///< Interrupt serivce routine label 
  #define VcpTxDma_IP           _DMA3IP ///< Transmit DMA interrupt priority register
  #define VcpTxDma_IE           _DMA3IE ///< Transmit DMA interrupt enable bit
  #define VcpTxDma_IF           _DMA3IF ///< Transmit DMA interrupt flag bit

#endif

#if  (VCP_RX_DMA == 0)

  #define VcpRxDmaInterrupt     _DMA0Interrupt///< Interrupt serivce routine label 
  #define VcpRxDma_IP           _DMA0IP ///< Transmit DMA interrupt priority register
  #define VcpRxDma_IE           _DMA0IE ///< Transmit DMA interrupt enable bit
  #define VcpRxDma_IF           _DMA0IF ///< Transmit DMA interrupt flag bit

#elif (VCP_RX_DMA == 1)

  #define VcpRxDmaInterrupt     _DMA1Interrupt///< Interrupt serivce routine label 
  #define VcpRxDma_IP           _DMA1IP ///< Transmit DMA interrupt priority register
  #define VcpRxDma_IE           _DMA1IE ///< Transmit DMA interrupt enable bit
  #define VcpRxDma_IF           _DMA1IF ///< Transmit DMA interrupt flag bit

#elif (VCP_RX_DMA == 2)

  #define VcpRxDmaInterrupt     _DMA2Interrupt///< Interrupt serivce routine label 
  #define VcpRxDma_IP           _DMA2IP ///< Transmit DMA interrupt priority register
  #define VcpRxDma_IE           _DMA2IE ///< Transmit DMA interrupt enable bit
  #define VcpRxDma_IF           _DMA2IF ///< Transmit DMA interrupt flag bit

#elif (VCP_RX_DMA == 3)

  #define VcpRxDmaInterrupt     _DMA3Interrupt///< Interrupt serivce routine label 
  #define VcpRxDma_IP           _DMA3IP ///< Transmit DMA interrupt priority register
  #define VcpRxDma_IE           _DMA3IE ///< Transmit DMA interrupt enable bit
  #define VcpRxDma_IF           _DMA3IF ///< Transmit DMA interrupt flag bit

#endif

#define CrcInterrupt            _CRCInterrupt ///< Interrupt serivce routine label 
#define CRC_IP                  _CRCIP ///< CRC interrupt priority register
#define CRC_IE                  _CRCIE ///< CRC interrupt enable bit
#define CRC_IF                  _CRCIF ///< CRC interrupt flag bit
#define CRCFIFO_FULL            _CRCFUL ///< CRC FIFO Full flag bit

#define VCP_RX_ISR_PRIORITY     2U ///< Receive interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)
#define VCP_TX_ISR_PRIORITY     2U ///< Transmit interrupt vector priority (valid settings between 0...6 with 6 being the highest priority)

/** @} */ // end of group comm-interrupt-vector-declarations

/**
 * @ingroup circuit-gpio-mcal-internal-i2c
 * @{
 */

// Debug Pin #1 on EPC9528 at External I2C, pin #1
#define ASCL                    PIN_RC5 ///< Alternate I2C Clock GPIO, overlaps with Debug Pin #1 on EPC9528
#define ASDA                    PIN_RC4 ///< Alternate I2C Data GPIO, overlaps with Debug Pin #2 on EPC9528

/** @} */ // end of group circuit-gpio-mcal-internal-i2c

/**
 * @ingroup circuit-gpio-mcal-internal-debug
 * @{
 */

// Debug Pin #1 on EPC9528 at External I2C, pin #1
#define DBGPIN1                 PIN_RC5 ///< Debug Pin #1 on EPC9528, overlaps with alternate I2C Clock GPIO
#define DBGPIN2                 PIN_RC4 ///< Debug Pin #2 on EPC9528, overlaps with alternate I2C Data GPIO

// Ref LED on EPC9528 at External I2C, pin #3
#define DBGLED                  PIN_RC4 ///< Debug LED overlapping with DBGPIN2

#define DBGLED_ON               1U  ///< pin state when debugging LED is on (1=HIGH, 0=LOW)
#define DBGLED_OFF              0U  ///< pin state when debugging LED is off (1=HIGH, 0=LOW)

/** @} */ // end of group circuit-gpio-mcal-internal-debug

/**************************************************************************************************
 * @ingroup circuit-peripheral-mcal
 * @{
 * @brief User-declaration of global defines for PWM signal generator settings
 * 
 * @details
 * This section defines fundamental PWM settings required for the interleaved buck converter
 * of the EPC9163 48V-to-12V rail power module development board. These settings are determined 
 * by hardware and defined using physical quantities. Pre-compiler macros are used to convert 
 * physical values into binary (integer) numbers to be written to Special Function Registers (SFR).
 * 
 **************************************************************************************************/

#define REF_DAC_INSTANCE         1U ///< Digital-To-Analog Converter instance used to generate current semse amplifier reference voltage
#define REF_BUFFER_OPA_INSTANCE  2U ///< Operational amplifier instance used to generate current semse amplifier reference voltage

/** @} */ // end of group circuit-peripheral-mcal

/**************************************************************************************************
 * @ingroup circuit-gpio-mcal-external
 * @{
 **************************************************************************************************/

#define CEC01   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #1  (is GND pin)
#define CEC02   PIN_RB10    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #2
#define CEC03   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #3  (is GND pin)
#define CEC04   PIN_RB11    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #4
#define CEC05   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #5  (is GND pin)
#define CEC06   PIN_RB12    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #6
#define CEC07   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #7  (is GND pin)
#define CEC08   PIN_RB13    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #8
#define CEC09   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #9  (is GND pin)
#define CEC10   PIN_RB14    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #10
#define CEC11   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #11 (is GND pin)
#define CEC12   PIN_RB15    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #12
#define CEC13   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #13 (is GND pin)
#define CEC14   PIN_RB6     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #14
// ---------
#define CEC15   PIN_NONE    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #15 => Card edge connector gap (no pins available)
#define CEC16   PIN_NONE    ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #16 => Card edge connector gap (no pins available)
// ---------
#define CEC17   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #17 (is GND pin)
#define CEC18   PIN_RB5     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #18
#define CEC19   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #19 (is GND pin)
#define CEC20   PIN_RC3     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #20
#define CEC21   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #21 (is GND pin)
#define CEC22   PIN_RB8     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #22
#define CEC23   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #23 (is GND pin)
#define CEC24   PIN_RA3     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #24
#define CEC25   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #25 (is GND pin)
#define CEC26   PIN_RB9     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #26
#define CEC27   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #27 (is GND pin)
#define CEC28   PIN_RC2     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #28
#define CEC29   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #29 (is GND pin)
#define CEC30   PIN_RC1     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #30
#define CEC31   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #31 (is GND pin)
#define CEC32   PIN_RB2     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #32
#define CEC33   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #33 (is GND pin)
#define CEC34   PIN_RB7     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #34
#define CEC35   PIN_VSS     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #35 (is GND pin)
#define CEC36   PIN_RA2     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #36
#define CEC37   PIN_RB1     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #37
#define CEC38   PIN_RA1     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #38
#define CEC39   PIN_RB0     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #39
#define CEC40   PIN_RC0     ///< EPC9528 Processor Plug-In Module Card Edge Connector Pin #40

/** @} */ // end of group circuit-gpio-mcal-external

#endif	/* HAL_EPC9528_R30_H */

// ________________________
// end if file
