/**
 * @file    init_dac.c
 * @brief   User configuration of on-chip Digital-To-Analog Converter (DAC)
 * @author  M91406
 * @date    05/19/2020
 * @version 1.0.3
 */

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdint.h> // include standard integer types header file
#include <stdbool.h> // include standard boolean types header file

#include "config/hal.h" // include common hardware abstraction header file
#include "init_dac.h" // include custom peripheral configuration header file

/***********************************************************************************
 * @ingroup dac-initialization
 * @brief   DAC Module Base Register Initialization
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function initializes the DAC module base registers for being operated 
 *  by the auxiliary oscillator at an input clock frequency of 500 MHz. 
 * 
 *  In this application the DAC is used to provide a constant reference voltage
 *  of VDD/2 = 1.65 V to the external current sense shunt amplifier devices.
 *  Neither, built-in comparator nor Pulse-Density Modulator (PDM) ramp generator
 *  are used.
 *
 **********************************************************************************/

uint16_t sysDacModule_Initialize(void) 
{
    int retval = 1;

    retval &= p33c_DacModule_SetConfig(dacModuleConfigClear);
    retval &= p33c_DacModule_SetConfig(dacModuleConfigDefault);

    return (retval);
    
}

/***********************************************************************************
 * @ingroup dac-initialization
 * @brief   Initializes a user specified DAC instance with DAC output enabled
 * @param   dacInstance Index of the selected DAC Instance (1=DAC1, 2=DAC2, etc.)
 * @param   initValue Integer value at startup of the DAC peripheral instance
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function clears all registers of the declared DAC instance and 
 *  initializes the registers required by a static Digital-to-Analog Converter
 *  output with the given default values.
 * 
 * @note
 *  The initialization function does not enable the DAC instance. Please use
 *  function sysDacOutput_Enable() to turn on the DAC instance at the appropriate
 *  time.
 *
 **********************************************************************************/

uint16_t sysDacOutput_Initialize(uint16_t dacInstance, uint16_t initValue)
{
    int retval = 1;
    volatile P33C_DAC_INSTANCE_t* dac;
    
    dac = p33c_DacInstance_GetHandle(dacInstance);
    retval &= p33c_DacInstance_Disable(dacInstance);
    
    dac->DACxCONL.bits.DACOEN = 1; // Enable DAC output pin
    dac->DACxCONH.bits.TMCB  = LEB_PERIOD; // Set Leading Edge blanking interval period
    dac->DACxDATH.value = initValue; // Set DAC output to initial value
    
    return (retval);
}

/***********************************************************************************
 * @ingroup dac-initialization
 * @brief   Enables a user specified DAC output
 * @param   dacInstance Index of the selected DAC Instance (1=DAC1, 2=DAC2, etc.)
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function enables the DAC output and DAC module specified in parameter
 *  dacInstance. This function should be called after the DAC instance has been 
 *  initialized by calling function sysDacOutput_Initialize().
 *
 **********************************************************************************/

uint16_t sysDacOutput_Enable(uint16_t dacInstance) {
    
    int retval = 1;
    volatile P33C_DAC_MODULE_t* dacmod;
    volatile P33C_DAC_INSTANCE_t* dacinst;
    
    // Capture DAC module SFR set
    dacmod = p33c_DacModule_GetHandle();
    dacmod->DACCTRL1L.bits.DACON = true; // Ensure DAC module is enabled

    // Capture specified DAC instance SFR set
    dacinst = p33c_DacInstance_GetHandle(dacInstance);
    dacinst->DACxCONL.bits.DACEN = true; // Enable DAC instance
    dacinst->DACxCONL.bits.DACOEN = true; // Enable DAC output pin
    
    return (retval);
}

/***********************************************************************************
 * @ingroup dac-initialization
 * @brief   Disables a user specified DAC output
 * @param   dacInstance Index of the selected DAC Instance (1=DAC1, 2=DAC2, etc.)
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  This function disables the DAC output and DAC module specified in parameter
 *  dacInstance. 
 *
 **********************************************************************************/

uint16_t sysDacOutput_Disable(uint16_t dacInstance) {
    
    int retval = 1;
    volatile P33C_DAC_INSTANCE_t* dacinst;
    
    // Capture specified DAC instance SFR set
    dacinst = p33c_DacInstance_GetHandle(dacInstance);
    dacinst->DACxCONL.bits.DACOEN = false; // Disable DAC output pin
    dacinst->DACxCONL.bits.DACEN = false; // Disable DAC instance

    return (retval);
}

// _______________________
// end of file
