/**
 * @file    init_opa.c
 * @brief   User configuration of on-chip Operational Amplifier (OPA)
 * @author  M91406
 * @date    05/19/2020
 * @version 1.0.3
 */

#include <xc_pral.h> // include device peripheral register abstraction layer
#include "init_opa.h" // include custom peripheral configuration header file

/***********************************************************************************
 * @ingroup op-amp-initialization
 * @brief   Initializes the on-chip operational amplifier module
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  The EPC9143 300W 16th brick power module reference designs uses one of the on-chip 
 *  operational amplifiers as buffer op-amp for the current reference signal applied
 *  to external high-side current sense shunt amplifier devices. This operational
 *  amplifier is initialized here.
 * 
 **********************************************************************************/

uint16_t sysOpAmp_Initialize(uint16_t opaInstance, bool disable_n_channel) 
{
    int retval = 1;
    uint16_t idum=0;
    volatile P33C_OPA_MODULE_t* opa = p33c_OpaModule_GetHandle();

    // If instance is set = 0, op-amp will be skipped and op-amps will be turned off
    if(opaInstance > 0)
    {
        // Disables Op Amp N channel input stage
        if (disable_n_channel)
        {
            idum = (((uint16_t)disable_n_channel) << (opaInstance - 1));
            opa->AMPCON1H.value |= idum;
        }
        else
        {
            idum = (1 << (opaInstance - 1));
            opa->AMPCON1H.value &= (~idum);
        }
        
        // Enable op-amp instance
        idum = (0x0001 << (opaInstance - 1));
        opa->AMPCON1L.value |= idum;
    }
    
    return (retval);
}

/***********************************************************************************
 * @ingroup op-amp-initialization
 * @brief   Enables the on-chip operational amplifier module
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  The on-chip operational amplifier module allows the configuration of multiple 
 *  operational amplifier instances. The module itself provides an additional 
 *  master enable bit to turn on/off the entire module.
 * 
 **********************************************************************************/

uint16_t sysOpAmp_ModuleEnable(void)
{
    volatile P33C_OPA_MODULE_t* opa = p33c_OpaModule_GetHandle();
    opa->AMPCON1L.bits.AMPON = true; // Enable Op-Amp module
    return (opa->AMPCON1L.bits.AMPON);
}


/***********************************************************************************
 * @ingroup op-amp-initialization
 * @brief   Resets the on-chip operational amplifier module
 * @return  unsigned integer (0=failure, 1=success)
 * 
 * @details
 *  The on-chip operational amplifier module allows the configuration of multiple 
 *  operational amplifier instances. The module itself provides an additional 
 *  master enable bit to turn on/off the entire module as well as to enable/disable 
 *  the N-inputs of individual op-amps. This function resets the op-amp module 
 *  to its default disabling the N-channel inputs of all op-amps, disables all
 *  individual op-amps as well as turns of the op-amp module master switch.
 * 
 **********************************************************************************/

uint16_t sysOpAmp_ModuleReset(void)
{
    volatile P33C_OPA_MODULE_t* opa = p33c_OpaModule_GetHandle();

    // Disable and reset op-amp module
    *opa = opaModuleDefault;    // Load configuration defaults
    opa->AMPCON1L.value = 0;    // Disable and reset Op-Amp module

    return (1-opa->AMPCON1L.bits.AMPON);
}

// _____________________________
// end of file
